/* testbidirlist.c generated by valac 0.56.3, the Vala compiler
 * generated from testbidirlist.vala, do not modify */

/* testbidirlist.vala
 *
 * Copyright (C) 2011  Maciej Piechotka
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Maciej Piechotka <uzytkownik2@gmail.com>
 */

#include <glib-object.h>
#include <gee.h>
#include <stdlib.h>
#include <string.h>
#include <glib.h>

#if !defined(VALA_EXTERN)
#if defined(_MSC_VER)
#define VALA_EXTERN __declspec(dllexport) extern
#elif __GNUC__ >= 4
#define VALA_EXTERN __attribute__((visibility("default"))) extern
#else
#define VALA_EXTERN extern
#endif
#endif

#define GEE_TYPE_TEST_CASE (gee_test_case_get_type ())
#define GEE_TEST_CASE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_TEST_CASE, GeeTestCase))
#define GEE_TEST_CASE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_TYPE_TEST_CASE, GeeTestCaseClass))
#define GEE_IS_TEST_CASE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_TEST_CASE))
#define GEE_IS_TEST_CASE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEE_TYPE_TEST_CASE))
#define GEE_TEST_CASE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_TYPE_TEST_CASE, GeeTestCaseClass))

typedef struct _GeeTestCase GeeTestCase;
typedef struct _GeeTestCaseClass GeeTestCaseClass;
typedef struct _GeeTestCasePrivate GeeTestCasePrivate;

#define TYPE_COLLECTION_TESTS (collection_tests_get_type ())
#define COLLECTION_TESTS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_COLLECTION_TESTS, CollectionTests))
#define COLLECTION_TESTS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_COLLECTION_TESTS, CollectionTestsClass))
#define IS_COLLECTION_TESTS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_COLLECTION_TESTS))
#define IS_COLLECTION_TESTS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_COLLECTION_TESTS))
#define COLLECTION_TESTS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_COLLECTION_TESTS, CollectionTestsClass))

typedef struct _CollectionTests CollectionTests;
typedef struct _CollectionTestsClass CollectionTestsClass;
typedef struct _CollectionTestsPrivate CollectionTestsPrivate;

#define TYPE_LIST_TESTS (list_tests_get_type ())
#define LIST_TESTS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_LIST_TESTS, ListTests))
#define LIST_TESTS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_LIST_TESTS, ListTestsClass))
#define IS_LIST_TESTS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_LIST_TESTS))
#define IS_LIST_TESTS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_LIST_TESTS))
#define LIST_TESTS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_LIST_TESTS, ListTestsClass))

typedef struct _ListTests ListTests;
typedef struct _ListTestsClass ListTestsClass;
typedef struct _ListTestsPrivate ListTestsPrivate;

#define TYPE_BIDIR_LIST_TESTS (bidir_list_tests_get_type ())
#define BIDIR_LIST_TESTS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_BIDIR_LIST_TESTS, BidirListTests))
#define BIDIR_LIST_TESTS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_BIDIR_LIST_TESTS, BidirListTestsClass))
#define IS_BIDIR_LIST_TESTS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_BIDIR_LIST_TESTS))
#define IS_BIDIR_LIST_TESTS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_BIDIR_LIST_TESTS))
#define BIDIR_LIST_TESTS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_BIDIR_LIST_TESTS, BidirListTestsClass))

typedef struct _BidirListTests BidirListTests;
typedef struct _BidirListTestsClass BidirListTestsClass;
typedef struct _BidirListTestsPrivate BidirListTestsPrivate;
enum  {
	BIDIR_LIST_TESTS_0_PROPERTY,
	BIDIR_LIST_TESTS_NUM_PROPERTIES
};
static GParamSpec* bidir_list_tests_properties[BIDIR_LIST_TESTS_NUM_PROPERTIES];
typedef void (*GeeTestCaseTestMethod) (gpointer user_data);
#define _g_object_unref0(var) ((var == NULL) ? NULL : (var = (g_object_unref (var), NULL)))
#define _g_free0(var) (var = (g_free (var), NULL))
#define _vala_assert(expr, msg) if G_LIKELY (expr) ; else g_assertion_message_expr (G_LOG_DOMAIN, __FILE__, __LINE__, G_STRFUNC, msg);
#define _vala_return_if_fail(expr, msg) if G_LIKELY (expr) ; else { g_return_if_fail_warning (G_LOG_DOMAIN, G_STRFUNC, msg); return; }
#define _vala_return_val_if_fail(expr, msg, val) if G_LIKELY (expr) ; else { g_return_if_fail_warning (G_LOG_DOMAIN, G_STRFUNC, msg); return val; }
#define _vala_warn_if_fail(expr, msg) if G_LIKELY (expr) ; else g_warn_message (G_LOG_DOMAIN, __FILE__, __LINE__, G_STRFUNC, msg);

struct _GeeTestCase {
	GObject parent_instance;
	GeeTestCasePrivate * priv;
};

struct _GeeTestCaseClass {
	GObjectClass parent_class;
	void (*set_up) (GeeTestCase* self);
	void (*tear_down) (GeeTestCase* self);
};

struct _CollectionTests {
	GeeTestCase parent_instance;
	CollectionTestsPrivate * priv;
	GeeCollection* test_collection;
};

struct _CollectionTestsClass {
	GeeTestCaseClass parent_class;
};

struct _ListTests {
	CollectionTests parent_instance;
	ListTestsPrivate * priv;
};

struct _ListTestsClass {
	CollectionTestsClass parent_class;
	void (*test_duplicates_are_retained) (ListTests* self);
};

struct _BidirListTests {
	ListTests parent_instance;
	BidirListTestsPrivate * priv;
};

struct _BidirListTestsClass {
	ListTestsClass parent_class;
};

static gpointer bidir_list_tests_parent_class = NULL;

VALA_EXTERN GType gee_test_case_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (GeeTestCase, g_object_unref)
VALA_EXTERN GType collection_tests_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (CollectionTests, g_object_unref)
VALA_EXTERN GType list_tests_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (ListTests, g_object_unref)
VALA_EXTERN GType bidir_list_tests_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (BidirListTests, g_object_unref)
VALA_EXTERN BidirListTests* bidir_list_tests_construct (GType object_type,
                                            const gchar* name);
VALA_EXTERN ListTests* list_tests_construct (GType object_type,
                                 const gchar* name);
VALA_EXTERN void gee_test_case_add_test (GeeTestCase* self,
                             const gchar* name,
                             GeeTestCaseTestMethod test,
                             gpointer test_target,
                             GDestroyNotify test_target_destroy_notify);
VALA_EXTERN void bidir_list_tests_test_bidir_list_iterator (BidirListTests* self);
static void _bidir_list_tests_test_bidir_list_iterator_gee_test_case_test_method (gpointer self);
static GType bidir_list_tests_get_type_once (void);

static void
_bidir_list_tests_test_bidir_list_iterator_gee_test_case_test_method (gpointer self)
{
	bidir_list_tests_test_bidir_list_iterator ((BidirListTests*) self);
}

BidirListTests*
bidir_list_tests_construct (GType object_type,
                            const gchar* name)
{
	BidirListTests * self = NULL;
	g_return_val_if_fail (name != NULL, NULL);
	self = (BidirListTests*) list_tests_construct (object_type, name);
	gee_test_case_add_test ((GeeTestCase*) self, "[List] bi-directional list iterator", _bidir_list_tests_test_bidir_list_iterator_gee_test_case_test_method, g_object_ref (self), g_object_unref);
	return self;
}

static gpointer
_g_object_ref0 (gpointer self)
{
	return self ? g_object_ref (self) : NULL;
}

void
bidir_list_tests_test_bidir_list_iterator (BidirListTests* self)
{
	GeeBidirList* test_list = NULL;
	GeeCollection* _tmp0_;
	GeeBidirList* _tmp1_;
	GeeBidirListIterator* iterator = NULL;
	GeeBidirListIterator* _tmp2_;
	GeeBidirListIterator* _tmp3_;
	GeeBidirListIterator* _tmp4_;
	GeeBidirListIterator* _tmp5_;
	GeeBidirListIterator* _tmp6_;
	GeeBidirListIterator* _tmp7_;
	GeeBidirListIterator* _tmp8_;
	GeeBidirListIterator* _tmp9_;
	GeeBidirListIterator* _tmp10_;
	GeeBidirListIterator* _tmp11_;
	gpointer _tmp12_;
	gchar* _tmp13_;
	GeeBidirListIterator* _tmp14_;
	GeeBidirListIterator* _tmp15_;
	GeeBidirListIterator* _tmp16_;
	GeeBidirListIterator* _tmp17_;
	gpointer _tmp18_;
	gchar* _tmp19_;
	GeeBidirListIterator* _tmp20_;
	GeeBidirListIterator* _tmp21_;
	gint _tmp22_;
	gint _tmp23_;
	GeeBidirListIterator* _tmp24_;
	GeeBidirListIterator* _tmp25_;
	gint _tmp26_;
	gint _tmp27_;
	GeeBidirListIterator* _tmp28_;
	GeeBidirListIterator* _tmp29_;
	gint _tmp30_;
	gint _tmp31_;
	GeeBidirListIterator* _tmp32_;
	GeeBidirListIterator* _tmp33_;
	GeeBidirListIterator* _tmp34_;
	gpointer _tmp35_;
	gchar* _tmp36_;
	GeeBidirListIterator* _tmp37_;
	GeeBidirListIterator* _tmp38_;
	GeeBidirListIterator* _tmp39_;
	GeeBidirListIterator* _tmp40_;
	GeeBidirListIterator* _tmp41_;
	GeeBidirListIterator* _tmp42_;
	gpointer _tmp43_;
	gchar* _tmp44_;
	GeeBidirListIterator* _tmp45_;
	GeeBidirListIterator* _tmp46_;
	GeeBidirListIterator* _tmp47_;
	GeeBidirListIterator* _tmp48_;
	GeeBidirListIterator* _tmp49_;
	gpointer _tmp50_;
	gchar* _tmp51_;
	GeeBidirListIterator* _tmp52_;
	GeeBidirListIterator* _tmp53_;
	GeeBidirListIterator* _tmp54_;
	GeeBidirListIterator* _tmp55_;
	GeeBidirListIterator* _tmp56_;
	GeeBidirListIterator* _tmp57_;
	gpointer _tmp58_;
	gchar* _tmp59_;
	GeeBidirListIterator* _tmp60_;
	GeeBidirListIterator* _tmp61_;
	GeeBidirListIterator* _tmp62_;
	GeeBidirListIterator* _tmp63_;
	gpointer _tmp64_;
	gchar* _tmp65_;
	GeeBidirListIterator* _tmp66_;
	GeeBidirListIterator* _tmp67_;
	GeeBidirListIterator* _tmp68_;
	GeeBidirListIterator* _tmp69_;
	gpointer _tmp70_;
	gchar* _tmp71_;
	GeeBidirListIterator* _tmp72_;
	GeeBidirListIterator* _tmp73_;
	GeeBidirListIterator* _tmp74_;
	GeeBidirListIterator* _tmp75_;
	gpointer _tmp76_;
	gchar* _tmp77_;
	GeeBidirListIterator* _tmp78_;
	g_return_if_fail (self != NULL);
	_tmp0_ = ((CollectionTests*) self)->test_collection;
	_tmp1_ = _g_object_ref0 (G_TYPE_CHECK_INSTANCE_TYPE (_tmp0_, GEE_TYPE_BIDIR_LIST) ? ((GeeBidirList*) _tmp0_) : NULL);
	test_list = _tmp1_;
	_vala_assert (test_list != NULL, "test_list != null");
	_tmp2_ = gee_bidir_list_bidir_list_iterator (test_list);
	iterator = _tmp2_;
	_tmp3_ = iterator;
	_vala_assert (!gee_iterator_has_next ((GeeIterator*) _tmp3_), "! iterator.has_next ()");
	_tmp4_ = iterator;
	_vala_assert (!gee_iterator_next ((GeeIterator*) _tmp4_), "! iterator.next ()");
	_tmp5_ = iterator;
	_vala_assert (!gee_bidir_iterator_has_previous ((GeeBidirIterator*) _tmp5_), "! iterator.has_previous ()");
	_tmp6_ = iterator;
	_vala_assert (!gee_bidir_iterator_previous ((GeeBidirIterator*) _tmp6_), "! iterator.previous ()");
	_tmp7_ = iterator;
	_vala_assert (!gee_bidir_iterator_first ((GeeBidirIterator*) _tmp7_), "! iterator.first ()");
	_tmp8_ = iterator;
	_vala_assert (!gee_bidir_iterator_last ((GeeBidirIterator*) _tmp8_), "! iterator.last ()");
	_vala_assert (gee_collection_add ((GeeCollection*) test_list, "one"), "test_list.add (\"one\")");
	_vala_assert (gee_collection_add ((GeeCollection*) test_list, "two"), "test_list.add (\"two\")");
	_vala_assert (gee_collection_add ((GeeCollection*) test_list, "three"), "test_list.add (\"three\")");
	_tmp9_ = gee_bidir_list_bidir_list_iterator (test_list);
	_g_object_unref0 (iterator);
	iterator = _tmp9_;
	_tmp10_ = iterator;
	_vala_assert (gee_iterator_next ((GeeIterator*) _tmp10_), "iterator.next()");
	_tmp11_ = iterator;
	_tmp12_ = gee_iterator_get ((GeeIterator*) _tmp11_);
	_tmp13_ = (gchar*) _tmp12_;
	_vala_assert (g_strcmp0 (_tmp13_, "one") == 0, "iterator.get () == \"one\"");
	_g_free0 (_tmp13_);
	_tmp14_ = iterator;
	_vala_assert (gee_list_iterator_index ((GeeListIterator*) _tmp14_) == 0, "iterator.index () == 0");
	_tmp15_ = iterator;
	gee_list_iterator_set (G_TYPE_CHECK_INSTANCE_CAST (_tmp15_, GEE_TYPE_LIST_ITERATOR, GeeListIterator), "new one");
	_tmp16_ = iterator;
	_vala_assert (gee_iterator_next ((GeeIterator*) _tmp16_), "iterator.next()");
	_tmp17_ = iterator;
	_tmp18_ = gee_iterator_get ((GeeIterator*) _tmp17_);
	_tmp19_ = (gchar*) _tmp18_;
	_vala_assert (g_strcmp0 (_tmp19_, "two") == 0, "iterator.get () == \"two\"");
	_g_free0 (_tmp19_);
	_tmp20_ = iterator;
	_vala_assert (gee_list_iterator_index ((GeeListIterator*) _tmp20_) == 1, "iterator.index () == 1");
	_tmp21_ = iterator;
	gee_list_iterator_set (G_TYPE_CHECK_INSTANCE_CAST (_tmp21_, GEE_TYPE_LIST_ITERATOR, GeeListIterator), "new two");
	_tmp22_ = gee_collection_get_size ((GeeCollection*) test_list);
	_tmp23_ = _tmp22_;
	_vala_assert (_tmp23_ == 3, "test_list.size == 3");
	_tmp24_ = iterator;
	_vala_assert (gee_list_iterator_index ((GeeListIterator*) _tmp24_) == 1, "iterator.index () == 1");
	_tmp25_ = iterator;
	gee_bidir_list_iterator_insert (_tmp25_, "before two");
	_tmp26_ = gee_collection_get_size ((GeeCollection*) test_list);
	_tmp27_ = _tmp26_;
	_vala_assert (_tmp27_ == 4, "test_list.size == 4");
	_tmp28_ = iterator;
	_vala_assert (gee_list_iterator_index ((GeeListIterator*) _tmp28_) == 2, "iterator.index () == 2");
	_tmp29_ = iterator;
	gee_list_iterator_add ((GeeListIterator*) _tmp29_, "after two");
	_tmp30_ = gee_collection_get_size ((GeeCollection*) test_list);
	_tmp31_ = _tmp30_;
	_vala_assert (_tmp31_ == 5, "test_list.size == 5");
	_tmp32_ = iterator;
	_vala_assert (gee_list_iterator_index ((GeeListIterator*) _tmp32_) == 3, "iterator.index () == 3");
	_tmp33_ = iterator;
	_vala_assert (gee_iterator_next ((GeeIterator*) _tmp33_), "iterator.next()");
	_tmp34_ = iterator;
	_tmp35_ = gee_iterator_get ((GeeIterator*) _tmp34_);
	_tmp36_ = (gchar*) _tmp35_;
	_vala_assert (g_strcmp0 (_tmp36_, "three") == 0, "iterator.get () == \"three\"");
	_g_free0 (_tmp36_);
	_tmp37_ = iterator;
	_vala_assert (gee_list_iterator_index ((GeeListIterator*) _tmp37_) == 4, "iterator.index () == 4");
	_tmp38_ = iterator;
	gee_list_iterator_set (G_TYPE_CHECK_INSTANCE_CAST (_tmp38_, GEE_TYPE_LIST_ITERATOR, GeeListIterator), "new three");
	_tmp39_ = iterator;
	_vala_assert (!gee_iterator_has_next ((GeeIterator*) _tmp39_), "! iterator.has_next ()");
	_tmp40_ = iterator;
	_vala_assert (!gee_iterator_next ((GeeIterator*) _tmp40_), "! iterator.next ()");
	_tmp41_ = iterator;
	_vala_assert (gee_bidir_iterator_first ((GeeBidirIterator*) _tmp41_), "iterator.first ()");
	_tmp42_ = iterator;
	_tmp43_ = gee_iterator_get ((GeeIterator*) _tmp42_);
	_tmp44_ = (gchar*) _tmp43_;
	_vala_assert (g_strcmp0 (_tmp44_, "new one") == 0, "iterator.get () == \"new one\"");
	_g_free0 (_tmp44_);
	_tmp45_ = iterator;
	_vala_assert (gee_list_iterator_index ((GeeListIterator*) _tmp45_) == 0, "iterator.index () == 0");
	_tmp46_ = iterator;
	_vala_assert (!gee_bidir_iterator_has_previous ((GeeBidirIterator*) _tmp46_), "! iterator.has_previous ()");
	_tmp47_ = iterator;
	_vala_assert (!gee_bidir_iterator_previous ((GeeBidirIterator*) _tmp47_), "! iterator.previous ()");
	_tmp48_ = iterator;
	_vala_assert (gee_bidir_iterator_last ((GeeBidirIterator*) _tmp48_), "iterator.last ()");
	_tmp49_ = iterator;
	_tmp50_ = gee_iterator_get ((GeeIterator*) _tmp49_);
	_tmp51_ = (gchar*) _tmp50_;
	_vala_assert (g_strcmp0 (_tmp51_, "new three") == 0, "iterator.get () == \"new three\"");
	_g_free0 (_tmp51_);
	_tmp52_ = iterator;
	_vala_assert (gee_list_iterator_index ((GeeListIterator*) _tmp52_) == 4, "iterator.index () == 4");
	_tmp53_ = iterator;
	_vala_assert (!gee_iterator_has_next ((GeeIterator*) _tmp53_), "! iterator.has_next ()");
	_tmp54_ = iterator;
	_vala_assert (!gee_iterator_next ((GeeIterator*) _tmp54_), "! iterator.next ()");
	_tmp55_ = iterator;
	_vala_assert (gee_bidir_iterator_has_previous ((GeeBidirIterator*) _tmp55_), "iterator.has_previous ()");
	_tmp56_ = iterator;
	_vala_assert (gee_bidir_iterator_previous ((GeeBidirIterator*) _tmp56_), "iterator.previous ()");
	_tmp57_ = iterator;
	_tmp58_ = gee_iterator_get ((GeeIterator*) _tmp57_);
	_tmp59_ = (gchar*) _tmp58_;
	_vala_assert (g_strcmp0 (_tmp59_, "after two") == 0, "iterator.get () == \"after two\"");
	_g_free0 (_tmp59_);
	_tmp60_ = iterator;
	_vala_assert (gee_list_iterator_index ((GeeListIterator*) _tmp60_) == 3, "iterator.index () == 3");
	_tmp61_ = iterator;
	_vala_assert (gee_bidir_iterator_has_previous ((GeeBidirIterator*) _tmp61_), "iterator.has_previous ()");
	_tmp62_ = iterator;
	_vala_assert (gee_bidir_iterator_previous ((GeeBidirIterator*) _tmp62_), "iterator.previous ()");
	_tmp63_ = iterator;
	_tmp64_ = gee_iterator_get ((GeeIterator*) _tmp63_);
	_tmp65_ = (gchar*) _tmp64_;
	_vala_assert (g_strcmp0 (_tmp65_, "new two") == 0, "iterator.get () == \"new two\"");
	_g_free0 (_tmp65_);
	_tmp66_ = iterator;
	_vala_assert (gee_list_iterator_index ((GeeListIterator*) _tmp66_) == 2, "iterator.index () == 2");
	_tmp67_ = iterator;
	_vala_assert (gee_bidir_iterator_has_previous ((GeeBidirIterator*) _tmp67_), "iterator.has_previous ()");
	_tmp68_ = iterator;
	_vala_assert (gee_bidir_iterator_previous ((GeeBidirIterator*) _tmp68_), "iterator.previous ()");
	_tmp69_ = iterator;
	_tmp70_ = gee_iterator_get ((GeeIterator*) _tmp69_);
	_tmp71_ = (gchar*) _tmp70_;
	_vala_assert (g_strcmp0 (_tmp71_, "before two") == 0, "iterator.get () == \"before two\"");
	_g_free0 (_tmp71_);
	_tmp72_ = iterator;
	_vala_assert (gee_list_iterator_index ((GeeListIterator*) _tmp72_) == 1, "iterator.index () == 1");
	_tmp73_ = iterator;
	_vala_assert (gee_bidir_iterator_has_previous ((GeeBidirIterator*) _tmp73_), "iterator.has_previous ()");
	_tmp74_ = iterator;
	_vala_assert (gee_bidir_iterator_previous ((GeeBidirIterator*) _tmp74_), "iterator.previous ()");
	_tmp75_ = iterator;
	_tmp76_ = gee_iterator_get ((GeeIterator*) _tmp75_);
	_tmp77_ = (gchar*) _tmp76_;
	_vala_assert (g_strcmp0 (_tmp77_, "new one") == 0, "iterator.get () == \"new one\"");
	_g_free0 (_tmp77_);
	_tmp78_ = iterator;
	_vala_assert (gee_list_iterator_index ((GeeListIterator*) _tmp78_) == 0, "iterator.index () == 0");
	_g_object_unref0 (iterator);
	_g_object_unref0 (test_list);
}

static void
bidir_list_tests_class_init (BidirListTestsClass * klass,
                             gpointer klass_data)
{
	bidir_list_tests_parent_class = g_type_class_peek_parent (klass);
}

static void
bidir_list_tests_instance_init (BidirListTests * self,
                                gpointer klass)
{
}

static GType
bidir_list_tests_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (BidirListTestsClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) bidir_list_tests_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (BidirListTests), 0, (GInstanceInitFunc) bidir_list_tests_instance_init, NULL };
	GType bidir_list_tests_type_id;
	bidir_list_tests_type_id = g_type_register_static (TYPE_LIST_TESTS, "BidirListTests", &g_define_type_info, G_TYPE_FLAG_ABSTRACT);
	return bidir_list_tests_type_id;
}

GType
bidir_list_tests_get_type (void)
{
	static volatile gsize bidir_list_tests_type_id__once = 0;
	if (g_once_init_enter (&bidir_list_tests_type_id__once)) {
		GType bidir_list_tests_type_id;
		bidir_list_tests_type_id = bidir_list_tests_get_type_once ();
		g_once_init_leave (&bidir_list_tests_type_id__once, bidir_list_tests_type_id);
	}
	return bidir_list_tests_type_id__once;
}

