/* bidirlist.c generated by valac 0.56.3, the Vala compiler
 * generated from bidirlist.vala, do not modify */

/* bidirlist.vala
 *
 * Copyright (C) 2011  Maciej Piechotka
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Maciej Piechotka <uzytkownik2@gmail.com>
 */

#include "gee.h"
#include <glib-object.h>
#include <glib.h>

static GType gee_bidir_list_get_type_once (void);

/**
 * Returns a BidirListIterator that can be used for iteration over this list.
 *
 * @return a BidirListIterator that can be used for iteration over this list
 */
GeeBidirListIterator*
gee_bidir_list_bidir_list_iterator (GeeBidirList* self)
{
	GeeBidirListIface* _iface_;
	g_return_val_if_fail (self != NULL, NULL);
	_iface_ = GEE_BIDIR_LIST_GET_INTERFACE (self);
	if (_iface_->bidir_list_iterator) {
		return _iface_->bidir_list_iterator (self);
	}
	return NULL;
}

GeeBidirList*
gee_bidir_list_get_read_only_view (GeeBidirList* self)
{
	GeeBidirListIface* _iface_;
	g_return_val_if_fail (self != NULL, NULL);
	_iface_ = GEE_BIDIR_LIST_GET_INTERFACE (self);
	if (_iface_->get_read_only_view) {
		return _iface_->get_read_only_view (self);
	}
	return NULL;
}

static void
gee_bidir_list_default_init (GeeBidirListIface * iface,
                             gpointer iface_data)
{
	/**
	 * The read-only view of this list.
	 */
	g_object_interface_install_property (iface, g_param_spec_object ("read-only-view", "read-only-view", "read-only-view", GEE_TYPE_BIDIR_LIST, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE));
}

static GType
gee_bidir_list_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (GeeBidirListIface), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) gee_bidir_list_default_init, (GClassFinalizeFunc) NULL, NULL, 0, 0, (GInstanceInitFunc) NULL, NULL };
	GType gee_bidir_list_type_id;
	gee_bidir_list_type_id = g_type_register_static (G_TYPE_INTERFACE, "GeeBidirList", &g_define_type_info, 0);
	g_type_interface_add_prerequisite (gee_bidir_list_type_id, GEE_TYPE_LIST);
	return gee_bidir_list_type_id;
}

GType
gee_bidir_list_get_type (void)
{
	static volatile gsize gee_bidir_list_type_id__once = 0;
	if (g_once_init_enter (&gee_bidir_list_type_id__once)) {
		GType gee_bidir_list_type_id;
		gee_bidir_list_type_id = gee_bidir_list_get_type_once ();
		g_once_init_leave (&gee_bidir_list_type_id__once, gee_bidir_list_type_id);
	}
	return gee_bidir_list_type_id__once;
}

