/* abstractbidirsortedmap.c generated by valac 0.56.3, the Vala compiler
 * generated from abstractbidirsortedmap.vala, do not modify */

/* abstractbidirsortedmap.vala
 *
 * Copyright (C) 2012  Maciej Piechotka
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Maciej Piechotka <uzytkownik2@gmail.com>
 */

#include "gee.h"
#include <glib-object.h>
#include <glib.h>

enum  {
	GEE_ABSTRACT_BIDIR_SORTED_MAP_0_PROPERTY,
	GEE_ABSTRACT_BIDIR_SORTED_MAP_K_TYPE,
	GEE_ABSTRACT_BIDIR_SORTED_MAP_K_DUP_FUNC,
	GEE_ABSTRACT_BIDIR_SORTED_MAP_K_DESTROY_FUNC,
	GEE_ABSTRACT_BIDIR_SORTED_MAP_V_TYPE,
	GEE_ABSTRACT_BIDIR_SORTED_MAP_V_DUP_FUNC,
	GEE_ABSTRACT_BIDIR_SORTED_MAP_V_DESTROY_FUNC,
	GEE_ABSTRACT_BIDIR_SORTED_MAP_READ_ONLY_VIEW_PROPERTY,
	GEE_ABSTRACT_BIDIR_SORTED_MAP_NUM_PROPERTIES
};
static GParamSpec* gee_abstract_bidir_sorted_map_properties[GEE_ABSTRACT_BIDIR_SORTED_MAP_NUM_PROPERTIES];

#define GEE_TYPE_READ_ONLY_MAP (gee_read_only_map_get_type ())
#define GEE_READ_ONLY_MAP(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_READ_ONLY_MAP, GeeReadOnlyMap))
#define GEE_READ_ONLY_MAP_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_TYPE_READ_ONLY_MAP, GeeReadOnlyMapClass))
#define GEE_IS_READ_ONLY_MAP(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_READ_ONLY_MAP))
#define GEE_IS_READ_ONLY_MAP_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEE_TYPE_READ_ONLY_MAP))
#define GEE_READ_ONLY_MAP_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_TYPE_READ_ONLY_MAP, GeeReadOnlyMapClass))

typedef struct _GeeReadOnlyMap GeeReadOnlyMap;
typedef struct _GeeReadOnlyMapClass GeeReadOnlyMapClass;

#define GEE_TYPE_READ_ONLY_SORTED_MAP (gee_read_only_sorted_map_get_type ())
#define GEE_READ_ONLY_SORTED_MAP(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_READ_ONLY_SORTED_MAP, GeeReadOnlySortedMap))
#define GEE_READ_ONLY_SORTED_MAP_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_TYPE_READ_ONLY_SORTED_MAP, GeeReadOnlySortedMapClass))
#define GEE_IS_READ_ONLY_SORTED_MAP(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_READ_ONLY_SORTED_MAP))
#define GEE_IS_READ_ONLY_SORTED_MAP_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEE_TYPE_READ_ONLY_SORTED_MAP))
#define GEE_READ_ONLY_SORTED_MAP_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_TYPE_READ_ONLY_SORTED_MAP, GeeReadOnlySortedMapClass))

typedef struct _GeeReadOnlySortedMap GeeReadOnlySortedMap;
typedef struct _GeeReadOnlySortedMapClass GeeReadOnlySortedMapClass;

#define GEE_TYPE_READ_ONLY_BIDIR_SORTED_MAP (gee_read_only_bidir_sorted_map_get_type ())
#define GEE_READ_ONLY_BIDIR_SORTED_MAP(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_READ_ONLY_BIDIR_SORTED_MAP, GeeReadOnlyBidirSortedMap))
#define GEE_READ_ONLY_BIDIR_SORTED_MAP_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_TYPE_READ_ONLY_BIDIR_SORTED_MAP, GeeReadOnlyBidirSortedMapClass))
#define GEE_IS_READ_ONLY_BIDIR_SORTED_MAP(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_READ_ONLY_BIDIR_SORTED_MAP))
#define GEE_IS_READ_ONLY_BIDIR_SORTED_MAP_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEE_TYPE_READ_ONLY_BIDIR_SORTED_MAP))
#define GEE_READ_ONLY_BIDIR_SORTED_MAP_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_TYPE_READ_ONLY_BIDIR_SORTED_MAP, GeeReadOnlyBidirSortedMapClass))

typedef struct _GeeReadOnlyBidirSortedMap GeeReadOnlyBidirSortedMap;
typedef struct _GeeReadOnlyBidirSortedMapClass GeeReadOnlyBidirSortedMapClass;
#define _g_object_unref0(var) ((var == NULL) ? NULL : (var = (g_object_unref (var), NULL)))

struct _GeeAbstractBidirSortedMapPrivate {
	GType k_type;
	GBoxedCopyFunc k_dup_func;
	GDestroyNotify k_destroy_func;
	GType v_type;
	GBoxedCopyFunc v_dup_func;
	GDestroyNotify v_destroy_func;
	GWeakRef _read_only_view;
};

static gint GeeAbstractBidirSortedMap_private_offset;
static gpointer gee_abstract_bidir_sorted_map_parent_class = NULL;
static GeeBidirSortedMapIface * gee_abstract_bidir_sorted_map_gee_bidir_sorted_map_parent_iface = NULL;

 G_GNUC_INTERNAL void gee_abstract_bidir_sorted_map_reserved0 (GeeAbstractBidirSortedMap* self);
 G_GNUC_INTERNAL void gee_abstract_bidir_sorted_map_reserved1 (GeeAbstractBidirSortedMap* self);
 G_GNUC_INTERNAL void gee_abstract_bidir_sorted_map_reserved2 (GeeAbstractBidirSortedMap* self);
 G_GNUC_INTERNAL void gee_abstract_bidir_sorted_map_reserved3 (GeeAbstractBidirSortedMap* self);
 G_GNUC_INTERNAL void gee_abstract_bidir_sorted_map_reserved4 (GeeAbstractBidirSortedMap* self);
 G_GNUC_INTERNAL void gee_abstract_bidir_sorted_map_reserved5 (GeeAbstractBidirSortedMap* self);
 G_GNUC_INTERNAL void gee_abstract_bidir_sorted_map_reserved6 (GeeAbstractBidirSortedMap* self);
 G_GNUC_INTERNAL void gee_abstract_bidir_sorted_map_reserved7 (GeeAbstractBidirSortedMap* self);
 G_GNUC_INTERNAL void gee_abstract_bidir_sorted_map_reserved8 (GeeAbstractBidirSortedMap* self);
 G_GNUC_INTERNAL void gee_abstract_bidir_sorted_map_reserved9 (GeeAbstractBidirSortedMap* self);
static GeeBidirMapIterator* gee_abstract_bidir_sorted_map_real_bidir_map_iterator (GeeAbstractBidirSortedMap* self);
static void gee_abstract_bidir_sorted_map_real_reserved0 (GeeAbstractBidirSortedMap* self);
static void gee_abstract_bidir_sorted_map_real_reserved1 (GeeAbstractBidirSortedMap* self);
static void gee_abstract_bidir_sorted_map_real_reserved2 (GeeAbstractBidirSortedMap* self);
static void gee_abstract_bidir_sorted_map_real_reserved3 (GeeAbstractBidirSortedMap* self);
static void gee_abstract_bidir_sorted_map_real_reserved4 (GeeAbstractBidirSortedMap* self);
static void gee_abstract_bidir_sorted_map_real_reserved5 (GeeAbstractBidirSortedMap* self);
static void gee_abstract_bidir_sorted_map_real_reserved6 (GeeAbstractBidirSortedMap* self);
static void gee_abstract_bidir_sorted_map_real_reserved7 (GeeAbstractBidirSortedMap* self);
static void gee_abstract_bidir_sorted_map_real_reserved8 (GeeAbstractBidirSortedMap* self);
static void gee_abstract_bidir_sorted_map_real_reserved9 (GeeAbstractBidirSortedMap* self);
 G_GNUC_INTERNAL GeeReadOnlyBidirSortedMap* gee_read_only_bidir_sorted_map_new (GType k_type,
                                                               GBoxedCopyFunc k_dup_func,
                                                               GDestroyNotify k_destroy_func,
                                                               GType v_type,
                                                               GBoxedCopyFunc v_dup_func,
                                                               GDestroyNotify v_destroy_func,
                                                               GeeBidirSortedMap* map);
 G_GNUC_INTERNAL GeeReadOnlyBidirSortedMap* gee_read_only_bidir_sorted_map_construct (GType object_type,
                                                                     GType k_type,
                                                                     GBoxedCopyFunc k_dup_func,
                                                                     GDestroyNotify k_destroy_func,
                                                                     GType v_type,
                                                                     GBoxedCopyFunc v_dup_func,
                                                                     GDestroyNotify v_destroy_func,
                                                                     GeeBidirSortedMap* map);
 G_GNUC_INTERNAL GType gee_read_only_map_get_type (void) G_GNUC_CONST  G_GNUC_UNUSED ;
 G_GNUC_INTERNAL GType gee_read_only_sorted_map_get_type (void) G_GNUC_CONST  G_GNUC_UNUSED ;
 G_GNUC_INTERNAL GType gee_read_only_bidir_sorted_map_get_type (void) G_GNUC_CONST  G_GNUC_UNUSED ;
static GObject * gee_abstract_bidir_sorted_map_constructor (GType type,
                                                     guint n_construct_properties,
                                                     GObjectConstructParam * construct_properties);
static void gee_abstract_bidir_sorted_map_finalize (GObject * obj);
static GType gee_abstract_bidir_sorted_map_get_type_once (void);
static void _vala_gee_abstract_bidir_sorted_map_get_property (GObject * object,
                                                       guint property_id,
                                                       GValue * value,
                                                       GParamSpec * pspec);
static void _vala_gee_abstract_bidir_sorted_map_set_property (GObject * object,
                                                       guint property_id,
                                                       const GValue * value,
                                                       GParamSpec * pspec);

static inline gpointer
gee_abstract_bidir_sorted_map_get_instance_private (GeeAbstractBidirSortedMap* self)
{
	return G_STRUCT_MEMBER_P (self, GeeAbstractBidirSortedMap_private_offset);
}

/**
 * {@inheritDoc}
 */
static GeeBidirMapIterator*
gee_abstract_bidir_sorted_map_real_bidir_map_iterator (GeeAbstractBidirSortedMap* self)
{
	g_critical ("Type `%s' does not implement abstract method `gee_abstract_bidir_sorted_map_bidir_map_iterator'", g_type_name (G_TYPE_FROM_INSTANCE (self)));
	return NULL;
}

GeeBidirMapIterator*
gee_abstract_bidir_sorted_map_bidir_map_iterator (GeeAbstractBidirSortedMap* self)
{
	GeeAbstractBidirSortedMapClass* _klass_;
	g_return_val_if_fail (self != NULL, NULL);
	_klass_ = GEE_ABSTRACT_BIDIR_SORTED_MAP_GET_CLASS (self);
	if (_klass_->bidir_map_iterator) {
		return _klass_->bidir_map_iterator (self);
	}
	return NULL;
}

static void
gee_abstract_bidir_sorted_map_real_reserved0 (GeeAbstractBidirSortedMap* self)
{
}

void
gee_abstract_bidir_sorted_map_reserved0 (GeeAbstractBidirSortedMap* self)
{
	GeeAbstractBidirSortedMapClass* _klass_;
	g_return_if_fail (self != NULL);
	_klass_ = GEE_ABSTRACT_BIDIR_SORTED_MAP_GET_CLASS (self);
	if (_klass_->reserved0) {
		_klass_->reserved0 (self);
	}
}

static void
gee_abstract_bidir_sorted_map_real_reserved1 (GeeAbstractBidirSortedMap* self)
{
}

void
gee_abstract_bidir_sorted_map_reserved1 (GeeAbstractBidirSortedMap* self)
{
	GeeAbstractBidirSortedMapClass* _klass_;
	g_return_if_fail (self != NULL);
	_klass_ = GEE_ABSTRACT_BIDIR_SORTED_MAP_GET_CLASS (self);
	if (_klass_->reserved1) {
		_klass_->reserved1 (self);
	}
}

static void
gee_abstract_bidir_sorted_map_real_reserved2 (GeeAbstractBidirSortedMap* self)
{
}

void
gee_abstract_bidir_sorted_map_reserved2 (GeeAbstractBidirSortedMap* self)
{
	GeeAbstractBidirSortedMapClass* _klass_;
	g_return_if_fail (self != NULL);
	_klass_ = GEE_ABSTRACT_BIDIR_SORTED_MAP_GET_CLASS (self);
	if (_klass_->reserved2) {
		_klass_->reserved2 (self);
	}
}

static void
gee_abstract_bidir_sorted_map_real_reserved3 (GeeAbstractBidirSortedMap* self)
{
}

void
gee_abstract_bidir_sorted_map_reserved3 (GeeAbstractBidirSortedMap* self)
{
	GeeAbstractBidirSortedMapClass* _klass_;
	g_return_if_fail (self != NULL);
	_klass_ = GEE_ABSTRACT_BIDIR_SORTED_MAP_GET_CLASS (self);
	if (_klass_->reserved3) {
		_klass_->reserved3 (self);
	}
}

static void
gee_abstract_bidir_sorted_map_real_reserved4 (GeeAbstractBidirSortedMap* self)
{
}

void
gee_abstract_bidir_sorted_map_reserved4 (GeeAbstractBidirSortedMap* self)
{
	GeeAbstractBidirSortedMapClass* _klass_;
	g_return_if_fail (self != NULL);
	_klass_ = GEE_ABSTRACT_BIDIR_SORTED_MAP_GET_CLASS (self);
	if (_klass_->reserved4) {
		_klass_->reserved4 (self);
	}
}

static void
gee_abstract_bidir_sorted_map_real_reserved5 (GeeAbstractBidirSortedMap* self)
{
}

void
gee_abstract_bidir_sorted_map_reserved5 (GeeAbstractBidirSortedMap* self)
{
	GeeAbstractBidirSortedMapClass* _klass_;
	g_return_if_fail (self != NULL);
	_klass_ = GEE_ABSTRACT_BIDIR_SORTED_MAP_GET_CLASS (self);
	if (_klass_->reserved5) {
		_klass_->reserved5 (self);
	}
}

static void
gee_abstract_bidir_sorted_map_real_reserved6 (GeeAbstractBidirSortedMap* self)
{
}

void
gee_abstract_bidir_sorted_map_reserved6 (GeeAbstractBidirSortedMap* self)
{
	GeeAbstractBidirSortedMapClass* _klass_;
	g_return_if_fail (self != NULL);
	_klass_ = GEE_ABSTRACT_BIDIR_SORTED_MAP_GET_CLASS (self);
	if (_klass_->reserved6) {
		_klass_->reserved6 (self);
	}
}

static void
gee_abstract_bidir_sorted_map_real_reserved7 (GeeAbstractBidirSortedMap* self)
{
}

void
gee_abstract_bidir_sorted_map_reserved7 (GeeAbstractBidirSortedMap* self)
{
	GeeAbstractBidirSortedMapClass* _klass_;
	g_return_if_fail (self != NULL);
	_klass_ = GEE_ABSTRACT_BIDIR_SORTED_MAP_GET_CLASS (self);
	if (_klass_->reserved7) {
		_klass_->reserved7 (self);
	}
}

static void
gee_abstract_bidir_sorted_map_real_reserved8 (GeeAbstractBidirSortedMap* self)
{
}

void
gee_abstract_bidir_sorted_map_reserved8 (GeeAbstractBidirSortedMap* self)
{
	GeeAbstractBidirSortedMapClass* _klass_;
	g_return_if_fail (self != NULL);
	_klass_ = GEE_ABSTRACT_BIDIR_SORTED_MAP_GET_CLASS (self);
	if (_klass_->reserved8) {
		_klass_->reserved8 (self);
	}
}

static void
gee_abstract_bidir_sorted_map_real_reserved9 (GeeAbstractBidirSortedMap* self)
{
}

void
gee_abstract_bidir_sorted_map_reserved9 (GeeAbstractBidirSortedMap* self)
{
	GeeAbstractBidirSortedMapClass* _klass_;
	g_return_if_fail (self != NULL);
	_klass_ = GEE_ABSTRACT_BIDIR_SORTED_MAP_GET_CLASS (self);
	if (_klass_->reserved9) {
		_klass_->reserved9 (self);
	}
}

GeeAbstractBidirSortedMap*
gee_abstract_bidir_sorted_map_construct (GType object_type,
                                         GType k_type,
                                         GBoxedCopyFunc k_dup_func,
                                         GDestroyNotify k_destroy_func,
                                         GType v_type,
                                         GBoxedCopyFunc v_dup_func,
                                         GDestroyNotify v_destroy_func)
{
	GeeAbstractBidirSortedMap * self = NULL;
	self = (GeeAbstractBidirSortedMap*) gee_abstract_sorted_map_construct (object_type, k_type, (GBoxedCopyFunc) k_dup_func, (GDestroyNotify) k_destroy_func, v_type, (GBoxedCopyFunc) v_dup_func, (GDestroyNotify) v_destroy_func);
	self->priv->k_type = k_type;
	self->priv->k_dup_func = k_dup_func;
	self->priv->k_destroy_func = k_destroy_func;
	self->priv->v_type = v_type;
	self->priv->v_dup_func = v_dup_func;
	self->priv->v_destroy_func = v_destroy_func;
	return self;
}

GeeBidirSortedMap*
gee_abstract_bidir_sorted_map_get_read_only_view (GeeAbstractBidirSortedMap* self)
{
	GeeAbstractBidirSortedMapClass* _klass_;
	g_return_val_if_fail (self != NULL, NULL);
	_klass_ = GEE_ABSTRACT_BIDIR_SORTED_MAP_GET_CLASS (self);
	if (_klass_->get_read_only_view) {
		return _klass_->get_read_only_view (self);
	}
	return NULL;
}

static GeeBidirSortedMap*
gee_abstract_bidir_sorted_map_real_get_read_only_view (GeeAbstractBidirSortedMap* base)
{
	GeeBidirSortedMap* result;
	GeeAbstractBidirSortedMap* self;
	GeeBidirSortedMap* instance = NULL;
	GObject* _tmp0_;
	GeeBidirSortedMap* _tmp1_;
	self = base;
	_tmp0_ = g_weak_ref_get (&self->priv->_read_only_view);
	instance = G_TYPE_CHECK_INSTANCE_CAST (_tmp0_, GEE_TYPE_BIDIR_SORTED_MAP, GeeBidirSortedMap);
	_tmp1_ = instance;
	if (_tmp1_ == NULL) {
		GeeReadOnlyBidirSortedMap* _tmp2_;
		GeeBidirSortedMap* _tmp3_;
		_tmp2_ = gee_read_only_bidir_sorted_map_new (self->priv->k_type, (GBoxedCopyFunc) self->priv->k_dup_func, (GDestroyNotify) self->priv->k_destroy_func, self->priv->v_type, (GBoxedCopyFunc) self->priv->v_dup_func, (GDestroyNotify) self->priv->v_destroy_func, (GeeBidirSortedMap*) self);
		_g_object_unref0 (instance);
		instance = (GeeBidirSortedMap*) _tmp2_;
		_tmp3_ = instance;
		g_weak_ref_set (&self->priv->_read_only_view, (GObject*) _tmp3_);
	}
	result = instance;
	return result;
}

static GObject *
gee_abstract_bidir_sorted_map_constructor (GType type,
                                           guint n_construct_properties,
                                           GObjectConstructParam * construct_properties)
{
	GObject * obj;
	GObjectClass * parent_class;
	GeeAbstractBidirSortedMap * self;
	parent_class = G_OBJECT_CLASS (gee_abstract_bidir_sorted_map_parent_class);
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, GEE_TYPE_ABSTRACT_BIDIR_SORTED_MAP, GeeAbstractBidirSortedMap);
	g_weak_ref_clear (&self->priv->_read_only_view);
	g_weak_ref_init (&self->priv->_read_only_view, NULL);
	return obj;
}

static void
gee_abstract_bidir_sorted_map_class_init (GeeAbstractBidirSortedMapClass * klass,
                                          gpointer klass_data)
{
	gee_abstract_bidir_sorted_map_parent_class = g_type_class_peek_parent (klass);
	g_type_class_adjust_private_offset (klass, &GeeAbstractBidirSortedMap_private_offset);
	((GeeAbstractBidirSortedMapClass *) klass)->bidir_map_iterator = (GeeBidirMapIterator* (*) (GeeAbstractBidirSortedMap*)) gee_abstract_bidir_sorted_map_real_bidir_map_iterator;
	((GeeAbstractBidirSortedMapClass *) klass)->reserved0 = (void (*) (GeeAbstractBidirSortedMap*)) gee_abstract_bidir_sorted_map_real_reserved0;
	((GeeAbstractBidirSortedMapClass *) klass)->reserved1 = (void (*) (GeeAbstractBidirSortedMap*)) gee_abstract_bidir_sorted_map_real_reserved1;
	((GeeAbstractBidirSortedMapClass *) klass)->reserved2 = (void (*) (GeeAbstractBidirSortedMap*)) gee_abstract_bidir_sorted_map_real_reserved2;
	((GeeAbstractBidirSortedMapClass *) klass)->reserved3 = (void (*) (GeeAbstractBidirSortedMap*)) gee_abstract_bidir_sorted_map_real_reserved3;
	((GeeAbstractBidirSortedMapClass *) klass)->reserved4 = (void (*) (GeeAbstractBidirSortedMap*)) gee_abstract_bidir_sorted_map_real_reserved4;
	((GeeAbstractBidirSortedMapClass *) klass)->reserved5 = (void (*) (GeeAbstractBidirSortedMap*)) gee_abstract_bidir_sorted_map_real_reserved5;
	((GeeAbstractBidirSortedMapClass *) klass)->reserved6 = (void (*) (GeeAbstractBidirSortedMap*)) gee_abstract_bidir_sorted_map_real_reserved6;
	((GeeAbstractBidirSortedMapClass *) klass)->reserved7 = (void (*) (GeeAbstractBidirSortedMap*)) gee_abstract_bidir_sorted_map_real_reserved7;
	((GeeAbstractBidirSortedMapClass *) klass)->reserved8 = (void (*) (GeeAbstractBidirSortedMap*)) gee_abstract_bidir_sorted_map_real_reserved8;
	((GeeAbstractBidirSortedMapClass *) klass)->reserved9 = (void (*) (GeeAbstractBidirSortedMap*)) gee_abstract_bidir_sorted_map_real_reserved9;
	GEE_ABSTRACT_BIDIR_SORTED_MAP_CLASS (klass)->get_read_only_view = gee_abstract_bidir_sorted_map_real_get_read_only_view;
	G_OBJECT_CLASS (klass)->get_property = _vala_gee_abstract_bidir_sorted_map_get_property;
	G_OBJECT_CLASS (klass)->set_property = _vala_gee_abstract_bidir_sorted_map_set_property;
	G_OBJECT_CLASS (klass)->constructor = gee_abstract_bidir_sorted_map_constructor;
	G_OBJECT_CLASS (klass)->finalize = gee_abstract_bidir_sorted_map_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_ABSTRACT_BIDIR_SORTED_MAP_K_TYPE, g_param_spec_gtype ("k-type", "type", "type", G_TYPE_NONE, G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_ABSTRACT_BIDIR_SORTED_MAP_K_DUP_FUNC, g_param_spec_pointer ("k-dup-func", "dup func", "dup func", G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_ABSTRACT_BIDIR_SORTED_MAP_K_DESTROY_FUNC, g_param_spec_pointer ("k-destroy-func", "destroy func", "destroy func", G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_ABSTRACT_BIDIR_SORTED_MAP_V_TYPE, g_param_spec_gtype ("v-type", "type", "type", G_TYPE_NONE, G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_ABSTRACT_BIDIR_SORTED_MAP_V_DUP_FUNC, g_param_spec_pointer ("v-dup-func", "dup func", "dup func", G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_ABSTRACT_BIDIR_SORTED_MAP_V_DESTROY_FUNC, g_param_spec_pointer ("v-destroy-func", "destroy func", "destroy func", G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	/**
	 * {@inheritDoc}
	 */
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_ABSTRACT_BIDIR_SORTED_MAP_READ_ONLY_VIEW_PROPERTY, gee_abstract_bidir_sorted_map_properties[GEE_ABSTRACT_BIDIR_SORTED_MAP_READ_ONLY_VIEW_PROPERTY] = g_param_spec_object ("read-only-view", "read-only-view", "read-only-view", GEE_TYPE_BIDIR_SORTED_MAP, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE));
}

static GType
gee_abstract_bidir_sorted_map_gee_bidir_sorted_map_get_k_type (GeeAbstractBidirSortedMap* self)
{
	return self->priv->k_type;
}

static GBoxedCopyFunc
gee_abstract_bidir_sorted_map_gee_bidir_sorted_map_get_k_dup_func (GeeAbstractBidirSortedMap* self)
{
	return self->priv->k_dup_func;
}

static GDestroyNotify
gee_abstract_bidir_sorted_map_gee_bidir_sorted_map_get_k_destroy_func (GeeAbstractBidirSortedMap* self)
{
	return self->priv->k_destroy_func;
}

static GType
gee_abstract_bidir_sorted_map_gee_bidir_sorted_map_get_v_type (GeeAbstractBidirSortedMap* self)
{
	return self->priv->v_type;
}

static GBoxedCopyFunc
gee_abstract_bidir_sorted_map_gee_bidir_sorted_map_get_v_dup_func (GeeAbstractBidirSortedMap* self)
{
	return self->priv->v_dup_func;
}

static GDestroyNotify
gee_abstract_bidir_sorted_map_gee_bidir_sorted_map_get_v_destroy_func (GeeAbstractBidirSortedMap* self)
{
	return self->priv->v_destroy_func;
}

static void
gee_abstract_bidir_sorted_map_gee_bidir_sorted_map_interface_init (GeeBidirSortedMapIface * iface,
                                                                   gpointer iface_data)
{
	gee_abstract_bidir_sorted_map_gee_bidir_sorted_map_parent_iface = g_type_interface_peek_parent (iface);
	iface->bidir_map_iterator = (GeeBidirMapIterator* (*) (GeeBidirSortedMap*)) gee_abstract_bidir_sorted_map_bidir_map_iterator;
	iface->get_k_type = (GType (*) (GeeBidirSortedMap *)) gee_abstract_bidir_sorted_map_gee_bidir_sorted_map_get_k_type;
	iface->get_k_dup_func = (GBoxedCopyFunc (*) (GeeBidirSortedMap *)) gee_abstract_bidir_sorted_map_gee_bidir_sorted_map_get_k_dup_func;
	iface->get_k_destroy_func = (GDestroyNotify (*) (GeeBidirSortedMap *)) gee_abstract_bidir_sorted_map_gee_bidir_sorted_map_get_k_destroy_func;
	iface->get_v_type = (GType (*) (GeeBidirSortedMap *)) gee_abstract_bidir_sorted_map_gee_bidir_sorted_map_get_v_type;
	iface->get_v_dup_func = (GBoxedCopyFunc (*) (GeeBidirSortedMap *)) gee_abstract_bidir_sorted_map_gee_bidir_sorted_map_get_v_dup_func;
	iface->get_v_destroy_func = (GDestroyNotify (*) (GeeBidirSortedMap *)) gee_abstract_bidir_sorted_map_gee_bidir_sorted_map_get_v_destroy_func;
	iface->get_read_only_view = (GeeBidirSortedMap* (*) (GeeBidirSortedMap *)) gee_abstract_bidir_sorted_map_get_read_only_view;
}

static void
gee_abstract_bidir_sorted_map_instance_init (GeeAbstractBidirSortedMap * self,
                                             gpointer klass)
{
	self->priv = gee_abstract_bidir_sorted_map_get_instance_private (self);
}

static void
gee_abstract_bidir_sorted_map_finalize (GObject * obj)
{
	GeeAbstractBidirSortedMap * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, GEE_TYPE_ABSTRACT_BIDIR_SORTED_MAP, GeeAbstractBidirSortedMap);
	g_weak_ref_clear (&self->priv->_read_only_view);
	G_OBJECT_CLASS (gee_abstract_bidir_sorted_map_parent_class)->finalize (obj);
}

/**
 * Skeletal implementation of the {@link BidirSortedSet} interface.
 *
 * Contains common code shared by all set implementations.
 *
 * @see TreeSet
 */
static GType
gee_abstract_bidir_sorted_map_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (GeeAbstractBidirSortedMapClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) gee_abstract_bidir_sorted_map_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GeeAbstractBidirSortedMap), 0, (GInstanceInitFunc) gee_abstract_bidir_sorted_map_instance_init, NULL };
	static const GInterfaceInfo gee_bidir_sorted_map_info = { (GInterfaceInitFunc) gee_abstract_bidir_sorted_map_gee_bidir_sorted_map_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
	GType gee_abstract_bidir_sorted_map_type_id;
	gee_abstract_bidir_sorted_map_type_id = g_type_register_static (GEE_TYPE_ABSTRACT_SORTED_MAP, "GeeAbstractBidirSortedMap", &g_define_type_info, G_TYPE_FLAG_ABSTRACT);
	g_type_add_interface_static (gee_abstract_bidir_sorted_map_type_id, GEE_TYPE_BIDIR_SORTED_MAP, &gee_bidir_sorted_map_info);
	GeeAbstractBidirSortedMap_private_offset = g_type_add_instance_private (gee_abstract_bidir_sorted_map_type_id, sizeof (GeeAbstractBidirSortedMapPrivate));
	return gee_abstract_bidir_sorted_map_type_id;
}

GType
gee_abstract_bidir_sorted_map_get_type (void)
{
	static volatile gsize gee_abstract_bidir_sorted_map_type_id__once = 0;
	if (g_once_init_enter (&gee_abstract_bidir_sorted_map_type_id__once)) {
		GType gee_abstract_bidir_sorted_map_type_id;
		gee_abstract_bidir_sorted_map_type_id = gee_abstract_bidir_sorted_map_get_type_once ();
		g_once_init_leave (&gee_abstract_bidir_sorted_map_type_id__once, gee_abstract_bidir_sorted_map_type_id);
	}
	return gee_abstract_bidir_sorted_map_type_id__once;
}

static void
_vala_gee_abstract_bidir_sorted_map_get_property (GObject * object,
                                                  guint property_id,
                                                  GValue * value,
                                                  GParamSpec * pspec)
{
	GeeAbstractBidirSortedMap * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, GEE_TYPE_ABSTRACT_BIDIR_SORTED_MAP, GeeAbstractBidirSortedMap);
	switch (property_id) {
		case GEE_ABSTRACT_BIDIR_SORTED_MAP_READ_ONLY_VIEW_PROPERTY:
		g_value_take_object (value, gee_abstract_bidir_sorted_map_get_read_only_view (self));
		break;
		case GEE_ABSTRACT_BIDIR_SORTED_MAP_K_TYPE:
		g_value_set_gtype (value, self->priv->k_type);
		break;
		case GEE_ABSTRACT_BIDIR_SORTED_MAP_K_DUP_FUNC:
		g_value_set_pointer (value, self->priv->k_dup_func);
		break;
		case GEE_ABSTRACT_BIDIR_SORTED_MAP_K_DESTROY_FUNC:
		g_value_set_pointer (value, self->priv->k_destroy_func);
		break;
		case GEE_ABSTRACT_BIDIR_SORTED_MAP_V_TYPE:
		g_value_set_gtype (value, self->priv->v_type);
		break;
		case GEE_ABSTRACT_BIDIR_SORTED_MAP_V_DUP_FUNC:
		g_value_set_pointer (value, self->priv->v_dup_func);
		break;
		case GEE_ABSTRACT_BIDIR_SORTED_MAP_V_DESTROY_FUNC:
		g_value_set_pointer (value, self->priv->v_destroy_func);
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

static void
_vala_gee_abstract_bidir_sorted_map_set_property (GObject * object,
                                                  guint property_id,
                                                  const GValue * value,
                                                  GParamSpec * pspec)
{
	GeeAbstractBidirSortedMap * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, GEE_TYPE_ABSTRACT_BIDIR_SORTED_MAP, GeeAbstractBidirSortedMap);
	switch (property_id) {
		case GEE_ABSTRACT_BIDIR_SORTED_MAP_K_TYPE:
		self->priv->k_type = g_value_get_gtype (value);
		break;
		case GEE_ABSTRACT_BIDIR_SORTED_MAP_K_DUP_FUNC:
		self->priv->k_dup_func = g_value_get_pointer (value);
		break;
		case GEE_ABSTRACT_BIDIR_SORTED_MAP_K_DESTROY_FUNC:
		self->priv->k_destroy_func = g_value_get_pointer (value);
		break;
		case GEE_ABSTRACT_BIDIR_SORTED_MAP_V_TYPE:
		self->priv->v_type = g_value_get_gtype (value);
		break;
		case GEE_ABSTRACT_BIDIR_SORTED_MAP_V_DUP_FUNC:
		self->priv->v_dup_func = g_value_get_pointer (value);
		break;
		case GEE_ABSTRACT_BIDIR_SORTED_MAP_V_DESTROY_FUNC:
		self->priv->v_destroy_func = g_value_get_pointer (value);
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

