/* comparable.c generated by valac 0.56.3, the Vala compiler
 * generated from comparable.vala, do not modify */

/* comparable.vala
 *
 * Copyright (C) 2009  Didier Villevalois
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Didier 'Ptitjes Villevalois <ptitjes@free.fr>
 */

#include "gee.h"
#include <glib-object.h>
#include <glib.h>

static GType gee_comparable_get_type_once (void);

/**
 * Compares this object with the specified object.
 *
 * @return a negative integer, zero, or a positive integer as this object
 *         is less than, equal to, or greater than the specified object
 */
gint
gee_comparable_compare_to (GeeComparable* self,
                           gconstpointer object)
{
	GeeComparableIface* _iface_;
	g_return_val_if_fail (self != NULL, 0);
	_iface_ = GEE_COMPARABLE_GET_INTERFACE (self);
	if (_iface_->compare_to) {
		return _iface_->compare_to (self, object);
	}
	return -1;
}

static void
gee_comparable_default_init (GeeComparableIface * iface,
                             gpointer iface_data)
{
}

/**
 * This interface defines a total ordering among instances of each class
 * implementing it.
 *
 * In other words:
 *
 *   * It's irreflexive: For all `a` it holds that `a.compare_to(a) == 0`
 *   * It's transitive: For all `a`, `b` and `c` if `a.compare_to(b) < 0` and
 *     `b.compare_to(c) < 0` then `a.compare_to(c) < 0`.
 *   * It's trichotomous: For all `a` and `b` it holds that
 *     `a.compare_to(b) = -b.compare_to(a)`.
 *
 * Note: The relationship must be immutable. In other words if at one point of
 *   program `a.compare_to(b)` had certain value then call `a.compare_to(b)`
 *   //must always// return the original value until end of `a` and `b` lifetime.
 *
 * @see Hashable
 */
static GType
gee_comparable_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (GeeComparableIface), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) gee_comparable_default_init, (GClassFinalizeFunc) NULL, NULL, 0, 0, (GInstanceInitFunc) NULL, NULL };
	GType gee_comparable_type_id;
	gee_comparable_type_id = g_type_register_static (G_TYPE_INTERFACE, "GeeComparable", &g_define_type_info, 0);
	g_type_interface_add_prerequisite (gee_comparable_type_id, G_TYPE_OBJECT);
	return gee_comparable_type_id;
}

GType
gee_comparable_get_type (void)
{
	static volatile gsize gee_comparable_type_id__once = 0;
	if (g_once_init_enter (&gee_comparable_type_id__once)) {
		GType gee_comparable_type_id;
		gee_comparable_type_id = gee_comparable_get_type_once ();
		g_once_init_leave (&gee_comparable_type_id__once, gee_comparable_type_id);
	}
	return gee_comparable_type_id__once;
}

