/* readonlyset.c generated by valac 0.56.3, the Vala compiler
 * generated from readonlyset.vala, do not modify */

/* readonlyset.vala
 *
 * Copyright (C) 2007-2008  Jürg Billeter
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Jürg Billeter <j@bitron.ch>
 */

#include <glib-object.h>
#include "gee.h"
#include <glib.h>

#define GEE_TYPE_READ_ONLY_COLLECTION (gee_read_only_collection_get_type ())
#define GEE_READ_ONLY_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_READ_ONLY_COLLECTION, GeeReadOnlyCollection))
#define GEE_READ_ONLY_COLLECTION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_TYPE_READ_ONLY_COLLECTION, GeeReadOnlyCollectionClass))
#define GEE_IS_READ_ONLY_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_READ_ONLY_COLLECTION))
#define GEE_IS_READ_ONLY_COLLECTION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEE_TYPE_READ_ONLY_COLLECTION))
#define GEE_READ_ONLY_COLLECTION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_TYPE_READ_ONLY_COLLECTION, GeeReadOnlyCollectionClass))

typedef struct _GeeReadOnlyCollection GeeReadOnlyCollection;
typedef struct _GeeReadOnlyCollectionClass GeeReadOnlyCollectionClass;
typedef struct _GeeReadOnlyCollectionPrivate GeeReadOnlyCollectionPrivate;

#define GEE_TYPE_READ_ONLY_SET (gee_read_only_set_get_type ())
#define GEE_READ_ONLY_SET(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_READ_ONLY_SET, GeeReadOnlySet))
#define GEE_READ_ONLY_SET_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_TYPE_READ_ONLY_SET, GeeReadOnlySetClass))
#define GEE_IS_READ_ONLY_SET(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_READ_ONLY_SET))
#define GEE_IS_READ_ONLY_SET_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEE_TYPE_READ_ONLY_SET))
#define GEE_READ_ONLY_SET_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_TYPE_READ_ONLY_SET, GeeReadOnlySetClass))

typedef struct _GeeReadOnlySet GeeReadOnlySet;
typedef struct _GeeReadOnlySetClass GeeReadOnlySetClass;
typedef struct _GeeReadOnlySetPrivate GeeReadOnlySetPrivate;
enum  {
	GEE_READ_ONLY_SET_0_PROPERTY,
	GEE_READ_ONLY_SET_G_TYPE,
	GEE_READ_ONLY_SET_G_DUP_FUNC,
	GEE_READ_ONLY_SET_G_DESTROY_FUNC,
	GEE_READ_ONLY_SET_READ_ONLY_VIEW_PROPERTY,
	GEE_READ_ONLY_SET_NUM_PROPERTIES
};
static GParamSpec* gee_read_only_set_properties[GEE_READ_ONLY_SET_NUM_PROPERTIES];

struct _GeeReadOnlyCollection {
	GObject parent_instance;
	GeeReadOnlyCollectionPrivate * priv;
	GeeCollection* _collection;
};

struct _GeeReadOnlyCollectionClass {
	GObjectClass parent_class;
	GeeCollection* (*get_read_only_view) (GeeReadOnlyCollection* self);
};

struct _GeeReadOnlySet {
	GeeReadOnlyCollection parent_instance;
	GeeReadOnlySetPrivate * priv;
};

struct _GeeReadOnlySetClass {
	GeeReadOnlyCollectionClass parent_class;
	GeeSet* (*get_read_only_view) (GeeReadOnlySet* self);
};

struct _GeeReadOnlySetPrivate {
	GType g_type;
	GBoxedCopyFunc g_dup_func;
	GDestroyNotify g_destroy_func;
};

static gint GeeReadOnlySet_private_offset;
static gpointer gee_read_only_set_parent_class = NULL;
static GeeSetIface * gee_read_only_set_gee_set_parent_iface = NULL;

 G_GNUC_INTERNAL GType gee_read_only_collection_get_type (void) G_GNUC_CONST  G_GNUC_UNUSED ;
 G_GNUC_INTERNAL GType gee_read_only_set_get_type (void) G_GNUC_CONST  G_GNUC_UNUSED ;
 G_GNUC_INTERNAL GeeSet* gee_read_only_set_get_read_only_view (GeeReadOnlySet* self);
 G_GNUC_INTERNAL GeeReadOnlySet* gee_read_only_set_new (GType g_type,
                                       GBoxedCopyFunc g_dup_func,
                                       GDestroyNotify g_destroy_func,
                                       GeeSet* set);
 G_GNUC_INTERNAL GeeReadOnlySet* gee_read_only_set_construct (GType object_type,
                                             GType g_type,
                                             GBoxedCopyFunc g_dup_func,
                                             GDestroyNotify g_destroy_func,
                                             GeeSet* set);
 G_GNUC_INTERNAL GeeReadOnlyCollection* gee_read_only_collection_new (GType g_type,
                                                     GBoxedCopyFunc g_dup_func,
                                                     GDestroyNotify g_destroy_func,
                                                     GeeCollection* collection);
 G_GNUC_INTERNAL GeeReadOnlyCollection* gee_read_only_collection_construct (GType object_type,
                                                           GType g_type,
                                                           GBoxedCopyFunc g_dup_func,
                                                           GDestroyNotify g_destroy_func,
                                                           GeeCollection* collection);
static GType gee_read_only_set_get_type_once (void);
static void _vala_gee_read_only_set_get_property (GObject * object,
                                           guint property_id,
                                           GValue * value,
                                           GParamSpec * pspec);
static void _vala_gee_read_only_set_set_property (GObject * object,
                                           guint property_id,
                                           const GValue * value,
                                           GParamSpec * pspec);

static inline gpointer
gee_read_only_set_get_instance_private (GeeReadOnlySet* self)
{
	return G_STRUCT_MEMBER_P (self, GeeReadOnlySet_private_offset);
}

/**
 * Constructs a read-only set that mirrors the content of the specified set.
 *
 * @param set the set to decorate.
 */
 G_GNUC_INTERNAL GeeReadOnlySet*
gee_read_only_set_construct (GType object_type,
                             GType g_type,
                             GBoxedCopyFunc g_dup_func,
                             GDestroyNotify g_destroy_func,
                             GeeSet* set)
{
	GeeReadOnlySet * self = NULL;
	g_return_val_if_fail (set != NULL, NULL);
	self = (GeeReadOnlySet*) gee_read_only_collection_construct (object_type, g_type, (GBoxedCopyFunc) g_dup_func, (GDestroyNotify) g_destroy_func, (GeeCollection*) set);
	self->priv->g_type = g_type;
	self->priv->g_dup_func = g_dup_func;
	self->priv->g_destroy_func = g_destroy_func;
	return self;
}

 G_GNUC_INTERNAL GeeReadOnlySet*
gee_read_only_set_new (GType g_type,
                       GBoxedCopyFunc g_dup_func,
                       GDestroyNotify g_destroy_func,
                       GeeSet* set)
{
	return gee_read_only_set_construct (GEE_TYPE_READ_ONLY_SET, g_type, g_dup_func, g_destroy_func, set);
}

 G_GNUC_INTERNAL GeeSet*
gee_read_only_set_get_read_only_view (GeeReadOnlySet* self)
{
	GeeReadOnlySetClass* _klass_;
	g_return_val_if_fail (self != NULL, NULL);
	_klass_ = GEE_READ_ONLY_SET_GET_CLASS (self);
	if (_klass_->get_read_only_view) {
		return _klass_->get_read_only_view (self);
	}
	return NULL;
}

static gpointer
_g_object_ref0 (gpointer self)
{
	return self ? g_object_ref (self) : NULL;
}

static GeeSet*
gee_read_only_set_real_get_read_only_view (GeeReadOnlySet* base)
{
	GeeSet* result;
	GeeReadOnlySet* self;
	GeeSet* _tmp0_;
	self = base;
	_tmp0_ = _g_object_ref0 ((GeeSet*) self);
	result = _tmp0_;
	return result;
}

static void
gee_read_only_set_class_init (GeeReadOnlySetClass * klass,
                              gpointer klass_data)
{
	gee_read_only_set_parent_class = g_type_class_peek_parent (klass);
	g_type_class_adjust_private_offset (klass, &GeeReadOnlySet_private_offset);
	GEE_READ_ONLY_SET_CLASS (klass)->get_read_only_view = gee_read_only_set_real_get_read_only_view;
	G_OBJECT_CLASS (klass)->get_property = _vala_gee_read_only_set_get_property;
	G_OBJECT_CLASS (klass)->set_property = _vala_gee_read_only_set_set_property;
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_READ_ONLY_SET_G_TYPE, g_param_spec_gtype ("g-type", "type", "type", G_TYPE_NONE, G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_READ_ONLY_SET_G_DUP_FUNC, g_param_spec_pointer ("g-dup-func", "dup func", "dup func", G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_READ_ONLY_SET_G_DESTROY_FUNC, g_param_spec_pointer ("g-destroy-func", "destroy func", "destroy func", G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_READ_ONLY_SET_READ_ONLY_VIEW_PROPERTY, gee_read_only_set_properties[GEE_READ_ONLY_SET_READ_ONLY_VIEW_PROPERTY] = g_param_spec_object ("read-only-view", "read-only-view", "read-only-view", GEE_TYPE_SET, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE));
}

static GType
gee_read_only_set_gee_set_get_g_type (GeeReadOnlySet* self)
{
	return self->priv->g_type;
}

static GBoxedCopyFunc
gee_read_only_set_gee_set_get_g_dup_func (GeeReadOnlySet* self)
{
	return self->priv->g_dup_func;
}

static GDestroyNotify
gee_read_only_set_gee_set_get_g_destroy_func (GeeReadOnlySet* self)
{
	return self->priv->g_destroy_func;
}

static void
gee_read_only_set_gee_set_interface_init (GeeSetIface * iface,
                                          gpointer iface_data)
{
	gee_read_only_set_gee_set_parent_iface = g_type_interface_peek_parent (iface);
	iface->get_g_type = (GType (*) (GeeSet *)) gee_read_only_set_gee_set_get_g_type;
	iface->get_g_dup_func = (GBoxedCopyFunc (*) (GeeSet *)) gee_read_only_set_gee_set_get_g_dup_func;
	iface->get_g_destroy_func = (GDestroyNotify (*) (GeeSet *)) gee_read_only_set_gee_set_get_g_destroy_func;
	iface->get_read_only_view = (GeeSet* (*) (GeeSet *)) gee_read_only_set_get_read_only_view;
}

static void
gee_read_only_set_instance_init (GeeReadOnlySet * self,
                                 gpointer klass)
{
	self->priv = gee_read_only_set_get_instance_private (self);
}

/**
 * Read-only view for {@link Set} collections.
 *
 * This class decorates any class which implements the {@link Set} interface
 * by making it read only. Any method which normally modify data will throw an
 * error.
 *
 * @see Set
 */
static GType
gee_read_only_set_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (GeeReadOnlySetClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) gee_read_only_set_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GeeReadOnlySet), 0, (GInstanceInitFunc) gee_read_only_set_instance_init, NULL };
	static const GInterfaceInfo gee_set_info = { (GInterfaceInitFunc) gee_read_only_set_gee_set_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
	GType gee_read_only_set_type_id;
	gee_read_only_set_type_id = g_type_register_static (GEE_TYPE_READ_ONLY_COLLECTION, "GeeReadOnlySet", &g_define_type_info, 0);
	g_type_add_interface_static (gee_read_only_set_type_id, GEE_TYPE_SET, &gee_set_info);
	GeeReadOnlySet_private_offset = g_type_add_instance_private (gee_read_only_set_type_id, sizeof (GeeReadOnlySetPrivate));
	return gee_read_only_set_type_id;
}

 G_GNUC_INTERNAL GType
gee_read_only_set_get_type (void)
{
	static volatile gsize gee_read_only_set_type_id__once = 0;
	if (g_once_init_enter (&gee_read_only_set_type_id__once)) {
		GType gee_read_only_set_type_id;
		gee_read_only_set_type_id = gee_read_only_set_get_type_once ();
		g_once_init_leave (&gee_read_only_set_type_id__once, gee_read_only_set_type_id);
	}
	return gee_read_only_set_type_id__once;
}

static void
_vala_gee_read_only_set_get_property (GObject * object,
                                      guint property_id,
                                      GValue * value,
                                      GParamSpec * pspec)
{
	GeeReadOnlySet * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, GEE_TYPE_READ_ONLY_SET, GeeReadOnlySet);
	switch (property_id) {
		case GEE_READ_ONLY_SET_READ_ONLY_VIEW_PROPERTY:
		g_value_take_object (value, gee_read_only_set_get_read_only_view (self));
		break;
		case GEE_READ_ONLY_SET_G_TYPE:
		g_value_set_gtype (value, self->priv->g_type);
		break;
		case GEE_READ_ONLY_SET_G_DUP_FUNC:
		g_value_set_pointer (value, self->priv->g_dup_func);
		break;
		case GEE_READ_ONLY_SET_G_DESTROY_FUNC:
		g_value_set_pointer (value, self->priv->g_destroy_func);
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

static void
_vala_gee_read_only_set_set_property (GObject * object,
                                      guint property_id,
                                      const GValue * value,
                                      GParamSpec * pspec)
{
	GeeReadOnlySet * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, GEE_TYPE_READ_ONLY_SET, GeeReadOnlySet);
	switch (property_id) {
		case GEE_READ_ONLY_SET_G_TYPE:
		self->priv->g_type = g_value_get_gtype (value);
		break;
		case GEE_READ_ONLY_SET_G_DUP_FUNC:
		self->priv->g_dup_func = g_value_get_pointer (value);
		break;
		case GEE_READ_ONLY_SET_G_DESTROY_FUNC:
		self->priv->g_destroy_func = g_value_get_pointer (value);
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

