/* readonlylist.c generated by valac 0.56.3, the Vala compiler
 * generated from readonlylist.vala, do not modify */

/* readonlylist.vala
 *
 * Copyright (C) 2007-2008  Jürg Billeter
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Jürg Billeter <j@bitron.ch>
 */

#include <glib-object.h>
#include "gee.h"
#include <glib.h>

#define GEE_TYPE_READ_ONLY_COLLECTION (gee_read_only_collection_get_type ())
#define GEE_READ_ONLY_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_READ_ONLY_COLLECTION, GeeReadOnlyCollection))
#define GEE_READ_ONLY_COLLECTION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_TYPE_READ_ONLY_COLLECTION, GeeReadOnlyCollectionClass))
#define GEE_IS_READ_ONLY_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_READ_ONLY_COLLECTION))
#define GEE_IS_READ_ONLY_COLLECTION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEE_TYPE_READ_ONLY_COLLECTION))
#define GEE_READ_ONLY_COLLECTION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_TYPE_READ_ONLY_COLLECTION, GeeReadOnlyCollectionClass))

typedef struct _GeeReadOnlyCollection GeeReadOnlyCollection;
typedef struct _GeeReadOnlyCollectionClass GeeReadOnlyCollectionClass;
typedef struct _GeeReadOnlyCollectionPrivate GeeReadOnlyCollectionPrivate;

#define GEE_TYPE_READ_ONLY_LIST (gee_read_only_list_get_type ())
#define GEE_READ_ONLY_LIST(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_READ_ONLY_LIST, GeeReadOnlyList))
#define GEE_READ_ONLY_LIST_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_TYPE_READ_ONLY_LIST, GeeReadOnlyListClass))
#define GEE_IS_READ_ONLY_LIST(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_READ_ONLY_LIST))
#define GEE_IS_READ_ONLY_LIST_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEE_TYPE_READ_ONLY_LIST))
#define GEE_READ_ONLY_LIST_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_TYPE_READ_ONLY_LIST, GeeReadOnlyListClass))

typedef struct _GeeReadOnlyList GeeReadOnlyList;
typedef struct _GeeReadOnlyListClass GeeReadOnlyListClass;
typedef struct _GeeReadOnlyListPrivate GeeReadOnlyListPrivate;
enum  {
	GEE_READ_ONLY_LIST_0_PROPERTY,
	GEE_READ_ONLY_LIST_G_TYPE,
	GEE_READ_ONLY_LIST_G_DUP_FUNC,
	GEE_READ_ONLY_LIST_G_DESTROY_FUNC,
	GEE_READ_ONLY_LIST_READ_ONLY_VIEW_PROPERTY,
	GEE_READ_ONLY_LIST_NUM_PROPERTIES
};
static GParamSpec* gee_read_only_list_properties[GEE_READ_ONLY_LIST_NUM_PROPERTIES];

#define GEE_READ_ONLY_COLLECTION_TYPE_ITERATOR (gee_read_only_collection_iterator_get_type ())
#define GEE_READ_ONLY_COLLECTION_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_READ_ONLY_COLLECTION_TYPE_ITERATOR, GeeReadOnlyCollectionIterator))
#define GEE_READ_ONLY_COLLECTION_ITERATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_READ_ONLY_COLLECTION_TYPE_ITERATOR, GeeReadOnlyCollectionIteratorClass))
#define GEE_READ_ONLY_COLLECTION_IS_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_READ_ONLY_COLLECTION_TYPE_ITERATOR))
#define GEE_READ_ONLY_COLLECTION_IS_ITERATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEE_READ_ONLY_COLLECTION_TYPE_ITERATOR))
#define GEE_READ_ONLY_COLLECTION_ITERATOR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_READ_ONLY_COLLECTION_TYPE_ITERATOR, GeeReadOnlyCollectionIteratorClass))

typedef struct _GeeReadOnlyCollectionIterator GeeReadOnlyCollectionIterator;
typedef struct _GeeReadOnlyCollectionIteratorClass GeeReadOnlyCollectionIteratorClass;

#define GEE_READ_ONLY_LIST_TYPE_ITERATOR (gee_read_only_list_iterator_get_type ())
#define GEE_READ_ONLY_LIST_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_READ_ONLY_LIST_TYPE_ITERATOR, GeeReadOnlyListIterator))
#define GEE_READ_ONLY_LIST_ITERATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_READ_ONLY_LIST_TYPE_ITERATOR, GeeReadOnlyListIteratorClass))
#define GEE_READ_ONLY_LIST_IS_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_READ_ONLY_LIST_TYPE_ITERATOR))
#define GEE_READ_ONLY_LIST_IS_ITERATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEE_READ_ONLY_LIST_TYPE_ITERATOR))
#define GEE_READ_ONLY_LIST_ITERATOR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_READ_ONLY_LIST_TYPE_ITERATOR, GeeReadOnlyListIteratorClass))

typedef struct _GeeReadOnlyListIterator GeeReadOnlyListIterator;
typedef struct _GeeReadOnlyListIteratorClass GeeReadOnlyListIteratorClass;
#define _g_object_unref0(var) ((var == NULL) ? NULL : (var = (g_object_unref (var), NULL)))
typedef struct _GeeReadOnlyCollectionIteratorPrivate GeeReadOnlyCollectionIteratorPrivate;
typedef struct _GeeReadOnlyListIteratorPrivate GeeReadOnlyListIteratorPrivate;
enum  {
	GEE_READ_ONLY_LIST_ITERATOR_0_PROPERTY,
	GEE_READ_ONLY_LIST_ITERATOR_G_TYPE,
	GEE_READ_ONLY_LIST_ITERATOR_G_DUP_FUNC,
	GEE_READ_ONLY_LIST_ITERATOR_G_DESTROY_FUNC,
	GEE_READ_ONLY_LIST_ITERATOR_NUM_PROPERTIES
};
static GParamSpec* gee_read_only_list_iterator_properties[GEE_READ_ONLY_LIST_ITERATOR_NUM_PROPERTIES];

struct _GeeReadOnlyCollection {
	GObject parent_instance;
	GeeReadOnlyCollectionPrivate * priv;
	GeeCollection* _collection;
};

struct _GeeReadOnlyCollectionClass {
	GObjectClass parent_class;
	GeeCollection* (*get_read_only_view) (GeeReadOnlyCollection* self);
};

struct _GeeReadOnlyList {
	GeeReadOnlyCollection parent_instance;
	GeeReadOnlyListPrivate * priv;
};

struct _GeeReadOnlyListClass {
	GeeReadOnlyCollectionClass parent_class;
	GeeList* (*get_read_only_view) (GeeReadOnlyList* self);
};

struct _GeeReadOnlyListPrivate {
	GType g_type;
	GBoxedCopyFunc g_dup_func;
	GDestroyNotify g_destroy_func;
};

struct _GeeReadOnlyCollectionIterator {
	GObject parent_instance;
	GeeReadOnlyCollectionIteratorPrivate * priv;
	GeeIterator* _iter;
};

struct _GeeReadOnlyCollectionIteratorClass {
	GObjectClass parent_class;
};

struct _GeeReadOnlyListIterator {
	GeeReadOnlyCollectionIterator parent_instance;
	GeeReadOnlyListIteratorPrivate * priv;
};

struct _GeeReadOnlyListIteratorClass {
	GeeReadOnlyCollectionIteratorClass parent_class;
};

struct _GeeReadOnlyListIteratorPrivate {
	GType g_type;
	GBoxedCopyFunc g_dup_func;
	GDestroyNotify g_destroy_func;
};

static gint GeeReadOnlyList_private_offset;
static gpointer gee_read_only_list_parent_class = NULL;
static gint GeeReadOnlyListIterator_private_offset;
static gpointer gee_read_only_list_iterator_parent_class = NULL;
static GeeListIteratorIface * gee_read_only_list_iterator_gee_list_iterator_parent_iface = NULL;
static GeeListIface * gee_read_only_list_gee_list_parent_iface = NULL;

 G_GNUC_INTERNAL GType gee_read_only_collection_get_type (void) G_GNUC_CONST  G_GNUC_UNUSED ;
 G_GNUC_INTERNAL GType gee_read_only_list_get_type (void) G_GNUC_CONST  G_GNUC_UNUSED ;
 G_GNUC_INTERNAL GeeList* gee_read_only_list_get_read_only_view (GeeReadOnlyList* self);
 G_GNUC_INTERNAL GeeReadOnlyList* gee_read_only_list_new (GType g_type,
                                         GBoxedCopyFunc g_dup_func,
                                         GDestroyNotify g_destroy_func,
                                         GeeList* list);
 G_GNUC_INTERNAL GeeReadOnlyList* gee_read_only_list_construct (GType object_type,
                                               GType g_type,
                                               GBoxedCopyFunc g_dup_func,
                                               GDestroyNotify g_destroy_func,
                                               GeeList* list);
 G_GNUC_INTERNAL GeeReadOnlyCollection* gee_read_only_collection_new (GType g_type,
                                                     GBoxedCopyFunc g_dup_func,
                                                     GDestroyNotify g_destroy_func,
                                                     GeeCollection* collection);
 G_GNUC_INTERNAL GeeReadOnlyCollection* gee_read_only_collection_construct (GType object_type,
                                                           GType g_type,
                                                           GBoxedCopyFunc g_dup_func,
                                                           GDestroyNotify g_destroy_func,
                                                           GeeCollection* collection);
static GeeListIterator* gee_read_only_list_real_list_iterator (GeeList* base);
 G_GNUC_INTERNAL GeeReadOnlyListIterator* gee_read_only_list_iterator_new (GType g_type,
                                                          GBoxedCopyFunc g_dup_func,
                                                          GDestroyNotify g_destroy_func,
                                                          GeeListIterator* iterator);
 G_GNUC_INTERNAL GeeReadOnlyListIterator* gee_read_only_list_iterator_construct (GType object_type,
                                                                GType g_type,
                                                                GBoxedCopyFunc g_dup_func,
                                                                GDestroyNotify g_destroy_func,
                                                                GeeListIterator* iterator);
VALA_EXTERN GType gee_read_only_collection_iterator_get_type (void) G_GNUC_CONST ;
VALA_EXTERN GType gee_read_only_list_iterator_get_type (void) G_GNUC_CONST ;
static gint gee_read_only_list_real_index_of (GeeList* base,
                                       gconstpointer item);
static void gee_read_only_list_real_insert (GeeList* base,
                                     gint index,
                                     gconstpointer item);
static gpointer gee_read_only_list_real_remove_at (GeeList* base,
                                            gint index);
static gpointer gee_read_only_list_real_get (GeeList* base,
                                      gint index);
static void gee_read_only_list_real_set (GeeList* base,
                                  gint index,
                                  gconstpointer o);
static GeeList* gee_read_only_list_real_slice (GeeList* base,
                                        gint start,
                                        gint stop);
static gpointer gee_read_only_list_real_first (GeeList* base);
static gpointer gee_read_only_list_real_last (GeeList* base);
static void gee_read_only_list_real_insert_all (GeeList* base,
                                         gint index,
                                         GeeCollection* collection);
static void gee_read_only_list_real_sort (GeeList* base,
                                   GCompareDataFunc compare,
                                   gpointer compare_target,
                                   GDestroyNotify compare_target_destroy_notify);
 G_GNUC_INTERNAL GeeReadOnlyCollectionIterator* gee_read_only_collection_iterator_new (GType g_type,
                                                                      GBoxedCopyFunc g_dup_func,
                                                                      GDestroyNotify g_destroy_func,
                                                                      GeeIterator* iterator);
 G_GNUC_INTERNAL GeeReadOnlyCollectionIterator* gee_read_only_collection_iterator_construct (GType object_type,
                                                                            GType g_type,
                                                                            GBoxedCopyFunc g_dup_func,
                                                                            GDestroyNotify g_destroy_func,
                                                                            GeeIterator* iterator);
static void gee_read_only_list_iterator_real_set (GeeListIterator* base,
                                           gconstpointer item);
static void gee_read_only_list_iterator_real_add (GeeListIterator* base,
                                           gconstpointer item);
static gint gee_read_only_list_iterator_real_index (GeeListIterator* base);
static GType gee_read_only_list_iterator_get_type_once (void);
static void _vala_gee_read_only_list_iterator_get_property (GObject * object,
                                                     guint property_id,
                                                     GValue * value,
                                                     GParamSpec * pspec);
static void _vala_gee_read_only_list_iterator_set_property (GObject * object,
                                                     guint property_id,
                                                     const GValue * value,
                                                     GParamSpec * pspec);
static GType gee_read_only_list_get_type_once (void);
static void _vala_gee_read_only_list_get_property (GObject * object,
                                            guint property_id,
                                            GValue * value,
                                            GParamSpec * pspec);
static void _vala_gee_read_only_list_set_property (GObject * object,
                                            guint property_id,
                                            const GValue * value,
                                            GParamSpec * pspec);

static inline gpointer
gee_read_only_list_get_instance_private (GeeReadOnlyList* self)
{
	return G_STRUCT_MEMBER_P (self, GeeReadOnlyList_private_offset);
}

/**
 * Constructs a read-only list that mirrors the content of the specified
 * list.
 *
 * @param list the list to decorate.
 */
 G_GNUC_INTERNAL GeeReadOnlyList*
gee_read_only_list_construct (GType object_type,
                              GType g_type,
                              GBoxedCopyFunc g_dup_func,
                              GDestroyNotify g_destroy_func,
                              GeeList* list)
{
	GeeReadOnlyList * self = NULL;
	g_return_val_if_fail (list != NULL, NULL);
	self = (GeeReadOnlyList*) gee_read_only_collection_construct (object_type, g_type, (GBoxedCopyFunc) g_dup_func, (GDestroyNotify) g_destroy_func, (GeeCollection*) list);
	self->priv->g_type = g_type;
	self->priv->g_dup_func = g_dup_func;
	self->priv->g_destroy_func = g_destroy_func;
	return self;
}

 G_GNUC_INTERNAL GeeReadOnlyList*
gee_read_only_list_new (GType g_type,
                        GBoxedCopyFunc g_dup_func,
                        GDestroyNotify g_destroy_func,
                        GeeList* list)
{
	return gee_read_only_list_construct (GEE_TYPE_READ_ONLY_LIST, g_type, g_dup_func, g_destroy_func, list);
}

/**
 * {@inheritDoc}
 */
static GeeListIterator*
gee_read_only_list_real_list_iterator (GeeList* base)
{
	GeeReadOnlyList * self;
	GeeCollection* _tmp0_;
	GeeListIterator* _tmp1_;
	GeeListIterator* _tmp2_;
	GeeReadOnlyListIterator* _tmp3_;
	GeeListIterator* _tmp4_;
	GeeListIterator* result;
	self = (GeeReadOnlyList*) base;
	_tmp0_ = ((GeeReadOnlyCollection*) self)->_collection;
	_tmp1_ = gee_list_list_iterator (G_TYPE_CHECK_INSTANCE_CAST (_tmp0_, GEE_TYPE_LIST, GeeList));
	_tmp2_ = _tmp1_;
	_tmp3_ = gee_read_only_list_iterator_new (self->priv->g_type, (GBoxedCopyFunc) self->priv->g_dup_func, (GDestroyNotify) self->priv->g_destroy_func, _tmp2_);
	_tmp4_ = (GeeListIterator*) _tmp3_;
	_g_object_unref0 (_tmp2_);
	result = _tmp4_;
	return result;
}

/**
 * {@inheritDoc}
 */
static gint
gee_read_only_list_real_index_of (GeeList* base,
                                  gconstpointer item)
{
	GeeReadOnlyList * self;
	GeeCollection* _tmp0_;
	gint result;
	self = (GeeReadOnlyList*) base;
	_tmp0_ = ((GeeReadOnlyCollection*) self)->_collection;
	result = gee_list_index_of (G_TYPE_CHECK_INSTANCE_CAST (_tmp0_, GEE_TYPE_LIST, GeeList), item);
	return result;
}

/**
 * Unimplemented method (read only list).
 */
static void
gee_read_only_list_real_insert (GeeList* base,
                                gint index,
                                gconstpointer item)
{
	GeeReadOnlyList * self;
	self = (GeeReadOnlyList*) base;
	g_assert_not_reached ();
}

/**
 * Unimplemented method (read only list).
 */
static gpointer
gee_read_only_list_real_remove_at (GeeList* base,
                                   gint index)
{
	GeeReadOnlyList * self;
	self = (GeeReadOnlyList*) base;
	g_assert_not_reached ();
}

/**
 * {@inheritDoc}
 */
static gpointer
gee_read_only_list_real_get (GeeList* base,
                             gint index)
{
	GeeReadOnlyList * self;
	GeeCollection* _tmp0_;
	gpointer _tmp1_;
	gpointer result;
	self = (GeeReadOnlyList*) base;
	_tmp0_ = ((GeeReadOnlyCollection*) self)->_collection;
	_tmp1_ = gee_list_get (G_TYPE_CHECK_INSTANCE_CAST (_tmp0_, GEE_TYPE_LIST, GeeList), index);
	result = _tmp1_;
	return result;
}

/**
 * Unimplemented method (read only list).
 */
static void
gee_read_only_list_real_set (GeeList* base,
                             gint index,
                             gconstpointer o)
{
	GeeReadOnlyList * self;
	self = (GeeReadOnlyList*) base;
	g_assert_not_reached ();
}

/**
 * {@inheritDoc}
 */
static GeeList*
gee_read_only_list_real_slice (GeeList* base,
                               gint start,
                               gint stop)
{
	GeeReadOnlyList * self;
	GeeCollection* _tmp0_;
	GeeList* _tmp1_;
	GeeList* result;
	self = (GeeReadOnlyList*) base;
	_tmp0_ = ((GeeReadOnlyCollection*) self)->_collection;
	_tmp1_ = gee_list_slice (G_TYPE_CHECK_INSTANCE_CAST (_tmp0_, GEE_TYPE_LIST, GeeList), start, stop);
	result = _tmp1_;
	return result;
}

/**
 * {@inheritDoc}
 */
static gpointer
gee_read_only_list_real_first (GeeList* base)
{
	GeeReadOnlyList * self;
	GeeCollection* _tmp0_;
	gpointer _tmp1_;
	gpointer result;
	self = (GeeReadOnlyList*) base;
	_tmp0_ = ((GeeReadOnlyCollection*) self)->_collection;
	_tmp1_ = gee_list_first (G_TYPE_CHECK_INSTANCE_CAST (_tmp0_, GEE_TYPE_LIST, GeeList));
	result = _tmp1_;
	return result;
}

/**
 * {@inheritDoc}
 */
static gpointer
gee_read_only_list_real_last (GeeList* base)
{
	GeeReadOnlyList * self;
	GeeCollection* _tmp0_;
	gpointer _tmp1_;
	gpointer result;
	self = (GeeReadOnlyList*) base;
	_tmp0_ = ((GeeReadOnlyCollection*) self)->_collection;
	_tmp1_ = gee_list_last (G_TYPE_CHECK_INSTANCE_CAST (_tmp0_, GEE_TYPE_LIST, GeeList));
	result = _tmp1_;
	return result;
}

/**
 * Unimplemented method (read only list).
 */
static void
gee_read_only_list_real_insert_all (GeeList* base,
                                    gint index,
                                    GeeCollection* collection)
{
	GeeReadOnlyList * self;
	self = (GeeReadOnlyList*) base;
	g_return_if_fail (collection != NULL);
	g_assert_not_reached ();
}

/**
 * {@inheritDoc}
 */
static void
gee_read_only_list_real_sort (GeeList* base,
                              GCompareDataFunc compare,
                              gpointer compare_target,
                              GDestroyNotify compare_target_destroy_notify)
{
	GeeReadOnlyList * self;
	self = (GeeReadOnlyList*) base;
	g_assert_not_reached ();
}

 G_GNUC_INTERNAL GeeList*
gee_read_only_list_get_read_only_view (GeeReadOnlyList* self)
{
	GeeReadOnlyListClass* _klass_;
	g_return_val_if_fail (self != NULL, NULL);
	_klass_ = GEE_READ_ONLY_LIST_GET_CLASS (self);
	if (_klass_->get_read_only_view) {
		return _klass_->get_read_only_view (self);
	}
	return NULL;
}

static gpointer
_g_object_ref0 (gpointer self)
{
	return self ? g_object_ref (self) : NULL;
}

static GeeList*
gee_read_only_list_real_get_read_only_view (GeeReadOnlyList* base)
{
	GeeList* result;
	GeeReadOnlyList* self;
	GeeList* _tmp0_;
	self = base;
	_tmp0_ = _g_object_ref0 ((GeeList*) self);
	result = _tmp0_;
	return result;
}

static inline gpointer
gee_read_only_list_iterator_get_instance_private (GeeReadOnlyListIterator* self)
{
	return G_STRUCT_MEMBER_P (self, GeeReadOnlyListIterator_private_offset);
}

 G_GNUC_INTERNAL GeeReadOnlyListIterator*
gee_read_only_list_iterator_construct (GType object_type,
                                       GType g_type,
                                       GBoxedCopyFunc g_dup_func,
                                       GDestroyNotify g_destroy_func,
                                       GeeListIterator* iterator)
{
	GeeReadOnlyListIterator * self = NULL;
	g_return_val_if_fail (iterator != NULL, NULL);
	self = (GeeReadOnlyListIterator*) gee_read_only_collection_iterator_construct (object_type, g_type, (GBoxedCopyFunc) g_dup_func, (GDestroyNotify) g_destroy_func, (GeeIterator*) iterator);
	self->priv->g_type = g_type;
	self->priv->g_dup_func = g_dup_func;
	self->priv->g_destroy_func = g_destroy_func;
	return self;
}

 G_GNUC_INTERNAL GeeReadOnlyListIterator*
gee_read_only_list_iterator_new (GType g_type,
                                 GBoxedCopyFunc g_dup_func,
                                 GDestroyNotify g_destroy_func,
                                 GeeListIterator* iterator)
{
	return gee_read_only_list_iterator_construct (GEE_READ_ONLY_LIST_TYPE_ITERATOR, g_type, g_dup_func, g_destroy_func, iterator);
}

static void
gee_read_only_list_iterator_real_set (GeeListIterator* base,
                                      gconstpointer item)
{
	GeeReadOnlyListIterator * self;
	self = (GeeReadOnlyListIterator*) base;
	g_assert_not_reached ();
}

static void
gee_read_only_list_iterator_real_add (GeeListIterator* base,
                                      gconstpointer item)
{
	GeeReadOnlyListIterator * self;
	self = (GeeReadOnlyListIterator*) base;
	g_assert_not_reached ();
}

static gint
gee_read_only_list_iterator_real_index (GeeListIterator* base)
{
	GeeReadOnlyListIterator * self;
	GeeIterator* _tmp0_;
	gint result;
	self = (GeeReadOnlyListIterator*) base;
	_tmp0_ = ((GeeReadOnlyCollectionIterator*) self)->_iter;
	result = gee_list_iterator_index (G_TYPE_CHECK_INSTANCE_CAST (_tmp0_, GEE_TYPE_LIST_ITERATOR, GeeListIterator));
	return result;
}

static void
gee_read_only_list_iterator_class_init (GeeReadOnlyListIteratorClass * klass,
                                        gpointer klass_data)
{
	gee_read_only_list_iterator_parent_class = g_type_class_peek_parent (klass);
	g_type_class_adjust_private_offset (klass, &GeeReadOnlyListIterator_private_offset);
	G_OBJECT_CLASS (klass)->get_property = _vala_gee_read_only_list_iterator_get_property;
	G_OBJECT_CLASS (klass)->set_property = _vala_gee_read_only_list_iterator_set_property;
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_READ_ONLY_LIST_ITERATOR_G_TYPE, g_param_spec_gtype ("g-type", "type", "type", G_TYPE_NONE, G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_READ_ONLY_LIST_ITERATOR_G_DUP_FUNC, g_param_spec_pointer ("g-dup-func", "dup func", "dup func", G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_READ_ONLY_LIST_ITERATOR_G_DESTROY_FUNC, g_param_spec_pointer ("g-destroy-func", "destroy func", "destroy func", G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
}

static void
gee_read_only_list_iterator_gee_list_iterator_interface_init (GeeListIteratorIface * iface,
                                                              gpointer iface_data)
{
	gee_read_only_list_iterator_gee_list_iterator_parent_iface = g_type_interface_peek_parent (iface);
	iface->set = (void (*) (GeeListIterator*, gconstpointer)) gee_read_only_list_iterator_real_set;
	iface->add = (void (*) (GeeListIterator*, gconstpointer)) gee_read_only_list_iterator_real_add;
	iface->index = (gint (*) (GeeListIterator*)) gee_read_only_list_iterator_real_index;
}

static void
gee_read_only_list_iterator_instance_init (GeeReadOnlyListIterator * self,
                                           gpointer klass)
{
	self->priv = gee_read_only_list_iterator_get_instance_private (self);
}

static GType
gee_read_only_list_iterator_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (GeeReadOnlyListIteratorClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) gee_read_only_list_iterator_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GeeReadOnlyListIterator), 0, (GInstanceInitFunc) gee_read_only_list_iterator_instance_init, NULL };
	static const GInterfaceInfo gee_list_iterator_info = { (GInterfaceInitFunc) gee_read_only_list_iterator_gee_list_iterator_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
	GType gee_read_only_list_iterator_type_id;
	gee_read_only_list_iterator_type_id = g_type_register_static (GEE_READ_ONLY_COLLECTION_TYPE_ITERATOR, "GeeReadOnlyListIterator", &g_define_type_info, 0);
	g_type_add_interface_static (gee_read_only_list_iterator_type_id, GEE_TYPE_LIST_ITERATOR, &gee_list_iterator_info);
	GeeReadOnlyListIterator_private_offset = g_type_add_instance_private (gee_read_only_list_iterator_type_id, sizeof (GeeReadOnlyListIteratorPrivate));
	return gee_read_only_list_iterator_type_id;
}

GType
gee_read_only_list_iterator_get_type (void)
{
	static volatile gsize gee_read_only_list_iterator_type_id__once = 0;
	if (g_once_init_enter (&gee_read_only_list_iterator_type_id__once)) {
		GType gee_read_only_list_iterator_type_id;
		gee_read_only_list_iterator_type_id = gee_read_only_list_iterator_get_type_once ();
		g_once_init_leave (&gee_read_only_list_iterator_type_id__once, gee_read_only_list_iterator_type_id);
	}
	return gee_read_only_list_iterator_type_id__once;
}

static void
_vala_gee_read_only_list_iterator_get_property (GObject * object,
                                                guint property_id,
                                                GValue * value,
                                                GParamSpec * pspec)
{
	GeeReadOnlyListIterator * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, GEE_READ_ONLY_LIST_TYPE_ITERATOR, GeeReadOnlyListIterator);
	switch (property_id) {
		case GEE_READ_ONLY_LIST_ITERATOR_G_TYPE:
		g_value_set_gtype (value, self->priv->g_type);
		break;
		case GEE_READ_ONLY_LIST_ITERATOR_G_DUP_FUNC:
		g_value_set_pointer (value, self->priv->g_dup_func);
		break;
		case GEE_READ_ONLY_LIST_ITERATOR_G_DESTROY_FUNC:
		g_value_set_pointer (value, self->priv->g_destroy_func);
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

static void
_vala_gee_read_only_list_iterator_set_property (GObject * object,
                                                guint property_id,
                                                const GValue * value,
                                                GParamSpec * pspec)
{
	GeeReadOnlyListIterator * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, GEE_READ_ONLY_LIST_TYPE_ITERATOR, GeeReadOnlyListIterator);
	switch (property_id) {
		case GEE_READ_ONLY_LIST_ITERATOR_G_TYPE:
		self->priv->g_type = g_value_get_gtype (value);
		break;
		case GEE_READ_ONLY_LIST_ITERATOR_G_DUP_FUNC:
		self->priv->g_dup_func = g_value_get_pointer (value);
		break;
		case GEE_READ_ONLY_LIST_ITERATOR_G_DESTROY_FUNC:
		self->priv->g_destroy_func = g_value_get_pointer (value);
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

static void
gee_read_only_list_class_init (GeeReadOnlyListClass * klass,
                               gpointer klass_data)
{
	gee_read_only_list_parent_class = g_type_class_peek_parent (klass);
	g_type_class_adjust_private_offset (klass, &GeeReadOnlyList_private_offset);
	GEE_READ_ONLY_LIST_CLASS (klass)->get_read_only_view = gee_read_only_list_real_get_read_only_view;
	G_OBJECT_CLASS (klass)->get_property = _vala_gee_read_only_list_get_property;
	G_OBJECT_CLASS (klass)->set_property = _vala_gee_read_only_list_set_property;
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_READ_ONLY_LIST_G_TYPE, g_param_spec_gtype ("g-type", "type", "type", G_TYPE_NONE, G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_READ_ONLY_LIST_G_DUP_FUNC, g_param_spec_pointer ("g-dup-func", "dup func", "dup func", G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_READ_ONLY_LIST_G_DESTROY_FUNC, g_param_spec_pointer ("g-destroy-func", "destroy func", "destroy func", G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	/**
	 * {@inheritDoc}
	 */
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_READ_ONLY_LIST_READ_ONLY_VIEW_PROPERTY, gee_read_only_list_properties[GEE_READ_ONLY_LIST_READ_ONLY_VIEW_PROPERTY] = g_param_spec_object ("read-only-view", "read-only-view", "read-only-view", GEE_TYPE_LIST, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE));
}

static GType
gee_read_only_list_gee_list_get_g_type (GeeReadOnlyList* self)
{
	return self->priv->g_type;
}

static GBoxedCopyFunc
gee_read_only_list_gee_list_get_g_dup_func (GeeReadOnlyList* self)
{
	return self->priv->g_dup_func;
}

static GDestroyNotify
gee_read_only_list_gee_list_get_g_destroy_func (GeeReadOnlyList* self)
{
	return self->priv->g_destroy_func;
}

static void
gee_read_only_list_gee_list_interface_init (GeeListIface * iface,
                                            gpointer iface_data)
{
	gee_read_only_list_gee_list_parent_iface = g_type_interface_peek_parent (iface);
	iface->list_iterator = (GeeListIterator* (*) (GeeList*)) gee_read_only_list_real_list_iterator;
	iface->index_of = (gint (*) (GeeList*, gconstpointer)) gee_read_only_list_real_index_of;
	iface->insert = (void (*) (GeeList*, gint, gconstpointer)) gee_read_only_list_real_insert;
	iface->remove_at = (gpointer (*) (GeeList*, gint)) gee_read_only_list_real_remove_at;
	iface->get = (gpointer (*) (GeeList*, gint)) gee_read_only_list_real_get;
	iface->set = (void (*) (GeeList*, gint, gconstpointer)) gee_read_only_list_real_set;
	iface->slice = (GeeList* (*) (GeeList*, gint, gint)) gee_read_only_list_real_slice;
	iface->first = (gpointer (*) (GeeList*)) gee_read_only_list_real_first;
	iface->last = (gpointer (*) (GeeList*)) gee_read_only_list_real_last;
	iface->insert_all = (void (*) (GeeList*, gint, GeeCollection*)) gee_read_only_list_real_insert_all;
	iface->sort = (void (*) (GeeList*, GCompareDataFunc, gpointer, GDestroyNotify)) gee_read_only_list_real_sort;
	iface->get_g_type = (GType (*) (GeeList *)) gee_read_only_list_gee_list_get_g_type;
	iface->get_g_dup_func = (GBoxedCopyFunc (*) (GeeList *)) gee_read_only_list_gee_list_get_g_dup_func;
	iface->get_g_destroy_func = (GDestroyNotify (*) (GeeList *)) gee_read_only_list_gee_list_get_g_destroy_func;
	iface->get_read_only_view = (GeeList* (*) (GeeList *)) gee_read_only_list_get_read_only_view;
}

static void
gee_read_only_list_instance_init (GeeReadOnlyList * self,
                                  gpointer klass)
{
	self->priv = gee_read_only_list_get_instance_private (self);
}

/**
 * Read-only view for {@link List} collections.
 *
 * This class decorates any class which implements the {@link List}
 * interface by making it read only. Any method which normally modify data will
 * throw an error.
 *
 * @see List
 */
static GType
gee_read_only_list_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (GeeReadOnlyListClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) gee_read_only_list_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GeeReadOnlyList), 0, (GInstanceInitFunc) gee_read_only_list_instance_init, NULL };
	static const GInterfaceInfo gee_list_info = { (GInterfaceInitFunc) gee_read_only_list_gee_list_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
	GType gee_read_only_list_type_id;
	gee_read_only_list_type_id = g_type_register_static (GEE_TYPE_READ_ONLY_COLLECTION, "GeeReadOnlyList", &g_define_type_info, 0);
	g_type_add_interface_static (gee_read_only_list_type_id, GEE_TYPE_LIST, &gee_list_info);
	GeeReadOnlyList_private_offset = g_type_add_instance_private (gee_read_only_list_type_id, sizeof (GeeReadOnlyListPrivate));
	return gee_read_only_list_type_id;
}

 G_GNUC_INTERNAL GType
gee_read_only_list_get_type (void)
{
	static volatile gsize gee_read_only_list_type_id__once = 0;
	if (g_once_init_enter (&gee_read_only_list_type_id__once)) {
		GType gee_read_only_list_type_id;
		gee_read_only_list_type_id = gee_read_only_list_get_type_once ();
		g_once_init_leave (&gee_read_only_list_type_id__once, gee_read_only_list_type_id);
	}
	return gee_read_only_list_type_id__once;
}

static void
_vala_gee_read_only_list_get_property (GObject * object,
                                       guint property_id,
                                       GValue * value,
                                       GParamSpec * pspec)
{
	GeeReadOnlyList * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, GEE_TYPE_READ_ONLY_LIST, GeeReadOnlyList);
	switch (property_id) {
		case GEE_READ_ONLY_LIST_READ_ONLY_VIEW_PROPERTY:
		g_value_take_object (value, gee_read_only_list_get_read_only_view (self));
		break;
		case GEE_READ_ONLY_LIST_G_TYPE:
		g_value_set_gtype (value, self->priv->g_type);
		break;
		case GEE_READ_ONLY_LIST_G_DUP_FUNC:
		g_value_set_pointer (value, self->priv->g_dup_func);
		break;
		case GEE_READ_ONLY_LIST_G_DESTROY_FUNC:
		g_value_set_pointer (value, self->priv->g_destroy_func);
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

static void
_vala_gee_read_only_list_set_property (GObject * object,
                                       guint property_id,
                                       const GValue * value,
                                       GParamSpec * pspec)
{
	GeeReadOnlyList * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, GEE_TYPE_READ_ONLY_LIST, GeeReadOnlyList);
	switch (property_id) {
		case GEE_READ_ONLY_LIST_G_TYPE:
		self->priv->g_type = g_value_get_gtype (value);
		break;
		case GEE_READ_ONLY_LIST_G_DUP_FUNC:
		self->priv->g_dup_func = g_value_get_pointer (value);
		break;
		case GEE_READ_ONLY_LIST_G_DESTROY_FUNC:
		self->priv->g_destroy_func = g_value_get_pointer (value);
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

