/* testpriorityqueue.c generated by valac 0.56.3, the Vala compiler
 * generated from testpriorityqueue.vala, do not modify */

/* testpriorityqueue.vala
 *
 * Copyright (C) 2009  Didier Villevalois
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Authors:
 * 	Didier 'Ptitjes Villevalois <ptitjes@free.fr>
 */

#include <glib-object.h>
#include <gee.h>
#include <stdlib.h>
#include <string.h>
#include <glib.h>

#if !defined(VALA_EXTERN)
#if defined(_MSC_VER)
#define VALA_EXTERN __declspec(dllexport) extern
#elif __GNUC__ >= 4
#define VALA_EXTERN __attribute__((visibility("default"))) extern
#else
#define VALA_EXTERN extern
#endif
#endif

#define GEE_TYPE_TEST_CASE (gee_test_case_get_type ())
#define GEE_TEST_CASE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_TEST_CASE, GeeTestCase))
#define GEE_TEST_CASE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_TYPE_TEST_CASE, GeeTestCaseClass))
#define GEE_IS_TEST_CASE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_TEST_CASE))
#define GEE_IS_TEST_CASE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEE_TYPE_TEST_CASE))
#define GEE_TEST_CASE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_TYPE_TEST_CASE, GeeTestCaseClass))

typedef struct _GeeTestCase GeeTestCase;
typedef struct _GeeTestCaseClass GeeTestCaseClass;
typedef struct _GeeTestCasePrivate GeeTestCasePrivate;

#define TYPE_COLLECTION_TESTS (collection_tests_get_type ())
#define COLLECTION_TESTS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_COLLECTION_TESTS, CollectionTests))
#define COLLECTION_TESTS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_COLLECTION_TESTS, CollectionTestsClass))
#define IS_COLLECTION_TESTS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_COLLECTION_TESTS))
#define IS_COLLECTION_TESTS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_COLLECTION_TESTS))
#define COLLECTION_TESTS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_COLLECTION_TESTS, CollectionTestsClass))

typedef struct _CollectionTests CollectionTests;
typedef struct _CollectionTestsClass CollectionTestsClass;
typedef struct _CollectionTestsPrivate CollectionTestsPrivate;

#define TYPE_QUEUE_TESTS (queue_tests_get_type ())
#define QUEUE_TESTS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_QUEUE_TESTS, QueueTests))
#define QUEUE_TESTS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_QUEUE_TESTS, QueueTestsClass))
#define IS_QUEUE_TESTS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_QUEUE_TESTS))
#define IS_QUEUE_TESTS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_QUEUE_TESTS))
#define QUEUE_TESTS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_QUEUE_TESTS, QueueTestsClass))

typedef struct _QueueTests QueueTests;
typedef struct _QueueTestsClass QueueTestsClass;
typedef struct _QueueTestsPrivate QueueTestsPrivate;

#define TYPE_PRIORITY_QUEUE_TESTS (priority_queue_tests_get_type ())
#define PRIORITY_QUEUE_TESTS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_PRIORITY_QUEUE_TESTS, PriorityQueueTests))
#define PRIORITY_QUEUE_TESTS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_PRIORITY_QUEUE_TESTS, PriorityQueueTestsClass))
#define IS_PRIORITY_QUEUE_TESTS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_PRIORITY_QUEUE_TESTS))
#define IS_PRIORITY_QUEUE_TESTS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_PRIORITY_QUEUE_TESTS))
#define PRIORITY_QUEUE_TESTS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_PRIORITY_QUEUE_TESTS, PriorityQueueTestsClass))

typedef struct _PriorityQueueTests PriorityQueueTests;
typedef struct _PriorityQueueTestsClass PriorityQueueTestsClass;
typedef struct _PriorityQueueTestsPrivate PriorityQueueTestsPrivate;
enum  {
	PRIORITY_QUEUE_TESTS_0_PROPERTY,
	PRIORITY_QUEUE_TESTS_NUM_PROPERTIES
};
static GParamSpec* priority_queue_tests_properties[PRIORITY_QUEUE_TESTS_NUM_PROPERTIES];
typedef void (*GeeTestCaseTestMethod) (gpointer user_data);
#define _g_object_unref0(var) ((var == NULL) ? NULL : (var = (g_object_unref (var), NULL)))
#define _g_free0(var) (var = (g_free (var), NULL))
#define _vala_assert(expr, msg) if G_LIKELY (expr) ; else g_assertion_message_expr (G_LOG_DOMAIN, __FILE__, __LINE__, G_STRFUNC, msg);
#define _vala_return_if_fail(expr, msg) if G_LIKELY (expr) ; else { g_return_if_fail_warning (G_LOG_DOMAIN, G_STRFUNC, msg); return; }
#define _vala_return_val_if_fail(expr, msg, val) if G_LIKELY (expr) ; else { g_return_if_fail_warning (G_LOG_DOMAIN, G_STRFUNC, msg); return val; }
#define _vala_warn_if_fail(expr, msg) if G_LIKELY (expr) ; else g_warn_message (G_LOG_DOMAIN, __FILE__, __LINE__, G_STRFUNC, msg);

struct _GeeTestCase {
	GObject parent_instance;
	GeeTestCasePrivate * priv;
};

struct _GeeTestCaseClass {
	GObjectClass parent_class;
	void (*set_up) (GeeTestCase* self);
	void (*tear_down) (GeeTestCase* self);
};

struct _CollectionTests {
	GeeTestCase parent_instance;
	CollectionTestsPrivate * priv;
	GeeCollection* test_collection;
};

struct _CollectionTestsClass {
	GeeTestCaseClass parent_class;
};

struct _QueueTests {
	CollectionTests parent_instance;
	QueueTestsPrivate * priv;
};

struct _QueueTestsClass {
	CollectionTestsClass parent_class;
};

struct _PriorityQueueTests {
	QueueTests parent_instance;
	PriorityQueueTestsPrivate * priv;
};

struct _PriorityQueueTestsClass {
	QueueTestsClass parent_class;
};

static gpointer priority_queue_tests_parent_class = NULL;

VALA_EXTERN GType gee_test_case_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (GeeTestCase, g_object_unref)
VALA_EXTERN GType collection_tests_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (CollectionTests, g_object_unref)
VALA_EXTERN GType queue_tests_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (QueueTests, g_object_unref)
VALA_EXTERN GType priority_queue_tests_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PriorityQueueTests, g_object_unref)
VALA_EXTERN void gee_test_case_set_up (GeeTestCase* self);
VALA_EXTERN void gee_test_case_tear_down (GeeTestCase* self);
VALA_EXTERN PriorityQueueTests* priority_queue_tests_new (void);
VALA_EXTERN PriorityQueueTests* priority_queue_tests_construct (GType object_type);
VALA_EXTERN QueueTests* queue_tests_construct (GType object_type,
                                   const gchar* name);
VALA_EXTERN void gee_test_case_add_test (GeeTestCase* self,
                             const gchar* name,
                             GeeTestCaseTestMethod test,
                             gpointer test_target,
                             GDestroyNotify test_target_destroy_notify);
static void priority_queue_tests_test_poll_gives_minimum (PriorityQueueTests* self);
static void _priority_queue_tests_test_poll_gives_minimum_gee_test_case_test_method (gpointer self);
static void priority_queue_tests_real_set_up (GeeTestCase* base);
static void priority_queue_tests_real_tear_down (GeeTestCase* base);
static GType priority_queue_tests_get_type_once (void);

static void
_priority_queue_tests_test_poll_gives_minimum_gee_test_case_test_method (gpointer self)
{
	priority_queue_tests_test_poll_gives_minimum ((PriorityQueueTests*) self);
}

PriorityQueueTests*
priority_queue_tests_construct (GType object_type)
{
	PriorityQueueTests * self = NULL;
	self = (PriorityQueueTests*) queue_tests_construct (object_type, "PriorityQueue");
	gee_test_case_add_test ((GeeTestCase*) self, "[PriorityQueue] poll gives minimum", _priority_queue_tests_test_poll_gives_minimum_gee_test_case_test_method, g_object_ref (self), g_object_unref);
	return self;
}

PriorityQueueTests*
priority_queue_tests_new (void)
{
	return priority_queue_tests_construct (TYPE_PRIORITY_QUEUE_TESTS);
}

static void
priority_queue_tests_real_set_up (GeeTestCase* base)
{
	PriorityQueueTests * self;
	GeePriorityQueue* _tmp0_;
	self = (PriorityQueueTests*) base;
	_tmp0_ = gee_priority_queue_new (G_TYPE_STRING, (GBoxedCopyFunc) g_strdup, (GDestroyNotify) g_free, NULL, NULL, NULL);
	_g_object_unref0 (((CollectionTests*) self)->test_collection);
	((CollectionTests*) self)->test_collection = (GeeCollection*) _tmp0_;
}

static void
priority_queue_tests_real_tear_down (GeeTestCase* base)
{
	PriorityQueueTests * self;
	self = (PriorityQueueTests*) base;
	_g_object_unref0 (((CollectionTests*) self)->test_collection);
	((CollectionTests*) self)->test_collection = NULL;
}

static gpointer
_g_object_ref0 (gpointer self)
{
	return self ? g_object_ref (self) : NULL;
}

static void
priority_queue_tests_test_poll_gives_minimum (PriorityQueueTests* self)
{
	GeeQueue* test_queue = NULL;
	GeeCollection* _tmp0_;
	GeeQueue* _tmp1_;
	gpointer _tmp2_;
	gchar* _tmp3_;
	gpointer _tmp4_;
	gchar* _tmp5_;
	gpointer _tmp6_;
	gchar* _tmp7_;
	gpointer _tmp8_;
	gchar* _tmp9_;
	gpointer _tmp10_;
	gchar* _tmp11_;
	gpointer _tmp12_;
	gchar* _tmp13_;
	gpointer _tmp14_;
	gchar* _tmp15_;
	gpointer _tmp16_;
	gchar* _tmp17_;
	gpointer _tmp18_;
	gchar* _tmp19_;
	gpointer _tmp20_;
	gchar* _tmp21_;
	gpointer _tmp22_;
	gchar* _tmp23_;
	gpointer _tmp24_;
	gchar* _tmp25_;
	gpointer _tmp26_;
	gchar* _tmp27_;
	gpointer _tmp28_;
	gchar* _tmp29_;
	gpointer _tmp30_;
	gchar* _tmp31_;
	gpointer _tmp32_;
	gchar* _tmp33_;
	gpointer _tmp34_;
	gchar* _tmp35_;
	gpointer _tmp36_;
	gchar* _tmp37_;
	gpointer _tmp38_;
	gchar* _tmp39_;
	gpointer _tmp40_;
	gchar* _tmp41_;
	gpointer _tmp42_;
	gchar* _tmp43_;
	gpointer _tmp44_;
	gchar* _tmp45_;
	gpointer _tmp46_;
	gchar* _tmp47_;
	gpointer _tmp48_;
	gchar* _tmp49_;
	gpointer _tmp50_;
	gchar* _tmp51_;
	gpointer _tmp52_;
	gchar* _tmp53_;
	gpointer _tmp54_;
	gchar* _tmp55_;
	gpointer _tmp56_;
	gchar* _tmp57_;
	gpointer _tmp58_;
	gchar* _tmp59_;
	gpointer _tmp60_;
	gchar* _tmp61_;
	gpointer _tmp62_;
	gchar* _tmp63_;
	g_return_if_fail (self != NULL);
	_tmp0_ = ((CollectionTests*) self)->test_collection;
	_tmp1_ = _g_object_ref0 (G_TYPE_CHECK_INSTANCE_TYPE (_tmp0_, GEE_TYPE_QUEUE) ? ((GeeQueue*) _tmp0_) : NULL);
	test_queue = _tmp1_;
	_vala_assert (test_queue != NULL, "test_queue != null");
	_vala_assert (gee_queue_offer (test_queue, "one"), "test_queue.offer (\"one\")");
	_vala_assert (gee_queue_offer (test_queue, "two"), "test_queue.offer (\"two\")");
	_tmp2_ = gee_queue_peek (test_queue);
	_tmp3_ = (gchar*) _tmp2_;
	_vala_assert (g_strcmp0 (_tmp3_, "one") == 0, "test_queue.peek () == \"one\"");
	_g_free0 (_tmp3_);
	_vala_assert (gee_collection_remove ((GeeCollection*) test_queue, "two"), "test_queue.remove (\"two\")");
	_tmp4_ = gee_queue_peek (test_queue);
	_tmp5_ = (gchar*) _tmp4_;
	_vala_assert (g_strcmp0 (_tmp5_, "one") == 0, "test_queue.peek () == \"one\"");
	_g_free0 (_tmp5_);
	_tmp6_ = gee_queue_poll (test_queue);
	_tmp7_ = (gchar*) _tmp6_;
	_vala_assert (g_strcmp0 (_tmp7_, "one") == 0, "test_queue.poll () == \"one\"");
	_g_free0 (_tmp7_);
	_vala_assert (gee_queue_offer (test_queue, "one"), "test_queue.offer (\"one\")");
	_vala_assert (gee_queue_offer (test_queue, "two"), "test_queue.offer (\"two\")");
	_vala_assert (gee_queue_offer (test_queue, "three"), "test_queue.offer (\"three\")");
	_vala_assert (gee_queue_offer (test_queue, "four"), "test_queue.offer (\"four\")");
	_vala_assert (gee_queue_offer (test_queue, "five"), "test_queue.offer (\"five\")");
	_vala_assert (gee_queue_offer (test_queue, "six"), "test_queue.offer (\"six\")");
	_vala_assert (gee_queue_offer (test_queue, "seven"), "test_queue.offer (\"seven\")");
	_vala_assert (gee_queue_offer (test_queue, "eight"), "test_queue.offer (\"eight\")");
	_vala_assert (gee_queue_offer (test_queue, "nine"), "test_queue.offer (\"nine\")");
	_vala_assert (gee_queue_offer (test_queue, "ten"), "test_queue.offer (\"ten\")");
	_vala_assert (gee_queue_offer (test_queue, "eleven"), "test_queue.offer (\"eleven\")");
	_vala_assert (gee_queue_offer (test_queue, "twelve"), "test_queue.offer (\"twelve\")");
	_tmp8_ = gee_queue_peek (test_queue);
	_tmp9_ = (gchar*) _tmp8_;
	_vala_assert (g_strcmp0 (_tmp9_, "eight") == 0, "test_queue.peek () == \"eight\"");
	_g_free0 (_tmp9_);
	_tmp10_ = gee_queue_poll (test_queue);
	_tmp11_ = (gchar*) _tmp10_;
	_vala_assert (g_strcmp0 (_tmp11_, "eight") == 0, "test_queue.poll () == \"eight\"");
	_g_free0 (_tmp11_);
	_tmp12_ = gee_queue_peek (test_queue);
	_tmp13_ = (gchar*) _tmp12_;
	_vala_assert (g_strcmp0 (_tmp13_, "eleven") == 0, "test_queue.peek () == \"eleven\"");
	_g_free0 (_tmp13_);
	_tmp14_ = gee_queue_poll (test_queue);
	_tmp15_ = (gchar*) _tmp14_;
	_vala_assert (g_strcmp0 (_tmp15_, "eleven") == 0, "test_queue.poll () == \"eleven\"");
	_g_free0 (_tmp15_);
	_tmp16_ = gee_queue_peek (test_queue);
	_tmp17_ = (gchar*) _tmp16_;
	_vala_assert (g_strcmp0 (_tmp17_, "five") == 0, "test_queue.peek () == \"five\"");
	_g_free0 (_tmp17_);
	_tmp18_ = gee_queue_poll (test_queue);
	_tmp19_ = (gchar*) _tmp18_;
	_vala_assert (g_strcmp0 (_tmp19_, "five") == 0, "test_queue.poll () == \"five\"");
	_g_free0 (_tmp19_);
	_tmp20_ = gee_queue_peek (test_queue);
	_tmp21_ = (gchar*) _tmp20_;
	_vala_assert (g_strcmp0 (_tmp21_, "four") == 0, "test_queue.peek () == \"four\"");
	_g_free0 (_tmp21_);
	_tmp22_ = gee_queue_poll (test_queue);
	_tmp23_ = (gchar*) _tmp22_;
	_vala_assert (g_strcmp0 (_tmp23_, "four") == 0, "test_queue.poll () == \"four\"");
	_g_free0 (_tmp23_);
	_tmp24_ = gee_queue_peek (test_queue);
	_tmp25_ = (gchar*) _tmp24_;
	_vala_assert (g_strcmp0 (_tmp25_, "nine") == 0, "test_queue.peek () == \"nine\"");
	_g_free0 (_tmp25_);
	_tmp26_ = gee_queue_poll (test_queue);
	_tmp27_ = (gchar*) _tmp26_;
	_vala_assert (g_strcmp0 (_tmp27_, "nine") == 0, "test_queue.poll () == \"nine\"");
	_g_free0 (_tmp27_);
	_tmp28_ = gee_queue_peek (test_queue);
	_tmp29_ = (gchar*) _tmp28_;
	_vala_assert (g_strcmp0 (_tmp29_, "one") == 0, "test_queue.peek () == \"one\"");
	_g_free0 (_tmp29_);
	_tmp30_ = gee_queue_poll (test_queue);
	_tmp31_ = (gchar*) _tmp30_;
	_vala_assert (g_strcmp0 (_tmp31_, "one") == 0, "test_queue.poll () == \"one\"");
	_g_free0 (_tmp31_);
	_tmp32_ = gee_queue_peek (test_queue);
	_tmp33_ = (gchar*) _tmp32_;
	_vala_assert (g_strcmp0 (_tmp33_, "seven") == 0, "test_queue.peek () == \"seven\"");
	_g_free0 (_tmp33_);
	_tmp34_ = gee_queue_poll (test_queue);
	_tmp35_ = (gchar*) _tmp34_;
	_vala_assert (g_strcmp0 (_tmp35_, "seven") == 0, "test_queue.poll () == \"seven\"");
	_g_free0 (_tmp35_);
	_tmp36_ = gee_queue_peek (test_queue);
	_tmp37_ = (gchar*) _tmp36_;
	_vala_assert (g_strcmp0 (_tmp37_, "six") == 0, "test_queue.peek () == \"six\"");
	_g_free0 (_tmp37_);
	_tmp38_ = gee_queue_poll (test_queue);
	_tmp39_ = (gchar*) _tmp38_;
	_vala_assert (g_strcmp0 (_tmp39_, "six") == 0, "test_queue.poll () == \"six\"");
	_g_free0 (_tmp39_);
	_tmp40_ = gee_queue_peek (test_queue);
	_tmp41_ = (gchar*) _tmp40_;
	_vala_assert (g_strcmp0 (_tmp41_, "ten") == 0, "test_queue.peek () == \"ten\"");
	_g_free0 (_tmp41_);
	_tmp42_ = gee_queue_poll (test_queue);
	_tmp43_ = (gchar*) _tmp42_;
	_vala_assert (g_strcmp0 (_tmp43_, "ten") == 0, "test_queue.poll () == \"ten\"");
	_g_free0 (_tmp43_);
	_tmp44_ = gee_queue_peek (test_queue);
	_tmp45_ = (gchar*) _tmp44_;
	_vala_assert (g_strcmp0 (_tmp45_, "three") == 0, "test_queue.peek () == \"three\"");
	_g_free0 (_tmp45_);
	_tmp46_ = gee_queue_poll (test_queue);
	_tmp47_ = (gchar*) _tmp46_;
	_vala_assert (g_strcmp0 (_tmp47_, "three") == 0, "test_queue.poll () == \"three\"");
	_g_free0 (_tmp47_);
	_tmp48_ = gee_queue_peek (test_queue);
	_tmp49_ = (gchar*) _tmp48_;
	_vala_assert (g_strcmp0 (_tmp49_, "twelve") == 0, "test_queue.peek () == \"twelve\"");
	_g_free0 (_tmp49_);
	_tmp50_ = gee_queue_poll (test_queue);
	_tmp51_ = (gchar*) _tmp50_;
	_vala_assert (g_strcmp0 (_tmp51_, "twelve") == 0, "test_queue.poll () == \"twelve\"");
	_g_free0 (_tmp51_);
	_tmp52_ = gee_queue_peek (test_queue);
	_tmp53_ = (gchar*) _tmp52_;
	_vala_assert (g_strcmp0 (_tmp53_, "two") == 0, "test_queue.peek () == \"two\"");
	_g_free0 (_tmp53_);
	_tmp54_ = gee_queue_poll (test_queue);
	_tmp55_ = (gchar*) _tmp54_;
	_vala_assert (g_strcmp0 (_tmp55_, "two") == 0, "test_queue.poll () == \"two\"");
	_g_free0 (_tmp55_);
	_vala_assert (gee_queue_offer (test_queue, "2"), "test_queue.offer (\"2\")");
	_vala_assert (gee_queue_offer (test_queue, "1"), "test_queue.offer (\"1\")");
	_tmp56_ = gee_queue_peek (test_queue);
	_tmp57_ = (gchar*) _tmp56_;
	_vala_assert (g_strcmp0 (_tmp57_, "1") == 0, "test_queue.peek () == \"1\"");
	_g_free0 (_tmp57_);
	_tmp58_ = gee_queue_poll (test_queue);
	_tmp59_ = (gchar*) _tmp58_;
	_vala_assert (g_strcmp0 (_tmp59_, "1") == 0, "test_queue.poll () == \"1\"");
	_g_free0 (_tmp59_);
	_tmp60_ = gee_queue_peek (test_queue);
	_tmp61_ = (gchar*) _tmp60_;
	_vala_assert (g_strcmp0 (_tmp61_, "2") == 0, "test_queue.peek () == \"2\"");
	_g_free0 (_tmp61_);
	_tmp62_ = gee_queue_poll (test_queue);
	_tmp63_ = (gchar*) _tmp62_;
	_vala_assert (g_strcmp0 (_tmp63_, "2") == 0, "test_queue.poll () == \"2\"");
	_g_free0 (_tmp63_);
	_g_object_unref0 (test_queue);
}

static void
priority_queue_tests_class_init (PriorityQueueTestsClass * klass,
                                 gpointer klass_data)
{
	priority_queue_tests_parent_class = g_type_class_peek_parent (klass);
	((GeeTestCaseClass *) klass)->set_up = (void (*) (GeeTestCase*)) priority_queue_tests_real_set_up;
	((GeeTestCaseClass *) klass)->tear_down = (void (*) (GeeTestCase*)) priority_queue_tests_real_tear_down;
}

static void
priority_queue_tests_instance_init (PriorityQueueTests * self,
                                    gpointer klass)
{
}

static GType
priority_queue_tests_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (PriorityQueueTestsClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) priority_queue_tests_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (PriorityQueueTests), 0, (GInstanceInitFunc) priority_queue_tests_instance_init, NULL };
	GType priority_queue_tests_type_id;
	priority_queue_tests_type_id = g_type_register_static (TYPE_QUEUE_TESTS, "PriorityQueueTests", &g_define_type_info, 0);
	return priority_queue_tests_type_id;
}

GType
priority_queue_tests_get_type (void)
{
	static volatile gsize priority_queue_tests_type_id__once = 0;
	if (g_once_init_enter (&priority_queue_tests_type_id__once)) {
		GType priority_queue_tests_type_id;
		priority_queue_tests_type_id = priority_queue_tests_get_type_once ();
		g_once_init_leave (&priority_queue_tests_type_id__once, priority_queue_tests_type_id);
	}
	return priority_queue_tests_type_id__once;
}

