/* treemultimap.c generated by valac 0.56.3, the Vala compiler
 * generated from treemultimap.vala, do not modify */

/* treemultimap.vala
 *
 * Copyright (C) 2009  Didier Villevalois
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Didier 'Ptitjes Villevalois <ptitjes@free.fr>
 */

#include "gee.h"
#include <glib-object.h>
#include <glib.h>

#define GEE_FUNCTIONS_TYPE_COMPARE_DATA_FUNC_CLOSURE (gee_functions_compare_data_func_closure_get_type ())
#define GEE_FUNCTIONS_COMPARE_DATA_FUNC_CLOSURE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_FUNCTIONS_TYPE_COMPARE_DATA_FUNC_CLOSURE, GeeFunctionsCompareDataFuncClosure))
#define GEE_FUNCTIONS_COMPARE_DATA_FUNC_CLOSURE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_FUNCTIONS_TYPE_COMPARE_DATA_FUNC_CLOSURE, GeeFunctionsCompareDataFuncClosureClass))
#define GEE_FUNCTIONS_IS_COMPARE_DATA_FUNC_CLOSURE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_FUNCTIONS_TYPE_COMPARE_DATA_FUNC_CLOSURE))
#define GEE_FUNCTIONS_IS_COMPARE_DATA_FUNC_CLOSURE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEE_FUNCTIONS_TYPE_COMPARE_DATA_FUNC_CLOSURE))
#define GEE_FUNCTIONS_COMPARE_DATA_FUNC_CLOSURE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_FUNCTIONS_TYPE_COMPARE_DATA_FUNC_CLOSURE, GeeFunctionsCompareDataFuncClosureClass))

typedef struct _GeeFunctionsCompareDataFuncClosure GeeFunctionsCompareDataFuncClosure;
typedef struct _GeeFunctionsCompareDataFuncClosureClass GeeFunctionsCompareDataFuncClosureClass;
enum  {
	GEE_TREE_MULTI_MAP_0_PROPERTY,
	GEE_TREE_MULTI_MAP_K_TYPE,
	GEE_TREE_MULTI_MAP_K_DUP_FUNC,
	GEE_TREE_MULTI_MAP_K_DESTROY_FUNC,
	GEE_TREE_MULTI_MAP_V_TYPE,
	GEE_TREE_MULTI_MAP_V_DUP_FUNC,
	GEE_TREE_MULTI_MAP_V_DESTROY_FUNC,
	GEE_TREE_MULTI_MAP_NUM_PROPERTIES
};
static GParamSpec* gee_tree_multi_map_properties[GEE_TREE_MULTI_MAP_NUM_PROPERTIES];
#define _gee_functions_compare_data_func_closure_unref0(var) ((var == NULL) ? NULL : (var = (gee_functions_compare_data_func_closure_unref (var), NULL)))
#define _g_object_unref0(var) ((var == NULL) ? NULL : (var = (g_object_unref (var), NULL)))
typedef struct _GeeFunctionsCompareDataFuncClosurePrivate GeeFunctionsCompareDataFuncClosurePrivate;

struct _GeeTreeMultiMapPrivate {
	GType k_type;
	GBoxedCopyFunc k_dup_func;
	GDestroyNotify k_destroy_func;
	GType v_type;
	GBoxedCopyFunc v_dup_func;
	GDestroyNotify v_destroy_func;
	GeeFunctionsCompareDataFuncClosure* _value_compare_func;
};

struct _GeeFunctionsCompareDataFuncClosure {
	GTypeInstance parent_instance;
	volatile int ref_count;
	GeeFunctionsCompareDataFuncClosurePrivate * priv;
	GCompareDataFunc func;
	gpointer func_target;
	GDestroyNotify func_target_destroy_notify;
};

struct _GeeFunctionsCompareDataFuncClosureClass {
	GTypeClass parent_class;
	void (*finalize) (GeeFunctionsCompareDataFuncClosure *self);
};

static gint GeeTreeMultiMap_private_offset;
static gpointer gee_tree_multi_map_parent_class = NULL;

 G_GNUC_INTERNAL gpointer gee_functions_compare_data_func_closure_ref (gpointer instance);
 G_GNUC_INTERNAL void gee_functions_compare_data_func_closure_unref (gpointer instance);
 G_GNUC_INTERNAL GParamSpec* gee_functions_param_spec_compare_data_func_closure (const gchar* name,
                                                                const gchar* nick,
                                                                const gchar* blurb,
                                                                GType object_type,
                                                                GParamFlags flags);
 G_GNUC_INTERNAL void gee_functions_value_set_compare_data_func_closure (GValue* value,
                                                        gpointer v_object) G_GNUC_UNUSED ;
 G_GNUC_INTERNAL void gee_functions_value_take_compare_data_func_closure (GValue* value,
                                                         gpointer v_object);
 G_GNUC_INTERNAL gpointer gee_functions_value_get_compare_data_func_closure (const GValue* value) G_GNUC_UNUSED ;
 G_GNUC_INTERNAL GType gee_functions_compare_data_func_closure_get_type (void) G_GNUC_CONST  G_GNUC_UNUSED ;
 G_GNUC_INTERNAL GeeFunctionsCompareDataFuncClosure* gee_functions_compare_data_func_closure_new (GType g_type,
                                                                                 GBoxedCopyFunc g_dup_func,
                                                                                 GDestroyNotify g_destroy_func,
                                                                                 GCompareDataFunc func,
                                                                                 gpointer func_target,
                                                                                 GDestroyNotify func_target_destroy_notify);
 G_GNUC_INTERNAL GeeFunctionsCompareDataFuncClosure* gee_functions_compare_data_func_closure_construct (GType object_type,
                                                                                       GType g_type,
                                                                                       GBoxedCopyFunc g_dup_func,
                                                                                       GDestroyNotify g_destroy_func,
                                                                                       GCompareDataFunc func,
                                                                                       gpointer func_target,
                                                                                       GDestroyNotify func_target_destroy_notify);
static GeeCollection* gee_tree_multi_map_real_create_value_storage (GeeAbstractMultiMap* base);
 G_GNUC_INTERNAL GeeTreeSet* gee_tree_set_new_with_closures (GType g_type,
                                            GBoxedCopyFunc g_dup_func,
                                            GDestroyNotify g_destroy_func,
                                            GeeFunctionsCompareDataFuncClosure* compare_func);
 G_GNUC_INTERNAL GeeTreeSet* gee_tree_set_construct_with_closures (GType object_type,
                                                  GType g_type,
                                                  GBoxedCopyFunc g_dup_func,
                                                  GDestroyNotify g_destroy_func,
                                                  GeeFunctionsCompareDataFuncClosure* compare_func);
static GeeMultiSet* gee_tree_multi_map_real_create_multi_key_set (GeeAbstractMultiMap* base);
 G_GNUC_INTERNAL GeeFunctionsCompareDataFuncClosure* gee_tree_map_get_key_compare_func_closure (GeeTreeMap* self);
 G_GNUC_INTERNAL GeeTreeMultiSet* gee_tree_multi_set_new_with_closures (GType g_type,
                                                       GBoxedCopyFunc g_dup_func,
                                                       GDestroyNotify g_destroy_func,
                                                       GeeFunctionsCompareDataFuncClosure* compare_func);
 G_GNUC_INTERNAL GeeTreeMultiSet* gee_tree_multi_set_construct_with_closures (GType object_type,
                                                             GType g_type,
                                                             GBoxedCopyFunc g_dup_func,
                                                             GDestroyNotify g_destroy_func,
                                                             GeeFunctionsCompareDataFuncClosure* compare_func);
static GeeEqualDataFunc gee_tree_multi_map_real_get_value_equal_func (GeeAbstractMultiMap* base,
                                                               gpointer* result_target,
                                                               GDestroyNotify* result_target_destroy_notify);
static void gee_tree_multi_map_set_value_compare_func (GeeTreeMultiMap* self,
                                                GCompareDataFunc value,
                                                gpointer value_target);
static void gee_tree_multi_map_finalize (GObject * obj);
static GType gee_tree_multi_map_get_type_once (void);
static void _vala_gee_tree_multi_map_get_property (GObject * object,
                                            guint property_id,
                                            GValue * value,
                                            GParamSpec * pspec);
static void _vala_gee_tree_multi_map_set_property (GObject * object,
                                            guint property_id,
                                            const GValue * value,
                                            GParamSpec * pspec);

static inline gpointer
gee_tree_multi_map_get_instance_private (GeeTreeMultiMap* self)
{
	return G_STRUCT_MEMBER_P (self, GeeTreeMultiMap_private_offset);
}

/**
 * Constructs a new, empty tree multimap.
 *
 * If not provided, the functions parameters are requested to the
 * {@link Functions} function factory methods.
 *
 * @param key_compare_func an optional key comparator function
 * @param value_compare_func an optional value comparator function
 */
GeeTreeMultiMap*
gee_tree_multi_map_construct (GType object_type,
                              GType k_type,
                              GBoxedCopyFunc k_dup_func,
                              GDestroyNotify k_destroy_func,
                              GType v_type,
                              GBoxedCopyFunc v_dup_func,
                              GDestroyNotify v_destroy_func,
                              GCompareDataFunc key_compare_func,
                              gpointer key_compare_func_target,
                              GDestroyNotify key_compare_func_target_destroy_notify,
                              GCompareDataFunc value_compare_func,
                              gpointer value_compare_func_target,
                              GDestroyNotify value_compare_func_target_destroy_notify)
{
	GeeTreeMultiMap * self = NULL;
	GCompareDataFunc _tmp0_;
	gpointer _tmp0__target;
	GDestroyNotify _tmp0__target_destroy_notify;
	gpointer _tmp1_ = NULL;
	GDestroyNotify _tmp2_ = NULL;
	GeeEqualDataFunc _tmp3_;
	GeeTreeMap* _tmp4_;
	GeeTreeMap* _tmp5_;
	GCompareDataFunc _tmp9_;
	gpointer _tmp9__target;
	GDestroyNotify _tmp9__target_destroy_notify;
	GeeFunctionsCompareDataFuncClosure* _tmp10_;
	_tmp0_ = key_compare_func;
	_tmp0__target = key_compare_func_target;
	_tmp0__target_destroy_notify = key_compare_func_target_destroy_notify;
	key_compare_func = NULL;
	key_compare_func_target = NULL;
	key_compare_func_target_destroy_notify = NULL;
	_tmp3_ = gee_functions_get_equal_func_for (GEE_TYPE_SET, &_tmp1_, &_tmp2_);
	_tmp4_ = gee_tree_map_new (k_type, (GBoxedCopyFunc) k_dup_func, (GDestroyNotify) k_destroy_func, GEE_TYPE_SET, (GBoxedCopyFunc) g_object_ref, (GDestroyNotify) g_object_unref, _tmp0_, _tmp0__target, _tmp0__target_destroy_notify, _tmp3_, _tmp1_, _tmp2_);
	_tmp5_ = _tmp4_;
	self = (GeeTreeMultiMap*) gee_abstract_multi_map_construct (object_type, k_type, (GBoxedCopyFunc) k_dup_func, (GDestroyNotify) k_destroy_func, v_type, (GBoxedCopyFunc) v_dup_func, (GDestroyNotify) v_destroy_func, (GeeMap*) _tmp5_);
	self->priv->k_type = k_type;
	self->priv->k_dup_func = k_dup_func;
	self->priv->k_destroy_func = k_destroy_func;
	self->priv->v_type = v_type;
	self->priv->v_dup_func = v_dup_func;
	self->priv->v_destroy_func = v_destroy_func;
	_g_object_unref0 (_tmp5_);
	if (value_compare_func == NULL) {
		gpointer _tmp6_ = NULL;
		GDestroyNotify _tmp7_ = NULL;
		GCompareDataFunc _tmp8_;
		_tmp8_ = gee_functions_get_compare_func_for (v_type, &_tmp6_, &_tmp7_);
		(value_compare_func_target_destroy_notify == NULL) ? NULL : (value_compare_func_target_destroy_notify (value_compare_func_target), NULL);
		value_compare_func = NULL;
		value_compare_func_target = NULL;
		value_compare_func_target_destroy_notify = NULL;
		value_compare_func = _tmp8_;
		value_compare_func_target = _tmp6_;
		value_compare_func_target_destroy_notify = _tmp7_;
	}
	_tmp9_ = value_compare_func;
	_tmp9__target = value_compare_func_target;
	_tmp9__target_destroy_notify = value_compare_func_target_destroy_notify;
	value_compare_func = NULL;
	value_compare_func_target = NULL;
	value_compare_func_target_destroy_notify = NULL;
	_tmp10_ = gee_functions_compare_data_func_closure_new (v_type, (GBoxedCopyFunc) v_dup_func, (GDestroyNotify) v_destroy_func, _tmp9_, _tmp9__target, _tmp9__target_destroy_notify);
	_gee_functions_compare_data_func_closure_unref0 (self->priv->_value_compare_func);
	self->priv->_value_compare_func = _tmp10_;
	(key_compare_func_target_destroy_notify == NULL) ? NULL : (key_compare_func_target_destroy_notify (key_compare_func_target), NULL);
	key_compare_func = NULL;
	key_compare_func_target = NULL;
	key_compare_func_target_destroy_notify = NULL;
	(value_compare_func_target_destroy_notify == NULL) ? NULL : (value_compare_func_target_destroy_notify (value_compare_func_target), NULL);
	value_compare_func = NULL;
	value_compare_func_target = NULL;
	value_compare_func_target_destroy_notify = NULL;
	return self;
}

GeeTreeMultiMap*
gee_tree_multi_map_new (GType k_type,
                        GBoxedCopyFunc k_dup_func,
                        GDestroyNotify k_destroy_func,
                        GType v_type,
                        GBoxedCopyFunc v_dup_func,
                        GDestroyNotify v_destroy_func,
                        GCompareDataFunc key_compare_func,
                        gpointer key_compare_func_target,
                        GDestroyNotify key_compare_func_target_destroy_notify,
                        GCompareDataFunc value_compare_func,
                        gpointer value_compare_func_target,
                        GDestroyNotify value_compare_func_target_destroy_notify)
{
	return gee_tree_multi_map_construct (GEE_TYPE_TREE_MULTI_MAP, k_type, k_dup_func, k_destroy_func, v_type, v_dup_func, v_destroy_func, key_compare_func, key_compare_func_target, key_compare_func_target_destroy_notify, value_compare_func, value_compare_func_target, value_compare_func_target_destroy_notify);
}

static gpointer
_gee_functions_compare_data_func_closure_ref0 (gpointer self)
{
	return self ? gee_functions_compare_data_func_closure_ref (self) : NULL;
}

static GeeCollection*
gee_tree_multi_map_real_create_value_storage (GeeAbstractMultiMap* base)
{
	GeeTreeMultiMap * self;
	GeeFunctionsCompareDataFuncClosure* _tmp0_;
	GeeFunctionsCompareDataFuncClosure* _tmp1_;
	GeeTreeSet* _tmp2_;
	GeeCollection* result;
	self = (GeeTreeMultiMap*) base;
	_tmp0_ = self->priv->_value_compare_func;
	_tmp1_ = _gee_functions_compare_data_func_closure_ref0 (_tmp0_);
	_tmp2_ = gee_tree_set_new_with_closures (self->priv->v_type, (GBoxedCopyFunc) self->priv->v_dup_func, (GDestroyNotify) self->priv->v_destroy_func, _tmp1_);
	result = (GeeCollection*) _tmp2_;
	return result;
}

static GeeMultiSet*
gee_tree_multi_map_real_create_multi_key_set (GeeAbstractMultiMap* base)
{
	GeeTreeMultiMap * self;
	GeeMap* _tmp0_;
	GeeFunctionsCompareDataFuncClosure* _tmp1_;
	GeeTreeMultiSet* _tmp2_;
	GeeMultiSet* result;
	self = (GeeTreeMultiMap*) base;
	_tmp0_ = ((GeeAbstractMultiMap*) self)->_storage_map;
	_tmp1_ = gee_tree_map_get_key_compare_func_closure (G_TYPE_CHECK_INSTANCE_CAST (_tmp0_, GEE_TYPE_TREE_MAP, GeeTreeMap));
	_tmp2_ = gee_tree_multi_set_new_with_closures (self->priv->k_type, (GBoxedCopyFunc) self->priv->k_dup_func, (GDestroyNotify) self->priv->k_destroy_func, _tmp1_);
	result = (GeeMultiSet*) _tmp2_;
	return result;
}

static GeeEqualDataFunc
gee_tree_multi_map_real_get_value_equal_func (GeeAbstractMultiMap* base,
                                              gpointer* result_target,
                                              GDestroyNotify* result_target_destroy_notify)
{
	GeeTreeMultiMap * self;
	gpointer _tmp0_ = NULL;
	GDestroyNotify _tmp1_ = NULL;
	GeeEqualDataFunc _tmp2_;
	GeeEqualDataFunc _tmp3_;
	gpointer _tmp3__target;
	GDestroyNotify _tmp3__target_destroy_notify;
	GeeEqualDataFunc result;
	self = (GeeTreeMultiMap*) base;
	_tmp2_ = gee_functions_get_equal_func_for (self->priv->v_type, &_tmp0_, &_tmp1_);
	_tmp3_ = _tmp2_;
	_tmp3__target = _tmp0_;
	_tmp3__target_destroy_notify = _tmp1_;
	*result_target = _tmp3__target;
	*result_target_destroy_notify = _tmp3__target_destroy_notify;
	result = _tmp3_;
	return result;
}

GCompareDataFunc
gee_tree_multi_map_get_key_compare_func (GeeTreeMultiMap* self,
                                         gpointer* result_target)
{
	GCompareDataFunc result;
	GeeMap* _tmp0_;
	GCompareDataFunc _tmp1_ = NULL;
	gpointer _tmp1__target = NULL;
	GCompareDataFunc _tmp2_;
	gpointer _tmp2__target;
	GCompareDataFunc _tmp3_;
	gpointer _tmp3__target;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = ((GeeAbstractMultiMap*) self)->_storage_map;
	_tmp1_ = gee_tree_map_get_key_compare_func (G_TYPE_CHECK_INSTANCE_CAST (_tmp0_, GEE_TYPE_TREE_MAP, GeeTreeMap), &_tmp1__target);
	_tmp2_ = _tmp1_;
	_tmp2__target = _tmp1__target;
	_tmp3_ = _tmp2_;
	_tmp3__target = _tmp2__target;
	*result_target = _tmp3__target;
	result = _tmp3_;
	return result;
}

GCompareDataFunc
gee_tree_multi_map_get_value_compare_func (GeeTreeMultiMap* self,
                                           gpointer* result_target)
{
	GCompareDataFunc result;
	GeeFunctionsCompareDataFuncClosure* _tmp0_;
	GCompareDataFunc _tmp1_;
	gpointer _tmp1__target;
	GCompareDataFunc _tmp2_;
	gpointer _tmp2__target;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = self->priv->_value_compare_func;
	_tmp1_ = _tmp0_->func;
	_tmp1__target = _tmp0_->func_target;
	_tmp2_ = _tmp1_;
	_tmp2__target = _tmp1__target;
	*result_target = _tmp2__target;
	result = _tmp2_;
	return result;
}

static void
gee_tree_multi_map_set_value_compare_func (GeeTreeMultiMap* self,
                                           GCompareDataFunc value,
                                           gpointer value_target)
{
	g_return_if_fail (self != NULL);
}

static void
gee_tree_multi_map_class_init (GeeTreeMultiMapClass * klass,
                               gpointer klass_data)
{
	gee_tree_multi_map_parent_class = g_type_class_peek_parent (klass);
	g_type_class_adjust_private_offset (klass, &GeeTreeMultiMap_private_offset);
	((GeeAbstractMultiMapClass *) klass)->create_value_storage = (GeeCollection* (*) (GeeAbstractMultiMap*)) gee_tree_multi_map_real_create_value_storage;
	((GeeAbstractMultiMapClass *) klass)->create_multi_key_set = (GeeMultiSet* (*) (GeeAbstractMultiMap*)) gee_tree_multi_map_real_create_multi_key_set;
	((GeeAbstractMultiMapClass *) klass)->get_value_equal_func = (GeeEqualDataFunc (*) (GeeAbstractMultiMap*, gpointer*, GDestroyNotify*)) gee_tree_multi_map_real_get_value_equal_func;
	G_OBJECT_CLASS (klass)->get_property = _vala_gee_tree_multi_map_get_property;
	G_OBJECT_CLASS (klass)->set_property = _vala_gee_tree_multi_map_set_property;
	G_OBJECT_CLASS (klass)->finalize = gee_tree_multi_map_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_TREE_MULTI_MAP_K_TYPE, g_param_spec_gtype ("k-type", "type", "type", G_TYPE_NONE, G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_TREE_MULTI_MAP_K_DUP_FUNC, g_param_spec_pointer ("k-dup-func", "dup func", "dup func", G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_TREE_MULTI_MAP_K_DESTROY_FUNC, g_param_spec_pointer ("k-destroy-func", "destroy func", "destroy func", G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_TREE_MULTI_MAP_V_TYPE, g_param_spec_gtype ("v-type", "type", "type", G_TYPE_NONE, G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_TREE_MULTI_MAP_V_DUP_FUNC, g_param_spec_pointer ("v-dup-func", "dup func", "dup func", G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_TREE_MULTI_MAP_V_DESTROY_FUNC, g_param_spec_pointer ("v-destroy-func", "destroy func", "destroy func", G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
}

static void
gee_tree_multi_map_instance_init (GeeTreeMultiMap * self,
                                  gpointer klass)
{
	self->priv = gee_tree_multi_map_get_instance_private (self);
}

static void
gee_tree_multi_map_finalize (GObject * obj)
{
	GeeTreeMultiMap * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, GEE_TYPE_TREE_MULTI_MAP, GeeTreeMultiMap);
	_gee_functions_compare_data_func_closure_unref0 (self->priv->_value_compare_func);
	G_OBJECT_CLASS (gee_tree_multi_map_parent_class)->finalize (obj);
}

/**
 * Left-leaning red-black tree implementation of the {@link MultiMap}
 * interface.
 */
static GType
gee_tree_multi_map_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (GeeTreeMultiMapClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) gee_tree_multi_map_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GeeTreeMultiMap), 0, (GInstanceInitFunc) gee_tree_multi_map_instance_init, NULL };
	GType gee_tree_multi_map_type_id;
	gee_tree_multi_map_type_id = g_type_register_static (GEE_TYPE_ABSTRACT_MULTI_MAP, "GeeTreeMultiMap", &g_define_type_info, 0);
	GeeTreeMultiMap_private_offset = g_type_add_instance_private (gee_tree_multi_map_type_id, sizeof (GeeTreeMultiMapPrivate));
	return gee_tree_multi_map_type_id;
}

GType
gee_tree_multi_map_get_type (void)
{
	static volatile gsize gee_tree_multi_map_type_id__once = 0;
	if (g_once_init_enter (&gee_tree_multi_map_type_id__once)) {
		GType gee_tree_multi_map_type_id;
		gee_tree_multi_map_type_id = gee_tree_multi_map_get_type_once ();
		g_once_init_leave (&gee_tree_multi_map_type_id__once, gee_tree_multi_map_type_id);
	}
	return gee_tree_multi_map_type_id__once;
}

static void
_vala_gee_tree_multi_map_get_property (GObject * object,
                                       guint property_id,
                                       GValue * value,
                                       GParamSpec * pspec)
{
	GeeTreeMultiMap * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, GEE_TYPE_TREE_MULTI_MAP, GeeTreeMultiMap);
	switch (property_id) {
		case GEE_TREE_MULTI_MAP_K_TYPE:
		g_value_set_gtype (value, self->priv->k_type);
		break;
		case GEE_TREE_MULTI_MAP_K_DUP_FUNC:
		g_value_set_pointer (value, self->priv->k_dup_func);
		break;
		case GEE_TREE_MULTI_MAP_K_DESTROY_FUNC:
		g_value_set_pointer (value, self->priv->k_destroy_func);
		break;
		case GEE_TREE_MULTI_MAP_V_TYPE:
		g_value_set_gtype (value, self->priv->v_type);
		break;
		case GEE_TREE_MULTI_MAP_V_DUP_FUNC:
		g_value_set_pointer (value, self->priv->v_dup_func);
		break;
		case GEE_TREE_MULTI_MAP_V_DESTROY_FUNC:
		g_value_set_pointer (value, self->priv->v_destroy_func);
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

static void
_vala_gee_tree_multi_map_set_property (GObject * object,
                                       guint property_id,
                                       const GValue * value,
                                       GParamSpec * pspec)
{
	GeeTreeMultiMap * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, GEE_TYPE_TREE_MULTI_MAP, GeeTreeMultiMap);
	switch (property_id) {
		case GEE_TREE_MULTI_MAP_K_TYPE:
		self->priv->k_type = g_value_get_gtype (value);
		break;
		case GEE_TREE_MULTI_MAP_K_DUP_FUNC:
		self->priv->k_dup_func = g_value_get_pointer (value);
		break;
		case GEE_TREE_MULTI_MAP_K_DESTROY_FUNC:
		self->priv->k_destroy_func = g_value_get_pointer (value);
		break;
		case GEE_TREE_MULTI_MAP_V_TYPE:
		self->priv->v_type = g_value_get_gtype (value);
		break;
		case GEE_TREE_MULTI_MAP_V_DUP_FUNC:
		self->priv->v_dup_func = g_value_get_pointer (value);
		break;
		case GEE_TREE_MULTI_MAP_V_DESTROY_FUNC:
		self->priv->v_destroy_func = g_value_get_pointer (value);
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

