/* readonlymap.c generated by valac 0.56.3, the Vala compiler
 * generated from readonlymap.vala, do not modify */

/* readonlymap.vala
 *
 * Copyright (C) 2007-2008  Jürg Billeter
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Jürg Billeter <j@bitron.ch>
 */

#include <glib-object.h>
#include "gee.h"
#include <glib.h>

#define GEE_TYPE_READ_ONLY_MAP (gee_read_only_map_get_type ())
#define GEE_READ_ONLY_MAP(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_READ_ONLY_MAP, GeeReadOnlyMap))
#define GEE_READ_ONLY_MAP_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_TYPE_READ_ONLY_MAP, GeeReadOnlyMapClass))
#define GEE_IS_READ_ONLY_MAP(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_READ_ONLY_MAP))
#define GEE_IS_READ_ONLY_MAP_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEE_TYPE_READ_ONLY_MAP))
#define GEE_READ_ONLY_MAP_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_TYPE_READ_ONLY_MAP, GeeReadOnlyMapClass))

typedef struct _GeeReadOnlyMap GeeReadOnlyMap;
typedef struct _GeeReadOnlyMapClass GeeReadOnlyMapClass;
typedef struct _GeeReadOnlyMapPrivate GeeReadOnlyMapPrivate;
enum  {
	GEE_READ_ONLY_MAP_0_PROPERTY,
	GEE_READ_ONLY_MAP_K_TYPE,
	GEE_READ_ONLY_MAP_K_DUP_FUNC,
	GEE_READ_ONLY_MAP_K_DESTROY_FUNC,
	GEE_READ_ONLY_MAP_V_TYPE,
	GEE_READ_ONLY_MAP_V_DUP_FUNC,
	GEE_READ_ONLY_MAP_V_DESTROY_FUNC,
	GEE_READ_ONLY_MAP_SIZE_PROPERTY,
	GEE_READ_ONLY_MAP_READ_ONLY_PROPERTY,
	GEE_READ_ONLY_MAP_KEYS_PROPERTY,
	GEE_READ_ONLY_MAP_VALUES_PROPERTY,
	GEE_READ_ONLY_MAP_ENTRIES_PROPERTY,
	GEE_READ_ONLY_MAP_READ_ONLY_VIEW_PROPERTY,
	GEE_READ_ONLY_MAP_KEY_TYPE_PROPERTY,
	GEE_READ_ONLY_MAP_VALUE_TYPE_PROPERTY,
	GEE_READ_ONLY_MAP_NUM_PROPERTIES
};
static GParamSpec* gee_read_only_map_properties[GEE_READ_ONLY_MAP_NUM_PROPERTIES];
#define _g_object_unref0(var) ((var == NULL) ? NULL : (var = (g_object_unref (var), NULL)))

#define GEE_READ_ONLY_MAP_TYPE_MAP_ITERATOR (gee_read_only_map_map_iterator_get_type ())
#define GEE_READ_ONLY_MAP_MAP_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_READ_ONLY_MAP_TYPE_MAP_ITERATOR, GeeReadOnlyMapMapIterator))
#define GEE_READ_ONLY_MAP_MAP_ITERATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_READ_ONLY_MAP_TYPE_MAP_ITERATOR, GeeReadOnlyMapMapIteratorClass))
#define GEE_READ_ONLY_MAP_IS_MAP_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_READ_ONLY_MAP_TYPE_MAP_ITERATOR))
#define GEE_READ_ONLY_MAP_IS_MAP_ITERATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEE_READ_ONLY_MAP_TYPE_MAP_ITERATOR))
#define GEE_READ_ONLY_MAP_MAP_ITERATOR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_READ_ONLY_MAP_TYPE_MAP_ITERATOR, GeeReadOnlyMapMapIteratorClass))

typedef struct _GeeReadOnlyMapMapIterator GeeReadOnlyMapMapIterator;
typedef struct _GeeReadOnlyMapMapIteratorClass GeeReadOnlyMapMapIteratorClass;
typedef struct _GeeReadOnlyMapMapIteratorPrivate GeeReadOnlyMapMapIteratorPrivate;
enum  {
	GEE_READ_ONLY_MAP_MAP_ITERATOR_0_PROPERTY,
	GEE_READ_ONLY_MAP_MAP_ITERATOR_K_TYPE,
	GEE_READ_ONLY_MAP_MAP_ITERATOR_K_DUP_FUNC,
	GEE_READ_ONLY_MAP_MAP_ITERATOR_K_DESTROY_FUNC,
	GEE_READ_ONLY_MAP_MAP_ITERATOR_V_TYPE,
	GEE_READ_ONLY_MAP_MAP_ITERATOR_V_DUP_FUNC,
	GEE_READ_ONLY_MAP_MAP_ITERATOR_V_DESTROY_FUNC,
	GEE_READ_ONLY_MAP_MAP_ITERATOR_READ_ONLY_PROPERTY,
	GEE_READ_ONLY_MAP_MAP_ITERATOR_MUTABLE_PROPERTY,
	GEE_READ_ONLY_MAP_MAP_ITERATOR_VALID_PROPERTY,
	GEE_READ_ONLY_MAP_MAP_ITERATOR_NUM_PROPERTIES
};
static GParamSpec* gee_read_only_map_map_iterator_properties[GEE_READ_ONLY_MAP_MAP_ITERATOR_NUM_PROPERTIES];

struct _GeeReadOnlyMap {
	GObject parent_instance;
	GeeReadOnlyMapPrivate * priv;
	GeeMap* _map;
};

struct _GeeReadOnlyMapClass {
	GObjectClass parent_class;
	GeeMap* (*get_read_only_view) (GeeReadOnlyMap* self);
};

struct _GeeReadOnlyMapPrivate {
	GType k_type;
	GBoxedCopyFunc k_dup_func;
	GDestroyNotify k_destroy_func;
	GType v_type;
	GBoxedCopyFunc v_dup_func;
	GDestroyNotify v_destroy_func;
};

struct _GeeReadOnlyMapMapIterator {
	GObject parent_instance;
	GeeReadOnlyMapMapIteratorPrivate * priv;
	GeeMapIterator* _iter;
};

struct _GeeReadOnlyMapMapIteratorClass {
	GObjectClass parent_class;
};

struct _GeeReadOnlyMapMapIteratorPrivate {
	GType k_type;
	GBoxedCopyFunc k_dup_func;
	GDestroyNotify k_destroy_func;
	GType v_type;
	GBoxedCopyFunc v_dup_func;
	GDestroyNotify v_destroy_func;
};

static gint GeeReadOnlyMap_private_offset;
static gpointer gee_read_only_map_parent_class = NULL;
static gint GeeReadOnlyMapMapIterator_private_offset;
static gpointer gee_read_only_map_map_iterator_parent_class = NULL;
static GeeMapIteratorIface * gee_read_only_map_map_iterator_gee_map_iterator_parent_iface = NULL;
static GeeTraversableIface * gee_read_only_map_gee_traversable_parent_iface = NULL;
static GeeIterableIface * gee_read_only_map_gee_iterable_parent_iface = NULL;
static GeeMapIface * gee_read_only_map_gee_map_parent_iface = NULL;

 G_GNUC_INTERNAL GType gee_read_only_map_get_type (void) G_GNUC_CONST  G_GNUC_UNUSED ;
 G_GNUC_INTERNAL GeeMap* gee_read_only_map_get_read_only_view (GeeReadOnlyMap* self);
 G_GNUC_INTERNAL GeeReadOnlyMap* gee_read_only_map_new (GType k_type,
                                       GBoxedCopyFunc k_dup_func,
                                       GDestroyNotify k_destroy_func,
                                       GType v_type,
                                       GBoxedCopyFunc v_dup_func,
                                       GDestroyNotify v_destroy_func,
                                       GeeMap* map);
 G_GNUC_INTERNAL GeeReadOnlyMap* gee_read_only_map_construct (GType object_type,
                                             GType k_type,
                                             GBoxedCopyFunc k_dup_func,
                                             GDestroyNotify k_destroy_func,
                                             GType v_type,
                                             GBoxedCopyFunc v_dup_func,
                                             GDestroyNotify v_destroy_func,
                                             GeeMap* map);
static gboolean gee_read_only_map_real_has_key (GeeMap* base,
                                         gconstpointer key);
 G_GNUC_INTERNAL gboolean gee_read_only_map_contains (GeeReadOnlyMap* self,
                                     gconstpointer key);
static gboolean gee_read_only_map_real_has (GeeMap* base,
                                     gconstpointer key,
                                     gconstpointer value);
static gpointer gee_read_only_map_real_get (GeeMap* base,
                                     gconstpointer key);
static void gee_read_only_map_real_set (GeeMap* base,
                                 gconstpointer key,
                                 gconstpointer value);
static gboolean gee_read_only_map_real_unset (GeeMap* base,
                                       gconstpointer key,
                                       gpointer* value);
 G_GNUC_INTERNAL gboolean gee_read_only_map_remove (GeeReadOnlyMap* self,
                                   gconstpointer key,
                                   gpointer* value);
static void gee_read_only_map_real_clear (GeeMap* base);
static GeeMapIterator* gee_read_only_map_real_map_iterator (GeeMap* base);
 G_GNUC_INTERNAL GeeReadOnlyMapMapIterator* gee_read_only_map_map_iterator_new (GType k_type,
                                                               GBoxedCopyFunc k_dup_func,
                                                               GDestroyNotify k_destroy_func,
                                                               GType v_type,
                                                               GBoxedCopyFunc v_dup_func,
                                                               GDestroyNotify v_destroy_func,
                                                               GeeMapIterator* iterator);
 G_GNUC_INTERNAL GeeReadOnlyMapMapIterator* gee_read_only_map_map_iterator_construct (GType object_type,
                                                                     GType k_type,
                                                                     GBoxedCopyFunc k_dup_func,
                                                                     GDestroyNotify k_destroy_func,
                                                                     GType v_type,
                                                                     GBoxedCopyFunc v_dup_func,
                                                                     GDestroyNotify v_destroy_func,
                                                                     GeeMapIterator* iterator);
 G_GNUC_INTERNAL GType gee_read_only_map_map_iterator_get_type (void) G_GNUC_CONST  G_GNUC_UNUSED ;
static void gee_read_only_map_real_set_all (GeeMap* base,
                                     GeeMap* map);
static gboolean gee_read_only_map_real_unset_all (GeeMap* base,
                                           GeeMap* map);
 G_GNUC_INTERNAL gboolean gee_read_only_map_remove_all (GeeReadOnlyMap* self,
                                       GeeMap* map);
static gboolean gee_read_only_map_real_has_all (GeeMap* base,
                                         GeeMap* map);
 G_GNUC_INTERNAL gboolean gee_read_only_map_contains_all (GeeReadOnlyMap* self,
                                         GeeMap* map);
static GeeIterator* gee_read_only_map_real_iterator (GeeIterable* base);
static gboolean gee_read_only_map_real_foreach (GeeTraversable* base,
                                         GeeForallFunc f,
                                         gpointer f_target);
static GeeIterator* gee_read_only_map_real_stream (GeeTraversable* base,
                                            GType a_type,
                                            GBoxedCopyFunc a_dup_func,
                                            GDestroyNotify a_destroy_func,
                                            GeeStreamFunc f,
                                            gpointer f_target,
                                            GDestroyNotify f_target_destroy_notify);
static GeeIterator* gee_read_only_map_real_filter (GeeTraversable* base,
                                            GeePredicate f,
                                            gpointer f_target,
                                            GDestroyNotify f_target_destroy_notify);
static GeeIterator* gee_read_only_map_real_chop (GeeTraversable* base,
                                          gint offset,
                                          gint length);
 G_GNUC_INTERNAL GType gee_read_only_map_get_key_type (GeeReadOnlyMap* self);
 G_GNUC_INTERNAL GType gee_read_only_map_get_value_type (GeeReadOnlyMap* self);
static gboolean gee_read_only_map_map_iterator_real_next (GeeMapIterator* base);
static gboolean gee_read_only_map_map_iterator_real_has_next (GeeMapIterator* base);
static gpointer gee_read_only_map_map_iterator_real_get_key (GeeMapIterator* base);
static gpointer gee_read_only_map_map_iterator_real_get_value (GeeMapIterator* base);
static void gee_read_only_map_map_iterator_real_set_value (GeeMapIterator* base,
                                                    gconstpointer value);
static void gee_read_only_map_map_iterator_real_unset (GeeMapIterator* base);
static void gee_read_only_map_map_iterator_finalize (GObject * obj);
static GType gee_read_only_map_map_iterator_get_type_once (void);
static void _vala_gee_read_only_map_map_iterator_get_property (GObject * object,
                                                        guint property_id,
                                                        GValue * value,
                                                        GParamSpec * pspec);
static void _vala_gee_read_only_map_map_iterator_set_property (GObject * object,
                                                        guint property_id,
                                                        const GValue * value,
                                                        GParamSpec * pspec);
static void gee_read_only_map_finalize (GObject * obj);
static GType gee_read_only_map_get_type_once (void);
static void _vala_gee_read_only_map_get_property (GObject * object,
                                           guint property_id,
                                           GValue * value,
                                           GParamSpec * pspec);
static void _vala_gee_read_only_map_set_property (GObject * object,
                                           guint property_id,
                                           const GValue * value,
                                           GParamSpec * pspec);

static inline gpointer
gee_read_only_map_get_instance_private (GeeReadOnlyMap* self)
{
	return G_STRUCT_MEMBER_P (self, GeeReadOnlyMap_private_offset);
}

/**
 * Constructs a read-only map that mirrors the content of the specified map.
 *
 * @param map the map to decorate.
 */
static gpointer
_g_object_ref0 (gpointer self)
{
	return self ? g_object_ref (self) : NULL;
}

 G_GNUC_INTERNAL GeeReadOnlyMap*
gee_read_only_map_construct (GType object_type,
                             GType k_type,
                             GBoxedCopyFunc k_dup_func,
                             GDestroyNotify k_destroy_func,
                             GType v_type,
                             GBoxedCopyFunc v_dup_func,
                             GDestroyNotify v_destroy_func,
                             GeeMap* map)
{
	GeeReadOnlyMap * self = NULL;
	GeeMap* _tmp0_;
	g_return_val_if_fail (map != NULL, NULL);
	self = (GeeReadOnlyMap*) g_object_new (object_type, "k-type", k_type, "k-dup-func", k_dup_func, "k-destroy-func", k_destroy_func, "v-type", v_type, "v-dup-func", v_dup_func, "v-destroy-func", v_destroy_func, NULL);
	self->priv->k_type = k_type;
	self->priv->k_dup_func = k_dup_func;
	self->priv->k_destroy_func = k_destroy_func;
	self->priv->v_type = v_type;
	self->priv->v_dup_func = v_dup_func;
	self->priv->v_destroy_func = v_destroy_func;
	_tmp0_ = _g_object_ref0 (map);
	_g_object_unref0 (self->_map);
	self->_map = _tmp0_;
	return self;
}

 G_GNUC_INTERNAL GeeReadOnlyMap*
gee_read_only_map_new (GType k_type,
                       GBoxedCopyFunc k_dup_func,
                       GDestroyNotify k_destroy_func,
                       GType v_type,
                       GBoxedCopyFunc v_dup_func,
                       GDestroyNotify v_destroy_func,
                       GeeMap* map)
{
	return gee_read_only_map_construct (GEE_TYPE_READ_ONLY_MAP, k_type, k_dup_func, k_destroy_func, v_type, v_dup_func, v_destroy_func, map);
}

/**
 * {@inheritDoc}
 */
static gboolean
gee_read_only_map_real_has_key (GeeMap* base,
                                gconstpointer key)
{
	GeeReadOnlyMap * self;
	GeeMap* _tmp0_;
	gboolean result;
	self = (GeeReadOnlyMap*) base;
	_tmp0_ = self->_map;
	result = gee_map_has_key (_tmp0_, key);
	return result;
}

/**
 * {@inheritDoc}
 */
 G_GNUC_INTERNAL gboolean
gee_read_only_map_contains (GeeReadOnlyMap* self,
                            gconstpointer key)
{
	GeeMap* _tmp0_;
	gboolean result;
	g_return_val_if_fail (self != NULL, FALSE);
	_tmp0_ = self->_map;
	result = gee_map_has_key (_tmp0_, key);
	return result;
}

/**
 * {@inheritDoc}
 */
static gboolean
gee_read_only_map_real_has (GeeMap* base,
                            gconstpointer key,
                            gconstpointer value)
{
	GeeReadOnlyMap * self;
	GeeMap* _tmp0_;
	gboolean result;
	self = (GeeReadOnlyMap*) base;
	_tmp0_ = self->_map;
	result = gee_map_has (_tmp0_, key, value);
	return result;
}

/**
 * {@inheritDoc}
 */
static gpointer
gee_read_only_map_real_get (GeeMap* base,
                            gconstpointer key)
{
	GeeReadOnlyMap * self;
	GeeMap* _tmp0_;
	gpointer _tmp1_;
	gpointer result;
	self = (GeeReadOnlyMap*) base;
	_tmp0_ = self->_map;
	_tmp1_ = gee_map_get (_tmp0_, key);
	result = _tmp1_;
	return result;
}

/**
 * Unimplemented method (read only map).
 */
static void
gee_read_only_map_real_set (GeeMap* base,
                            gconstpointer key,
                            gconstpointer value)
{
	GeeReadOnlyMap * self;
	self = (GeeReadOnlyMap*) base;
	g_assert_not_reached ();
}

/**
 * Unimplemented method (read only map).
 */
static gboolean
gee_read_only_map_real_unset (GeeMap* base,
                              gconstpointer key,
                              gpointer* value)
{
	GeeReadOnlyMap * self;
	gpointer _vala_value = NULL;
	self = (GeeReadOnlyMap*) base;
	g_assert_not_reached ();
}

/**
 * Unimplemented method (read only map).
 */
 G_GNUC_INTERNAL gboolean
gee_read_only_map_remove (GeeReadOnlyMap* self,
                          gconstpointer key,
                          gpointer* value)
{
	gpointer _vala_value = NULL;
	g_return_val_if_fail (self != NULL, FALSE);
	g_assert_not_reached ();
}

/**
 * Unimplemented method (read only map).
 */
static void
gee_read_only_map_real_clear (GeeMap* base)
{
	GeeReadOnlyMap * self;
	self = (GeeReadOnlyMap*) base;
	g_assert_not_reached ();
}

/**
 * {@inheritDoc}
 */
static GeeMapIterator*
gee_read_only_map_real_map_iterator (GeeMap* base)
{
	GeeReadOnlyMap * self;
	GeeMap* _tmp0_;
	GeeMapIterator* _tmp1_;
	GeeMapIterator* _tmp2_;
	GeeReadOnlyMapMapIterator* _tmp3_;
	GeeMapIterator* _tmp4_;
	GeeMapIterator* result;
	self = (GeeReadOnlyMap*) base;
	_tmp0_ = self->_map;
	_tmp1_ = gee_map_map_iterator (_tmp0_);
	_tmp2_ = _tmp1_;
	_tmp3_ = gee_read_only_map_map_iterator_new (self->priv->k_type, (GBoxedCopyFunc) self->priv->k_dup_func, (GDestroyNotify) self->priv->k_destroy_func, self->priv->v_type, (GBoxedCopyFunc) self->priv->v_dup_func, (GDestroyNotify) self->priv->v_destroy_func, _tmp2_);
	_tmp4_ = (GeeMapIterator*) _tmp3_;
	_g_object_unref0 (_tmp2_);
	result = _tmp4_;
	return result;
}

/**
 * Unimplemented method (read only map).
 */
static void
gee_read_only_map_real_set_all (GeeMap* base,
                                GeeMap* map)
{
	GeeReadOnlyMap * self;
	self = (GeeReadOnlyMap*) base;
	g_return_if_fail (map != NULL);
	g_assert_not_reached ();
}

/**
 * Unimplemented method (read only map).
 */
static gboolean
gee_read_only_map_real_unset_all (GeeMap* base,
                                  GeeMap* map)
{
	GeeReadOnlyMap * self;
	self = (GeeReadOnlyMap*) base;
	g_return_val_if_fail (map != NULL, FALSE);
	g_assert_not_reached ();
}

/**
 * Unimplemented method (read only map).
 */
 G_GNUC_INTERNAL gboolean
gee_read_only_map_remove_all (GeeReadOnlyMap* self,
                              GeeMap* map)
{
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (map != NULL, FALSE);
	g_assert_not_reached ();
}

/**
 * {@inheritDoc}
 */
static gboolean
gee_read_only_map_real_has_all (GeeMap* base,
                                GeeMap* map)
{
	GeeReadOnlyMap * self;
	GeeMap* _tmp0_;
	gboolean result;
	self = (GeeReadOnlyMap*) base;
	g_return_val_if_fail (map != NULL, FALSE);
	_tmp0_ = self->_map;
	result = gee_map_has_all (_tmp0_, map);
	return result;
}

/**
 * {@inheritDoc}
 */
 G_GNUC_INTERNAL gboolean
gee_read_only_map_contains_all (GeeReadOnlyMap* self,
                                GeeMap* map)
{
	GeeMap* _tmp0_;
	gboolean result;
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (map != NULL, FALSE);
	_tmp0_ = self->_map;
	result = gee_map_has_all (_tmp0_, map);
	return result;
}

/**
 * {@inheritDoc}
 */
static GeeIterator*
gee_read_only_map_real_iterator (GeeIterable* base)
{
	GeeReadOnlyMap * self;
	GeeSet* _tmp0_;
	GeeSet* _tmp1_;
	GeeSet* _tmp2_;
	GeeIterator* _tmp3_;
	GeeIterator* _tmp4_;
	GeeIterator* result;
	self = (GeeReadOnlyMap*) base;
	_tmp0_ = gee_map_get_entries ((GeeMap*) self);
	_tmp1_ = _tmp0_;
	_tmp2_ = _tmp1_;
	_tmp3_ = gee_iterable_iterator ((GeeIterable*) _tmp2_);
	_tmp4_ = _tmp3_;
	_g_object_unref0 (_tmp2_);
	result = _tmp4_;
	return result;
}

/**
 * {@inheritDoc}
 */
static gboolean
gee_read_only_map_real_foreach (GeeTraversable* base,
                                GeeForallFunc f,
                                gpointer f_target)
{
	GeeReadOnlyMap * self;
	GeeMap* _tmp0_;
	gboolean result;
	self = (GeeReadOnlyMap*) base;
	_tmp0_ = self->_map;
	result = gee_traversable_foreach ((GeeTraversable*) _tmp0_, f, f_target);
	return result;
}

/**
 * {@inheritDoc}
 */
static GeeIterator*
gee_read_only_map_real_stream (GeeTraversable* base,
                               GType a_type,
                               GBoxedCopyFunc a_dup_func,
                               GDestroyNotify a_destroy_func,
                               GeeStreamFunc f,
                               gpointer f_target,
                               GDestroyNotify f_target_destroy_notify)
{
	GeeReadOnlyMap * self;
	GeeMap* _tmp0_;
	GeeStreamFunc _tmp1_;
	gpointer _tmp1__target;
	GDestroyNotify _tmp1__target_destroy_notify;
	GeeIterator* _tmp2_;
	GeeIterator* result;
	self = (GeeReadOnlyMap*) base;
	_tmp0_ = self->_map;
	_tmp1_ = f;
	_tmp1__target = f_target;
	_tmp1__target_destroy_notify = f_target_destroy_notify;
	f = NULL;
	f_target = NULL;
	f_target_destroy_notify = NULL;
	_tmp2_ = gee_traversable_stream ((GeeTraversable*) _tmp0_, a_type, (GBoxedCopyFunc) a_dup_func, (GDestroyNotify) a_destroy_func, _tmp1_, _tmp1__target, _tmp1__target_destroy_notify);
	result = _tmp2_;
	(f_target_destroy_notify == NULL) ? NULL : (f_target_destroy_notify (f_target), NULL);
	f = NULL;
	f_target = NULL;
	f_target_destroy_notify = NULL;
	return result;
}

static GeeIterator*
gee_read_only_map_real_filter (GeeTraversable* base,
                               GeePredicate f,
                               gpointer f_target,
                               GDestroyNotify f_target_destroy_notify)
{
	GeeReadOnlyMap * self;
	GeeMap* _tmp0_;
	GeePredicate _tmp1_;
	gpointer _tmp1__target;
	GDestroyNotify _tmp1__target_destroy_notify;
	GeeIterator* _tmp2_;
	GeeIterator* result;
	self = (GeeReadOnlyMap*) base;
	_tmp0_ = self->_map;
	_tmp1_ = f;
	_tmp1__target = f_target;
	_tmp1__target_destroy_notify = f_target_destroy_notify;
	f = NULL;
	f_target = NULL;
	f_target_destroy_notify = NULL;
	_tmp2_ = gee_traversable_filter ((GeeTraversable*) _tmp0_, _tmp1_, _tmp1__target, _tmp1__target_destroy_notify);
	result = _tmp2_;
	(f_target_destroy_notify == NULL) ? NULL : (f_target_destroy_notify (f_target), NULL);
	f = NULL;
	f_target = NULL;
	f_target_destroy_notify = NULL;
	return result;
}

static GeeIterator*
gee_read_only_map_real_chop (GeeTraversable* base,
                             gint offset,
                             gint length)
{
	GeeReadOnlyMap * self;
	GeeMap* _tmp0_;
	GeeIterator* _tmp1_;
	GeeIterator* result;
	self = (GeeReadOnlyMap*) base;
	_tmp0_ = self->_map;
	_tmp1_ = gee_traversable_chop ((GeeTraversable*) _tmp0_, offset, length);
	result = _tmp1_;
	return result;
}

static gint
gee_read_only_map_real_get_size (GeeMap* base)
{
	gint result;
	GeeReadOnlyMap* self;
	GeeMap* _tmp0_;
	gint _tmp1_;
	gint _tmp2_;
	self = (GeeReadOnlyMap*) base;
	_tmp0_ = self->_map;
	_tmp1_ = gee_map_get_size (_tmp0_);
	_tmp2_ = _tmp1_;
	result = _tmp2_;
	return result;
}

static gboolean
gee_read_only_map_real_get_is_empty (GeeMap* base)
{
	gboolean result;
	GeeReadOnlyMap* self;
	GeeMap* _tmp0_;
	gboolean _tmp1_;
	gboolean _tmp2_;
	self = (GeeReadOnlyMap*) base;
	_tmp0_ = self->_map;
	_tmp1_ = gee_map_get_is_empty (_tmp0_);
	_tmp2_ = _tmp1_;
	result = _tmp2_;
	return result;
}

static gboolean
gee_read_only_map_real_get_read_only (GeeMap* base)
{
	gboolean result;
	GeeReadOnlyMap* self;
	self = (GeeReadOnlyMap*) base;
	result = TRUE;
	return result;
}

static GeeSet*
gee_read_only_map_real_get_keys (GeeMap* base)
{
	GeeSet* result;
	GeeReadOnlyMap* self;
	GeeMap* _tmp0_;
	GeeSet* _tmp1_;
	GeeSet* _tmp2_;
	self = (GeeReadOnlyMap*) base;
	_tmp0_ = self->_map;
	_tmp1_ = gee_map_get_keys (_tmp0_);
	_tmp2_ = _tmp1_;
	result = _tmp2_;
	return result;
}

static GeeCollection*
gee_read_only_map_real_get_values (GeeMap* base)
{
	GeeCollection* result;
	GeeReadOnlyMap* self;
	GeeMap* _tmp0_;
	GeeCollection* _tmp1_;
	GeeCollection* _tmp2_;
	self = (GeeReadOnlyMap*) base;
	_tmp0_ = self->_map;
	_tmp1_ = gee_map_get_values (_tmp0_);
	_tmp2_ = _tmp1_;
	result = _tmp2_;
	return result;
}

static GeeSet*
gee_read_only_map_real_get_entries (GeeMap* base)
{
	GeeSet* result;
	GeeReadOnlyMap* self;
	GeeMap* _tmp0_;
	GeeSet* _tmp1_;
	GeeSet* _tmp2_;
	self = (GeeReadOnlyMap*) base;
	_tmp0_ = self->_map;
	_tmp1_ = gee_map_get_entries (_tmp0_);
	_tmp2_ = _tmp1_;
	result = _tmp2_;
	return result;
}

 G_GNUC_INTERNAL GeeMap*
gee_read_only_map_get_read_only_view (GeeReadOnlyMap* self)
{
	GeeReadOnlyMapClass* _klass_;
	g_return_val_if_fail (self != NULL, NULL);
	_klass_ = GEE_READ_ONLY_MAP_GET_CLASS (self);
	if (_klass_->get_read_only_view) {
		return _klass_->get_read_only_view (self);
	}
	return NULL;
}

static GeeMap*
gee_read_only_map_real_get_read_only_view (GeeReadOnlyMap* base)
{
	GeeMap* result;
	GeeReadOnlyMap* self;
	GeeMap* _tmp0_;
	self = base;
	_tmp0_ = _g_object_ref0 ((GeeMap*) self);
	result = _tmp0_;
	return result;
}

 G_GNUC_INTERNAL GType
gee_read_only_map_get_key_type (GeeReadOnlyMap* self)
{
	GType result;
	g_return_val_if_fail (self != NULL, 0UL);
	result = self->priv->k_type;
	return result;
}

 G_GNUC_INTERNAL GType
gee_read_only_map_get_value_type (GeeReadOnlyMap* self)
{
	GType result;
	g_return_val_if_fail (self != NULL, 0UL);
	result = self->priv->v_type;
	return result;
}

static GType
gee_read_only_map_real_get_element_type (GeeTraversable* base)
{
	GType result;
	GeeReadOnlyMap* self;
	self = (GeeReadOnlyMap*) base;
	result = GEE_MAP_TYPE_ENTRY;
	return result;
}

static inline gpointer
gee_read_only_map_map_iterator_get_instance_private (GeeReadOnlyMapMapIterator* self)
{
	return G_STRUCT_MEMBER_P (self, GeeReadOnlyMapMapIterator_private_offset);
}

 G_GNUC_INTERNAL GeeReadOnlyMapMapIterator*
gee_read_only_map_map_iterator_construct (GType object_type,
                                          GType k_type,
                                          GBoxedCopyFunc k_dup_func,
                                          GDestroyNotify k_destroy_func,
                                          GType v_type,
                                          GBoxedCopyFunc v_dup_func,
                                          GDestroyNotify v_destroy_func,
                                          GeeMapIterator* iterator)
{
	GeeReadOnlyMapMapIterator * self = NULL;
	GeeMapIterator* _tmp0_;
	g_return_val_if_fail (iterator != NULL, NULL);
	self = (GeeReadOnlyMapMapIterator*) g_object_new (object_type, "k-type", k_type, "k-dup-func", k_dup_func, "k-destroy-func", k_destroy_func, "v-type", v_type, "v-dup-func", v_dup_func, "v-destroy-func", v_destroy_func, NULL);
	self->priv->k_type = k_type;
	self->priv->k_dup_func = k_dup_func;
	self->priv->k_destroy_func = k_destroy_func;
	self->priv->v_type = v_type;
	self->priv->v_dup_func = v_dup_func;
	self->priv->v_destroy_func = v_destroy_func;
	_tmp0_ = _g_object_ref0 (iterator);
	_g_object_unref0 (self->_iter);
	self->_iter = _tmp0_;
	return self;
}

 G_GNUC_INTERNAL GeeReadOnlyMapMapIterator*
gee_read_only_map_map_iterator_new (GType k_type,
                                    GBoxedCopyFunc k_dup_func,
                                    GDestroyNotify k_destroy_func,
                                    GType v_type,
                                    GBoxedCopyFunc v_dup_func,
                                    GDestroyNotify v_destroy_func,
                                    GeeMapIterator* iterator)
{
	return gee_read_only_map_map_iterator_construct (GEE_READ_ONLY_MAP_TYPE_MAP_ITERATOR, k_type, k_dup_func, k_destroy_func, v_type, v_dup_func, v_destroy_func, iterator);
}

static gboolean
gee_read_only_map_map_iterator_real_next (GeeMapIterator* base)
{
	GeeReadOnlyMapMapIterator * self;
	GeeMapIterator* _tmp0_;
	gboolean result;
	self = (GeeReadOnlyMapMapIterator*) base;
	_tmp0_ = self->_iter;
	result = gee_map_iterator_next (_tmp0_);
	return result;
}

static gboolean
gee_read_only_map_map_iterator_real_has_next (GeeMapIterator* base)
{
	GeeReadOnlyMapMapIterator * self;
	GeeMapIterator* _tmp0_;
	gboolean result;
	self = (GeeReadOnlyMapMapIterator*) base;
	_tmp0_ = self->_iter;
	result = gee_map_iterator_has_next (_tmp0_);
	return result;
}

static gpointer
gee_read_only_map_map_iterator_real_get_key (GeeMapIterator* base)
{
	GeeReadOnlyMapMapIterator * self;
	GeeMapIterator* _tmp0_;
	gpointer _tmp1_;
	gpointer result;
	self = (GeeReadOnlyMapMapIterator*) base;
	_tmp0_ = self->_iter;
	_tmp1_ = gee_map_iterator_get_key (_tmp0_);
	result = _tmp1_;
	return result;
}

static gpointer
gee_read_only_map_map_iterator_real_get_value (GeeMapIterator* base)
{
	GeeReadOnlyMapMapIterator * self;
	GeeMapIterator* _tmp0_;
	gpointer _tmp1_;
	gpointer result;
	self = (GeeReadOnlyMapMapIterator*) base;
	_tmp0_ = self->_iter;
	_tmp1_ = gee_map_iterator_get_value (_tmp0_);
	result = _tmp1_;
	return result;
}

static void
gee_read_only_map_map_iterator_real_set_value (GeeMapIterator* base,
                                               gconstpointer value)
{
	GeeReadOnlyMapMapIterator * self;
	self = (GeeReadOnlyMapMapIterator*) base;
	g_assert_not_reached ();
}

static void
gee_read_only_map_map_iterator_real_unset (GeeMapIterator* base)
{
	GeeReadOnlyMapMapIterator * self;
	self = (GeeReadOnlyMapMapIterator*) base;
	g_assert_not_reached ();
}

static gboolean
gee_read_only_map_map_iterator_real_get_read_only (GeeMapIterator* base)
{
	gboolean result;
	GeeReadOnlyMapMapIterator* self;
	self = (GeeReadOnlyMapMapIterator*) base;
	result = TRUE;
	return result;
}

static gboolean
gee_read_only_map_map_iterator_real_get_mutable (GeeMapIterator* base)
{
	gboolean result;
	GeeReadOnlyMapMapIterator* self;
	self = (GeeReadOnlyMapMapIterator*) base;
	result = FALSE;
	return result;
}

static gboolean
gee_read_only_map_map_iterator_real_get_valid (GeeMapIterator* base)
{
	gboolean result;
	GeeReadOnlyMapMapIterator* self;
	GeeMapIterator* _tmp0_;
	gboolean _tmp1_;
	gboolean _tmp2_;
	self = (GeeReadOnlyMapMapIterator*) base;
	_tmp0_ = self->_iter;
	_tmp1_ = gee_map_iterator_get_valid (_tmp0_);
	_tmp2_ = _tmp1_;
	result = _tmp2_;
	return result;
}

static void
gee_read_only_map_map_iterator_class_init (GeeReadOnlyMapMapIteratorClass * klass,
                                           gpointer klass_data)
{
	gee_read_only_map_map_iterator_parent_class = g_type_class_peek_parent (klass);
	g_type_class_adjust_private_offset (klass, &GeeReadOnlyMapMapIterator_private_offset);
	G_OBJECT_CLASS (klass)->get_property = _vala_gee_read_only_map_map_iterator_get_property;
	G_OBJECT_CLASS (klass)->set_property = _vala_gee_read_only_map_map_iterator_set_property;
	G_OBJECT_CLASS (klass)->finalize = gee_read_only_map_map_iterator_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_READ_ONLY_MAP_MAP_ITERATOR_K_TYPE, g_param_spec_gtype ("k-type", "type", "type", G_TYPE_NONE, G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_READ_ONLY_MAP_MAP_ITERATOR_K_DUP_FUNC, g_param_spec_pointer ("k-dup-func", "dup func", "dup func", G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_READ_ONLY_MAP_MAP_ITERATOR_K_DESTROY_FUNC, g_param_spec_pointer ("k-destroy-func", "destroy func", "destroy func", G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_READ_ONLY_MAP_MAP_ITERATOR_V_TYPE, g_param_spec_gtype ("v-type", "type", "type", G_TYPE_NONE, G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_READ_ONLY_MAP_MAP_ITERATOR_V_DUP_FUNC, g_param_spec_pointer ("v-dup-func", "dup func", "dup func", G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_READ_ONLY_MAP_MAP_ITERATOR_V_DESTROY_FUNC, g_param_spec_pointer ("v-destroy-func", "destroy func", "destroy func", G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_READ_ONLY_MAP_MAP_ITERATOR_READ_ONLY_PROPERTY, gee_read_only_map_map_iterator_properties[GEE_READ_ONLY_MAP_MAP_ITERATOR_READ_ONLY_PROPERTY] = g_param_spec_boolean ("read-only", "read-only", "read-only", FALSE, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_READ_ONLY_MAP_MAP_ITERATOR_MUTABLE_PROPERTY, gee_read_only_map_map_iterator_properties[GEE_READ_ONLY_MAP_MAP_ITERATOR_MUTABLE_PROPERTY] = g_param_spec_boolean ("mutable", "mutable", "mutable", FALSE, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_READ_ONLY_MAP_MAP_ITERATOR_VALID_PROPERTY, gee_read_only_map_map_iterator_properties[GEE_READ_ONLY_MAP_MAP_ITERATOR_VALID_PROPERTY] = g_param_spec_boolean ("valid", "valid", "valid", FALSE, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE));
}

static GType
gee_read_only_map_map_iterator_gee_map_iterator_get_k_type (GeeReadOnlyMapMapIterator* self)
{
	return self->priv->k_type;
}

static GBoxedCopyFunc
gee_read_only_map_map_iterator_gee_map_iterator_get_k_dup_func (GeeReadOnlyMapMapIterator* self)
{
	return self->priv->k_dup_func;
}

static GDestroyNotify
gee_read_only_map_map_iterator_gee_map_iterator_get_k_destroy_func (GeeReadOnlyMapMapIterator* self)
{
	return self->priv->k_destroy_func;
}

static GType
gee_read_only_map_map_iterator_gee_map_iterator_get_v_type (GeeReadOnlyMapMapIterator* self)
{
	return self->priv->v_type;
}

static GBoxedCopyFunc
gee_read_only_map_map_iterator_gee_map_iterator_get_v_dup_func (GeeReadOnlyMapMapIterator* self)
{
	return self->priv->v_dup_func;
}

static GDestroyNotify
gee_read_only_map_map_iterator_gee_map_iterator_get_v_destroy_func (GeeReadOnlyMapMapIterator* self)
{
	return self->priv->v_destroy_func;
}

static void
gee_read_only_map_map_iterator_gee_map_iterator_interface_init (GeeMapIteratorIface * iface,
                                                                gpointer iface_data)
{
	gee_read_only_map_map_iterator_gee_map_iterator_parent_iface = g_type_interface_peek_parent (iface);
	iface->next = (gboolean (*) (GeeMapIterator*)) gee_read_only_map_map_iterator_real_next;
	iface->has_next = (gboolean (*) (GeeMapIterator*)) gee_read_only_map_map_iterator_real_has_next;
	iface->get_key = (gpointer (*) (GeeMapIterator*)) gee_read_only_map_map_iterator_real_get_key;
	iface->get_value = (gpointer (*) (GeeMapIterator*)) gee_read_only_map_map_iterator_real_get_value;
	iface->set_value = (void (*) (GeeMapIterator*, gconstpointer)) gee_read_only_map_map_iterator_real_set_value;
	iface->unset = (void (*) (GeeMapIterator*)) gee_read_only_map_map_iterator_real_unset;
	iface->get_k_type = (GType (*) (GeeMapIterator *)) gee_read_only_map_map_iterator_gee_map_iterator_get_k_type;
	iface->get_k_dup_func = (GBoxedCopyFunc (*) (GeeMapIterator *)) gee_read_only_map_map_iterator_gee_map_iterator_get_k_dup_func;
	iface->get_k_destroy_func = (GDestroyNotify (*) (GeeMapIterator *)) gee_read_only_map_map_iterator_gee_map_iterator_get_k_destroy_func;
	iface->get_v_type = (GType (*) (GeeMapIterator *)) gee_read_only_map_map_iterator_gee_map_iterator_get_v_type;
	iface->get_v_dup_func = (GBoxedCopyFunc (*) (GeeMapIterator *)) gee_read_only_map_map_iterator_gee_map_iterator_get_v_dup_func;
	iface->get_v_destroy_func = (GDestroyNotify (*) (GeeMapIterator *)) gee_read_only_map_map_iterator_gee_map_iterator_get_v_destroy_func;
	iface->get_read_only = gee_read_only_map_map_iterator_real_get_read_only;
	iface->get_mutable = gee_read_only_map_map_iterator_real_get_mutable;
	iface->get_valid = gee_read_only_map_map_iterator_real_get_valid;
}

static void
gee_read_only_map_map_iterator_instance_init (GeeReadOnlyMapMapIterator * self,
                                              gpointer klass)
{
	self->priv = gee_read_only_map_map_iterator_get_instance_private (self);
}

static void
gee_read_only_map_map_iterator_finalize (GObject * obj)
{
	GeeReadOnlyMapMapIterator * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, GEE_READ_ONLY_MAP_TYPE_MAP_ITERATOR, GeeReadOnlyMapMapIterator);
	_g_object_unref0 (self->_iter);
	G_OBJECT_CLASS (gee_read_only_map_map_iterator_parent_class)->finalize (obj);
}

static GType
gee_read_only_map_map_iterator_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (GeeReadOnlyMapMapIteratorClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) gee_read_only_map_map_iterator_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GeeReadOnlyMapMapIterator), 0, (GInstanceInitFunc) gee_read_only_map_map_iterator_instance_init, NULL };
	static const GInterfaceInfo gee_map_iterator_info = { (GInterfaceInitFunc) gee_read_only_map_map_iterator_gee_map_iterator_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
	GType gee_read_only_map_map_iterator_type_id;
	gee_read_only_map_map_iterator_type_id = g_type_register_static (G_TYPE_OBJECT, "GeeReadOnlyMapMapIterator", &g_define_type_info, 0);
	g_type_add_interface_static (gee_read_only_map_map_iterator_type_id, GEE_TYPE_MAP_ITERATOR, &gee_map_iterator_info);
	GeeReadOnlyMapMapIterator_private_offset = g_type_add_instance_private (gee_read_only_map_map_iterator_type_id, sizeof (GeeReadOnlyMapMapIteratorPrivate));
	return gee_read_only_map_map_iterator_type_id;
}

 G_GNUC_INTERNAL GType
gee_read_only_map_map_iterator_get_type (void)
{
	static volatile gsize gee_read_only_map_map_iterator_type_id__once = 0;
	if (g_once_init_enter (&gee_read_only_map_map_iterator_type_id__once)) {
		GType gee_read_only_map_map_iterator_type_id;
		gee_read_only_map_map_iterator_type_id = gee_read_only_map_map_iterator_get_type_once ();
		g_once_init_leave (&gee_read_only_map_map_iterator_type_id__once, gee_read_only_map_map_iterator_type_id);
	}
	return gee_read_only_map_map_iterator_type_id__once;
}

static void
_vala_gee_read_only_map_map_iterator_get_property (GObject * object,
                                                   guint property_id,
                                                   GValue * value,
                                                   GParamSpec * pspec)
{
	GeeReadOnlyMapMapIterator * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, GEE_READ_ONLY_MAP_TYPE_MAP_ITERATOR, GeeReadOnlyMapMapIterator);
	switch (property_id) {
		case GEE_READ_ONLY_MAP_MAP_ITERATOR_READ_ONLY_PROPERTY:
		g_value_set_boolean (value, gee_map_iterator_get_read_only ((GeeMapIterator*) self));
		break;
		case GEE_READ_ONLY_MAP_MAP_ITERATOR_MUTABLE_PROPERTY:
		g_value_set_boolean (value, gee_map_iterator_get_mutable ((GeeMapIterator*) self));
		break;
		case GEE_READ_ONLY_MAP_MAP_ITERATOR_VALID_PROPERTY:
		g_value_set_boolean (value, gee_map_iterator_get_valid ((GeeMapIterator*) self));
		break;
		case GEE_READ_ONLY_MAP_MAP_ITERATOR_K_TYPE:
		g_value_set_gtype (value, self->priv->k_type);
		break;
		case GEE_READ_ONLY_MAP_MAP_ITERATOR_K_DUP_FUNC:
		g_value_set_pointer (value, self->priv->k_dup_func);
		break;
		case GEE_READ_ONLY_MAP_MAP_ITERATOR_K_DESTROY_FUNC:
		g_value_set_pointer (value, self->priv->k_destroy_func);
		break;
		case GEE_READ_ONLY_MAP_MAP_ITERATOR_V_TYPE:
		g_value_set_gtype (value, self->priv->v_type);
		break;
		case GEE_READ_ONLY_MAP_MAP_ITERATOR_V_DUP_FUNC:
		g_value_set_pointer (value, self->priv->v_dup_func);
		break;
		case GEE_READ_ONLY_MAP_MAP_ITERATOR_V_DESTROY_FUNC:
		g_value_set_pointer (value, self->priv->v_destroy_func);
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

static void
_vala_gee_read_only_map_map_iterator_set_property (GObject * object,
                                                   guint property_id,
                                                   const GValue * value,
                                                   GParamSpec * pspec)
{
	GeeReadOnlyMapMapIterator * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, GEE_READ_ONLY_MAP_TYPE_MAP_ITERATOR, GeeReadOnlyMapMapIterator);
	switch (property_id) {
		case GEE_READ_ONLY_MAP_MAP_ITERATOR_K_TYPE:
		self->priv->k_type = g_value_get_gtype (value);
		break;
		case GEE_READ_ONLY_MAP_MAP_ITERATOR_K_DUP_FUNC:
		self->priv->k_dup_func = g_value_get_pointer (value);
		break;
		case GEE_READ_ONLY_MAP_MAP_ITERATOR_K_DESTROY_FUNC:
		self->priv->k_destroy_func = g_value_get_pointer (value);
		break;
		case GEE_READ_ONLY_MAP_MAP_ITERATOR_V_TYPE:
		self->priv->v_type = g_value_get_gtype (value);
		break;
		case GEE_READ_ONLY_MAP_MAP_ITERATOR_V_DUP_FUNC:
		self->priv->v_dup_func = g_value_get_pointer (value);
		break;
		case GEE_READ_ONLY_MAP_MAP_ITERATOR_V_DESTROY_FUNC:
		self->priv->v_destroy_func = g_value_get_pointer (value);
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

static void
gee_read_only_map_class_init (GeeReadOnlyMapClass * klass,
                              gpointer klass_data)
{
	gee_read_only_map_parent_class = g_type_class_peek_parent (klass);
	g_type_class_adjust_private_offset (klass, &GeeReadOnlyMap_private_offset);
	GEE_READ_ONLY_MAP_CLASS (klass)->get_read_only_view = gee_read_only_map_real_get_read_only_view;
	G_OBJECT_CLASS (klass)->get_property = _vala_gee_read_only_map_get_property;
	G_OBJECT_CLASS (klass)->set_property = _vala_gee_read_only_map_set_property;
	G_OBJECT_CLASS (klass)->finalize = gee_read_only_map_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_READ_ONLY_MAP_K_TYPE, g_param_spec_gtype ("k-type", "type", "type", G_TYPE_NONE, G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_READ_ONLY_MAP_K_DUP_FUNC, g_param_spec_pointer ("k-dup-func", "dup func", "dup func", G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_READ_ONLY_MAP_K_DESTROY_FUNC, g_param_spec_pointer ("k-destroy-func", "destroy func", "destroy func", G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_READ_ONLY_MAP_V_TYPE, g_param_spec_gtype ("v-type", "type", "type", G_TYPE_NONE, G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_READ_ONLY_MAP_V_DUP_FUNC, g_param_spec_pointer ("v-dup-func", "dup func", "dup func", G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_READ_ONLY_MAP_V_DESTROY_FUNC, g_param_spec_pointer ("v-destroy-func", "destroy func", "destroy func", G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	/**
	 * {@inheritDoc}
	 */
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_READ_ONLY_MAP_SIZE_PROPERTY, gee_read_only_map_properties[GEE_READ_ONLY_MAP_SIZE_PROPERTY] = g_param_spec_int ("size", "size", "size", G_MININT, G_MAXINT, 0, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE));
	/**
	 * {@inheritDoc}
	 */
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_READ_ONLY_MAP_READ_ONLY_PROPERTY, gee_read_only_map_properties[GEE_READ_ONLY_MAP_READ_ONLY_PROPERTY] = g_param_spec_boolean ("read-only", "read-only", "read-only", FALSE, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE));
	/**
	 * {@inheritDoc}
	 */
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_READ_ONLY_MAP_KEYS_PROPERTY, gee_read_only_map_properties[GEE_READ_ONLY_MAP_KEYS_PROPERTY] = g_param_spec_object ("keys", "keys", "keys", GEE_TYPE_SET, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE));
	/**
	 * {@inheritDoc}
	 */
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_READ_ONLY_MAP_VALUES_PROPERTY, gee_read_only_map_properties[GEE_READ_ONLY_MAP_VALUES_PROPERTY] = g_param_spec_object ("values", "values", "values", GEE_TYPE_COLLECTION, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE));
	/**
	 * {@inheritDoc}
	 */
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_READ_ONLY_MAP_ENTRIES_PROPERTY, gee_read_only_map_properties[GEE_READ_ONLY_MAP_ENTRIES_PROPERTY] = g_param_spec_object ("entries", "entries", "entries", GEE_TYPE_SET, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_READ_ONLY_MAP_READ_ONLY_VIEW_PROPERTY, gee_read_only_map_properties[GEE_READ_ONLY_MAP_READ_ONLY_VIEW_PROPERTY] = g_param_spec_object ("read-only-view", "read-only-view", "read-only-view", GEE_TYPE_MAP, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE));
	/**
	 * {@inheritDoc}
	 */
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_READ_ONLY_MAP_KEY_TYPE_PROPERTY, gee_read_only_map_properties[GEE_READ_ONLY_MAP_KEY_TYPE_PROPERTY] = g_param_spec_gtype ("key-type", "key-type", "key-type", G_TYPE_NONE, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE));
	/**
	 * {@inheritDoc}
	 */
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_READ_ONLY_MAP_VALUE_TYPE_PROPERTY, gee_read_only_map_properties[GEE_READ_ONLY_MAP_VALUE_TYPE_PROPERTY] = g_param_spec_gtype ("value-type", "value-type", "value-type", G_TYPE_NONE, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE));
}

static GType
gee_read_only_map_gee_traversable_get_g_type (GeeReadOnlyMap* self)
{
	return GEE_MAP_TYPE_ENTRY;
}

static GBoxedCopyFunc
gee_read_only_map_gee_traversable_get_g_dup_func (GeeReadOnlyMap* self)
{
	return g_object_ref;
}

static GDestroyNotify
gee_read_only_map_gee_traversable_get_g_destroy_func (GeeReadOnlyMap* self)
{
	return g_object_unref;
}

static void
gee_read_only_map_gee_traversable_interface_init (GeeTraversableIface * iface,
                                                  gpointer iface_data)
{
	gee_read_only_map_gee_traversable_parent_iface = g_type_interface_peek_parent (iface);
	iface->foreach = (gboolean (*) (GeeTraversable*, GeeForallFunc, gpointer)) gee_read_only_map_real_foreach;
	iface->stream = (GeeIterator* (*) (GeeTraversable*, GType, GBoxedCopyFunc, GDestroyNotify, GeeStreamFunc, gpointer, GDestroyNotify)) gee_read_only_map_real_stream;
	iface->filter = (GeeIterator* (*) (GeeTraversable*, GeePredicate, gpointer, GDestroyNotify)) gee_read_only_map_real_filter;
	iface->chop = (GeeIterator* (*) (GeeTraversable*, gint, gint)) gee_read_only_map_real_chop;
	iface->get_g_type = (GType (*) (GeeTraversable *)) gee_read_only_map_gee_traversable_get_g_type;
	iface->get_g_dup_func = (GBoxedCopyFunc (*) (GeeTraversable *)) gee_read_only_map_gee_traversable_get_g_dup_func;
	iface->get_g_destroy_func = (GDestroyNotify (*) (GeeTraversable *)) gee_read_only_map_gee_traversable_get_g_destroy_func;
	iface->get_element_type = gee_read_only_map_real_get_element_type;
}

static GType
gee_read_only_map_gee_iterable_get_g_type (GeeReadOnlyMap* self)
{
	return GEE_MAP_TYPE_ENTRY;
}

static GBoxedCopyFunc
gee_read_only_map_gee_iterable_get_g_dup_func (GeeReadOnlyMap* self)
{
	return g_object_ref;
}

static GDestroyNotify
gee_read_only_map_gee_iterable_get_g_destroy_func (GeeReadOnlyMap* self)
{
	return g_object_unref;
}

static void
gee_read_only_map_gee_iterable_interface_init (GeeIterableIface * iface,
                                               gpointer iface_data)
{
	gee_read_only_map_gee_iterable_parent_iface = g_type_interface_peek_parent (iface);
	iface->iterator = (GeeIterator* (*) (GeeIterable*)) gee_read_only_map_real_iterator;
	iface->get_g_type = (GType (*) (GeeIterable *)) gee_read_only_map_gee_iterable_get_g_type;
	iface->get_g_dup_func = (GBoxedCopyFunc (*) (GeeIterable *)) gee_read_only_map_gee_iterable_get_g_dup_func;
	iface->get_g_destroy_func = (GDestroyNotify (*) (GeeIterable *)) gee_read_only_map_gee_iterable_get_g_destroy_func;
}

static GType
gee_read_only_map_gee_map_get_k_type (GeeReadOnlyMap* self)
{
	return self->priv->k_type;
}

static GBoxedCopyFunc
gee_read_only_map_gee_map_get_k_dup_func (GeeReadOnlyMap* self)
{
	return self->priv->k_dup_func;
}

static GDestroyNotify
gee_read_only_map_gee_map_get_k_destroy_func (GeeReadOnlyMap* self)
{
	return self->priv->k_destroy_func;
}

static GType
gee_read_only_map_gee_map_get_v_type (GeeReadOnlyMap* self)
{
	return self->priv->v_type;
}

static GBoxedCopyFunc
gee_read_only_map_gee_map_get_v_dup_func (GeeReadOnlyMap* self)
{
	return self->priv->v_dup_func;
}

static GDestroyNotify
gee_read_only_map_gee_map_get_v_destroy_func (GeeReadOnlyMap* self)
{
	return self->priv->v_destroy_func;
}

static void
gee_read_only_map_gee_map_interface_init (GeeMapIface * iface,
                                          gpointer iface_data)
{
	gee_read_only_map_gee_map_parent_iface = g_type_interface_peek_parent (iface);
	iface->has_key = (gboolean (*) (GeeMap*, gconstpointer)) gee_read_only_map_real_has_key;
	iface->has = (gboolean (*) (GeeMap*, gconstpointer, gconstpointer)) gee_read_only_map_real_has;
	iface->get = (gpointer (*) (GeeMap*, gconstpointer)) gee_read_only_map_real_get;
	iface->set = (void (*) (GeeMap*, gconstpointer, gconstpointer)) gee_read_only_map_real_set;
	iface->unset = (gboolean (*) (GeeMap*, gconstpointer, gpointer*)) gee_read_only_map_real_unset;
	iface->clear = (void (*) (GeeMap*)) gee_read_only_map_real_clear;
	iface->map_iterator = (GeeMapIterator* (*) (GeeMap*)) gee_read_only_map_real_map_iterator;
	iface->set_all = (void (*) (GeeMap*, GeeMap*)) gee_read_only_map_real_set_all;
	iface->unset_all = (gboolean (*) (GeeMap*, GeeMap*)) gee_read_only_map_real_unset_all;
	iface->has_all = (gboolean (*) (GeeMap*, GeeMap*)) gee_read_only_map_real_has_all;
	iface->get_k_type = (GType (*) (GeeMap *)) gee_read_only_map_gee_map_get_k_type;
	iface->get_k_dup_func = (GBoxedCopyFunc (*) (GeeMap *)) gee_read_only_map_gee_map_get_k_dup_func;
	iface->get_k_destroy_func = (GDestroyNotify (*) (GeeMap *)) gee_read_only_map_gee_map_get_k_destroy_func;
	iface->get_v_type = (GType (*) (GeeMap *)) gee_read_only_map_gee_map_get_v_type;
	iface->get_v_dup_func = (GBoxedCopyFunc (*) (GeeMap *)) gee_read_only_map_gee_map_get_v_dup_func;
	iface->get_v_destroy_func = (GDestroyNotify (*) (GeeMap *)) gee_read_only_map_gee_map_get_v_destroy_func;
	iface->get_size = gee_read_only_map_real_get_size;
	iface->get_is_empty = gee_read_only_map_real_get_is_empty;
	iface->get_read_only = gee_read_only_map_real_get_read_only;
	iface->get_keys = gee_read_only_map_real_get_keys;
	iface->get_values = gee_read_only_map_real_get_values;
	iface->get_entries = gee_read_only_map_real_get_entries;
	iface->get_read_only_view = (GeeMap* (*) (GeeMap *)) gee_read_only_map_get_read_only_view;
}

static void
gee_read_only_map_instance_init (GeeReadOnlyMap * self,
                                 gpointer klass)
{
	self->priv = gee_read_only_map_get_instance_private (self);
}

static void
gee_read_only_map_finalize (GObject * obj)
{
	GeeReadOnlyMap * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, GEE_TYPE_READ_ONLY_MAP, GeeReadOnlyMap);
	_g_object_unref0 (self->_map);
	G_OBJECT_CLASS (gee_read_only_map_parent_class)->finalize (obj);
}

/**
 * Read-only view for {@link Map} collections.
 *
 * This class decorates any class which implements the {@link Map} interface
 * by making it read only. Any method which normally modify data will throw an
 * error.
 *
 * @see Map
 */
static GType
gee_read_only_map_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (GeeReadOnlyMapClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) gee_read_only_map_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GeeReadOnlyMap), 0, (GInstanceInitFunc) gee_read_only_map_instance_init, NULL };
	static const GInterfaceInfo gee_traversable_info = { (GInterfaceInitFunc) gee_read_only_map_gee_traversable_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
	static const GInterfaceInfo gee_iterable_info = { (GInterfaceInitFunc) gee_read_only_map_gee_iterable_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
	static const GInterfaceInfo gee_map_info = { (GInterfaceInitFunc) gee_read_only_map_gee_map_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
	GType gee_read_only_map_type_id;
	gee_read_only_map_type_id = g_type_register_static (G_TYPE_OBJECT, "GeeReadOnlyMap", &g_define_type_info, 0);
	g_type_add_interface_static (gee_read_only_map_type_id, GEE_TYPE_TRAVERSABLE, &gee_traversable_info);
	g_type_add_interface_static (gee_read_only_map_type_id, GEE_TYPE_ITERABLE, &gee_iterable_info);
	g_type_add_interface_static (gee_read_only_map_type_id, GEE_TYPE_MAP, &gee_map_info);
	GeeReadOnlyMap_private_offset = g_type_add_instance_private (gee_read_only_map_type_id, sizeof (GeeReadOnlyMapPrivate));
	return gee_read_only_map_type_id;
}

 G_GNUC_INTERNAL GType
gee_read_only_map_get_type (void)
{
	static volatile gsize gee_read_only_map_type_id__once = 0;
	if (g_once_init_enter (&gee_read_only_map_type_id__once)) {
		GType gee_read_only_map_type_id;
		gee_read_only_map_type_id = gee_read_only_map_get_type_once ();
		g_once_init_leave (&gee_read_only_map_type_id__once, gee_read_only_map_type_id);
	}
	return gee_read_only_map_type_id__once;
}

static void
_vala_gee_read_only_map_get_property (GObject * object,
                                      guint property_id,
                                      GValue * value,
                                      GParamSpec * pspec)
{
	GeeReadOnlyMap * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, GEE_TYPE_READ_ONLY_MAP, GeeReadOnlyMap);
	switch (property_id) {
		case GEE_READ_ONLY_MAP_SIZE_PROPERTY:
		g_value_set_int (value, gee_map_get_size ((GeeMap*) self));
		break;
		case GEE_READ_ONLY_MAP_READ_ONLY_PROPERTY:
		g_value_set_boolean (value, gee_map_get_read_only ((GeeMap*) self));
		break;
		case GEE_READ_ONLY_MAP_KEYS_PROPERTY:
		g_value_take_object (value, gee_map_get_keys ((GeeMap*) self));
		break;
		case GEE_READ_ONLY_MAP_VALUES_PROPERTY:
		g_value_take_object (value, gee_map_get_values ((GeeMap*) self));
		break;
		case GEE_READ_ONLY_MAP_ENTRIES_PROPERTY:
		g_value_take_object (value, gee_map_get_entries ((GeeMap*) self));
		break;
		case GEE_READ_ONLY_MAP_READ_ONLY_VIEW_PROPERTY:
		g_value_take_object (value, gee_read_only_map_get_read_only_view (self));
		break;
		case GEE_READ_ONLY_MAP_KEY_TYPE_PROPERTY:
		g_value_set_gtype (value, gee_read_only_map_get_key_type (self));
		break;
		case GEE_READ_ONLY_MAP_VALUE_TYPE_PROPERTY:
		g_value_set_gtype (value, gee_read_only_map_get_value_type (self));
		break;
		case GEE_READ_ONLY_MAP_K_TYPE:
		g_value_set_gtype (value, self->priv->k_type);
		break;
		case GEE_READ_ONLY_MAP_K_DUP_FUNC:
		g_value_set_pointer (value, self->priv->k_dup_func);
		break;
		case GEE_READ_ONLY_MAP_K_DESTROY_FUNC:
		g_value_set_pointer (value, self->priv->k_destroy_func);
		break;
		case GEE_READ_ONLY_MAP_V_TYPE:
		g_value_set_gtype (value, self->priv->v_type);
		break;
		case GEE_READ_ONLY_MAP_V_DUP_FUNC:
		g_value_set_pointer (value, self->priv->v_dup_func);
		break;
		case GEE_READ_ONLY_MAP_V_DESTROY_FUNC:
		g_value_set_pointer (value, self->priv->v_destroy_func);
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

static void
_vala_gee_read_only_map_set_property (GObject * object,
                                      guint property_id,
                                      const GValue * value,
                                      GParamSpec * pspec)
{
	GeeReadOnlyMap * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, GEE_TYPE_READ_ONLY_MAP, GeeReadOnlyMap);
	switch (property_id) {
		case GEE_READ_ONLY_MAP_K_TYPE:
		self->priv->k_type = g_value_get_gtype (value);
		break;
		case GEE_READ_ONLY_MAP_K_DUP_FUNC:
		self->priv->k_dup_func = g_value_get_pointer (value);
		break;
		case GEE_READ_ONLY_MAP_K_DESTROY_FUNC:
		self->priv->k_destroy_func = g_value_get_pointer (value);
		break;
		case GEE_READ_ONLY_MAP_V_TYPE:
		self->priv->v_type = g_value_get_gtype (value);
		break;
		case GEE_READ_ONLY_MAP_V_DUP_FUNC:
		self->priv->v_dup_func = g_value_get_pointer (value);
		break;
		case GEE_READ_ONLY_MAP_V_DESTROY_FUNC:
		self->priv->v_destroy_func = g_value_get_pointer (value);
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

