/* testbidirsortedmap.c generated by valac 0.56.3, the Vala compiler
 * generated from testbidirsortedmap.vala, do not modify */

/* testbidirsortedmap.vala
 *
 * Copyright (C) 2012  Maciej Piechotka
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Maciej Piechotka <uzytkownik2@gmail.com>
 */

#include <glib-object.h>
#include <gee.h>
#include <stdlib.h>
#include <string.h>
#include <glib.h>

#if !defined(VALA_EXTERN)
#if defined(_MSC_VER)
#define VALA_EXTERN __declspec(dllexport) extern
#elif __GNUC__ >= 4
#define VALA_EXTERN __attribute__((visibility("default"))) extern
#else
#define VALA_EXTERN extern
#endif
#endif

#define GEE_TYPE_TEST_CASE (gee_test_case_get_type ())
#define GEE_TEST_CASE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_TEST_CASE, GeeTestCase))
#define GEE_TEST_CASE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_TYPE_TEST_CASE, GeeTestCaseClass))
#define GEE_IS_TEST_CASE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_TEST_CASE))
#define GEE_IS_TEST_CASE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEE_TYPE_TEST_CASE))
#define GEE_TEST_CASE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_TYPE_TEST_CASE, GeeTestCaseClass))

typedef struct _GeeTestCase GeeTestCase;
typedef struct _GeeTestCaseClass GeeTestCaseClass;
typedef struct _GeeTestCasePrivate GeeTestCasePrivate;

#define TYPE_MAP_TESTS (map_tests_get_type ())
#define MAP_TESTS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_MAP_TESTS, MapTests))
#define MAP_TESTS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_MAP_TESTS, MapTestsClass))
#define IS_MAP_TESTS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_MAP_TESTS))
#define IS_MAP_TESTS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_MAP_TESTS))
#define MAP_TESTS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_MAP_TESTS, MapTestsClass))

typedef struct _MapTests MapTests;
typedef struct _MapTestsClass MapTestsClass;
typedef struct _MapTestsPrivate MapTestsPrivate;

#define GEE_TYPE_SORTED_MAP_TESTS (gee_sorted_map_tests_get_type ())
#define GEE_SORTED_MAP_TESTS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_SORTED_MAP_TESTS, GeeSortedMapTests))
#define GEE_SORTED_MAP_TESTS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_TYPE_SORTED_MAP_TESTS, GeeSortedMapTestsClass))
#define GEE_IS_SORTED_MAP_TESTS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_SORTED_MAP_TESTS))
#define GEE_IS_SORTED_MAP_TESTS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEE_TYPE_SORTED_MAP_TESTS))
#define GEE_SORTED_MAP_TESTS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_TYPE_SORTED_MAP_TESTS, GeeSortedMapTestsClass))

typedef struct _GeeSortedMapTests GeeSortedMapTests;
typedef struct _GeeSortedMapTestsClass GeeSortedMapTestsClass;
typedef struct _GeeSortedMapTestsPrivate GeeSortedMapTestsPrivate;

#define TYPE_BIDIR_SORTED_MAP_TESTS (bidir_sorted_map_tests_get_type ())
#define BIDIR_SORTED_MAP_TESTS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_BIDIR_SORTED_MAP_TESTS, BidirSortedMapTests))
#define BIDIR_SORTED_MAP_TESTS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_BIDIR_SORTED_MAP_TESTS, BidirSortedMapTestsClass))
#define IS_BIDIR_SORTED_MAP_TESTS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_BIDIR_SORTED_MAP_TESTS))
#define IS_BIDIR_SORTED_MAP_TESTS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_BIDIR_SORTED_MAP_TESTS))
#define BIDIR_SORTED_MAP_TESTS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_BIDIR_SORTED_MAP_TESTS, BidirSortedMapTestsClass))

typedef struct _BidirSortedMapTests BidirSortedMapTests;
typedef struct _BidirSortedMapTestsClass BidirSortedMapTestsClass;
typedef struct _BidirSortedMapTestsPrivate BidirSortedMapTestsPrivate;
enum  {
	BIDIR_SORTED_MAP_TESTS_0_PROPERTY,
	BIDIR_SORTED_MAP_TESTS_NUM_PROPERTIES
};
static GParamSpec* bidir_sorted_map_tests_properties[BIDIR_SORTED_MAP_TESTS_NUM_PROPERTIES];
typedef void (*GeeTestCaseTestMethod) (gpointer user_data);
typedef enum  {
	GEE_SORTED_MAP_TESTS_SUB_MAP_TESTS_TYPE_HEAD,
	GEE_SORTED_MAP_TESTS_SUB_MAP_TESTS_TYPE_TAIL,
	GEE_SORTED_MAP_TESTS_SUB_MAP_TESTS_TYPE_SUB,
	GEE_SORTED_MAP_TESTS_SUB_MAP_TESTS_TYPE_EMPTY
} GeeSortedMapTestsSubMapTestsType;

#define GEE_SORTED_MAP_TESTS_SUB_MAP_TESTS_TYPE_TYPE (gee_sorted_map_tests_sub_map_tests_type_get_type ())

#define BIDIR_SORTED_MAP_TESTS_TYPE_BIDIR_SUB_MAP_TESTS (bidir_sorted_map_tests_bidir_sub_map_tests_get_type ())
#define BIDIR_SORTED_MAP_TESTS_BIDIR_SUB_MAP_TESTS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BIDIR_SORTED_MAP_TESTS_TYPE_BIDIR_SUB_MAP_TESTS, BidirSortedMapTestsBidirSubMapTests))
#define BIDIR_SORTED_MAP_TESTS_BIDIR_SUB_MAP_TESTS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), BIDIR_SORTED_MAP_TESTS_TYPE_BIDIR_SUB_MAP_TESTS, BidirSortedMapTestsBidirSubMapTestsClass))
#define BIDIR_SORTED_MAP_TESTS_IS_BIDIR_SUB_MAP_TESTS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BIDIR_SORTED_MAP_TESTS_TYPE_BIDIR_SUB_MAP_TESTS))
#define BIDIR_SORTED_MAP_TESTS_IS_BIDIR_SUB_MAP_TESTS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), BIDIR_SORTED_MAP_TESTS_TYPE_BIDIR_SUB_MAP_TESTS))
#define BIDIR_SORTED_MAP_TESTS_BIDIR_SUB_MAP_TESTS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), BIDIR_SORTED_MAP_TESTS_TYPE_BIDIR_SUB_MAP_TESTS, BidirSortedMapTestsBidirSubMapTestsClass))

typedef struct _BidirSortedMapTestsBidirSubMapTests BidirSortedMapTestsBidirSubMapTests;
typedef struct _BidirSortedMapTestsBidirSubMapTestsClass BidirSortedMapTestsBidirSubMapTestsClass;
#define _g_object_unref0(var) ((var == NULL) ? NULL : (var = (g_object_unref (var), NULL)))
#define _g_free0(var) (var = (g_free (var), NULL))
typedef struct _BidirSortedMapTestsBidirSubMapTestsPrivate BidirSortedMapTestsBidirSubMapTestsPrivate;
enum  {
	BIDIR_SORTED_MAP_TESTS_BIDIR_SUB_MAP_TESTS_0_PROPERTY,
	BIDIR_SORTED_MAP_TESTS_BIDIR_SUB_MAP_TESTS_NUM_PROPERTIES
};
static GParamSpec* bidir_sorted_map_tests_bidir_sub_map_tests_properties[BIDIR_SORTED_MAP_TESTS_BIDIR_SUB_MAP_TESTS_NUM_PROPERTIES];
#define _vala_assert(expr, msg) if G_LIKELY (expr) ; else g_assertion_message_expr (G_LOG_DOMAIN, __FILE__, __LINE__, G_STRFUNC, msg);
#define _vala_return_if_fail(expr, msg) if G_LIKELY (expr) ; else { g_return_if_fail_warning (G_LOG_DOMAIN, G_STRFUNC, msg); return; }
#define _vala_return_val_if_fail(expr, msg, val) if G_LIKELY (expr) ; else { g_return_if_fail_warning (G_LOG_DOMAIN, G_STRFUNC, msg); return val; }
#define _vala_warn_if_fail(expr, msg) if G_LIKELY (expr) ; else g_warn_message (G_LOG_DOMAIN, __FILE__, __LINE__, G_STRFUNC, msg);

struct _GeeTestCase {
	GObject parent_instance;
	GeeTestCasePrivate * priv;
};

struct _GeeTestCaseClass {
	GObjectClass parent_class;
	void (*set_up) (GeeTestCase* self);
	void (*tear_down) (GeeTestCase* self);
};

struct _MapTests {
	GeeTestCase parent_instance;
	MapTestsPrivate * priv;
	GeeMap* test_map;
};

struct _MapTestsClass {
	GeeTestCaseClass parent_class;
};

struct _GeeSortedMapTests {
	MapTests parent_instance;
	GeeSortedMapTestsPrivate * priv;
};

struct _GeeSortedMapTestsClass {
	MapTestsClass parent_class;
};

struct _BidirSortedMapTests {
	GeeSortedMapTests parent_instance;
	BidirSortedMapTestsPrivate * priv;
};

struct _BidirSortedMapTestsClass {
	GeeSortedMapTestsClass parent_class;
};

struct _BidirSortedMapTestsBidirSubMapTests {
	GeeTestCase parent_instance;
	BidirSortedMapTestsBidirSubMapTestsPrivate * priv;
};

struct _BidirSortedMapTestsBidirSubMapTestsClass {
	GeeTestCaseClass parent_class;
};

struct _BidirSortedMapTestsBidirSubMapTestsPrivate {
	GeeBidirSortedMap* master;
	GeeBidirSortedMap* submap;
	BidirSortedMapTests* test;
	GeeSortedMapTestsSubMapTestsType type;
};

static gpointer bidir_sorted_map_tests_parent_class = NULL;
static gint BidirSortedMapTestsBidirSubMapTests_private_offset;
static gpointer bidir_sorted_map_tests_bidir_sub_map_tests_parent_class = NULL;

VALA_EXTERN GType gee_test_case_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (GeeTestCase, g_object_unref)
VALA_EXTERN GType map_tests_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (MapTests, g_object_unref)
VALA_EXTERN GType gee_sorted_map_tests_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (GeeSortedMapTests, g_object_unref)
VALA_EXTERN GType bidir_sorted_map_tests_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (BidirSortedMapTests, g_object_unref)
VALA_EXTERN BidirSortedMapTests* bidir_sorted_map_tests_construct (GType object_type,
                                                       const gchar* name);
VALA_EXTERN GeeSortedMapTests* gee_sorted_map_tests_construct (GType object_type,
                                                   const gchar* name);
VALA_EXTERN void gee_test_case_add_test (GeeTestCase* self,
                             const gchar* name,
                             GeeTestCaseTestMethod test,
                             gpointer test_target,
                             GDestroyNotify test_target_destroy_notify);
VALA_EXTERN void bidir_sorted_map_tests_test_bidir_iterator_can_go_backward (BidirSortedMapTests* self);
static void _bidir_sorted_map_tests_test_bidir_iterator_can_go_backward_gee_test_case_test_method (gpointer self);
VALA_EXTERN void bidir_sorted_map_tests_test_bidir_iterator_last (BidirSortedMapTests* self);
static void _bidir_sorted_map_tests_test_bidir_iterator_last_gee_test_case_test_method (gpointer self);
VALA_EXTERN GTestSuite* gee_test_case_get_suite (GeeTestCase* self);
VALA_EXTERN GType gee_sorted_map_tests_sub_map_tests_type_get_type (void) G_GNUC_CONST ;
VALA_EXTERN BidirSortedMapTestsBidirSubMapTests* bidir_sorted_map_tests_bidir_sub_map_tests_new (BidirSortedMapTests* test,
                                                                                     GeeSortedMapTestsSubMapTestsType type);
VALA_EXTERN BidirSortedMapTestsBidirSubMapTests* bidir_sorted_map_tests_bidir_sub_map_tests_construct (GType object_type,
                                                                                           BidirSortedMapTests* test,
                                                                                           GeeSortedMapTestsSubMapTestsType type);
VALA_EXTERN GType bidir_sorted_map_tests_bidir_sub_map_tests_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (BidirSortedMapTestsBidirSubMapTests, g_object_unref)
VALA_EXTERN void map_tests_assert_entry (GeeMapEntry* e,
                             const gchar* key,
                             const gchar* value);
VALA_EXTERN void gee_test_case_set_up (GeeTestCase* self);
VALA_EXTERN void gee_test_case_tear_down (GeeTestCase* self);
VALA_EXTERN const gchar* gee_sorted_map_tests_sub_map_tests_type_to_string (GeeSortedMapTestsSubMapTestsType self);
VALA_EXTERN GeeTestCase* gee_test_case_construct (GType object_type,
                                      const gchar* name);
VALA_EXTERN void bidir_sorted_map_tests_bidir_sub_map_tests_test_bidir_iterators (BidirSortedMapTestsBidirSubMapTests* self);
static void _bidir_sorted_map_tests_bidir_sub_map_tests_test_bidir_iterators_gee_test_case_test_method (gpointer self);
static void bidir_sorted_map_tests_bidir_sub_map_tests_real_set_up (GeeTestCase* base);
static void bidir_sorted_map_tests_bidir_sub_map_tests_real_tear_down (GeeTestCase* base);
VALA_EXTERN void bidir_sorted_map_tests_bidir_sub_map_tests_set_default_values (BidirSortedMapTestsBidirSubMapTests* self,
                                                                    gchar*** contains,
                                                                    gint* contains_length1,
                                                                    gchar*** not_contains,
                                                                    gint* not_contains_length1);
static void bidir_sorted_map_tests_bidir_sub_map_tests_finalize (GObject * obj);
static GType bidir_sorted_map_tests_bidir_sub_map_tests_get_type_once (void);
static GType bidir_sorted_map_tests_get_type_once (void);
static void _vala_array_destroy (gpointer array,
                          gssize array_length,
                          GDestroyNotify destroy_func);
static void _vala_array_free (gpointer array,
                       gssize array_length,
                       GDestroyNotify destroy_func);

static void
_bidir_sorted_map_tests_test_bidir_iterator_can_go_backward_gee_test_case_test_method (gpointer self)
{
	bidir_sorted_map_tests_test_bidir_iterator_can_go_backward ((BidirSortedMapTests*) self);
}

static void
_bidir_sorted_map_tests_test_bidir_iterator_last_gee_test_case_test_method (gpointer self)
{
	bidir_sorted_map_tests_test_bidir_iterator_last ((BidirSortedMapTests*) self);
}

BidirSortedMapTests*
bidir_sorted_map_tests_construct (GType object_type,
                                  const gchar* name)
{
	BidirSortedMapTests * self = NULL;
	GTestSuite* _tmp0_;
	BidirSortedMapTestsBidirSubMapTests* _tmp1_;
	BidirSortedMapTestsBidirSubMapTests* _tmp2_;
	GTestSuite* _tmp3_;
	GTestSuite* _tmp4_;
	BidirSortedMapTestsBidirSubMapTests* _tmp5_;
	BidirSortedMapTestsBidirSubMapTests* _tmp6_;
	GTestSuite* _tmp7_;
	GTestSuite* _tmp8_;
	BidirSortedMapTestsBidirSubMapTests* _tmp9_;
	BidirSortedMapTestsBidirSubMapTests* _tmp10_;
	GTestSuite* _tmp11_;
	GTestSuite* _tmp12_;
	BidirSortedMapTestsBidirSubMapTests* _tmp13_;
	BidirSortedMapTestsBidirSubMapTests* _tmp14_;
	GTestSuite* _tmp15_;
	g_return_val_if_fail (name != NULL, NULL);
	self = (BidirSortedMapTests*) gee_sorted_map_tests_construct (object_type, name);
	gee_test_case_add_test ((GeeTestCase*) self, "[SortedMap] bi-directional iterators can go backward", _bidir_sorted_map_tests_test_bidir_iterator_can_go_backward_gee_test_case_test_method, g_object_ref (self), g_object_unref);
	gee_test_case_add_test ((GeeTestCase*) self, "[SortedSet] bi-directional iterators can to end", _bidir_sorted_map_tests_test_bidir_iterator_last_gee_test_case_test_method, g_object_ref (self), g_object_unref);
	_tmp0_ = gee_test_case_get_suite ((GeeTestCase*) self);
	_tmp1_ = bidir_sorted_map_tests_bidir_sub_map_tests_new (self, GEE_SORTED_MAP_TESTS_SUB_MAP_TESTS_TYPE_HEAD);
	_tmp2_ = _tmp1_;
	_tmp3_ = gee_test_case_get_suite ((GeeTestCase*) _tmp2_);
	g_test_suite_add_suite (_tmp0_, _tmp3_);
	_g_object_unref0 (_tmp2_);
	_tmp4_ = gee_test_case_get_suite ((GeeTestCase*) self);
	_tmp5_ = bidir_sorted_map_tests_bidir_sub_map_tests_new (self, GEE_SORTED_MAP_TESTS_SUB_MAP_TESTS_TYPE_TAIL);
	_tmp6_ = _tmp5_;
	_tmp7_ = gee_test_case_get_suite ((GeeTestCase*) _tmp6_);
	g_test_suite_add_suite (_tmp4_, _tmp7_);
	_g_object_unref0 (_tmp6_);
	_tmp8_ = gee_test_case_get_suite ((GeeTestCase*) self);
	_tmp9_ = bidir_sorted_map_tests_bidir_sub_map_tests_new (self, GEE_SORTED_MAP_TESTS_SUB_MAP_TESTS_TYPE_SUB);
	_tmp10_ = _tmp9_;
	_tmp11_ = gee_test_case_get_suite ((GeeTestCase*) _tmp10_);
	g_test_suite_add_suite (_tmp8_, _tmp11_);
	_g_object_unref0 (_tmp10_);
	_tmp12_ = gee_test_case_get_suite ((GeeTestCase*) self);
	_tmp13_ = bidir_sorted_map_tests_bidir_sub_map_tests_new (self, GEE_SORTED_MAP_TESTS_SUB_MAP_TESTS_TYPE_EMPTY);
	_tmp14_ = _tmp13_;
	_tmp15_ = gee_test_case_get_suite ((GeeTestCase*) _tmp14_);
	g_test_suite_add_suite (_tmp12_, _tmp15_);
	_g_object_unref0 (_tmp14_);
	return self;
}

static gpointer
_g_object_ref0 (gpointer self)
{
	return self ? g_object_ref (self) : NULL;
}

void
bidir_sorted_map_tests_test_bidir_iterator_can_go_backward (BidirSortedMapTests* self)
{
	GeeBidirSortedMap* test_sorted_map = NULL;
	GeeMap* _tmp0_;
	GeeBidirSortedMap* _tmp1_;
	GeeBidirSortedSet* keys = NULL;
	GeeSortedSet* _tmp2_;
	GeeSortedSet* _tmp3_;
	GeeBidirSortedSet* _tmp4_;
	GeeBidirSortedSet* entries = NULL;
	GeeSortedSet* _tmp5_;
	GeeSortedSet* _tmp6_;
	GeeBidirSortedSet* _tmp7_;
	GeeBidirIterator* keys_iterator = NULL;
	GeeBidirIterator* _tmp8_;
	GeeBidirIterator* entries_iterator = NULL;
	GeeBidirIterator* _tmp9_;
	GeeBidirMapIterator* map_iterator = NULL;
	GeeBidirMapIterator* _tmp10_;
	GeeBidirIterator* _tmp11_;
	GeeBidirIterator* _tmp12_;
	GeeBidirMapIterator* _tmp13_;
	GeeBidirIterator* _tmp14_;
	GeeBidirIterator* _tmp15_;
	GeeBidirMapIterator* _tmp16_;
	GeeBidirIterator* _tmp17_;
	GeeBidirIterator* _tmp18_;
	GeeBidirMapIterator* _tmp19_;
	GeeBidirIterator* _tmp20_;
	GeeBidirIterator* _tmp21_;
	GeeBidirMapIterator* _tmp22_;
	GeeBidirIterator* _tmp23_;
	gpointer _tmp24_;
	gchar* _tmp25_;
	GeeBidirIterator* _tmp26_;
	gpointer _tmp27_;
	GeeMapEntry* _tmp28_;
	GeeBidirMapIterator* _tmp29_;
	gpointer _tmp30_;
	gchar* _tmp31_;
	GeeBidirMapIterator* _tmp32_;
	gpointer _tmp33_;
	gchar* _tmp34_;
	GeeBidirIterator* _tmp35_;
	GeeBidirIterator* _tmp36_;
	GeeBidirMapIterator* _tmp37_;
	GeeBidirIterator* _tmp38_;
	GeeBidirIterator* _tmp39_;
	GeeBidirMapIterator* _tmp40_;
	GeeBidirIterator* _tmp41_;
	gpointer _tmp42_;
	gchar* _tmp43_;
	GeeBidirIterator* _tmp44_;
	gpointer _tmp45_;
	GeeMapEntry* _tmp46_;
	GeeBidirMapIterator* _tmp47_;
	gpointer _tmp48_;
	gchar* _tmp49_;
	GeeBidirMapIterator* _tmp50_;
	gpointer _tmp51_;
	gchar* _tmp52_;
	GeeBidirIterator* _tmp53_;
	GeeBidirIterator* _tmp54_;
	GeeBidirMapIterator* _tmp55_;
	GeeBidirIterator* _tmp56_;
	GeeBidirIterator* _tmp57_;
	GeeBidirMapIterator* _tmp58_;
	GeeBidirIterator* _tmp59_;
	gpointer _tmp60_;
	gchar* _tmp61_;
	GeeBidirIterator* _tmp62_;
	gpointer _tmp63_;
	GeeMapEntry* _tmp64_;
	GeeBidirMapIterator* _tmp65_;
	gpointer _tmp66_;
	gchar* _tmp67_;
	GeeBidirMapIterator* _tmp68_;
	gpointer _tmp69_;
	gchar* _tmp70_;
	GeeBidirIterator* _tmp71_;
	GeeBidirIterator* _tmp72_;
	GeeBidirMapIterator* _tmp73_;
	GeeBidirIterator* _tmp74_;
	GeeBidirIterator* _tmp75_;
	GeeBidirMapIterator* _tmp76_;
	GeeBidirIterator* _tmp77_;
	gpointer _tmp78_;
	gchar* _tmp79_;
	GeeBidirIterator* _tmp80_;
	gpointer _tmp81_;
	GeeMapEntry* _tmp82_;
	GeeBidirMapIterator* _tmp83_;
	gpointer _tmp84_;
	gchar* _tmp85_;
	GeeBidirMapIterator* _tmp86_;
	gpointer _tmp87_;
	gchar* _tmp88_;
	GeeBidirIterator* _tmp89_;
	GeeBidirIterator* _tmp90_;
	GeeBidirMapIterator* _tmp91_;
	GeeBidirIterator* _tmp92_;
	GeeBidirIterator* _tmp93_;
	GeeBidirMapIterator* _tmp94_;
	GeeBidirIterator* _tmp95_;
	gpointer _tmp96_;
	gchar* _tmp97_;
	GeeBidirIterator* _tmp98_;
	gpointer _tmp99_;
	GeeMapEntry* _tmp100_;
	GeeBidirMapIterator* _tmp101_;
	gpointer _tmp102_;
	gchar* _tmp103_;
	GeeBidirMapIterator* _tmp104_;
	gpointer _tmp105_;
	gchar* _tmp106_;
	GeeBidirIterator* _tmp107_;
	GeeBidirIterator* _tmp108_;
	GeeBidirMapIterator* _tmp109_;
	GeeBidirIterator* _tmp110_;
	GeeBidirIterator* _tmp111_;
	GeeBidirMapIterator* _tmp112_;
	GeeBidirIterator* _tmp113_;
	gpointer _tmp114_;
	gchar* _tmp115_;
	GeeBidirIterator* _tmp116_;
	gpointer _tmp117_;
	GeeMapEntry* _tmp118_;
	GeeBidirMapIterator* _tmp119_;
	gpointer _tmp120_;
	gchar* _tmp121_;
	GeeBidirMapIterator* _tmp122_;
	gpointer _tmp123_;
	gchar* _tmp124_;
	GeeBidirIterator* _tmp125_;
	GeeBidirIterator* _tmp126_;
	GeeBidirMapIterator* _tmp127_;
	GeeBidirIterator* _tmp128_;
	GeeBidirIterator* _tmp129_;
	GeeBidirMapIterator* _tmp130_;
	GeeBidirIterator* _tmp131_;
	GeeBidirIterator* _tmp132_;
	GeeBidirMapIterator* _tmp133_;
	GeeBidirIterator* _tmp134_;
	gpointer _tmp135_;
	gchar* _tmp136_;
	GeeBidirIterator* _tmp137_;
	gpointer _tmp138_;
	GeeMapEntry* _tmp139_;
	GeeBidirMapIterator* _tmp140_;
	gpointer _tmp141_;
	gchar* _tmp142_;
	GeeBidirMapIterator* _tmp143_;
	gpointer _tmp144_;
	gchar* _tmp145_;
	GeeBidirIterator* _tmp146_;
	GeeBidirIterator* _tmp147_;
	GeeBidirMapIterator* _tmp148_;
	GeeBidirIterator* _tmp149_;
	gpointer _tmp150_;
	gchar* _tmp151_;
	GeeBidirIterator* _tmp152_;
	gpointer _tmp153_;
	GeeMapEntry* _tmp154_;
	GeeBidirMapIterator* _tmp155_;
	gpointer _tmp156_;
	gchar* _tmp157_;
	GeeBidirMapIterator* _tmp158_;
	gpointer _tmp159_;
	gchar* _tmp160_;
	GeeBidirIterator* _tmp161_;
	GeeBidirIterator* _tmp162_;
	GeeBidirMapIterator* _tmp163_;
	GeeBidirIterator* _tmp164_;
	gpointer _tmp165_;
	gchar* _tmp166_;
	GeeBidirIterator* _tmp167_;
	gpointer _tmp168_;
	GeeMapEntry* _tmp169_;
	GeeBidirMapIterator* _tmp170_;
	gpointer _tmp171_;
	gchar* _tmp172_;
	GeeBidirMapIterator* _tmp173_;
	gpointer _tmp174_;
	gchar* _tmp175_;
	GeeBidirIterator* _tmp176_;
	GeeBidirIterator* _tmp177_;
	GeeBidirMapIterator* _tmp178_;
	GeeBidirIterator* _tmp179_;
	gpointer _tmp180_;
	gchar* _tmp181_;
	GeeBidirIterator* _tmp182_;
	gpointer _tmp183_;
	GeeMapEntry* _tmp184_;
	GeeBidirMapIterator* _tmp185_;
	gpointer _tmp186_;
	gchar* _tmp187_;
	GeeBidirMapIterator* _tmp188_;
	gpointer _tmp189_;
	gchar* _tmp190_;
	GeeBidirIterator* _tmp191_;
	GeeBidirIterator* _tmp192_;
	GeeBidirMapIterator* _tmp193_;
	GeeBidirIterator* _tmp194_;
	gpointer _tmp195_;
	gchar* _tmp196_;
	GeeBidirIterator* _tmp197_;
	gpointer _tmp198_;
	GeeMapEntry* _tmp199_;
	GeeBidirMapIterator* _tmp200_;
	gpointer _tmp201_;
	gchar* _tmp202_;
	GeeBidirMapIterator* _tmp203_;
	gpointer _tmp204_;
	gchar* _tmp205_;
	GeeBidirIterator* _tmp206_;
	GeeBidirIterator* _tmp207_;
	GeeBidirMapIterator* _tmp208_;
	GeeBidirIterator* _tmp209_;
	gpointer _tmp210_;
	gchar* _tmp211_;
	GeeBidirIterator* _tmp212_;
	gpointer _tmp213_;
	GeeMapEntry* _tmp214_;
	GeeBidirMapIterator* _tmp215_;
	gpointer _tmp216_;
	gchar* _tmp217_;
	GeeBidirMapIterator* _tmp218_;
	gpointer _tmp219_;
	gchar* _tmp220_;
	g_return_if_fail (self != NULL);
	_tmp0_ = ((MapTests*) self)->test_map;
	_tmp1_ = _g_object_ref0 (G_TYPE_CHECK_INSTANCE_TYPE (_tmp0_, GEE_TYPE_BIDIR_SORTED_MAP) ? ((GeeBidirSortedMap*) _tmp0_) : NULL);
	test_sorted_map = _tmp1_;
	_tmp2_ = gee_sorted_map_get_ascending_keys ((GeeSortedMap*) test_sorted_map);
	_tmp3_ = _tmp2_;
	_tmp4_ = G_TYPE_CHECK_INSTANCE_TYPE (_tmp3_, GEE_TYPE_BIDIR_SORTED_SET) ? ((GeeBidirSortedSet*) _tmp3_) : NULL;
	if (_tmp4_ == NULL) {
		_g_object_unref0 (_tmp3_);
	}
	keys = _tmp4_;
	_tmp5_ = gee_sorted_map_get_ascending_entries ((GeeSortedMap*) test_sorted_map);
	_tmp6_ = _tmp5_;
	_tmp7_ = G_TYPE_CHECK_INSTANCE_TYPE (_tmp6_, GEE_TYPE_BIDIR_SORTED_SET) ? ((GeeBidirSortedSet*) _tmp6_) : NULL;
	if (_tmp7_ == NULL) {
		_g_object_unref0 (_tmp6_);
	}
	entries = _tmp7_;
	_tmp8_ = gee_bidir_sorted_set_bidir_iterator (keys);
	keys_iterator = _tmp8_;
	_tmp9_ = gee_bidir_sorted_set_bidir_iterator (entries);
	entries_iterator = _tmp9_;
	_tmp10_ = gee_bidir_sorted_map_bidir_map_iterator (test_sorted_map);
	map_iterator = _tmp10_;
	_tmp11_ = keys_iterator;
	_vala_assert (!gee_bidir_iterator_has_previous (_tmp11_), "!keys_iterator.has_previous ()");
	_tmp12_ = entries_iterator;
	_vala_assert (!gee_bidir_iterator_has_previous (_tmp12_), "!entries_iterator.has_previous ()");
	_tmp13_ = map_iterator;
	_vala_assert (!gee_bidir_map_iterator_has_previous (_tmp13_), "!map_iterator.has_previous ()");
	_tmp14_ = keys_iterator;
	_vala_assert (!gee_bidir_iterator_previous (_tmp14_), "!keys_iterator.previous ()");
	_tmp15_ = entries_iterator;
	_vala_assert (!gee_bidir_iterator_has_previous (_tmp15_), "!entries_iterator.has_previous ()");
	_tmp16_ = map_iterator;
	_vala_assert (!gee_bidir_map_iterator_previous (_tmp16_), "!map_iterator.previous ()");
	gee_map_set ((GeeMap*) test_sorted_map, "one", "one");
	gee_map_set ((GeeMap*) test_sorted_map, "two", "two");
	gee_map_set ((GeeMap*) test_sorted_map, "three", "three");
	gee_map_set ((GeeMap*) test_sorted_map, "four", "four");
	gee_map_set ((GeeMap*) test_sorted_map, "five", "five");
	gee_map_set ((GeeMap*) test_sorted_map, "six", "six");
	_tmp17_ = gee_bidir_sorted_set_bidir_iterator (keys);
	_g_object_unref0 (keys_iterator);
	keys_iterator = _tmp17_;
	_tmp18_ = gee_bidir_sorted_set_bidir_iterator (entries);
	_g_object_unref0 (entries_iterator);
	entries_iterator = _tmp18_;
	_tmp19_ = gee_bidir_sorted_map_bidir_map_iterator (test_sorted_map);
	_g_object_unref0 (map_iterator);
	map_iterator = _tmp19_;
	_tmp20_ = keys_iterator;
	_vala_assert (gee_iterator_next ((GeeIterator*) _tmp20_), "keys_iterator.next ()");
	_tmp21_ = entries_iterator;
	_vala_assert (gee_iterator_next ((GeeIterator*) _tmp21_), "entries_iterator.next ()");
	_tmp22_ = map_iterator;
	_vala_assert (gee_map_iterator_next ((GeeMapIterator*) _tmp22_), "map_iterator.next ()");
	_tmp23_ = keys_iterator;
	_tmp24_ = gee_iterator_get ((GeeIterator*) _tmp23_);
	_tmp25_ = (gchar*) _tmp24_;
	_vala_assert (g_strcmp0 (_tmp25_, "five") == 0, "keys_iterator.get () == \"five\"");
	_g_free0 (_tmp25_);
	_tmp26_ = entries_iterator;
	_tmp27_ = gee_iterator_get ((GeeIterator*) _tmp26_);
	_tmp28_ = (GeeMapEntry*) _tmp27_;
	map_tests_assert_entry (_tmp28_, "five", "five");
	_g_object_unref0 (_tmp28_);
	_tmp29_ = map_iterator;
	_tmp30_ = gee_map_iterator_get_key ((GeeMapIterator*) _tmp29_);
	_tmp31_ = (gchar*) _tmp30_;
	_vala_assert (g_strcmp0 (_tmp31_, "five") == 0, "map_iterator.get_key () == \"five\"");
	_g_free0 (_tmp31_);
	_tmp32_ = map_iterator;
	_tmp33_ = gee_map_iterator_get_value ((GeeMapIterator*) _tmp32_);
	_tmp34_ = (gchar*) _tmp33_;
	_vala_assert (g_strcmp0 (_tmp34_, "five") == 0, "map_iterator.get_value () == \"five\"");
	_g_free0 (_tmp34_);
	_tmp35_ = keys_iterator;
	_vala_assert (!gee_bidir_iterator_has_previous (_tmp35_), "!keys_iterator.has_previous ()");
	_tmp36_ = entries_iterator;
	_vala_assert (!gee_bidir_iterator_has_previous (_tmp36_), "!entries_iterator.has_previous ()");
	_tmp37_ = map_iterator;
	_vala_assert (!gee_bidir_map_iterator_has_previous (_tmp37_), "!map_iterator.has_previous ()");
	_tmp38_ = keys_iterator;
	_vala_assert (gee_iterator_next ((GeeIterator*) _tmp38_), "keys_iterator.next ()");
	_tmp39_ = entries_iterator;
	_vala_assert (gee_iterator_next ((GeeIterator*) _tmp39_), "entries_iterator.next ()");
	_tmp40_ = map_iterator;
	_vala_assert (gee_map_iterator_next ((GeeMapIterator*) _tmp40_), "map_iterator.next ()");
	_tmp41_ = keys_iterator;
	_tmp42_ = gee_iterator_get ((GeeIterator*) _tmp41_);
	_tmp43_ = (gchar*) _tmp42_;
	_vala_assert (g_strcmp0 (_tmp43_, "four") == 0, "keys_iterator.get () == \"four\"");
	_g_free0 (_tmp43_);
	_tmp44_ = entries_iterator;
	_tmp45_ = gee_iterator_get ((GeeIterator*) _tmp44_);
	_tmp46_ = (GeeMapEntry*) _tmp45_;
	map_tests_assert_entry (_tmp46_, "four", "four");
	_g_object_unref0 (_tmp46_);
	_tmp47_ = map_iterator;
	_tmp48_ = gee_map_iterator_get_key ((GeeMapIterator*) _tmp47_);
	_tmp49_ = (gchar*) _tmp48_;
	_vala_assert (g_strcmp0 (_tmp49_, "four") == 0, "map_iterator.get_key () == \"four\"");
	_g_free0 (_tmp49_);
	_tmp50_ = map_iterator;
	_tmp51_ = gee_map_iterator_get_value ((GeeMapIterator*) _tmp50_);
	_tmp52_ = (gchar*) _tmp51_;
	_vala_assert (g_strcmp0 (_tmp52_, "four") == 0, "map_iterator.get_value () == \"four\"");
	_g_free0 (_tmp52_);
	_tmp53_ = keys_iterator;
	_vala_assert (gee_bidir_iterator_has_previous (_tmp53_), "keys_iterator.has_previous ()");
	_tmp54_ = entries_iterator;
	_vala_assert (gee_bidir_iterator_has_previous (_tmp54_), "entries_iterator.has_previous ()");
	_tmp55_ = map_iterator;
	_vala_assert (gee_bidir_map_iterator_has_previous (_tmp55_), "map_iterator.has_previous ()");
	_tmp56_ = keys_iterator;
	_vala_assert (gee_iterator_next ((GeeIterator*) _tmp56_), "keys_iterator.next ()");
	_tmp57_ = entries_iterator;
	_vala_assert (gee_iterator_next ((GeeIterator*) _tmp57_), "entries_iterator.next ()");
	_tmp58_ = map_iterator;
	_vala_assert (gee_map_iterator_next ((GeeMapIterator*) _tmp58_), "map_iterator.next ()");
	_tmp59_ = keys_iterator;
	_tmp60_ = gee_iterator_get ((GeeIterator*) _tmp59_);
	_tmp61_ = (gchar*) _tmp60_;
	_vala_assert (g_strcmp0 (_tmp61_, "one") == 0, "keys_iterator.get () == \"one\"");
	_g_free0 (_tmp61_);
	_tmp62_ = entries_iterator;
	_tmp63_ = gee_iterator_get ((GeeIterator*) _tmp62_);
	_tmp64_ = (GeeMapEntry*) _tmp63_;
	map_tests_assert_entry (_tmp64_, "one", "one");
	_g_object_unref0 (_tmp64_);
	_tmp65_ = map_iterator;
	_tmp66_ = gee_map_iterator_get_key ((GeeMapIterator*) _tmp65_);
	_tmp67_ = (gchar*) _tmp66_;
	_vala_assert (g_strcmp0 (_tmp67_, "one") == 0, "map_iterator.get_key () == \"one\"");
	_g_free0 (_tmp67_);
	_tmp68_ = map_iterator;
	_tmp69_ = gee_map_iterator_get_value ((GeeMapIterator*) _tmp68_);
	_tmp70_ = (gchar*) _tmp69_;
	_vala_assert (g_strcmp0 (_tmp70_, "one") == 0, "map_iterator.get_value () == \"one\"");
	_g_free0 (_tmp70_);
	_tmp71_ = keys_iterator;
	_vala_assert (gee_bidir_iterator_has_previous (_tmp71_), "keys_iterator.has_previous ()");
	_tmp72_ = entries_iterator;
	_vala_assert (gee_bidir_iterator_has_previous (_tmp72_), "entries_iterator.has_previous ()");
	_tmp73_ = map_iterator;
	_vala_assert (gee_bidir_map_iterator_has_previous (_tmp73_), "map_iterator.has_previous ()");
	_tmp74_ = keys_iterator;
	_vala_assert (gee_iterator_next ((GeeIterator*) _tmp74_), "keys_iterator.next ()");
	_tmp75_ = entries_iterator;
	_vala_assert (gee_iterator_next ((GeeIterator*) _tmp75_), "entries_iterator.next ()");
	_tmp76_ = map_iterator;
	_vala_assert (gee_map_iterator_next ((GeeMapIterator*) _tmp76_), "map_iterator.next ()");
	_tmp77_ = keys_iterator;
	_tmp78_ = gee_iterator_get ((GeeIterator*) _tmp77_);
	_tmp79_ = (gchar*) _tmp78_;
	_vala_assert (g_strcmp0 (_tmp79_, "six") == 0, "keys_iterator.get () == \"six\"");
	_g_free0 (_tmp79_);
	_tmp80_ = entries_iterator;
	_tmp81_ = gee_iterator_get ((GeeIterator*) _tmp80_);
	_tmp82_ = (GeeMapEntry*) _tmp81_;
	map_tests_assert_entry (_tmp82_, "six", "six");
	_g_object_unref0 (_tmp82_);
	_tmp83_ = map_iterator;
	_tmp84_ = gee_map_iterator_get_key ((GeeMapIterator*) _tmp83_);
	_tmp85_ = (gchar*) _tmp84_;
	_vala_assert (g_strcmp0 (_tmp85_, "six") == 0, "map_iterator.get_key () == \"six\"");
	_g_free0 (_tmp85_);
	_tmp86_ = map_iterator;
	_tmp87_ = gee_map_iterator_get_value ((GeeMapIterator*) _tmp86_);
	_tmp88_ = (gchar*) _tmp87_;
	_vala_assert (g_strcmp0 (_tmp88_, "six") == 0, "map_iterator.get_value () == \"six\"");
	_g_free0 (_tmp88_);
	_tmp89_ = keys_iterator;
	_vala_assert (gee_bidir_iterator_has_previous (_tmp89_), "keys_iterator.has_previous ()");
	_tmp90_ = entries_iterator;
	_vala_assert (gee_bidir_iterator_has_previous (_tmp90_), "entries_iterator.has_previous ()");
	_tmp91_ = map_iterator;
	_vala_assert (gee_bidir_map_iterator_has_previous (_tmp91_), "map_iterator.has_previous ()");
	_tmp92_ = keys_iterator;
	_vala_assert (gee_iterator_next ((GeeIterator*) _tmp92_), "keys_iterator.next ()");
	_tmp93_ = entries_iterator;
	_vala_assert (gee_iterator_next ((GeeIterator*) _tmp93_), "entries_iterator.next ()");
	_tmp94_ = map_iterator;
	_vala_assert (gee_map_iterator_next ((GeeMapIterator*) _tmp94_), "map_iterator.next ()");
	_tmp95_ = keys_iterator;
	_tmp96_ = gee_iterator_get ((GeeIterator*) _tmp95_);
	_tmp97_ = (gchar*) _tmp96_;
	_vala_assert (g_strcmp0 (_tmp97_, "three") == 0, "keys_iterator.get () == \"three\"");
	_g_free0 (_tmp97_);
	_tmp98_ = entries_iterator;
	_tmp99_ = gee_iterator_get ((GeeIterator*) _tmp98_);
	_tmp100_ = (GeeMapEntry*) _tmp99_;
	map_tests_assert_entry (_tmp100_, "three", "three");
	_g_object_unref0 (_tmp100_);
	_tmp101_ = map_iterator;
	_tmp102_ = gee_map_iterator_get_key ((GeeMapIterator*) _tmp101_);
	_tmp103_ = (gchar*) _tmp102_;
	_vala_assert (g_strcmp0 (_tmp103_, "three") == 0, "map_iterator.get_key () == \"three\"");
	_g_free0 (_tmp103_);
	_tmp104_ = map_iterator;
	_tmp105_ = gee_map_iterator_get_value ((GeeMapIterator*) _tmp104_);
	_tmp106_ = (gchar*) _tmp105_;
	_vala_assert (g_strcmp0 (_tmp106_, "three") == 0, "map_iterator.get_value () == \"three\"");
	_g_free0 (_tmp106_);
	_tmp107_ = keys_iterator;
	_vala_assert (gee_bidir_iterator_has_previous (_tmp107_), "keys_iterator.has_previous ()");
	_tmp108_ = entries_iterator;
	_vala_assert (gee_bidir_iterator_has_previous (_tmp108_), "entries_iterator.has_previous ()");
	_tmp109_ = map_iterator;
	_vala_assert (gee_bidir_map_iterator_has_previous (_tmp109_), "map_iterator.has_previous ()");
	_tmp110_ = keys_iterator;
	_vala_assert (gee_iterator_next ((GeeIterator*) _tmp110_), "keys_iterator.next ()");
	_tmp111_ = entries_iterator;
	_vala_assert (gee_iterator_next ((GeeIterator*) _tmp111_), "entries_iterator.next ()");
	_tmp112_ = map_iterator;
	_vala_assert (gee_map_iterator_next ((GeeMapIterator*) _tmp112_), "map_iterator.next ()");
	_tmp113_ = keys_iterator;
	_tmp114_ = gee_iterator_get ((GeeIterator*) _tmp113_);
	_tmp115_ = (gchar*) _tmp114_;
	_vala_assert (g_strcmp0 (_tmp115_, "two") == 0, "keys_iterator.get () == \"two\"");
	_g_free0 (_tmp115_);
	_tmp116_ = entries_iterator;
	_tmp117_ = gee_iterator_get ((GeeIterator*) _tmp116_);
	_tmp118_ = (GeeMapEntry*) _tmp117_;
	map_tests_assert_entry (_tmp118_, "two", "two");
	_g_object_unref0 (_tmp118_);
	_tmp119_ = map_iterator;
	_tmp120_ = gee_map_iterator_get_key ((GeeMapIterator*) _tmp119_);
	_tmp121_ = (gchar*) _tmp120_;
	_vala_assert (g_strcmp0 (_tmp121_, "two") == 0, "map_iterator.get_key () == \"two\"");
	_g_free0 (_tmp121_);
	_tmp122_ = map_iterator;
	_tmp123_ = gee_map_iterator_get_value ((GeeMapIterator*) _tmp122_);
	_tmp124_ = (gchar*) _tmp123_;
	_vala_assert (g_strcmp0 (_tmp124_, "two") == 0, "map_iterator.get_value () == \"two\"");
	_g_free0 (_tmp124_);
	_tmp125_ = keys_iterator;
	_vala_assert (gee_bidir_iterator_has_previous (_tmp125_), "keys_iterator.has_previous ()");
	_tmp126_ = entries_iterator;
	_vala_assert (gee_bidir_iterator_has_previous (_tmp126_), "entries_iterator.has_previous ()");
	_tmp127_ = map_iterator;
	_vala_assert (gee_bidir_map_iterator_has_previous (_tmp127_), "map_iterator.has_previous ()");
	_tmp128_ = keys_iterator;
	_vala_assert (!gee_iterator_next ((GeeIterator*) _tmp128_), "!keys_iterator.next ()");
	_tmp129_ = entries_iterator;
	_vala_assert (!gee_iterator_next ((GeeIterator*) _tmp129_), "!entries_iterator.next ()");
	_tmp130_ = map_iterator;
	_vala_assert (!gee_map_iterator_next ((GeeMapIterator*) _tmp130_), "!map_iterator.next ()");
	_tmp131_ = keys_iterator;
	_vala_assert (gee_bidir_iterator_previous (_tmp131_), "keys_iterator.previous ()");
	_tmp132_ = entries_iterator;
	_vala_assert (gee_bidir_iterator_previous (_tmp132_), "entries_iterator.previous ()");
	_tmp133_ = map_iterator;
	_vala_assert (gee_bidir_map_iterator_previous (_tmp133_), "map_iterator.previous ()");
	_tmp134_ = keys_iterator;
	_tmp135_ = gee_iterator_get ((GeeIterator*) _tmp134_);
	_tmp136_ = (gchar*) _tmp135_;
	_vala_assert (g_strcmp0 (_tmp136_, "three") == 0, "keys_iterator.get () == \"three\"");
	_g_free0 (_tmp136_);
	_tmp137_ = entries_iterator;
	_tmp138_ = gee_iterator_get ((GeeIterator*) _tmp137_);
	_tmp139_ = (GeeMapEntry*) _tmp138_;
	map_tests_assert_entry (_tmp139_, "three", "three");
	_g_object_unref0 (_tmp139_);
	_tmp140_ = map_iterator;
	_tmp141_ = gee_map_iterator_get_key ((GeeMapIterator*) _tmp140_);
	_tmp142_ = (gchar*) _tmp141_;
	_vala_assert (g_strcmp0 (_tmp142_, "three") == 0, "map_iterator.get_key () == \"three\"");
	_g_free0 (_tmp142_);
	_tmp143_ = map_iterator;
	_tmp144_ = gee_map_iterator_get_value ((GeeMapIterator*) _tmp143_);
	_tmp145_ = (gchar*) _tmp144_;
	_vala_assert (g_strcmp0 (_tmp145_, "three") == 0, "map_iterator.get_value () == \"three\"");
	_g_free0 (_tmp145_);
	_tmp146_ = keys_iterator;
	_vala_assert (gee_bidir_iterator_previous (_tmp146_), "keys_iterator.previous ()");
	_tmp147_ = entries_iterator;
	_vala_assert (gee_bidir_iterator_previous (_tmp147_), "entries_iterator.previous ()");
	_tmp148_ = map_iterator;
	_vala_assert (gee_bidir_map_iterator_previous (_tmp148_), "map_iterator.previous ()");
	_tmp149_ = keys_iterator;
	_tmp150_ = gee_iterator_get ((GeeIterator*) _tmp149_);
	_tmp151_ = (gchar*) _tmp150_;
	_vala_assert (g_strcmp0 (_tmp151_, "six") == 0, "keys_iterator.get () == \"six\"");
	_g_free0 (_tmp151_);
	_tmp152_ = entries_iterator;
	_tmp153_ = gee_iterator_get ((GeeIterator*) _tmp152_);
	_tmp154_ = (GeeMapEntry*) _tmp153_;
	map_tests_assert_entry (_tmp154_, "six", "six");
	_g_object_unref0 (_tmp154_);
	_tmp155_ = map_iterator;
	_tmp156_ = gee_map_iterator_get_key ((GeeMapIterator*) _tmp155_);
	_tmp157_ = (gchar*) _tmp156_;
	_vala_assert (g_strcmp0 (_tmp157_, "six") == 0, "map_iterator.get_key () == \"six\"");
	_g_free0 (_tmp157_);
	_tmp158_ = map_iterator;
	_tmp159_ = gee_map_iterator_get_value ((GeeMapIterator*) _tmp158_);
	_tmp160_ = (gchar*) _tmp159_;
	_vala_assert (g_strcmp0 (_tmp160_, "six") == 0, "map_iterator.get_value () == \"six\"");
	_g_free0 (_tmp160_);
	_tmp161_ = keys_iterator;
	_vala_assert (gee_bidir_iterator_previous (_tmp161_), "keys_iterator.previous ()");
	_tmp162_ = entries_iterator;
	_vala_assert (gee_bidir_iterator_previous (_tmp162_), "entries_iterator.previous ()");
	_tmp163_ = map_iterator;
	_vala_assert (gee_bidir_map_iterator_previous (_tmp163_), "map_iterator.previous ()");
	_tmp164_ = keys_iterator;
	_tmp165_ = gee_iterator_get ((GeeIterator*) _tmp164_);
	_tmp166_ = (gchar*) _tmp165_;
	_vala_assert (g_strcmp0 (_tmp166_, "one") == 0, "keys_iterator.get () == \"one\"");
	_g_free0 (_tmp166_);
	_tmp167_ = entries_iterator;
	_tmp168_ = gee_iterator_get ((GeeIterator*) _tmp167_);
	_tmp169_ = (GeeMapEntry*) _tmp168_;
	map_tests_assert_entry (_tmp169_, "one", "one");
	_g_object_unref0 (_tmp169_);
	_tmp170_ = map_iterator;
	_tmp171_ = gee_map_iterator_get_key ((GeeMapIterator*) _tmp170_);
	_tmp172_ = (gchar*) _tmp171_;
	_vala_assert (g_strcmp0 (_tmp172_, "one") == 0, "map_iterator.get_key () == \"one\"");
	_g_free0 (_tmp172_);
	_tmp173_ = map_iterator;
	_tmp174_ = gee_map_iterator_get_value ((GeeMapIterator*) _tmp173_);
	_tmp175_ = (gchar*) _tmp174_;
	_vala_assert (g_strcmp0 (_tmp175_, "one") == 0, "map_iterator.get_value () == \"one\"");
	_g_free0 (_tmp175_);
	_tmp176_ = keys_iterator;
	_vala_assert (gee_bidir_iterator_previous (_tmp176_), "keys_iterator.previous ()");
	_tmp177_ = entries_iterator;
	_vala_assert (gee_bidir_iterator_previous (_tmp177_), "entries_iterator.previous ()");
	_tmp178_ = map_iterator;
	_vala_assert (gee_bidir_map_iterator_previous (_tmp178_), "map_iterator.previous ()");
	_tmp179_ = keys_iterator;
	_tmp180_ = gee_iterator_get ((GeeIterator*) _tmp179_);
	_tmp181_ = (gchar*) _tmp180_;
	_vala_assert (g_strcmp0 (_tmp181_, "four") == 0, "keys_iterator.get () == \"four\"");
	_g_free0 (_tmp181_);
	_tmp182_ = entries_iterator;
	_tmp183_ = gee_iterator_get ((GeeIterator*) _tmp182_);
	_tmp184_ = (GeeMapEntry*) _tmp183_;
	map_tests_assert_entry (_tmp184_, "four", "four");
	_g_object_unref0 (_tmp184_);
	_tmp185_ = map_iterator;
	_tmp186_ = gee_map_iterator_get_key ((GeeMapIterator*) _tmp185_);
	_tmp187_ = (gchar*) _tmp186_;
	_vala_assert (g_strcmp0 (_tmp187_, "four") == 0, "map_iterator.get_key () == \"four\"");
	_g_free0 (_tmp187_);
	_tmp188_ = map_iterator;
	_tmp189_ = gee_map_iterator_get_value ((GeeMapIterator*) _tmp188_);
	_tmp190_ = (gchar*) _tmp189_;
	_vala_assert (g_strcmp0 (_tmp190_, "four") == 0, "map_iterator.get_value () == \"four\"");
	_g_free0 (_tmp190_);
	_tmp191_ = keys_iterator;
	_vala_assert (gee_bidir_iterator_previous (_tmp191_), "keys_iterator.previous ()");
	_tmp192_ = entries_iterator;
	_vala_assert (gee_bidir_iterator_previous (_tmp192_), "entries_iterator.previous ()");
	_tmp193_ = map_iterator;
	_vala_assert (gee_bidir_map_iterator_previous (_tmp193_), "map_iterator.previous ()");
	_tmp194_ = keys_iterator;
	_tmp195_ = gee_iterator_get ((GeeIterator*) _tmp194_);
	_tmp196_ = (gchar*) _tmp195_;
	_vala_assert (g_strcmp0 (_tmp196_, "five") == 0, "keys_iterator.get () == \"five\"");
	_g_free0 (_tmp196_);
	_tmp197_ = entries_iterator;
	_tmp198_ = gee_iterator_get ((GeeIterator*) _tmp197_);
	_tmp199_ = (GeeMapEntry*) _tmp198_;
	map_tests_assert_entry (_tmp199_, "five", "five");
	_g_object_unref0 (_tmp199_);
	_tmp200_ = map_iterator;
	_tmp201_ = gee_map_iterator_get_key ((GeeMapIterator*) _tmp200_);
	_tmp202_ = (gchar*) _tmp201_;
	_vala_assert (g_strcmp0 (_tmp202_, "five") == 0, "map_iterator.get_key () == \"five\"");
	_g_free0 (_tmp202_);
	_tmp203_ = map_iterator;
	_tmp204_ = gee_map_iterator_get_value ((GeeMapIterator*) _tmp203_);
	_tmp205_ = (gchar*) _tmp204_;
	_vala_assert (g_strcmp0 (_tmp205_, "five") == 0, "map_iterator.get_value () == \"five\"");
	_g_free0 (_tmp205_);
	_tmp206_ = keys_iterator;
	_vala_assert (!gee_bidir_iterator_previous (_tmp206_), "!keys_iterator.previous ()");
	_tmp207_ = entries_iterator;
	_vala_assert (!gee_bidir_iterator_previous (_tmp207_), "!entries_iterator.previous ()");
	_tmp208_ = map_iterator;
	_vala_assert (!gee_bidir_map_iterator_previous (_tmp208_), "!map_iterator.previous ()");
	_tmp209_ = keys_iterator;
	_tmp210_ = gee_iterator_get ((GeeIterator*) _tmp209_);
	_tmp211_ = (gchar*) _tmp210_;
	_vala_assert (g_strcmp0 (_tmp211_, "five") == 0, "keys_iterator.get () == \"five\"");
	_g_free0 (_tmp211_);
	_tmp212_ = entries_iterator;
	_tmp213_ = gee_iterator_get ((GeeIterator*) _tmp212_);
	_tmp214_ = (GeeMapEntry*) _tmp213_;
	map_tests_assert_entry (_tmp214_, "five", "five");
	_g_object_unref0 (_tmp214_);
	_tmp215_ = map_iterator;
	_tmp216_ = gee_map_iterator_get_key ((GeeMapIterator*) _tmp215_);
	_tmp217_ = (gchar*) _tmp216_;
	_vala_assert (g_strcmp0 (_tmp217_, "five") == 0, "map_iterator.get_key () == \"five\"");
	_g_free0 (_tmp217_);
	_tmp218_ = map_iterator;
	_tmp219_ = gee_map_iterator_get_value ((GeeMapIterator*) _tmp218_);
	_tmp220_ = (gchar*) _tmp219_;
	_vala_assert (g_strcmp0 (_tmp220_, "five") == 0, "map_iterator.get_value () == \"five\"");
	_g_free0 (_tmp220_);
	_g_object_unref0 (map_iterator);
	_g_object_unref0 (entries_iterator);
	_g_object_unref0 (keys_iterator);
	_g_object_unref0 (entries);
	_g_object_unref0 (keys);
	_g_object_unref0 (test_sorted_map);
}

void
bidir_sorted_map_tests_test_bidir_iterator_last (BidirSortedMapTests* self)
{
	GeeBidirSortedMap* test_sorted_map = NULL;
	GeeMap* _tmp0_;
	GeeBidirSortedMap* _tmp1_;
	GeeBidirSortedSet* keys = NULL;
	GeeSortedSet* _tmp2_;
	GeeSortedSet* _tmp3_;
	GeeBidirSortedSet* _tmp4_;
	GeeBidirSortedSet* entries = NULL;
	GeeSortedSet* _tmp5_;
	GeeSortedSet* _tmp6_;
	GeeBidirSortedSet* _tmp7_;
	GeeBidirIterator* keys_iterator = NULL;
	GeeBidirIterator* _tmp8_;
	GeeBidirIterator* entries_iterator = NULL;
	GeeBidirIterator* _tmp9_;
	GeeBidirIterator* _tmp10_;
	GeeBidirIterator* _tmp11_;
	GeeBidirIterator* _tmp12_;
	GeeBidirIterator* _tmp13_;
	GeeBidirIterator* _tmp14_;
	GeeBidirIterator* _tmp15_;
	GeeBidirIterator* _tmp16_;
	gpointer _tmp17_;
	gchar* _tmp18_;
	GeeBidirIterator* _tmp19_;
	gpointer _tmp20_;
	GeeMapEntry* _tmp21_;
	g_return_if_fail (self != NULL);
	_tmp0_ = ((MapTests*) self)->test_map;
	_tmp1_ = _g_object_ref0 (G_TYPE_CHECK_INSTANCE_TYPE (_tmp0_, GEE_TYPE_BIDIR_SORTED_MAP) ? ((GeeBidirSortedMap*) _tmp0_) : NULL);
	test_sorted_map = _tmp1_;
	_tmp2_ = gee_sorted_map_get_ascending_keys ((GeeSortedMap*) test_sorted_map);
	_tmp3_ = _tmp2_;
	_tmp4_ = G_TYPE_CHECK_INSTANCE_TYPE (_tmp3_, GEE_TYPE_BIDIR_SORTED_SET) ? ((GeeBidirSortedSet*) _tmp3_) : NULL;
	if (_tmp4_ == NULL) {
		_g_object_unref0 (_tmp3_);
	}
	keys = _tmp4_;
	_tmp5_ = gee_sorted_map_get_ascending_entries ((GeeSortedMap*) test_sorted_map);
	_tmp6_ = _tmp5_;
	_tmp7_ = G_TYPE_CHECK_INSTANCE_TYPE (_tmp6_, GEE_TYPE_BIDIR_SORTED_SET) ? ((GeeBidirSortedSet*) _tmp6_) : NULL;
	if (_tmp7_ == NULL) {
		_g_object_unref0 (_tmp6_);
	}
	entries = _tmp7_;
	_tmp8_ = gee_bidir_sorted_set_bidir_iterator (keys);
	keys_iterator = _tmp8_;
	_tmp9_ = gee_bidir_sorted_set_bidir_iterator (entries);
	entries_iterator = _tmp9_;
	_tmp10_ = keys_iterator;
	_vala_assert (!gee_bidir_iterator_last (_tmp10_), "!keys_iterator.last ()");
	_tmp11_ = entries_iterator;
	_vala_assert (!gee_bidir_iterator_last (_tmp11_), "!entries_iterator.last ()");
	gee_map_set ((GeeMap*) test_sorted_map, "one", "one");
	gee_map_set ((GeeMap*) test_sorted_map, "two", "two");
	gee_map_set ((GeeMap*) test_sorted_map, "three", "three");
	gee_map_set ((GeeMap*) test_sorted_map, "four", "four");
	gee_map_set ((GeeMap*) test_sorted_map, "five", "five");
	gee_map_set ((GeeMap*) test_sorted_map, "six", "six");
	_tmp12_ = gee_bidir_sorted_set_bidir_iterator (keys);
	_g_object_unref0 (keys_iterator);
	keys_iterator = _tmp12_;
	_tmp13_ = gee_bidir_sorted_set_bidir_iterator (entries);
	_g_object_unref0 (entries_iterator);
	entries_iterator = _tmp13_;
	_tmp14_ = keys_iterator;
	_vala_assert (gee_bidir_iterator_last (_tmp14_), "keys_iterator.last ()");
	_tmp15_ = entries_iterator;
	_vala_assert (gee_bidir_iterator_last (_tmp15_), "entries_iterator.last ()");
	_tmp16_ = keys_iterator;
	_tmp17_ = gee_iterator_get ((GeeIterator*) _tmp16_);
	_tmp18_ = (gchar*) _tmp17_;
	_vala_assert (g_strcmp0 (_tmp18_, "two") == 0, "keys_iterator.get () == \"two\"");
	_g_free0 (_tmp18_);
	_tmp19_ = entries_iterator;
	_tmp20_ = gee_iterator_get ((GeeIterator*) _tmp19_);
	_tmp21_ = (GeeMapEntry*) _tmp20_;
	map_tests_assert_entry (_tmp21_, "two", "two");
	_g_object_unref0 (_tmp21_);
	_g_object_unref0 (entries_iterator);
	_g_object_unref0 (keys_iterator);
	_g_object_unref0 (entries);
	_g_object_unref0 (keys);
	_g_object_unref0 (test_sorted_map);
}

static inline gpointer
bidir_sorted_map_tests_bidir_sub_map_tests_get_instance_private (BidirSortedMapTestsBidirSubMapTests* self)
{
	return G_STRUCT_MEMBER_P (self, BidirSortedMapTestsBidirSubMapTests_private_offset);
}

static void
_bidir_sorted_map_tests_bidir_sub_map_tests_test_bidir_iterators_gee_test_case_test_method (gpointer self)
{
	bidir_sorted_map_tests_bidir_sub_map_tests_test_bidir_iterators ((BidirSortedMapTestsBidirSubMapTests*) self);
}

BidirSortedMapTestsBidirSubMapTests*
bidir_sorted_map_tests_bidir_sub_map_tests_construct (GType object_type,
                                                      BidirSortedMapTests* test,
                                                      GeeSortedMapTestsSubMapTestsType type)
{
	BidirSortedMapTestsBidirSubMapTests * self = NULL;
	const gchar* _tmp0_;
	gchar* _tmp1_;
	gchar* _tmp2_;
	BidirSortedMapTests* _tmp3_;
	g_return_val_if_fail (test != NULL, NULL);
	_tmp0_ = gee_sorted_map_tests_sub_map_tests_type_to_string (type);
	_tmp1_ = g_strdup_printf ("%s Subset", _tmp0_);
	_tmp2_ = _tmp1_;
	self = (BidirSortedMapTestsBidirSubMapTests*) gee_test_case_construct (object_type, _tmp2_);
	_g_free0 (_tmp2_);
	_tmp3_ = _g_object_ref0 (test);
	_g_object_unref0 (self->priv->test);
	self->priv->test = _tmp3_;
	self->priv->type = type;
	gee_test_case_add_test ((GeeTestCase*) self, "[BidirSortedSet] bi-directional iterator", _bidir_sorted_map_tests_bidir_sub_map_tests_test_bidir_iterators_gee_test_case_test_method, g_object_ref (self), g_object_unref);
	return self;
}

BidirSortedMapTestsBidirSubMapTests*
bidir_sorted_map_tests_bidir_sub_map_tests_new (BidirSortedMapTests* test,
                                                GeeSortedMapTestsSubMapTestsType type)
{
	return bidir_sorted_map_tests_bidir_sub_map_tests_construct (BIDIR_SORTED_MAP_TESTS_TYPE_BIDIR_SUB_MAP_TESTS, test, type);
}

static void
bidir_sorted_map_tests_bidir_sub_map_tests_real_set_up (GeeTestCase* base)
{
	BidirSortedMapTestsBidirSubMapTests * self;
	BidirSortedMapTests* _tmp0_;
	BidirSortedMapTests* _tmp1_;
	GeeMap* _tmp2_;
	GeeBidirSortedMap* _tmp3_;
	self = (BidirSortedMapTestsBidirSubMapTests*) base;
	_tmp0_ = self->priv->test;
	gee_test_case_set_up ((GeeTestCase*) _tmp0_);
	_tmp1_ = self->priv->test;
	_tmp2_ = ((MapTests*) _tmp1_)->test_map;
	_tmp3_ = _g_object_ref0 (G_TYPE_CHECK_INSTANCE_TYPE (_tmp2_, GEE_TYPE_BIDIR_SORTED_MAP) ? ((GeeBidirSortedMap*) _tmp2_) : NULL);
	_g_object_unref0 (self->priv->master);
	self->priv->master = _tmp3_;
	switch (self->priv->type) {
		case GEE_SORTED_MAP_TESTS_SUB_MAP_TESTS_TYPE_HEAD:
		{
			GeeBidirSortedMap* _tmp4_;
			GeeSortedMap* _tmp5_;
			GeeBidirSortedMap* _tmp6_;
			_tmp4_ = self->priv->master;
			_tmp5_ = gee_sorted_map_head_map ((GeeSortedMap*) _tmp4_, "one");
			_tmp6_ = G_TYPE_CHECK_INSTANCE_TYPE (_tmp5_, GEE_TYPE_BIDIR_SORTED_MAP) ? ((GeeBidirSortedMap*) _tmp5_) : NULL;
			if (_tmp6_ == NULL) {
				_g_object_unref0 (_tmp5_);
			}
			_g_object_unref0 (self->priv->submap);
			self->priv->submap = _tmp6_;
			break;
		}
		case GEE_SORTED_MAP_TESTS_SUB_MAP_TESTS_TYPE_TAIL:
		{
			GeeBidirSortedMap* _tmp7_;
			GeeSortedMap* _tmp8_;
			GeeBidirSortedMap* _tmp9_;
			_tmp7_ = self->priv->master;
			_tmp8_ = gee_sorted_map_tail_map ((GeeSortedMap*) _tmp7_, "six");
			_tmp9_ = G_TYPE_CHECK_INSTANCE_TYPE (_tmp8_, GEE_TYPE_BIDIR_SORTED_MAP) ? ((GeeBidirSortedMap*) _tmp8_) : NULL;
			if (_tmp9_ == NULL) {
				_g_object_unref0 (_tmp8_);
			}
			_g_object_unref0 (self->priv->submap);
			self->priv->submap = _tmp9_;
			break;
		}
		case GEE_SORTED_MAP_TESTS_SUB_MAP_TESTS_TYPE_SUB:
		{
			GeeBidirSortedMap* _tmp10_;
			GeeSortedMap* _tmp11_;
			GeeBidirSortedMap* _tmp12_;
			_tmp10_ = self->priv->master;
			_tmp11_ = gee_sorted_map_sub_map ((GeeSortedMap*) _tmp10_, "four", "three");
			_tmp12_ = G_TYPE_CHECK_INSTANCE_TYPE (_tmp11_, GEE_TYPE_BIDIR_SORTED_MAP) ? ((GeeBidirSortedMap*) _tmp11_) : NULL;
			if (_tmp12_ == NULL) {
				_g_object_unref0 (_tmp11_);
			}
			_g_object_unref0 (self->priv->submap);
			self->priv->submap = _tmp12_;
			break;
		}
		case GEE_SORTED_MAP_TESTS_SUB_MAP_TESTS_TYPE_EMPTY:
		{
			GeeBidirSortedMap* _tmp13_;
			GeeSortedMap* _tmp14_;
			GeeBidirSortedMap* _tmp15_;
			_tmp13_ = self->priv->master;
			_tmp14_ = gee_sorted_map_sub_map ((GeeSortedMap*) _tmp13_, "three", "four");
			_tmp15_ = G_TYPE_CHECK_INSTANCE_TYPE (_tmp14_, GEE_TYPE_BIDIR_SORTED_MAP) ? ((GeeBidirSortedMap*) _tmp14_) : NULL;
			if (_tmp15_ == NULL) {
				_g_object_unref0 (_tmp14_);
			}
			_g_object_unref0 (self->priv->submap);
			self->priv->submap = _tmp15_;
			break;
		}
		default:
		{
			g_assert_not_reached ();
		}
	}
}

static void
bidir_sorted_map_tests_bidir_sub_map_tests_real_tear_down (GeeTestCase* base)
{
	BidirSortedMapTestsBidirSubMapTests * self;
	BidirSortedMapTests* _tmp0_;
	self = (BidirSortedMapTestsBidirSubMapTests*) base;
	_tmp0_ = self->priv->test;
	gee_test_case_tear_down ((GeeTestCase*) _tmp0_);
}

void
bidir_sorted_map_tests_bidir_sub_map_tests_set_default_values (BidirSortedMapTestsBidirSubMapTests* self,
                                                               gchar*** contains,
                                                               gint* contains_length1,
                                                               gchar*** not_contains,
                                                               gint* not_contains_length1)
{
	gchar** _vala_contains = NULL;
	gint _vala_contains_length1 = 0;
	gchar** _vala_not_contains = NULL;
	gint _vala_not_contains_length1 = 0;
	GeeBidirSortedMap* _tmp0_;
	GeeBidirSortedMap* _tmp1_;
	GeeBidirSortedMap* _tmp2_;
	GeeBidirSortedMap* _tmp3_;
	GeeBidirSortedMap* _tmp4_;
	GeeBidirSortedMap* _tmp5_;
	g_return_if_fail (self != NULL);
	_tmp0_ = self->priv->master;
	gee_map_set ((GeeMap*) _tmp0_, "one", "one");
	_tmp1_ = self->priv->master;
	gee_map_set ((GeeMap*) _tmp1_, "two", "two");
	_tmp2_ = self->priv->master;
	gee_map_set ((GeeMap*) _tmp2_, "three", "three");
	_tmp3_ = self->priv->master;
	gee_map_set ((GeeMap*) _tmp3_, "four", "four");
	_tmp4_ = self->priv->master;
	gee_map_set ((GeeMap*) _tmp4_, "five", "five");
	_tmp5_ = self->priv->master;
	gee_map_set ((GeeMap*) _tmp5_, "six", "six");
	switch (self->priv->type) {
		case GEE_SORTED_MAP_TESTS_SUB_MAP_TESTS_TYPE_HEAD:
		{
			gchar* _tmp6_;
			gchar* _tmp7_;
			gchar** _tmp8_;
			gchar* _tmp9_;
			gchar* _tmp10_;
			gchar* _tmp11_;
			gchar* _tmp12_;
			gchar** _tmp13_;
			_tmp6_ = g_strdup ("five");
			_tmp7_ = g_strdup ("four");
			_tmp8_ = g_new0 (gchar*, 2 + 1);
			_tmp8_[0] = _tmp6_;
			_tmp8_[1] = _tmp7_;
			_vala_contains = (_vala_array_free (_vala_contains, _vala_contains_length1, (GDestroyNotify) g_free), NULL);
			_vala_contains = _tmp8_;
			_vala_contains_length1 = 2;
			_tmp9_ = g_strdup ("one");
			_tmp10_ = g_strdup ("two");
			_tmp11_ = g_strdup ("three");
			_tmp12_ = g_strdup ("six");
			_tmp13_ = g_new0 (gchar*, 4 + 1);
			_tmp13_[0] = _tmp9_;
			_tmp13_[1] = _tmp10_;
			_tmp13_[2] = _tmp11_;
			_tmp13_[3] = _tmp12_;
			_vala_not_contains = (_vala_array_free (_vala_not_contains, _vala_not_contains_length1, (GDestroyNotify) g_free), NULL);
			_vala_not_contains = _tmp13_;
			_vala_not_contains_length1 = 4;
			break;
		}
		case GEE_SORTED_MAP_TESTS_SUB_MAP_TESTS_TYPE_TAIL:
		{
			gchar* _tmp14_;
			gchar* _tmp15_;
			gchar* _tmp16_;
			gchar** _tmp17_;
			gchar* _tmp18_;
			gchar* _tmp19_;
			gchar* _tmp20_;
			gchar** _tmp21_;
			_tmp14_ = g_strdup ("six");
			_tmp15_ = g_strdup ("three");
			_tmp16_ = g_strdup ("two");
			_tmp17_ = g_new0 (gchar*, 3 + 1);
			_tmp17_[0] = _tmp14_;
			_tmp17_[1] = _tmp15_;
			_tmp17_[2] = _tmp16_;
			_vala_contains = (_vala_array_free (_vala_contains, _vala_contains_length1, (GDestroyNotify) g_free), NULL);
			_vala_contains = _tmp17_;
			_vala_contains_length1 = 3;
			_tmp18_ = g_strdup ("one");
			_tmp19_ = g_strdup ("four");
			_tmp20_ = g_strdup ("five");
			_tmp21_ = g_new0 (gchar*, 3 + 1);
			_tmp21_[0] = _tmp18_;
			_tmp21_[1] = _tmp19_;
			_tmp21_[2] = _tmp20_;
			_vala_not_contains = (_vala_array_free (_vala_not_contains, _vala_not_contains_length1, (GDestroyNotify) g_free), NULL);
			_vala_not_contains = _tmp21_;
			_vala_not_contains_length1 = 3;
			break;
		}
		case GEE_SORTED_MAP_TESTS_SUB_MAP_TESTS_TYPE_SUB:
		{
			gchar* _tmp22_;
			gchar* _tmp23_;
			gchar* _tmp24_;
			gchar** _tmp25_;
			gchar* _tmp26_;
			gchar* _tmp27_;
			gchar* _tmp28_;
			gchar** _tmp29_;
			_tmp22_ = g_strdup ("four");
			_tmp23_ = g_strdup ("one");
			_tmp24_ = g_strdup ("six");
			_tmp25_ = g_new0 (gchar*, 3 + 1);
			_tmp25_[0] = _tmp22_;
			_tmp25_[1] = _tmp23_;
			_tmp25_[2] = _tmp24_;
			_vala_contains = (_vala_array_free (_vala_contains, _vala_contains_length1, (GDestroyNotify) g_free), NULL);
			_vala_contains = _tmp25_;
			_vala_contains_length1 = 3;
			_tmp26_ = g_strdup ("two");
			_tmp27_ = g_strdup ("three");
			_tmp28_ = g_strdup ("five");
			_tmp29_ = g_new0 (gchar*, 3 + 1);
			_tmp29_[0] = _tmp26_;
			_tmp29_[1] = _tmp27_;
			_tmp29_[2] = _tmp28_;
			_vala_not_contains = (_vala_array_free (_vala_not_contains, _vala_not_contains_length1, (GDestroyNotify) g_free), NULL);
			_vala_not_contains = _tmp29_;
			_vala_not_contains_length1 = 3;
			break;
		}
		case GEE_SORTED_MAP_TESTS_SUB_MAP_TESTS_TYPE_EMPTY:
		{
			gchar** _tmp30_;
			gchar* _tmp31_;
			gchar* _tmp32_;
			gchar* _tmp33_;
			gchar* _tmp34_;
			gchar* _tmp35_;
			gchar* _tmp36_;
			gchar** _tmp37_;
			_tmp30_ = g_new0 (gchar*, 0 + 1);
			_vala_contains = (_vala_array_free (_vala_contains, _vala_contains_length1, (GDestroyNotify) g_free), NULL);
			_vala_contains = _tmp30_;
			_vala_contains_length1 = 0;
			_tmp31_ = g_strdup ("one");
			_tmp32_ = g_strdup ("two");
			_tmp33_ = g_strdup ("three");
			_tmp34_ = g_strdup ("four");
			_tmp35_ = g_strdup ("five");
			_tmp36_ = g_strdup ("six");
			_tmp37_ = g_new0 (gchar*, 6 + 1);
			_tmp37_[0] = _tmp31_;
			_tmp37_[1] = _tmp32_;
			_tmp37_[2] = _tmp33_;
			_tmp37_[3] = _tmp34_;
			_tmp37_[4] = _tmp35_;
			_tmp37_[5] = _tmp36_;
			_vala_not_contains = (_vala_array_free (_vala_not_contains, _vala_not_contains_length1, (GDestroyNotify) g_free), NULL);
			_vala_not_contains = _tmp37_;
			_vala_not_contains_length1 = 6;
			break;
		}
		default:
		{
			g_assert_not_reached ();
		}
	}
	if (contains) {
		*contains = _vala_contains;
	} else {
		_vala_contains = (_vala_array_free (_vala_contains, _vala_contains_length1, (GDestroyNotify) g_free), NULL);
	}
	if (contains_length1) {
		*contains_length1 = _vala_contains_length1;
	}
	if (not_contains) {
		*not_contains = _vala_not_contains;
	} else {
		_vala_not_contains = (_vala_array_free (_vala_not_contains, _vala_not_contains_length1, (GDestroyNotify) g_free), NULL);
	}
	if (not_contains_length1) {
		*not_contains_length1 = _vala_not_contains_length1;
	}
}

void
bidir_sorted_map_tests_bidir_sub_map_tests_test_bidir_iterators (BidirSortedMapTestsBidirSubMapTests* self)
{
	gchar** contains = NULL;
	gint contains_length1 = 0;
	gint _contains_size_ = 0;
	gchar** not_contains = NULL;
	gint not_contains_length1 = 0;
	gint _not_contains_size_ = 0;
	GeeBidirMapIterator* _map_iter = NULL;
	GeeBidirSortedMap* _tmp0_;
	GeeBidirMapIterator* _tmp1_;
	GeeBidirMapIterator* _tmp2_;
	GeeBidirMapIterator* _tmp3_;
	GeeBidirMapIterator* _tmp4_;
	GeeBidirMapIterator* _tmp5_;
	gchar** _tmp6_ = NULL;
	gint _tmp7_ = 0;
	gchar** _tmp8_ = NULL;
	gint _tmp9_ = 0;
	gint i = 0;
	GeeBidirSortedMap* _tmp10_;
	GeeBidirMapIterator* _tmp11_;
	gchar** _tmp24_;
	gint _tmp24__length1;
	gchar** _tmp38_;
	gint _tmp38__length1;
	gchar** _tmp54_;
	gint _tmp54__length1;
	GeeBidirIterator* keys_iter = NULL;
	GeeBidirSortedMap* _tmp55_;
	GeeSortedSet* _tmp56_;
	GeeSortedSet* _tmp57_;
	GeeBidirSortedSet* _tmp58_;
	GeeBidirIterator* _tmp59_;
	GeeBidirIterator* _tmp60_;
	GeeBidirIterator* entries_iter = NULL;
	GeeBidirSortedMap* _tmp61_;
	GeeSortedSet* _tmp62_;
	GeeSortedSet* _tmp63_;
	GeeBidirSortedSet* _tmp64_;
	GeeBidirIterator* _tmp65_;
	GeeBidirIterator* _tmp66_;
	GeeBidirMapIterator* map_iter = NULL;
	GeeBidirSortedMap* _tmp67_;
	GeeBidirMapIterator* _tmp68_;
	g_return_if_fail (self != NULL);
	_tmp0_ = self->priv->submap;
	_tmp1_ = gee_bidir_sorted_map_bidir_map_iterator (_tmp0_);
	_map_iter = _tmp1_;
	_tmp2_ = _map_iter;
	_vala_assert (!gee_map_iterator_has_next ((GeeMapIterator*) _tmp2_), "!_map_iter.has_next ()");
	_tmp3_ = _map_iter;
	_vala_assert (!gee_map_iterator_next ((GeeMapIterator*) _tmp3_), "!_map_iter.next ()");
	_tmp4_ = _map_iter;
	_vala_assert (!gee_bidir_map_iterator_has_previous (_tmp4_), "!_map_iter.has_previous ()");
	_tmp5_ = _map_iter;
	_vala_assert (!gee_bidir_map_iterator_previous (_tmp5_), "!_map_iter.previous ()");
	bidir_sorted_map_tests_bidir_sub_map_tests_set_default_values (self, &_tmp6_, &_tmp7_, &_tmp8_, &_tmp9_);
	contains = (_vala_array_free (contains, contains_length1, (GDestroyNotify) g_free), NULL);
	contains = _tmp6_;
	contains_length1 = _tmp7_;
	_contains_size_ = contains_length1;
	not_contains = (_vala_array_free (not_contains, not_contains_length1, (GDestroyNotify) g_free), NULL);
	not_contains = _tmp8_;
	not_contains_length1 = _tmp9_;
	_not_contains_size_ = not_contains_length1;
	i = 0;
	_tmp10_ = self->priv->submap;
	_tmp11_ = gee_bidir_sorted_map_bidir_map_iterator (_tmp10_);
	_g_object_unref0 (_map_iter);
	_map_iter = _tmp11_;
	while (TRUE) {
		GeeBidirMapIterator* _tmp12_;
		GeeBidirMapIterator* _tmp13_;
		gpointer _tmp14_;
		gchar* _tmp15_;
		gchar** _tmp16_;
		gint _tmp16__length1;
		const gchar* _tmp17_;
		GeeBidirMapIterator* _tmp18_;
		gpointer _tmp19_;
		gchar* _tmp20_;
		gchar** _tmp21_;
		gint _tmp21__length1;
		const gchar* _tmp22_;
		gint _tmp23_;
		_tmp12_ = _map_iter;
		if (!gee_map_iterator_next ((GeeMapIterator*) _tmp12_)) {
			break;
		}
		_tmp13_ = _map_iter;
		_tmp14_ = gee_map_iterator_get_key ((GeeMapIterator*) _tmp13_);
		_tmp15_ = (gchar*) _tmp14_;
		_tmp16_ = contains;
		_tmp16__length1 = contains_length1;
		_tmp17_ = _tmp16_[i];
		_vala_assert (g_strcmp0 (_tmp15_, _tmp17_) == 0, "_map_iter.get_key () == contains[i]");
		_g_free0 (_tmp15_);
		_tmp18_ = _map_iter;
		_tmp19_ = gee_map_iterator_get_value ((GeeMapIterator*) _tmp18_);
		_tmp20_ = (gchar*) _tmp19_;
		_tmp21_ = contains;
		_tmp21__length1 = contains_length1;
		_tmp22_ = _tmp21_[i];
		_vala_assert (g_strcmp0 (_tmp20_, _tmp22_) == 0, "_map_iter.get_value () == contains[i]");
		_g_free0 (_tmp20_);
		_tmp23_ = i;
		i = _tmp23_ + 1;
	}
	_tmp24_ = contains;
	_tmp24__length1 = contains_length1;
	_vala_assert (i == _tmp24__length1, "i == contains.length");
	i = 0;
	{
		GeeIterator* _k_it = NULL;
		GeeBidirSortedMap* _tmp25_;
		GeeSet* _tmp26_;
		GeeSet* _tmp27_;
		GeeSet* _tmp28_;
		GeeIterator* _tmp29_;
		GeeIterator* _tmp30_;
		_tmp25_ = self->priv->submap;
		_tmp26_ = gee_map_get_keys ((GeeMap*) _tmp25_);
		_tmp27_ = _tmp26_;
		_tmp28_ = _tmp27_;
		_tmp29_ = gee_iterable_iterator ((GeeIterable*) _tmp28_);
		_tmp30_ = _tmp29_;
		_g_object_unref0 (_tmp28_);
		_k_it = _tmp30_;
		while (TRUE) {
			GeeIterator* _tmp31_;
			gchar* k = NULL;
			GeeIterator* _tmp32_;
			gpointer _tmp33_;
			const gchar* _tmp34_;
			gchar** _tmp35_;
			gint _tmp35__length1;
			gint _tmp36_;
			const gchar* _tmp37_;
			_tmp31_ = _k_it;
			if (!gee_iterator_next (_tmp31_)) {
				break;
			}
			_tmp32_ = _k_it;
			_tmp33_ = gee_iterator_get (_tmp32_);
			k = (gchar*) _tmp33_;
			_tmp34_ = k;
			_tmp35_ = contains;
			_tmp35__length1 = contains_length1;
			_tmp36_ = i;
			i = _tmp36_ + 1;
			_tmp37_ = _tmp35_[_tmp36_];
			_vala_assert (g_strcmp0 (_tmp34_, _tmp37_) == 0, "k == contains[i++]");
			_g_free0 (k);
		}
		_g_object_unref0 (_k_it);
	}
	_tmp38_ = contains;
	_tmp38__length1 = contains_length1;
	_vala_assert (i == _tmp38__length1, "i == contains.length");
	i = 0;
	{
		GeeIterator* _e_it = NULL;
		GeeBidirSortedMap* _tmp39_;
		GeeSet* _tmp40_;
		GeeSet* _tmp41_;
		GeeSet* _tmp42_;
		GeeIterator* _tmp43_;
		GeeIterator* _tmp44_;
		_tmp39_ = self->priv->submap;
		_tmp40_ = gee_map_get_entries ((GeeMap*) _tmp39_);
		_tmp41_ = _tmp40_;
		_tmp42_ = _tmp41_;
		_tmp43_ = gee_iterable_iterator ((GeeIterable*) _tmp42_);
		_tmp44_ = _tmp43_;
		_g_object_unref0 (_tmp42_);
		_e_it = _tmp44_;
		while (TRUE) {
			GeeIterator* _tmp45_;
			GeeMapEntry* e = NULL;
			GeeIterator* _tmp46_;
			gpointer _tmp47_;
			GeeMapEntry* _tmp48_;
			gchar** _tmp49_;
			gint _tmp49__length1;
			const gchar* _tmp50_;
			gchar** _tmp51_;
			gint _tmp51__length1;
			const gchar* _tmp52_;
			gint _tmp53_;
			_tmp45_ = _e_it;
			if (!gee_iterator_next (_tmp45_)) {
				break;
			}
			_tmp46_ = _e_it;
			_tmp47_ = gee_iterator_get (_tmp46_);
			e = (GeeMapEntry*) _tmp47_;
			_tmp48_ = e;
			_tmp49_ = contains;
			_tmp49__length1 = contains_length1;
			_tmp50_ = _tmp49_[i];
			_tmp51_ = contains;
			_tmp51__length1 = contains_length1;
			_tmp52_ = _tmp51_[i];
			map_tests_assert_entry (_tmp48_, _tmp50_, _tmp52_);
			_tmp53_ = i;
			i = _tmp53_ + 1;
			_g_object_unref0 (e);
		}
		_g_object_unref0 (_e_it);
	}
	_tmp54_ = contains;
	_tmp54__length1 = contains_length1;
	_vala_assert (i == _tmp54__length1, "i == contains.length");
	_tmp55_ = self->priv->submap;
	_tmp56_ = gee_sorted_map_get_ascending_keys ((GeeSortedMap*) _tmp55_);
	_tmp57_ = _tmp56_;
	_tmp58_ = G_TYPE_CHECK_INSTANCE_CAST (_tmp57_, GEE_TYPE_BIDIR_SORTED_SET, GeeBidirSortedSet);
	_tmp59_ = gee_bidir_sorted_set_bidir_iterator (_tmp58_);
	_tmp60_ = _tmp59_;
	_g_object_unref0 (_tmp58_);
	keys_iter = _tmp60_;
	_tmp61_ = self->priv->submap;
	_tmp62_ = gee_sorted_map_get_ascending_entries ((GeeSortedMap*) _tmp61_);
	_tmp63_ = _tmp62_;
	_tmp64_ = G_TYPE_CHECK_INSTANCE_CAST (_tmp63_, GEE_TYPE_BIDIR_SORTED_SET, GeeBidirSortedSet);
	_tmp65_ = gee_bidir_sorted_set_bidir_iterator (_tmp64_);
	_tmp66_ = _tmp65_;
	_g_object_unref0 (_tmp64_);
	entries_iter = _tmp66_;
	_tmp67_ = self->priv->submap;
	_tmp68_ = gee_bidir_sorted_map_bidir_map_iterator (_tmp67_);
	map_iter = _tmp68_;
	if (self->priv->type != GEE_SORTED_MAP_TESTS_SUB_MAP_TESTS_TYPE_EMPTY) {
		GeeBidirMapIterator* _tmp69_;
		GeeBidirMapIterator* _tmp70_;
		gpointer _tmp71_;
		gchar* _tmp72_;
		gchar** _tmp73_;
		gint _tmp73__length1;
		gchar** _tmp74_;
		gint _tmp74__length1;
		const gchar* _tmp75_;
		GeeBidirMapIterator* _tmp76_;
		gpointer _tmp77_;
		gchar* _tmp78_;
		gchar** _tmp79_;
		gint _tmp79__length1;
		gchar** _tmp80_;
		gint _tmp80__length1;
		const gchar* _tmp81_;
		GeeBidirSortedMap* _tmp82_;
		GeeBidirMapIterator* _tmp83_;
		GeeBidirMapIterator* _tmp84_;
		GeeBidirMapIterator* _tmp85_;
		gpointer _tmp86_;
		gchar* _tmp87_;
		gchar** _tmp88_;
		gint _tmp88__length1;
		const gchar* _tmp89_;
		GeeBidirMapIterator* _tmp90_;
		gpointer _tmp91_;
		gchar* _tmp92_;
		gchar** _tmp93_;
		gint _tmp93__length1;
		const gchar* _tmp94_;
		GeeBidirMapIterator* _tmp95_;
		GeeBidirMapIterator* _tmp96_;
		GeeBidirMapIterator* _tmp97_;
		gpointer _tmp98_;
		gchar* _tmp99_;
		gchar** _tmp100_;
		gint _tmp100__length1;
		const gchar* _tmp101_;
		GeeBidirMapIterator* _tmp102_;
		gpointer _tmp103_;
		gchar* _tmp104_;
		gchar** _tmp105_;
		gint _tmp105__length1;
		const gchar* _tmp106_;
		GeeBidirMapIterator* _tmp107_;
		GeeBidirMapIterator* _tmp108_;
		GeeBidirMapIterator* _tmp109_;
		GeeBidirMapIterator* _tmp112_;
		GeeBidirMapIterator* _tmp113_;
		gpointer _tmp114_;
		gchar* _tmp115_;
		gchar** _tmp116_;
		gint _tmp116__length1;
		const gchar* _tmp117_;
		GeeBidirMapIterator* _tmp118_;
		gpointer _tmp119_;
		gchar* _tmp120_;
		gchar** _tmp121_;
		gint _tmp121__length1;
		const gchar* _tmp122_;
		GeeBidirSortedMap* _tmp123_;
		gchar** _tmp124_ = NULL;
		gint _tmp125_ = 0;
		gchar** _tmp126_ = NULL;
		gint _tmp127_ = 0;
		GeeBidirIterator* _tmp128_;
		GeeBidirIterator* _tmp129_;
		gpointer _tmp130_;
		gchar* _tmp131_;
		gchar** _tmp132_;
		gint _tmp132__length1;
		gchar** _tmp133_;
		gint _tmp133__length1;
		const gchar* _tmp134_;
		GeeBidirIterator* _tmp135_;
		GeeBidirIterator* _tmp136_;
		gpointer _tmp137_;
		gchar* _tmp138_;
		gchar** _tmp139_;
		gint _tmp139__length1;
		const gchar* _tmp140_;
		GeeBidirIterator* _tmp141_;
		GeeBidirIterator* _tmp142_;
		GeeBidirIterator* _tmp143_;
		gpointer _tmp144_;
		gchar* _tmp145_;
		gchar** _tmp146_;
		gint _tmp146__length1;
		const gchar* _tmp147_;
		GeeBidirIterator* _tmp148_;
		GeeBidirIterator* _tmp150_;
		GeeBidirIterator* _tmp153_;
		GeeBidirIterator* _tmp154_;
		gpointer _tmp155_;
		gchar* _tmp156_;
		gchar** _tmp157_;
		gint _tmp157__length1;
		const gchar* _tmp158_;
		GeeBidirSortedMap* _tmp159_;
		gchar** _tmp160_ = NULL;
		gint _tmp161_ = 0;
		gchar** _tmp162_ = NULL;
		gint _tmp163_ = 0;
		GeeBidirIterator* _tmp164_;
		GeeBidirIterator* _tmp165_;
		gpointer _tmp166_;
		GeeMapEntry* _tmp167_;
		gchar** _tmp168_;
		gint _tmp168__length1;
		gchar** _tmp169_;
		gint _tmp169__length1;
		const gchar* _tmp170_;
		gchar** _tmp171_;
		gint _tmp171__length1;
		gchar** _tmp172_;
		gint _tmp172__length1;
		const gchar* _tmp173_;
		GeeBidirIterator* _tmp174_;
		GeeBidirIterator* _tmp175_;
		gpointer _tmp176_;
		GeeMapEntry* _tmp177_;
		gchar** _tmp178_;
		gint _tmp178__length1;
		const gchar* _tmp179_;
		gchar** _tmp180_;
		gint _tmp180__length1;
		const gchar* _tmp181_;
		GeeBidirIterator* _tmp182_;
		GeeBidirIterator* _tmp183_;
		GeeBidirIterator* _tmp184_;
		gpointer _tmp185_;
		GeeMapEntry* _tmp186_;
		gchar** _tmp187_;
		gint _tmp187__length1;
		const gchar* _tmp188_;
		gchar** _tmp189_;
		gint _tmp189__length1;
		const gchar* _tmp190_;
		GeeBidirIterator* _tmp191_;
		GeeBidirIterator* _tmp192_;
		GeeBidirIterator* _tmp193_;
		GeeBidirIterator* _tmp196_;
		GeeBidirIterator* _tmp197_;
		gpointer _tmp198_;
		GeeMapEntry* _tmp199_;
		gchar** _tmp200_;
		gint _tmp200__length1;
		const gchar* _tmp201_;
		gchar** _tmp202_;
		gint _tmp202__length1;
		const gchar* _tmp203_;
		_tmp69_ = map_iter;
		_vala_assert (gee_bidir_map_iterator_last (_tmp69_), "map_iter.last ()");
		_tmp70_ = map_iter;
		_tmp71_ = gee_map_iterator_get_key ((GeeMapIterator*) _tmp70_);
		_tmp72_ = (gchar*) _tmp71_;
		_tmp73_ = contains;
		_tmp73__length1 = contains_length1;
		_tmp74_ = contains;
		_tmp74__length1 = contains_length1;
		_tmp75_ = _tmp73_[_tmp74__length1 - 1];
		_vala_assert (g_strcmp0 (_tmp72_, _tmp75_) == 0, "map_iter.get_key () == contains[contains.length - 1]");
		_g_free0 (_tmp72_);
		_tmp76_ = map_iter;
		_tmp77_ = gee_map_iterator_get_value ((GeeMapIterator*) _tmp76_);
		_tmp78_ = (gchar*) _tmp77_;
		_tmp79_ = contains;
		_tmp79__length1 = contains_length1;
		_tmp80_ = contains;
		_tmp80__length1 = contains_length1;
		_tmp81_ = _tmp79_[_tmp80__length1 - 1];
		_vala_assert (g_strcmp0 (_tmp78_, _tmp81_) == 0, "map_iter.get_value () == contains[contains.length - 1]");
		_g_free0 (_tmp78_);
		_tmp82_ = self->priv->submap;
		_tmp83_ = gee_bidir_sorted_map_bidir_map_iterator (_tmp82_);
		_g_object_unref0 (map_iter);
		map_iter = _tmp83_;
		_tmp84_ = map_iter;
		_vala_assert (gee_map_iterator_next ((GeeMapIterator*) _tmp84_), "map_iter.next ()");
		_tmp85_ = map_iter;
		_tmp86_ = gee_map_iterator_get_key ((GeeMapIterator*) _tmp85_);
		_tmp87_ = (gchar*) _tmp86_;
		_tmp88_ = contains;
		_tmp88__length1 = contains_length1;
		_tmp89_ = _tmp88_[0];
		_vala_assert (g_strcmp0 (_tmp87_, _tmp89_) == 0, "map_iter.get_key () == contains[0]");
		_g_free0 (_tmp87_);
		_tmp90_ = map_iter;
		_tmp91_ = gee_map_iterator_get_value ((GeeMapIterator*) _tmp90_);
		_tmp92_ = (gchar*) _tmp91_;
		_tmp93_ = contains;
		_tmp93__length1 = contains_length1;
		_tmp94_ = _tmp93_[0];
		_vala_assert (g_strcmp0 (_tmp92_, _tmp94_) == 0, "map_iter.get_value () == contains[0]");
		_g_free0 (_tmp92_);
		_tmp95_ = map_iter;
		_vala_assert (gee_map_iterator_has_next ((GeeMapIterator*) _tmp95_), "map_iter.has_next ()");
		_tmp96_ = map_iter;
		_vala_assert (gee_map_iterator_next ((GeeMapIterator*) _tmp96_), "map_iter.next ()");
		_tmp97_ = map_iter;
		_tmp98_ = gee_map_iterator_get_key ((GeeMapIterator*) _tmp97_);
		_tmp99_ = (gchar*) _tmp98_;
		_tmp100_ = contains;
		_tmp100__length1 = contains_length1;
		_tmp101_ = _tmp100_[1];
		_vala_assert (g_strcmp0 (_tmp99_, _tmp101_) == 0, "map_iter.get_key () == contains[1]");
		_g_free0 (_tmp99_);
		_tmp102_ = map_iter;
		_tmp103_ = gee_map_iterator_get_value ((GeeMapIterator*) _tmp102_);
		_tmp104_ = (gchar*) _tmp103_;
		_tmp105_ = contains;
		_tmp105__length1 = contains_length1;
		_tmp106_ = _tmp105_[1];
		_vala_assert (g_strcmp0 (_tmp104_, _tmp106_) == 0, "map_iter.get_value () == contains[1]");
		_g_free0 (_tmp104_);
		_tmp107_ = map_iter;
		_vala_assert (gee_bidir_map_iterator_has_previous (_tmp107_), "map_iter.has_previous ()");
		_tmp108_ = map_iter;
		gee_map_iterator_unset ((GeeMapIterator*) _tmp108_);
		_tmp109_ = map_iter;
		_vala_assert (gee_bidir_map_iterator_has_previous (_tmp109_), "map_iter.has_previous ()");
		if (self->priv->type != GEE_SORTED_MAP_TESTS_SUB_MAP_TESTS_TYPE_HEAD) {
			GeeBidirMapIterator* _tmp110_;
			_tmp110_ = map_iter;
			_vala_assert (gee_map_iterator_has_next ((GeeMapIterator*) _tmp110_), "map_iter.has_next ()");
		} else {
			GeeBidirMapIterator* _tmp111_;
			_tmp111_ = map_iter;
			_vala_assert (!gee_map_iterator_has_next ((GeeMapIterator*) _tmp111_), "!map_iter.has_next ()");
		}
		_tmp112_ = map_iter;
		_vala_assert (gee_bidir_map_iterator_previous (_tmp112_), "map_iter.previous ()");
		_tmp113_ = map_iter;
		_tmp114_ = gee_map_iterator_get_key ((GeeMapIterator*) _tmp113_);
		_tmp115_ = (gchar*) _tmp114_;
		_tmp116_ = contains;
		_tmp116__length1 = contains_length1;
		_tmp117_ = _tmp116_[0];
		_vala_assert (g_strcmp0 (_tmp115_, _tmp117_) == 0, "map_iter.get_key () == contains[0]");
		_g_free0 (_tmp115_);
		_tmp118_ = map_iter;
		_tmp119_ = gee_map_iterator_get_value ((GeeMapIterator*) _tmp118_);
		_tmp120_ = (gchar*) _tmp119_;
		_tmp121_ = contains;
		_tmp121__length1 = contains_length1;
		_tmp122_ = _tmp121_[0];
		_vala_assert (g_strcmp0 (_tmp120_, _tmp122_) == 0, "map_iter.get_value () == contains[0]");
		_g_free0 (_tmp120_);
		_tmp123_ = self->priv->master;
		gee_map_clear ((GeeMap*) _tmp123_);
		bidir_sorted_map_tests_bidir_sub_map_tests_set_default_values (self, &_tmp124_, &_tmp125_, &_tmp126_, &_tmp127_);
		contains = (_vala_array_free (contains, contains_length1, (GDestroyNotify) g_free), NULL);
		contains = _tmp124_;
		contains_length1 = _tmp125_;
		_contains_size_ = contains_length1;
		not_contains = (_vala_array_free (not_contains, not_contains_length1, (GDestroyNotify) g_free), NULL);
		not_contains = _tmp126_;
		not_contains_length1 = _tmp127_;
		_not_contains_size_ = not_contains_length1;
		_tmp128_ = keys_iter;
		_vala_assert (gee_bidir_iterator_last (_tmp128_), "keys_iter.last ()");
		_tmp129_ = keys_iter;
		_tmp130_ = gee_iterator_get ((GeeIterator*) _tmp129_);
		_tmp131_ = (gchar*) _tmp130_;
		_tmp132_ = contains;
		_tmp132__length1 = contains_length1;
		_tmp133_ = contains;
		_tmp133__length1 = contains_length1;
		_tmp134_ = _tmp132_[_tmp133__length1 - 1];
		_vala_assert (g_strcmp0 (_tmp131_, _tmp134_) == 0, "keys_iter.get () == contains[contains.length - 1]");
		_g_free0 (_tmp131_);
		_tmp135_ = keys_iter;
		_vala_assert (gee_bidir_iterator_first (_tmp135_), "keys_iter.first ()");
		_tmp136_ = keys_iter;
		_tmp137_ = gee_iterator_get ((GeeIterator*) _tmp136_);
		_tmp138_ = (gchar*) _tmp137_;
		_tmp139_ = contains;
		_tmp139__length1 = contains_length1;
		_tmp140_ = _tmp139_[0];
		_vala_assert (g_strcmp0 (_tmp138_, _tmp140_) == 0, "keys_iter.get () == contains[0]");
		_g_free0 (_tmp138_);
		_tmp141_ = keys_iter;
		_vala_assert (gee_iterator_has_next ((GeeIterator*) _tmp141_), "keys_iter.has_next ()");
		_tmp142_ = keys_iter;
		_vala_assert (gee_iterator_next ((GeeIterator*) _tmp142_), "keys_iter.next ()");
		_tmp143_ = keys_iter;
		_tmp144_ = gee_iterator_get ((GeeIterator*) _tmp143_);
		_tmp145_ = (gchar*) _tmp144_;
		_tmp146_ = contains;
		_tmp146__length1 = contains_length1;
		_tmp147_ = _tmp146_[1];
		_vala_assert (g_strcmp0 (_tmp145_, _tmp147_) == 0, "keys_iter.get () == contains[1]");
		_g_free0 (_tmp145_);
		_tmp148_ = keys_iter;
		_vala_assert (gee_bidir_iterator_has_previous (_tmp148_), "keys_iter.has_previous ()");
		if (g_test_trap_fork ((guint64) 0, G_TEST_TRAP_SILENCE_STDOUT | G_TEST_TRAP_SILENCE_STDERR)) {
			GeeBidirIterator* _tmp149_;
			_tmp149_ = keys_iter;
			gee_iterator_remove ((GeeIterator*) _tmp149_);
			exit (0);
		}
		_tmp150_ = keys_iter;
		_vala_assert (gee_bidir_iterator_has_previous (_tmp150_), "keys_iter.has_previous ()");
		if (self->priv->type != GEE_SORTED_MAP_TESTS_SUB_MAP_TESTS_TYPE_HEAD) {
			GeeBidirIterator* _tmp151_;
			_tmp151_ = keys_iter;
			_vala_assert (gee_iterator_has_next ((GeeIterator*) _tmp151_), "keys_iter.has_next ()");
		} else {
			GeeBidirIterator* _tmp152_;
			_tmp152_ = keys_iter;
			_vala_assert (!gee_iterator_has_next ((GeeIterator*) _tmp152_), "!keys_iter.has_next ()");
		}
		_tmp153_ = keys_iter;
		_vala_assert (gee_bidir_iterator_previous (_tmp153_), "keys_iter.previous ()");
		_tmp154_ = keys_iter;
		_tmp155_ = gee_iterator_get ((GeeIterator*) _tmp154_);
		_tmp156_ = (gchar*) _tmp155_;
		_tmp157_ = contains;
		_tmp157__length1 = contains_length1;
		_tmp158_ = _tmp157_[0];
		_vala_assert (g_strcmp0 (_tmp156_, _tmp158_) == 0, "keys_iter.get () == contains[0]");
		_g_free0 (_tmp156_);
		_tmp159_ = self->priv->master;
		gee_map_clear ((GeeMap*) _tmp159_);
		bidir_sorted_map_tests_bidir_sub_map_tests_set_default_values (self, &_tmp160_, &_tmp161_, &_tmp162_, &_tmp163_);
		contains = (_vala_array_free (contains, contains_length1, (GDestroyNotify) g_free), NULL);
		contains = _tmp160_;
		contains_length1 = _tmp161_;
		_contains_size_ = contains_length1;
		not_contains = (_vala_array_free (not_contains, not_contains_length1, (GDestroyNotify) g_free), NULL);
		not_contains = _tmp162_;
		not_contains_length1 = _tmp163_;
		_not_contains_size_ = not_contains_length1;
		_tmp164_ = entries_iter;
		_vala_assert (gee_bidir_iterator_last (_tmp164_), "entries_iter.last ()");
		_tmp165_ = entries_iter;
		_tmp166_ = gee_iterator_get ((GeeIterator*) _tmp165_);
		_tmp167_ = (GeeMapEntry*) _tmp166_;
		_tmp168_ = contains;
		_tmp168__length1 = contains_length1;
		_tmp169_ = contains;
		_tmp169__length1 = contains_length1;
		_tmp170_ = _tmp168_[_tmp169__length1 - 1];
		_tmp171_ = contains;
		_tmp171__length1 = contains_length1;
		_tmp172_ = contains;
		_tmp172__length1 = contains_length1;
		_tmp173_ = _tmp171_[_tmp172__length1 - 1];
		map_tests_assert_entry (_tmp167_, _tmp170_, _tmp173_);
		_g_object_unref0 (_tmp167_);
		_tmp174_ = entries_iter;
		_vala_assert (gee_bidir_iterator_first (_tmp174_), "entries_iter.first ()");
		_tmp175_ = entries_iter;
		_tmp176_ = gee_iterator_get ((GeeIterator*) _tmp175_);
		_tmp177_ = (GeeMapEntry*) _tmp176_;
		_tmp178_ = contains;
		_tmp178__length1 = contains_length1;
		_tmp179_ = _tmp178_[0];
		_tmp180_ = contains;
		_tmp180__length1 = contains_length1;
		_tmp181_ = _tmp180_[0];
		map_tests_assert_entry (_tmp177_, _tmp179_, _tmp181_);
		_g_object_unref0 (_tmp177_);
		_tmp182_ = entries_iter;
		_vala_assert (gee_iterator_has_next ((GeeIterator*) _tmp182_), "entries_iter.has_next ()");
		_tmp183_ = entries_iter;
		_vala_assert (gee_iterator_next ((GeeIterator*) _tmp183_), "entries_iter.next ()");
		_tmp184_ = entries_iter;
		_tmp185_ = gee_iterator_get ((GeeIterator*) _tmp184_);
		_tmp186_ = (GeeMapEntry*) _tmp185_;
		_tmp187_ = contains;
		_tmp187__length1 = contains_length1;
		_tmp188_ = _tmp187_[1];
		_tmp189_ = contains;
		_tmp189__length1 = contains_length1;
		_tmp190_ = _tmp189_[1];
		map_tests_assert_entry (_tmp186_, _tmp188_, _tmp190_);
		_g_object_unref0 (_tmp186_);
		_tmp191_ = entries_iter;
		_vala_assert (gee_bidir_iterator_has_previous (_tmp191_), "entries_iter.has_previous ()");
		_tmp192_ = entries_iter;
		gee_iterator_remove ((GeeIterator*) _tmp192_);
		_tmp193_ = entries_iter;
		_vala_assert (gee_bidir_iterator_has_previous (_tmp193_), "entries_iter.has_previous ()");
		if (self->priv->type != GEE_SORTED_MAP_TESTS_SUB_MAP_TESTS_TYPE_HEAD) {
			GeeBidirIterator* _tmp194_;
			_tmp194_ = entries_iter;
			_vala_assert (gee_iterator_has_next ((GeeIterator*) _tmp194_), "entries_iter.has_next ()");
		} else {
			GeeBidirIterator* _tmp195_;
			_tmp195_ = entries_iter;
			_vala_assert (!gee_iterator_has_next ((GeeIterator*) _tmp195_), "!entries_iter.has_next ()");
		}
		_tmp196_ = entries_iter;
		_vala_assert (gee_bidir_iterator_previous (_tmp196_), "entries_iter.previous ()");
		_tmp197_ = entries_iter;
		_tmp198_ = gee_iterator_get ((GeeIterator*) _tmp197_);
		_tmp199_ = (GeeMapEntry*) _tmp198_;
		_tmp200_ = contains;
		_tmp200__length1 = contains_length1;
		_tmp201_ = _tmp200_[0];
		_tmp202_ = contains;
		_tmp202__length1 = contains_length1;
		_tmp203_ = _tmp202_[0];
		map_tests_assert_entry (_tmp199_, _tmp201_, _tmp203_);
		_g_object_unref0 (_tmp199_);
	} else {
		GeeBidirIterator* _tmp204_;
		GeeBidirIterator* _tmp205_;
		GeeBidirIterator* _tmp207_;
		_tmp204_ = keys_iter;
		_vala_assert (!gee_bidir_iterator_first (_tmp204_), "!keys_iter.first ()");
		_tmp205_ = keys_iter;
		_vala_assert (!gee_bidir_iterator_last (_tmp205_), "!keys_iter.last ()");
		if (g_test_trap_fork ((guint64) 0, G_TEST_TRAP_SILENCE_STDOUT | G_TEST_TRAP_SILENCE_STDERR)) {
			GeeBidirIterator* _tmp206_;
			_tmp206_ = keys_iter;
			gee_iterator_remove ((GeeIterator*) _tmp206_);
			exit (0);
		}
		g_test_trap_assert_failed ();
		_tmp207_ = entries_iter;
		_vala_assert (!gee_bidir_iterator_first (_tmp207_), "!entries_iter.first ()");
		if (g_test_trap_fork ((guint64) 0, G_TEST_TRAP_SILENCE_STDOUT | G_TEST_TRAP_SILENCE_STDERR)) {
			GeeBidirIterator* _tmp208_;
			_tmp208_ = entries_iter;
			gee_iterator_remove ((GeeIterator*) _tmp208_);
			exit (0);
		}
		g_test_trap_assert_failed ();
	}
	_g_object_unref0 (map_iter);
	_g_object_unref0 (entries_iter);
	_g_object_unref0 (keys_iter);
	_g_object_unref0 (_map_iter);
	not_contains = (_vala_array_free (not_contains, not_contains_length1, (GDestroyNotify) g_free), NULL);
	contains = (_vala_array_free (contains, contains_length1, (GDestroyNotify) g_free), NULL);
}

static void
bidir_sorted_map_tests_bidir_sub_map_tests_class_init (BidirSortedMapTestsBidirSubMapTestsClass * klass,
                                                       gpointer klass_data)
{
	bidir_sorted_map_tests_bidir_sub_map_tests_parent_class = g_type_class_peek_parent (klass);
	g_type_class_adjust_private_offset (klass, &BidirSortedMapTestsBidirSubMapTests_private_offset);
	((GeeTestCaseClass *) klass)->set_up = (void (*) (GeeTestCase*)) bidir_sorted_map_tests_bidir_sub_map_tests_real_set_up;
	((GeeTestCaseClass *) klass)->tear_down = (void (*) (GeeTestCase*)) bidir_sorted_map_tests_bidir_sub_map_tests_real_tear_down;
	G_OBJECT_CLASS (klass)->finalize = bidir_sorted_map_tests_bidir_sub_map_tests_finalize;
}

static void
bidir_sorted_map_tests_bidir_sub_map_tests_instance_init (BidirSortedMapTestsBidirSubMapTests * self,
                                                          gpointer klass)
{
	self->priv = bidir_sorted_map_tests_bidir_sub_map_tests_get_instance_private (self);
}

static void
bidir_sorted_map_tests_bidir_sub_map_tests_finalize (GObject * obj)
{
	BidirSortedMapTestsBidirSubMapTests * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, BIDIR_SORTED_MAP_TESTS_TYPE_BIDIR_SUB_MAP_TESTS, BidirSortedMapTestsBidirSubMapTests);
	_g_object_unref0 (self->priv->master);
	_g_object_unref0 (self->priv->submap);
	_g_object_unref0 (self->priv->test);
	G_OBJECT_CLASS (bidir_sorted_map_tests_bidir_sub_map_tests_parent_class)->finalize (obj);
}

static GType
bidir_sorted_map_tests_bidir_sub_map_tests_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (BidirSortedMapTestsBidirSubMapTestsClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) bidir_sorted_map_tests_bidir_sub_map_tests_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (BidirSortedMapTestsBidirSubMapTests), 0, (GInstanceInitFunc) bidir_sorted_map_tests_bidir_sub_map_tests_instance_init, NULL };
	GType bidir_sorted_map_tests_bidir_sub_map_tests_type_id;
	bidir_sorted_map_tests_bidir_sub_map_tests_type_id = g_type_register_static (GEE_TYPE_TEST_CASE, "BidirSortedMapTestsBidirSubMapTests", &g_define_type_info, 0);
	BidirSortedMapTestsBidirSubMapTests_private_offset = g_type_add_instance_private (bidir_sorted_map_tests_bidir_sub_map_tests_type_id, sizeof (BidirSortedMapTestsBidirSubMapTestsPrivate));
	return bidir_sorted_map_tests_bidir_sub_map_tests_type_id;
}

GType
bidir_sorted_map_tests_bidir_sub_map_tests_get_type (void)
{
	static volatile gsize bidir_sorted_map_tests_bidir_sub_map_tests_type_id__once = 0;
	if (g_once_init_enter (&bidir_sorted_map_tests_bidir_sub_map_tests_type_id__once)) {
		GType bidir_sorted_map_tests_bidir_sub_map_tests_type_id;
		bidir_sorted_map_tests_bidir_sub_map_tests_type_id = bidir_sorted_map_tests_bidir_sub_map_tests_get_type_once ();
		g_once_init_leave (&bidir_sorted_map_tests_bidir_sub_map_tests_type_id__once, bidir_sorted_map_tests_bidir_sub_map_tests_type_id);
	}
	return bidir_sorted_map_tests_bidir_sub_map_tests_type_id__once;
}

static void
bidir_sorted_map_tests_class_init (BidirSortedMapTestsClass * klass,
                                   gpointer klass_data)
{
	bidir_sorted_map_tests_parent_class = g_type_class_peek_parent (klass);
}

static void
bidir_sorted_map_tests_instance_init (BidirSortedMapTests * self,
                                      gpointer klass)
{
}

static GType
bidir_sorted_map_tests_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (BidirSortedMapTestsClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) bidir_sorted_map_tests_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (BidirSortedMapTests), 0, (GInstanceInitFunc) bidir_sorted_map_tests_instance_init, NULL };
	GType bidir_sorted_map_tests_type_id;
	bidir_sorted_map_tests_type_id = g_type_register_static (GEE_TYPE_SORTED_MAP_TESTS, "BidirSortedMapTests", &g_define_type_info, G_TYPE_FLAG_ABSTRACT);
	return bidir_sorted_map_tests_type_id;
}

GType
bidir_sorted_map_tests_get_type (void)
{
	static volatile gsize bidir_sorted_map_tests_type_id__once = 0;
	if (g_once_init_enter (&bidir_sorted_map_tests_type_id__once)) {
		GType bidir_sorted_map_tests_type_id;
		bidir_sorted_map_tests_type_id = bidir_sorted_map_tests_get_type_once ();
		g_once_init_leave (&bidir_sorted_map_tests_type_id__once, bidir_sorted_map_tests_type_id);
	}
	return bidir_sorted_map_tests_type_id__once;
}

static void
_vala_array_destroy (gpointer array,
                     gssize array_length,
                     GDestroyNotify destroy_func)
{
	if ((array != NULL) && (destroy_func != NULL)) {
		gssize i;
		for (i = 0; i < array_length; i = i + 1) {
			if (((gpointer*) array)[i] != NULL) {
				destroy_func (((gpointer*) array)[i]);
			}
		}
	}
}

static void
_vala_array_free (gpointer array,
                  gssize array_length,
                  GDestroyNotify destroy_func)
{
	_vala_array_destroy (array, array_length, destroy_func);
	g_free (array);
}

