/* readonlysortedset.c generated by valac 0.56.3, the Vala compiler
 * generated from readonlysortedset.vala, do not modify */

/* readonlysortedset.vala
 *
 * Copyright (C) 2009  Didier Villevalois, Maciej Piechotka
 * Copyright (C) 2011  Maciej Piechotka
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Maciej Piechotka <uzytkownik2@gmail.com>
 */

#include <glib-object.h>
#include "gee.h"
#include <glib.h>

#define GEE_TYPE_READ_ONLY_COLLECTION (gee_read_only_collection_get_type ())
#define GEE_READ_ONLY_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_READ_ONLY_COLLECTION, GeeReadOnlyCollection))
#define GEE_READ_ONLY_COLLECTION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_TYPE_READ_ONLY_COLLECTION, GeeReadOnlyCollectionClass))
#define GEE_IS_READ_ONLY_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_READ_ONLY_COLLECTION))
#define GEE_IS_READ_ONLY_COLLECTION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEE_TYPE_READ_ONLY_COLLECTION))
#define GEE_READ_ONLY_COLLECTION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_TYPE_READ_ONLY_COLLECTION, GeeReadOnlyCollectionClass))

typedef struct _GeeReadOnlyCollection GeeReadOnlyCollection;
typedef struct _GeeReadOnlyCollectionClass GeeReadOnlyCollectionClass;
typedef struct _GeeReadOnlyCollectionPrivate GeeReadOnlyCollectionPrivate;

#define GEE_TYPE_READ_ONLY_SET (gee_read_only_set_get_type ())
#define GEE_READ_ONLY_SET(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_READ_ONLY_SET, GeeReadOnlySet))
#define GEE_READ_ONLY_SET_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_TYPE_READ_ONLY_SET, GeeReadOnlySetClass))
#define GEE_IS_READ_ONLY_SET(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_READ_ONLY_SET))
#define GEE_IS_READ_ONLY_SET_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEE_TYPE_READ_ONLY_SET))
#define GEE_READ_ONLY_SET_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_TYPE_READ_ONLY_SET, GeeReadOnlySetClass))

typedef struct _GeeReadOnlySet GeeReadOnlySet;
typedef struct _GeeReadOnlySetClass GeeReadOnlySetClass;
typedef struct _GeeReadOnlySetPrivate GeeReadOnlySetPrivate;

#define GEE_TYPE_READ_ONLY_SORTED_SET (gee_read_only_sorted_set_get_type ())
#define GEE_READ_ONLY_SORTED_SET(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_READ_ONLY_SORTED_SET, GeeReadOnlySortedSet))
#define GEE_READ_ONLY_SORTED_SET_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_TYPE_READ_ONLY_SORTED_SET, GeeReadOnlySortedSetClass))
#define GEE_IS_READ_ONLY_SORTED_SET(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_READ_ONLY_SORTED_SET))
#define GEE_IS_READ_ONLY_SORTED_SET_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEE_TYPE_READ_ONLY_SORTED_SET))
#define GEE_READ_ONLY_SORTED_SET_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_TYPE_READ_ONLY_SORTED_SET, GeeReadOnlySortedSetClass))

typedef struct _GeeReadOnlySortedSet GeeReadOnlySortedSet;
typedef struct _GeeReadOnlySortedSetClass GeeReadOnlySortedSetClass;
typedef struct _GeeReadOnlySortedSetPrivate GeeReadOnlySortedSetPrivate;
enum  {
	GEE_READ_ONLY_SORTED_SET_0_PROPERTY,
	GEE_READ_ONLY_SORTED_SET_G_TYPE,
	GEE_READ_ONLY_SORTED_SET_G_DUP_FUNC,
	GEE_READ_ONLY_SORTED_SET_G_DESTROY_FUNC,
	GEE_READ_ONLY_SORTED_SET_READ_ONLY_VIEW_PROPERTY,
	GEE_READ_ONLY_SORTED_SET_NUM_PROPERTIES
};
static GParamSpec* gee_read_only_sorted_set_properties[GEE_READ_ONLY_SORTED_SET_NUM_PROPERTIES];

#define GEE_READ_ONLY_COLLECTION_TYPE_ITERATOR (gee_read_only_collection_iterator_get_type ())
#define GEE_READ_ONLY_COLLECTION_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_READ_ONLY_COLLECTION_TYPE_ITERATOR, GeeReadOnlyCollectionIterator))
#define GEE_READ_ONLY_COLLECTION_ITERATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_READ_ONLY_COLLECTION_TYPE_ITERATOR, GeeReadOnlyCollectionIteratorClass))
#define GEE_READ_ONLY_COLLECTION_IS_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_READ_ONLY_COLLECTION_TYPE_ITERATOR))
#define GEE_READ_ONLY_COLLECTION_IS_ITERATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEE_READ_ONLY_COLLECTION_TYPE_ITERATOR))
#define GEE_READ_ONLY_COLLECTION_ITERATOR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_READ_ONLY_COLLECTION_TYPE_ITERATOR, GeeReadOnlyCollectionIteratorClass))

typedef struct _GeeReadOnlyCollectionIterator GeeReadOnlyCollectionIterator;
typedef struct _GeeReadOnlyCollectionIteratorClass GeeReadOnlyCollectionIteratorClass;
#define _g_object_unref0(var) ((var == NULL) ? NULL : (var = (g_object_unref (var), NULL)))

struct _GeeReadOnlyCollection {
	GObject parent_instance;
	GeeReadOnlyCollectionPrivate * priv;
	GeeCollection* _collection;
};

struct _GeeReadOnlyCollectionClass {
	GObjectClass parent_class;
	GeeCollection* (*get_read_only_view) (GeeReadOnlyCollection* self);
};

struct _GeeReadOnlySet {
	GeeReadOnlyCollection parent_instance;
	GeeReadOnlySetPrivate * priv;
};

struct _GeeReadOnlySetClass {
	GeeReadOnlyCollectionClass parent_class;
	GeeSet* (*get_read_only_view) (GeeReadOnlySet* self);
};

struct _GeeReadOnlySortedSet {
	GeeReadOnlySet parent_instance;
	GeeReadOnlySortedSetPrivate * priv;
};

struct _GeeReadOnlySortedSetClass {
	GeeReadOnlySetClass parent_class;
};

struct _GeeReadOnlySortedSetPrivate {
	GType g_type;
	GBoxedCopyFunc g_dup_func;
	GDestroyNotify g_destroy_func;
};

static gint GeeReadOnlySortedSet_private_offset;
static gpointer gee_read_only_sorted_set_parent_class = NULL;
static GeeSortedSetIface * gee_read_only_sorted_set_gee_sorted_set_parent_iface = NULL;

 G_GNUC_INTERNAL GType gee_read_only_collection_get_type (void) G_GNUC_CONST  G_GNUC_UNUSED ;
 G_GNUC_INTERNAL GType gee_read_only_set_get_type (void) G_GNUC_CONST  G_GNUC_UNUSED ;
 G_GNUC_INTERNAL GType gee_read_only_sorted_set_get_type (void) G_GNUC_CONST  G_GNUC_UNUSED ;
 G_GNUC_INTERNAL GeeReadOnlySortedSet* gee_read_only_sorted_set_new (GType g_type,
                                                    GBoxedCopyFunc g_dup_func,
                                                    GDestroyNotify g_destroy_func,
                                                    GeeSortedSet* set);
 G_GNUC_INTERNAL GeeReadOnlySortedSet* gee_read_only_sorted_set_construct (GType object_type,
                                                          GType g_type,
                                                          GBoxedCopyFunc g_dup_func,
                                                          GDestroyNotify g_destroy_func,
                                                          GeeSortedSet* set);
 G_GNUC_INTERNAL GeeReadOnlySet* gee_read_only_set_new (GType g_type,
                                       GBoxedCopyFunc g_dup_func,
                                       GDestroyNotify g_destroy_func,
                                       GeeSet* set);
 G_GNUC_INTERNAL GeeReadOnlySet* gee_read_only_set_construct (GType object_type,
                                             GType g_type,
                                             GBoxedCopyFunc g_dup_func,
                                             GDestroyNotify g_destroy_func,
                                             GeeSet* set);
static gpointer gee_read_only_sorted_set_real_first (GeeSortedSet* base);
static gpointer gee_read_only_sorted_set_real_last (GeeSortedSet* base);
static GeeIterator* gee_read_only_sorted_set_real_iterator_at (GeeSortedSet* base,
                                                        gconstpointer element);
VALA_EXTERN GType gee_read_only_collection_iterator_get_type (void) G_GNUC_CONST ;
 G_GNUC_INTERNAL GeeReadOnlyCollectionIterator* gee_read_only_collection_iterator_new (GType g_type,
                                                                      GBoxedCopyFunc g_dup_func,
                                                                      GDestroyNotify g_destroy_func,
                                                                      GeeIterator* iterator);
 G_GNUC_INTERNAL GeeReadOnlyCollectionIterator* gee_read_only_collection_iterator_construct (GType object_type,
                                                                            GType g_type,
                                                                            GBoxedCopyFunc g_dup_func,
                                                                            GDestroyNotify g_destroy_func,
                                                                            GeeIterator* iterator);
static gpointer gee_read_only_sorted_set_real_lower (GeeSortedSet* base,
                                              gconstpointer element);
static gpointer gee_read_only_sorted_set_real_higher (GeeSortedSet* base,
                                               gconstpointer element);
static gpointer gee_read_only_sorted_set_real_floor (GeeSortedSet* base,
                                              gconstpointer element);
static gpointer gee_read_only_sorted_set_real_ceil (GeeSortedSet* base,
                                             gconstpointer element);
static GeeSortedSet* gee_read_only_sorted_set_real_head_set (GeeSortedSet* base,
                                                      gconstpointer before);
static GeeSortedSet* gee_read_only_sorted_set_real_tail_set (GeeSortedSet* base,
                                                      gconstpointer after);
static GeeSortedSet* gee_read_only_sorted_set_real_sub_set (GeeSortedSet* base,
                                                     gconstpointer from,
                                                     gconstpointer to);
static GType gee_read_only_sorted_set_get_type_once (void);
static void _vala_gee_read_only_sorted_set_get_property (GObject * object,
                                                  guint property_id,
                                                  GValue * value,
                                                  GParamSpec * pspec);
static void _vala_gee_read_only_sorted_set_set_property (GObject * object,
                                                  guint property_id,
                                                  const GValue * value,
                                                  GParamSpec * pspec);

static inline gpointer
gee_read_only_sorted_set_get_instance_private (GeeReadOnlySortedSet* self)
{
	return G_STRUCT_MEMBER_P (self, GeeReadOnlySortedSet_private_offset);
}

/**
 * Constructs a read-only set that mirrors the content of the specified set.
 *
 * @param set the set to decorate.
 */
 G_GNUC_INTERNAL GeeReadOnlySortedSet*
gee_read_only_sorted_set_construct (GType object_type,
                                    GType g_type,
                                    GBoxedCopyFunc g_dup_func,
                                    GDestroyNotify g_destroy_func,
                                    GeeSortedSet* set)
{
	GeeReadOnlySortedSet * self = NULL;
	g_return_val_if_fail (set != NULL, NULL);
	self = (GeeReadOnlySortedSet*) gee_read_only_set_construct (object_type, g_type, (GBoxedCopyFunc) g_dup_func, (GDestroyNotify) g_destroy_func, (GeeSet*) set);
	self->priv->g_type = g_type;
	self->priv->g_dup_func = g_dup_func;
	self->priv->g_destroy_func = g_destroy_func;
	return self;
}

 G_GNUC_INTERNAL GeeReadOnlySortedSet*
gee_read_only_sorted_set_new (GType g_type,
                              GBoxedCopyFunc g_dup_func,
                              GDestroyNotify g_destroy_func,
                              GeeSortedSet* set)
{
	return gee_read_only_sorted_set_construct (GEE_TYPE_READ_ONLY_SORTED_SET, g_type, g_dup_func, g_destroy_func, set);
}

/**
 * {@inheritDoc}
 */
static gpointer
gee_read_only_sorted_set_real_first (GeeSortedSet* base)
{
	GeeReadOnlySortedSet * self;
	GeeCollection* _tmp0_;
	gpointer _tmp1_;
	gpointer result;
	self = (GeeReadOnlySortedSet*) base;
	_tmp0_ = ((GeeReadOnlyCollection*) self)->_collection;
	_tmp1_ = gee_sorted_set_first (G_TYPE_CHECK_INSTANCE_CAST (_tmp0_, GEE_TYPE_SORTED_SET, GeeSortedSet));
	result = _tmp1_;
	return result;
}

/**
 * {@inheritDoc}
 */
static gpointer
gee_read_only_sorted_set_real_last (GeeSortedSet* base)
{
	GeeReadOnlySortedSet * self;
	GeeCollection* _tmp0_;
	gpointer _tmp1_;
	gpointer result;
	self = (GeeReadOnlySortedSet*) base;
	_tmp0_ = ((GeeReadOnlyCollection*) self)->_collection;
	_tmp1_ = gee_sorted_set_last (G_TYPE_CHECK_INSTANCE_CAST (_tmp0_, GEE_TYPE_SORTED_SET, GeeSortedSet));
	result = _tmp1_;
	return result;
}

/**
 * {@inheritDoc}
 */
static GeeIterator*
gee_read_only_sorted_set_real_iterator_at (GeeSortedSet* base,
                                           gconstpointer element)
{
	GeeReadOnlySortedSet * self;
	GeeIterator* iter = NULL;
	GeeCollection* _tmp0_;
	GeeIterator* _tmp1_;
	GeeReadOnlyCollectionIterator* _tmp2_ = NULL;
	GeeIterator* _tmp3_;
	GeeIterator* result;
	self = (GeeReadOnlySortedSet*) base;
	_tmp0_ = ((GeeReadOnlyCollection*) self)->_collection;
	_tmp1_ = gee_sorted_set_iterator_at (G_TYPE_CHECK_INSTANCE_CAST (_tmp0_, GEE_TYPE_SORTED_SET, GeeSortedSet), element);
	iter = _tmp1_;
	_tmp3_ = iter;
	if (_tmp3_ != NULL) {
		GeeIterator* _tmp4_;
		GeeReadOnlyCollectionIterator* _tmp5_;
		_tmp4_ = iter;
		_tmp5_ = gee_read_only_collection_iterator_new (self->priv->g_type, (GBoxedCopyFunc) self->priv->g_dup_func, (GDestroyNotify) self->priv->g_destroy_func, _tmp4_);
		_g_object_unref0 (_tmp2_);
		_tmp2_ = _tmp5_;
	} else {
		_g_object_unref0 (_tmp2_);
		_tmp2_ = NULL;
	}
	result = (GeeIterator*) _tmp2_;
	_g_object_unref0 (iter);
	return result;
}

/**
 * {@inheritDoc}
 */
static gpointer
gee_read_only_sorted_set_real_lower (GeeSortedSet* base,
                                     gconstpointer element)
{
	GeeReadOnlySortedSet * self;
	GeeCollection* _tmp0_;
	gpointer _tmp1_;
	gpointer result;
	self = (GeeReadOnlySortedSet*) base;
	_tmp0_ = ((GeeReadOnlyCollection*) self)->_collection;
	_tmp1_ = gee_sorted_set_lower (G_TYPE_CHECK_INSTANCE_CAST (_tmp0_, GEE_TYPE_SORTED_SET, GeeSortedSet), element);
	result = _tmp1_;
	return result;
}

/**
 * {@inheritDoc}
 */
static gpointer
gee_read_only_sorted_set_real_higher (GeeSortedSet* base,
                                      gconstpointer element)
{
	GeeReadOnlySortedSet * self;
	GeeCollection* _tmp0_;
	gpointer _tmp1_;
	gpointer result;
	self = (GeeReadOnlySortedSet*) base;
	_tmp0_ = ((GeeReadOnlyCollection*) self)->_collection;
	_tmp1_ = gee_sorted_set_higher (G_TYPE_CHECK_INSTANCE_CAST (_tmp0_, GEE_TYPE_SORTED_SET, GeeSortedSet), element);
	result = _tmp1_;
	return result;
}

/**
 * {@inheritDoc}
 */
static gpointer
gee_read_only_sorted_set_real_floor (GeeSortedSet* base,
                                     gconstpointer element)
{
	GeeReadOnlySortedSet * self;
	GeeCollection* _tmp0_;
	gpointer _tmp1_;
	gpointer result;
	self = (GeeReadOnlySortedSet*) base;
	_tmp0_ = ((GeeReadOnlyCollection*) self)->_collection;
	_tmp1_ = gee_sorted_set_floor (G_TYPE_CHECK_INSTANCE_CAST (_tmp0_, GEE_TYPE_SORTED_SET, GeeSortedSet), element);
	result = _tmp1_;
	return result;
}

/**
 * {@inheritDoc}
 */
static gpointer
gee_read_only_sorted_set_real_ceil (GeeSortedSet* base,
                                    gconstpointer element)
{
	GeeReadOnlySortedSet * self;
	GeeCollection* _tmp0_;
	gpointer _tmp1_;
	gpointer result;
	self = (GeeReadOnlySortedSet*) base;
	_tmp0_ = ((GeeReadOnlyCollection*) self)->_collection;
	_tmp1_ = gee_sorted_set_ceil (G_TYPE_CHECK_INSTANCE_CAST (_tmp0_, GEE_TYPE_SORTED_SET, GeeSortedSet), element);
	result = _tmp1_;
	return result;
}

/**
 * {@inheritDoc}
 */
static GeeSortedSet*
gee_read_only_sorted_set_real_head_set (GeeSortedSet* base,
                                        gconstpointer before)
{
	GeeReadOnlySortedSet * self;
	GeeCollection* _tmp0_;
	GeeSortedSet* _tmp1_;
	GeeSortedSet* _tmp2_;
	GeeSortedSet* _tmp3_;
	GeeSortedSet* _tmp4_;
	GeeSortedSet* _tmp5_;
	GeeSortedSet* result;
	self = (GeeReadOnlySortedSet*) base;
	_tmp0_ = ((GeeReadOnlyCollection*) self)->_collection;
	_tmp1_ = gee_sorted_set_head_set (G_TYPE_CHECK_INSTANCE_CAST (_tmp0_, GEE_TYPE_SORTED_SET, GeeSortedSet), before);
	_tmp2_ = _tmp1_;
	_tmp3_ = gee_sorted_set_get_read_only_view (_tmp2_);
	_tmp4_ = _tmp3_;
	_tmp5_ = _tmp4_;
	_g_object_unref0 (_tmp2_);
	result = _tmp5_;
	return result;
}

/**
 * {@inheritDoc}
 */
static GeeSortedSet*
gee_read_only_sorted_set_real_tail_set (GeeSortedSet* base,
                                        gconstpointer after)
{
	GeeReadOnlySortedSet * self;
	GeeCollection* _tmp0_;
	GeeSortedSet* _tmp1_;
	GeeSortedSet* _tmp2_;
	GeeSortedSet* _tmp3_;
	GeeSortedSet* _tmp4_;
	GeeSortedSet* _tmp5_;
	GeeSortedSet* result;
	self = (GeeReadOnlySortedSet*) base;
	_tmp0_ = ((GeeReadOnlyCollection*) self)->_collection;
	_tmp1_ = gee_sorted_set_tail_set (G_TYPE_CHECK_INSTANCE_CAST (_tmp0_, GEE_TYPE_SORTED_SET, GeeSortedSet), after);
	_tmp2_ = _tmp1_;
	_tmp3_ = gee_sorted_set_get_read_only_view (_tmp2_);
	_tmp4_ = _tmp3_;
	_tmp5_ = _tmp4_;
	_g_object_unref0 (_tmp2_);
	result = _tmp5_;
	return result;
}

/**
 * {@inheritDoc}
 */
static GeeSortedSet*
gee_read_only_sorted_set_real_sub_set (GeeSortedSet* base,
                                       gconstpointer from,
                                       gconstpointer to)
{
	GeeReadOnlySortedSet * self;
	GeeCollection* _tmp0_;
	GeeSortedSet* _tmp1_;
	GeeSortedSet* _tmp2_;
	GeeSortedSet* _tmp3_;
	GeeSortedSet* _tmp4_;
	GeeSortedSet* _tmp5_;
	GeeSortedSet* result;
	self = (GeeReadOnlySortedSet*) base;
	_tmp0_ = ((GeeReadOnlyCollection*) self)->_collection;
	_tmp1_ = gee_sorted_set_sub_set (G_TYPE_CHECK_INSTANCE_CAST (_tmp0_, GEE_TYPE_SORTED_SET, GeeSortedSet), from, to);
	_tmp2_ = _tmp1_;
	_tmp3_ = gee_sorted_set_get_read_only_view (_tmp2_);
	_tmp4_ = _tmp3_;
	_tmp5_ = _tmp4_;
	_g_object_unref0 (_tmp2_);
	result = _tmp5_;
	return result;
}

static gpointer
_g_object_ref0 (gpointer self)
{
	return self ? g_object_ref (self) : NULL;
}

static GeeSortedSet*
gee_read_only_sorted_set_real_get_read_only_view (GeeSortedSet* base)
{
	GeeSortedSet* result;
	GeeReadOnlySortedSet* self;
	GeeSortedSet* _tmp0_;
	self = (GeeReadOnlySortedSet*) base;
	_tmp0_ = _g_object_ref0 ((GeeSortedSet*) self);
	result = _tmp0_;
	return result;
}

static void
gee_read_only_sorted_set_class_init (GeeReadOnlySortedSetClass * klass,
                                     gpointer klass_data)
{
	gee_read_only_sorted_set_parent_class = g_type_class_peek_parent (klass);
	g_type_class_adjust_private_offset (klass, &GeeReadOnlySortedSet_private_offset);
	G_OBJECT_CLASS (klass)->get_property = _vala_gee_read_only_sorted_set_get_property;
	G_OBJECT_CLASS (klass)->set_property = _vala_gee_read_only_sorted_set_set_property;
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_READ_ONLY_SORTED_SET_G_TYPE, g_param_spec_gtype ("g-type", "type", "type", G_TYPE_NONE, G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_READ_ONLY_SORTED_SET_G_DUP_FUNC, g_param_spec_pointer ("g-dup-func", "dup func", "dup func", G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_READ_ONLY_SORTED_SET_G_DESTROY_FUNC, g_param_spec_pointer ("g-destroy-func", "destroy func", "destroy func", G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	/**
	 * {@inheritDoc}
	 */
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_READ_ONLY_SORTED_SET_READ_ONLY_VIEW_PROPERTY, gee_read_only_sorted_set_properties[GEE_READ_ONLY_SORTED_SET_READ_ONLY_VIEW_PROPERTY] = g_param_spec_object ("read-only-view", "read-only-view", "read-only-view", GEE_TYPE_SORTED_SET, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE));
}

static GType
gee_read_only_sorted_set_gee_sorted_set_get_g_type (GeeReadOnlySortedSet* self)
{
	return self->priv->g_type;
}

static GBoxedCopyFunc
gee_read_only_sorted_set_gee_sorted_set_get_g_dup_func (GeeReadOnlySortedSet* self)
{
	return self->priv->g_dup_func;
}

static GDestroyNotify
gee_read_only_sorted_set_gee_sorted_set_get_g_destroy_func (GeeReadOnlySortedSet* self)
{
	return self->priv->g_destroy_func;
}

static void
gee_read_only_sorted_set_gee_sorted_set_interface_init (GeeSortedSetIface * iface,
                                                        gpointer iface_data)
{
	gee_read_only_sorted_set_gee_sorted_set_parent_iface = g_type_interface_peek_parent (iface);
	iface->first = (gpointer (*) (GeeSortedSet*)) gee_read_only_sorted_set_real_first;
	iface->last = (gpointer (*) (GeeSortedSet*)) gee_read_only_sorted_set_real_last;
	iface->iterator_at = (GeeIterator* (*) (GeeSortedSet*, gconstpointer)) gee_read_only_sorted_set_real_iterator_at;
	iface->lower = (gpointer (*) (GeeSortedSet*, gconstpointer)) gee_read_only_sorted_set_real_lower;
	iface->higher = (gpointer (*) (GeeSortedSet*, gconstpointer)) gee_read_only_sorted_set_real_higher;
	iface->floor = (gpointer (*) (GeeSortedSet*, gconstpointer)) gee_read_only_sorted_set_real_floor;
	iface->ceil = (gpointer (*) (GeeSortedSet*, gconstpointer)) gee_read_only_sorted_set_real_ceil;
	iface->head_set = (GeeSortedSet* (*) (GeeSortedSet*, gconstpointer)) gee_read_only_sorted_set_real_head_set;
	iface->tail_set = (GeeSortedSet* (*) (GeeSortedSet*, gconstpointer)) gee_read_only_sorted_set_real_tail_set;
	iface->sub_set = (GeeSortedSet* (*) (GeeSortedSet*, gconstpointer, gconstpointer)) gee_read_only_sorted_set_real_sub_set;
	iface->get_g_type = (GType (*) (GeeSortedSet *)) gee_read_only_sorted_set_gee_sorted_set_get_g_type;
	iface->get_g_dup_func = (GBoxedCopyFunc (*) (GeeSortedSet *)) gee_read_only_sorted_set_gee_sorted_set_get_g_dup_func;
	iface->get_g_destroy_func = (GDestroyNotify (*) (GeeSortedSet *)) gee_read_only_sorted_set_gee_sorted_set_get_g_destroy_func;
	iface->get_read_only_view = gee_read_only_sorted_set_real_get_read_only_view;
}

static void
gee_read_only_sorted_set_instance_init (GeeReadOnlySortedSet * self,
                                        gpointer klass)
{
	self->priv = gee_read_only_sorted_set_get_instance_private (self);
}

/**
 * Read-only view for {@link SortedSet} collections.
 *
 * This class decorates any class which implements the {@link SortedSet} interface
 * by making it read only. Any method which normally modify data will throw an
 * error.
 *
 * @see SortedSet
 */
static GType
gee_read_only_sorted_set_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (GeeReadOnlySortedSetClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) gee_read_only_sorted_set_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GeeReadOnlySortedSet), 0, (GInstanceInitFunc) gee_read_only_sorted_set_instance_init, NULL };
	static const GInterfaceInfo gee_sorted_set_info = { (GInterfaceInitFunc) gee_read_only_sorted_set_gee_sorted_set_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
	GType gee_read_only_sorted_set_type_id;
	gee_read_only_sorted_set_type_id = g_type_register_static (GEE_TYPE_READ_ONLY_SET, "GeeReadOnlySortedSet", &g_define_type_info, 0);
	g_type_add_interface_static (gee_read_only_sorted_set_type_id, GEE_TYPE_SORTED_SET, &gee_sorted_set_info);
	GeeReadOnlySortedSet_private_offset = g_type_add_instance_private (gee_read_only_sorted_set_type_id, sizeof (GeeReadOnlySortedSetPrivate));
	return gee_read_only_sorted_set_type_id;
}

 G_GNUC_INTERNAL GType
gee_read_only_sorted_set_get_type (void)
{
	static volatile gsize gee_read_only_sorted_set_type_id__once = 0;
	if (g_once_init_enter (&gee_read_only_sorted_set_type_id__once)) {
		GType gee_read_only_sorted_set_type_id;
		gee_read_only_sorted_set_type_id = gee_read_only_sorted_set_get_type_once ();
		g_once_init_leave (&gee_read_only_sorted_set_type_id__once, gee_read_only_sorted_set_type_id);
	}
	return gee_read_only_sorted_set_type_id__once;
}

static void
_vala_gee_read_only_sorted_set_get_property (GObject * object,
                                             guint property_id,
                                             GValue * value,
                                             GParamSpec * pspec)
{
	GeeReadOnlySortedSet * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, GEE_TYPE_READ_ONLY_SORTED_SET, GeeReadOnlySortedSet);
	switch (property_id) {
		case GEE_READ_ONLY_SORTED_SET_READ_ONLY_VIEW_PROPERTY:
		g_value_take_object (value, gee_sorted_set_get_read_only_view ((GeeSortedSet*) self));
		break;
		case GEE_READ_ONLY_SORTED_SET_G_TYPE:
		g_value_set_gtype (value, self->priv->g_type);
		break;
		case GEE_READ_ONLY_SORTED_SET_G_DUP_FUNC:
		g_value_set_pointer (value, self->priv->g_dup_func);
		break;
		case GEE_READ_ONLY_SORTED_SET_G_DESTROY_FUNC:
		g_value_set_pointer (value, self->priv->g_destroy_func);
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

static void
_vala_gee_read_only_sorted_set_set_property (GObject * object,
                                             guint property_id,
                                             const GValue * value,
                                             GParamSpec * pspec)
{
	GeeReadOnlySortedSet * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, GEE_TYPE_READ_ONLY_SORTED_SET, GeeReadOnlySortedSet);
	switch (property_id) {
		case GEE_READ_ONLY_SORTED_SET_G_TYPE:
		self->priv->g_type = g_value_get_gtype (value);
		break;
		case GEE_READ_ONLY_SORTED_SET_G_DUP_FUNC:
		self->priv->g_dup_func = g_value_get_pointer (value);
		break;
		case GEE_READ_ONLY_SORTED_SET_G_DESTROY_FUNC:
		self->priv->g_destroy_func = g_value_get_pointer (value);
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

