/* readonlymultiset.c generated by valac 0.56.3, the Vala compiler
 * generated from readonlymultiset.vala, do not modify */

/* readonlymultiset.vala
 *
 * Copyright (C) 2013  Maciej Piechotka
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Maciej Piechotka <uzytkownik2@gmail.com>
 */

#include <glib-object.h>
#include "gee.h"
#include <glib.h>

#define GEE_TYPE_READ_ONLY_COLLECTION (gee_read_only_collection_get_type ())
#define GEE_READ_ONLY_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_READ_ONLY_COLLECTION, GeeReadOnlyCollection))
#define GEE_READ_ONLY_COLLECTION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_TYPE_READ_ONLY_COLLECTION, GeeReadOnlyCollectionClass))
#define GEE_IS_READ_ONLY_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_READ_ONLY_COLLECTION))
#define GEE_IS_READ_ONLY_COLLECTION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEE_TYPE_READ_ONLY_COLLECTION))
#define GEE_READ_ONLY_COLLECTION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_TYPE_READ_ONLY_COLLECTION, GeeReadOnlyCollectionClass))

typedef struct _GeeReadOnlyCollection GeeReadOnlyCollection;
typedef struct _GeeReadOnlyCollectionClass GeeReadOnlyCollectionClass;
typedef struct _GeeReadOnlyCollectionPrivate GeeReadOnlyCollectionPrivate;

#define GEE_TYPE_READ_ONLY_MULTI_SET (gee_read_only_multi_set_get_type ())
#define GEE_READ_ONLY_MULTI_SET(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_READ_ONLY_MULTI_SET, GeeReadOnlyMultiSet))
#define GEE_READ_ONLY_MULTI_SET_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_TYPE_READ_ONLY_MULTI_SET, GeeReadOnlyMultiSetClass))
#define GEE_IS_READ_ONLY_MULTI_SET(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_READ_ONLY_MULTI_SET))
#define GEE_IS_READ_ONLY_MULTI_SET_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEE_TYPE_READ_ONLY_MULTI_SET))
#define GEE_READ_ONLY_MULTI_SET_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_TYPE_READ_ONLY_MULTI_SET, GeeReadOnlyMultiSetClass))

typedef struct _GeeReadOnlyMultiSet GeeReadOnlyMultiSet;
typedef struct _GeeReadOnlyMultiSetClass GeeReadOnlyMultiSetClass;
typedef struct _GeeReadOnlyMultiSetPrivate GeeReadOnlyMultiSetPrivate;
enum  {
	GEE_READ_ONLY_MULTI_SET_0_PROPERTY,
	GEE_READ_ONLY_MULTI_SET_G_TYPE,
	GEE_READ_ONLY_MULTI_SET_G_DUP_FUNC,
	GEE_READ_ONLY_MULTI_SET_G_DESTROY_FUNC,
	GEE_READ_ONLY_MULTI_SET_NUM_PROPERTIES
};
static GParamSpec* gee_read_only_multi_set_properties[GEE_READ_ONLY_MULTI_SET_NUM_PROPERTIES];

struct _GeeReadOnlyCollection {
	GObject parent_instance;
	GeeReadOnlyCollectionPrivate * priv;
	GeeCollection* _collection;
};

struct _GeeReadOnlyCollectionClass {
	GObjectClass parent_class;
	GeeCollection* (*get_read_only_view) (GeeReadOnlyCollection* self);
};

struct _GeeReadOnlyMultiSet {
	GeeReadOnlyCollection parent_instance;
	GeeReadOnlyMultiSetPrivate * priv;
};

struct _GeeReadOnlyMultiSetClass {
	GeeReadOnlyCollectionClass parent_class;
	GeeMultiSet* (*get_read_only_view) (GeeReadOnlyMultiSet* self);
};

struct _GeeReadOnlyMultiSetPrivate {
	GType g_type;
	GBoxedCopyFunc g_dup_func;
	GDestroyNotify g_destroy_func;
};

static gint GeeReadOnlyMultiSet_private_offset;
static gpointer gee_read_only_multi_set_parent_class = NULL;
static GeeMultiSetIface * gee_read_only_multi_set_gee_multi_set_parent_iface = NULL;

 G_GNUC_INTERNAL GType gee_read_only_collection_get_type (void) G_GNUC_CONST  G_GNUC_UNUSED ;
 G_GNUC_INTERNAL GType gee_read_only_multi_set_get_type (void) G_GNUC_CONST  G_GNUC_UNUSED ;
 G_GNUC_INTERNAL GeeMultiSet* gee_read_only_multi_set_get_read_only_view (GeeReadOnlyMultiSet* self);
 G_GNUC_INTERNAL GeeReadOnlyMultiSet* gee_read_only_multi_set_new (GType g_type,
                                                  GBoxedCopyFunc g_dup_func,
                                                  GDestroyNotify g_destroy_func,
                                                  GeeMultiSet* multiset);
 G_GNUC_INTERNAL GeeReadOnlyMultiSet* gee_read_only_multi_set_construct (GType object_type,
                                                        GType g_type,
                                                        GBoxedCopyFunc g_dup_func,
                                                        GDestroyNotify g_destroy_func,
                                                        GeeMultiSet* multiset);
 G_GNUC_INTERNAL GeeReadOnlyCollection* gee_read_only_collection_new (GType g_type,
                                                     GBoxedCopyFunc g_dup_func,
                                                     GDestroyNotify g_destroy_func,
                                                     GeeCollection* collection);
 G_GNUC_INTERNAL GeeReadOnlyCollection* gee_read_only_collection_construct (GType object_type,
                                                           GType g_type,
                                                           GBoxedCopyFunc g_dup_func,
                                                           GDestroyNotify g_destroy_func,
                                                           GeeCollection* collection);
static gint gee_read_only_multi_set_real_count (GeeMultiSet* base,
                                         gconstpointer item);
static GType gee_read_only_multi_set_get_type_once (void);
static void _vala_gee_read_only_multi_set_get_property (GObject * object,
                                                 guint property_id,
                                                 GValue * value,
                                                 GParamSpec * pspec);
static void _vala_gee_read_only_multi_set_set_property (GObject * object,
                                                 guint property_id,
                                                 const GValue * value,
                                                 GParamSpec * pspec);

static inline gpointer
gee_read_only_multi_set_get_instance_private (GeeReadOnlyMultiSet* self)
{
	return G_STRUCT_MEMBER_P (self, GeeReadOnlyMultiSet_private_offset);
}

/**
 * Constructs a read-only multi-set that mirrors the content of the specified
 * list.
 *
 * @param multiset the multi-set to decorate.
 */
 G_GNUC_INTERNAL GeeReadOnlyMultiSet*
gee_read_only_multi_set_construct (GType object_type,
                                   GType g_type,
                                   GBoxedCopyFunc g_dup_func,
                                   GDestroyNotify g_destroy_func,
                                   GeeMultiSet* multiset)
{
	GeeReadOnlyMultiSet * self = NULL;
	g_return_val_if_fail (multiset != NULL, NULL);
	self = (GeeReadOnlyMultiSet*) gee_read_only_collection_construct (object_type, g_type, (GBoxedCopyFunc) g_dup_func, (GDestroyNotify) g_destroy_func, (GeeCollection*) multiset);
	self->priv->g_type = g_type;
	self->priv->g_dup_func = g_dup_func;
	self->priv->g_destroy_func = g_destroy_func;
	return self;
}

 G_GNUC_INTERNAL GeeReadOnlyMultiSet*
gee_read_only_multi_set_new (GType g_type,
                             GBoxedCopyFunc g_dup_func,
                             GDestroyNotify g_destroy_func,
                             GeeMultiSet* multiset)
{
	return gee_read_only_multi_set_construct (GEE_TYPE_READ_ONLY_MULTI_SET, g_type, g_dup_func, g_destroy_func, multiset);
}

/**
 * {@inheritDoc}
 */
static gint
gee_read_only_multi_set_real_count (GeeMultiSet* base,
                                    gconstpointer item)
{
	GeeReadOnlyMultiSet * self;
	GeeCollection* _tmp0_;
	gint result;
	self = (GeeReadOnlyMultiSet*) base;
	_tmp0_ = ((GeeReadOnlyCollection*) self)->_collection;
	result = gee_multi_set_count (G_TYPE_CHECK_INSTANCE_CAST (_tmp0_, GEE_TYPE_MULTI_SET, GeeMultiSet), item);
	return result;
}

 G_GNUC_INTERNAL GeeMultiSet*
gee_read_only_multi_set_get_read_only_view (GeeReadOnlyMultiSet* self)
{
	GeeReadOnlyMultiSetClass* _klass_;
	g_return_val_if_fail (self != NULL, NULL);
	_klass_ = GEE_READ_ONLY_MULTI_SET_GET_CLASS (self);
	if (_klass_->get_read_only_view) {
		return _klass_->get_read_only_view (self);
	}
	return NULL;
}

static gpointer
_g_object_ref0 (gpointer self)
{
	return self ? g_object_ref (self) : NULL;
}

static GeeMultiSet*
gee_read_only_multi_set_real_get_read_only_view (GeeReadOnlyMultiSet* base)
{
	GeeMultiSet* result;
	GeeReadOnlyMultiSet* self;
	GeeMultiSet* _tmp0_;
	self = base;
	_tmp0_ = _g_object_ref0 ((GeeMultiSet*) self);
	result = _tmp0_;
	return result;
}

static void
gee_read_only_multi_set_class_init (GeeReadOnlyMultiSetClass * klass,
                                    gpointer klass_data)
{
	gee_read_only_multi_set_parent_class = g_type_class_peek_parent (klass);
	g_type_class_adjust_private_offset (klass, &GeeReadOnlyMultiSet_private_offset);
	GEE_READ_ONLY_MULTI_SET_CLASS (klass)->get_read_only_view = gee_read_only_multi_set_real_get_read_only_view;
	G_OBJECT_CLASS (klass)->get_property = _vala_gee_read_only_multi_set_get_property;
	G_OBJECT_CLASS (klass)->set_property = _vala_gee_read_only_multi_set_set_property;
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_READ_ONLY_MULTI_SET_G_TYPE, g_param_spec_gtype ("g-type", "type", "type", G_TYPE_NONE, G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_READ_ONLY_MULTI_SET_G_DUP_FUNC, g_param_spec_pointer ("g-dup-func", "dup func", "dup func", G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_READ_ONLY_MULTI_SET_G_DESTROY_FUNC, g_param_spec_pointer ("g-destroy-func", "destroy func", "destroy func", G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
}

static GType
gee_read_only_multi_set_gee_multi_set_get_g_type (GeeReadOnlyMultiSet* self)
{
	return self->priv->g_type;
}

static GBoxedCopyFunc
gee_read_only_multi_set_gee_multi_set_get_g_dup_func (GeeReadOnlyMultiSet* self)
{
	return self->priv->g_dup_func;
}

static GDestroyNotify
gee_read_only_multi_set_gee_multi_set_get_g_destroy_func (GeeReadOnlyMultiSet* self)
{
	return self->priv->g_destroy_func;
}

static void
gee_read_only_multi_set_gee_multi_set_interface_init (GeeMultiSetIface * iface,
                                                      gpointer iface_data)
{
	gee_read_only_multi_set_gee_multi_set_parent_iface = g_type_interface_peek_parent (iface);
	iface->count = (gint (*) (GeeMultiSet*, gconstpointer)) gee_read_only_multi_set_real_count;
	iface->get_g_type = (GType (*) (GeeMultiSet *)) gee_read_only_multi_set_gee_multi_set_get_g_type;
	iface->get_g_dup_func = (GBoxedCopyFunc (*) (GeeMultiSet *)) gee_read_only_multi_set_gee_multi_set_get_g_dup_func;
	iface->get_g_destroy_func = (GDestroyNotify (*) (GeeMultiSet *)) gee_read_only_multi_set_gee_multi_set_get_g_destroy_func;
	iface->get_read_only_view = (GeeMultiSet* (*) (GeeMultiSet *)) gee_read_only_multi_set_get_read_only_view;
}

static void
gee_read_only_multi_set_instance_init (GeeReadOnlyMultiSet * self,
                                       gpointer klass)
{
	self->priv = gee_read_only_multi_set_get_instance_private (self);
}

/**
 * Read-only view for {@link MultiSet} collections.
 *
 * This class decorates any class which implements the {@link Collection}
 * interface by making it read only. Any method which normally modify data will
 * throw an error.
 *
 * @see MultiSet
 */
static GType
gee_read_only_multi_set_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (GeeReadOnlyMultiSetClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) gee_read_only_multi_set_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GeeReadOnlyMultiSet), 0, (GInstanceInitFunc) gee_read_only_multi_set_instance_init, NULL };
	static const GInterfaceInfo gee_multi_set_info = { (GInterfaceInitFunc) gee_read_only_multi_set_gee_multi_set_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
	GType gee_read_only_multi_set_type_id;
	gee_read_only_multi_set_type_id = g_type_register_static (GEE_TYPE_READ_ONLY_COLLECTION, "GeeReadOnlyMultiSet", &g_define_type_info, 0);
	g_type_add_interface_static (gee_read_only_multi_set_type_id, GEE_TYPE_MULTI_SET, &gee_multi_set_info);
	GeeReadOnlyMultiSet_private_offset = g_type_add_instance_private (gee_read_only_multi_set_type_id, sizeof (GeeReadOnlyMultiSetPrivate));
	return gee_read_only_multi_set_type_id;
}

 G_GNUC_INTERNAL GType
gee_read_only_multi_set_get_type (void)
{
	static volatile gsize gee_read_only_multi_set_type_id__once = 0;
	if (g_once_init_enter (&gee_read_only_multi_set_type_id__once)) {
		GType gee_read_only_multi_set_type_id;
		gee_read_only_multi_set_type_id = gee_read_only_multi_set_get_type_once ();
		g_once_init_leave (&gee_read_only_multi_set_type_id__once, gee_read_only_multi_set_type_id);
	}
	return gee_read_only_multi_set_type_id__once;
}

static void
_vala_gee_read_only_multi_set_get_property (GObject * object,
                                            guint property_id,
                                            GValue * value,
                                            GParamSpec * pspec)
{
	GeeReadOnlyMultiSet * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, GEE_TYPE_READ_ONLY_MULTI_SET, GeeReadOnlyMultiSet);
	switch (property_id) {
		case GEE_READ_ONLY_MULTI_SET_G_TYPE:
		g_value_set_gtype (value, self->priv->g_type);
		break;
		case GEE_READ_ONLY_MULTI_SET_G_DUP_FUNC:
		g_value_set_pointer (value, self->priv->g_dup_func);
		break;
		case GEE_READ_ONLY_MULTI_SET_G_DESTROY_FUNC:
		g_value_set_pointer (value, self->priv->g_destroy_func);
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

static void
_vala_gee_read_only_multi_set_set_property (GObject * object,
                                            guint property_id,
                                            const GValue * value,
                                            GParamSpec * pspec)
{
	GeeReadOnlyMultiSet * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, GEE_TYPE_READ_ONLY_MULTI_SET, GeeReadOnlyMultiSet);
	switch (property_id) {
		case GEE_READ_ONLY_MULTI_SET_G_TYPE:
		self->priv->g_type = g_value_get_gtype (value);
		break;
		case GEE_READ_ONLY_MULTI_SET_G_DUP_FUNC:
		self->priv->g_dup_func = g_value_get_pointer (value);
		break;
		case GEE_READ_ONLY_MULTI_SET_G_DESTROY_FUNC:
		self->priv->g_destroy_func = g_value_get_pointer (value);
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

