/* abstractmultiset.c generated by valac 0.56.3, the Vala compiler
 * generated from abstractmultiset.vala, do not modify */

/* abstractmultiset.vala
 *
 * Copyright (C) 2009  Ali Sabil
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Ali Sabil <ali.sabil@gmail.com>
 * 	Didier 'Ptitjes Villevalois <ptitjes@free.fr>
 */

#include "gee.h"
#include <glib.h>
#include <glib-object.h>

enum  {
	GEE_ABSTRACT_MULTI_SET_0_PROPERTY,
	GEE_ABSTRACT_MULTI_SET_G_TYPE,
	GEE_ABSTRACT_MULTI_SET_G_DUP_FUNC,
	GEE_ABSTRACT_MULTI_SET_G_DESTROY_FUNC,
	GEE_ABSTRACT_MULTI_SET_SIZE_PROPERTY,
	GEE_ABSTRACT_MULTI_SET_READ_ONLY_PROPERTY,
	GEE_ABSTRACT_MULTI_SET_NUM_PROPERTIES
};
static GParamSpec* gee_abstract_multi_set_properties[GEE_ABSTRACT_MULTI_SET_NUM_PROPERTIES];
#define _g_object_unref0(var) ((var == NULL) ? NULL : (var = (g_object_unref (var), NULL)))

#define GEE_ABSTRACT_MULTI_SET_TYPE_ITERATOR (gee_abstract_multi_set_iterator_get_type ())
#define GEE_ABSTRACT_MULTI_SET_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_ABSTRACT_MULTI_SET_TYPE_ITERATOR, GeeAbstractMultiSetIterator))
#define GEE_ABSTRACT_MULTI_SET_ITERATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_ABSTRACT_MULTI_SET_TYPE_ITERATOR, GeeAbstractMultiSetIteratorClass))
#define GEE_ABSTRACT_MULTI_SET_IS_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_ABSTRACT_MULTI_SET_TYPE_ITERATOR))
#define GEE_ABSTRACT_MULTI_SET_IS_ITERATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEE_ABSTRACT_MULTI_SET_TYPE_ITERATOR))
#define GEE_ABSTRACT_MULTI_SET_ITERATOR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_ABSTRACT_MULTI_SET_TYPE_ITERATOR, GeeAbstractMultiSetIteratorClass))

typedef struct _GeeAbstractMultiSetIterator GeeAbstractMultiSetIterator;
typedef struct _GeeAbstractMultiSetIteratorClass GeeAbstractMultiSetIteratorClass;

#define GEE_TYPE_READ_ONLY_COLLECTION (gee_read_only_collection_get_type ())
#define GEE_READ_ONLY_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_READ_ONLY_COLLECTION, GeeReadOnlyCollection))
#define GEE_READ_ONLY_COLLECTION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_TYPE_READ_ONLY_COLLECTION, GeeReadOnlyCollectionClass))
#define GEE_IS_READ_ONLY_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_READ_ONLY_COLLECTION))
#define GEE_IS_READ_ONLY_COLLECTION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEE_TYPE_READ_ONLY_COLLECTION))
#define GEE_READ_ONLY_COLLECTION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_TYPE_READ_ONLY_COLLECTION, GeeReadOnlyCollectionClass))

typedef struct _GeeReadOnlyCollection GeeReadOnlyCollection;
typedef struct _GeeReadOnlyCollectionClass GeeReadOnlyCollectionClass;

#define GEE_TYPE_READ_ONLY_MULTI_SET (gee_read_only_multi_set_get_type ())
#define GEE_READ_ONLY_MULTI_SET(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_READ_ONLY_MULTI_SET, GeeReadOnlyMultiSet))
#define GEE_READ_ONLY_MULTI_SET_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_TYPE_READ_ONLY_MULTI_SET, GeeReadOnlyMultiSetClass))
#define GEE_IS_READ_ONLY_MULTI_SET(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_READ_ONLY_MULTI_SET))
#define GEE_IS_READ_ONLY_MULTI_SET_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEE_TYPE_READ_ONLY_MULTI_SET))
#define GEE_READ_ONLY_MULTI_SET_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_TYPE_READ_ONLY_MULTI_SET, GeeReadOnlyMultiSetClass))

typedef struct _GeeReadOnlyMultiSet GeeReadOnlyMultiSet;
typedef struct _GeeReadOnlyMultiSetClass GeeReadOnlyMultiSetClass;
typedef struct _GeeAbstractMultiSetIteratorPrivate GeeAbstractMultiSetIteratorPrivate;
enum  {
	GEE_ABSTRACT_MULTI_SET_ITERATOR_0_PROPERTY,
	GEE_ABSTRACT_MULTI_SET_ITERATOR_G_TYPE,
	GEE_ABSTRACT_MULTI_SET_ITERATOR_G_DUP_FUNC,
	GEE_ABSTRACT_MULTI_SET_ITERATOR_G_DESTROY_FUNC,
	GEE_ABSTRACT_MULTI_SET_ITERATOR_READ_ONLY_PROPERTY,
	GEE_ABSTRACT_MULTI_SET_ITERATOR_VALID_PROPERTY,
	GEE_ABSTRACT_MULTI_SET_ITERATOR_NUM_PROPERTIES
};
static GParamSpec* gee_abstract_multi_set_iterator_properties[GEE_ABSTRACT_MULTI_SET_ITERATOR_NUM_PROPERTIES];
#define _vala_assert(expr, msg) if G_LIKELY (expr) ; else g_assertion_message_expr (G_LOG_DOMAIN, __FILE__, __LINE__, G_STRFUNC, msg);
#define _vala_return_if_fail(expr, msg) if G_LIKELY (expr) ; else { g_return_if_fail_warning (G_LOG_DOMAIN, G_STRFUNC, msg); return; }
#define _vala_return_val_if_fail(expr, msg, val) if G_LIKELY (expr) ; else { g_return_if_fail_warning (G_LOG_DOMAIN, G_STRFUNC, msg); return val; }
#define _vala_warn_if_fail(expr, msg) if G_LIKELY (expr) ; else g_warn_message (G_LOG_DOMAIN, __FILE__, __LINE__, G_STRFUNC, msg);

struct _GeeAbstractMultiSetPrivate {
	GType g_type;
	GBoxedCopyFunc g_dup_func;
	GDestroyNotify g_destroy_func;
	gint _nitems;
	GWeakRef _read_only_view;
};

struct _GeeAbstractMultiSetIterator {
	GObject parent_instance;
	GeeAbstractMultiSetIteratorPrivate * priv;
};

struct _GeeAbstractMultiSetIteratorClass {
	GObjectClass parent_class;
};

struct _GeeAbstractMultiSetIteratorPrivate {
	GType g_type;
	GBoxedCopyFunc g_dup_func;
	GDestroyNotify g_destroy_func;
	GeeAbstractMultiSet* _set;
	GeeMapIterator* _iter;
	gint _pending;
	gboolean _removed;
};

static gint GeeAbstractMultiSet_private_offset;
static gpointer gee_abstract_multi_set_parent_class = NULL;
static gint GeeAbstractMultiSetIterator_private_offset;
static gpointer gee_abstract_multi_set_iterator_parent_class = NULL;
static GeeTraversableIface * gee_abstract_multi_set_iterator_gee_traversable_parent_iface = NULL;
static GeeIteratorIface * gee_abstract_multi_set_iterator_gee_iterator_parent_iface = NULL;
static GeeMultiSetIface * gee_abstract_multi_set_gee_multi_set_parent_iface = NULL;

 G_GNUC_INTERNAL void gee_abstract_multi_set_reserved0 (GeeAbstractMultiSet* self);
 G_GNUC_INTERNAL void gee_abstract_multi_set_reserved1 (GeeAbstractMultiSet* self);
 G_GNUC_INTERNAL void gee_abstract_multi_set_reserved2 (GeeAbstractMultiSet* self);
 G_GNUC_INTERNAL void gee_abstract_multi_set_reserved3 (GeeAbstractMultiSet* self);
 G_GNUC_INTERNAL void gee_abstract_multi_set_reserved4 (GeeAbstractMultiSet* self);
 G_GNUC_INTERNAL void gee_abstract_multi_set_reserved5 (GeeAbstractMultiSet* self);
 G_GNUC_INTERNAL void gee_abstract_multi_set_reserved6 (GeeAbstractMultiSet* self);
 G_GNUC_INTERNAL void gee_abstract_multi_set_reserved7 (GeeAbstractMultiSet* self);
 G_GNUC_INTERNAL void gee_abstract_multi_set_reserved8 (GeeAbstractMultiSet* self);
static gint gee_abstract_multi_set_real_count (GeeMultiSet* base,
                                        gconstpointer item);
static gboolean gee_abstract_multi_set_real_contains (GeeAbstractCollection* base,
                                               gconstpointer item);
static GeeIterator* gee_abstract_multi_set_real_iterator (GeeAbstractCollection* base);
static GeeAbstractMultiSetIterator* gee_abstract_multi_set_iterator_new (GType g_type,
                                                                  GBoxedCopyFunc g_dup_func,
                                                                  GDestroyNotify g_destroy_func,
                                                                  GeeAbstractMultiSet* set);
static GeeAbstractMultiSetIterator* gee_abstract_multi_set_iterator_construct (GType object_type,
                                                                        GType g_type,
                                                                        GBoxedCopyFunc g_dup_func,
                                                                        GDestroyNotify g_destroy_func,
                                                                        GeeAbstractMultiSet* set);
static GType gee_abstract_multi_set_iterator_get_type (void) G_GNUC_CONST  G_GNUC_UNUSED ;
static gboolean gee_abstract_multi_set_real_add (GeeAbstractCollection* base,
                                          gconstpointer item);
static gboolean gee_abstract_multi_set_real_remove (GeeAbstractCollection* base,
                                             gconstpointer item);
static void gee_abstract_multi_set_real_clear (GeeAbstractCollection* base);
static void gee_abstract_multi_set_real_reserved0 (GeeAbstractMultiSet* self);
static void gee_abstract_multi_set_real_reserved1 (GeeAbstractMultiSet* self);
static void gee_abstract_multi_set_real_reserved2 (GeeAbstractMultiSet* self);
static void gee_abstract_multi_set_real_reserved3 (GeeAbstractMultiSet* self);
static void gee_abstract_multi_set_real_reserved4 (GeeAbstractMultiSet* self);
static void gee_abstract_multi_set_real_reserved5 (GeeAbstractMultiSet* self);
static void gee_abstract_multi_set_real_reserved6 (GeeAbstractMultiSet* self);
static void gee_abstract_multi_set_real_reserved7 (GeeAbstractMultiSet* self);
static void gee_abstract_multi_set_real_reserved8 (GeeAbstractMultiSet* self);
 G_GNUC_INTERNAL GeeReadOnlyMultiSet* gee_read_only_multi_set_new (GType g_type,
                                                  GBoxedCopyFunc g_dup_func,
                                                  GDestroyNotify g_destroy_func,
                                                  GeeMultiSet* multiset);
 G_GNUC_INTERNAL GeeReadOnlyMultiSet* gee_read_only_multi_set_construct (GType object_type,
                                                        GType g_type,
                                                        GBoxedCopyFunc g_dup_func,
                                                        GDestroyNotify g_destroy_func,
                                                        GeeMultiSet* multiset);
 G_GNUC_INTERNAL GType gee_read_only_collection_get_type (void) G_GNUC_CONST  G_GNUC_UNUSED ;
 G_GNUC_INTERNAL GType gee_read_only_multi_set_get_type (void) G_GNUC_CONST  G_GNUC_UNUSED ;
static gboolean gee_abstract_multi_set_iterator_real_next (GeeIterator* base);
static gboolean gee_abstract_multi_set_iterator_real_has_next (GeeIterator* base);
static gpointer gee_abstract_multi_set_iterator_real_get (GeeIterator* base);
static void gee_abstract_multi_set_iterator_real_remove (GeeIterator* base);
static gboolean gee_abstract_multi_set_iterator_real_foreach (GeeTraversable* base,
                                                       GeeForallFunc f,
                                                       gpointer f_target);
static void gee_abstract_multi_set_iterator_finalize (GObject * obj);
static GType gee_abstract_multi_set_iterator_get_type_once (void);
static void _vala_gee_abstract_multi_set_iterator_get_property (GObject * object,
                                                         guint property_id,
                                                         GValue * value,
                                                         GParamSpec * pspec);
static void _vala_gee_abstract_multi_set_iterator_set_property (GObject * object,
                                                         guint property_id,
                                                         const GValue * value,
                                                         GParamSpec * pspec);
static GObject * gee_abstract_multi_set_constructor (GType type,
                                              guint n_construct_properties,
                                              GObjectConstructParam * construct_properties);
static void gee_abstract_multi_set_finalize (GObject * obj);
static GType gee_abstract_multi_set_get_type_once (void);
static void _vala_gee_abstract_multi_set_get_property (GObject * object,
                                                guint property_id,
                                                GValue * value,
                                                GParamSpec * pspec);
static void _vala_gee_abstract_multi_set_set_property (GObject * object,
                                                guint property_id,
                                                const GValue * value,
                                                GParamSpec * pspec);

static inline gpointer
gee_abstract_multi_set_get_instance_private (GeeAbstractMultiSet* self)
{
	return G_STRUCT_MEMBER_P (self, GeeAbstractMultiSet_private_offset);
}

/**
 * Constructs a new, empty abstract multi set.
 */
static gpointer
_g_object_ref0 (gpointer self)
{
	return self ? g_object_ref (self) : NULL;
}

GeeAbstractMultiSet*
gee_abstract_multi_set_construct (GType object_type,
                                  GType g_type,
                                  GBoxedCopyFunc g_dup_func,
                                  GDestroyNotify g_destroy_func,
                                  GeeMap* storage_map)
{
	GeeAbstractMultiSet * self = NULL;
	GeeMap* _tmp0_;
	g_return_val_if_fail (storage_map != NULL, NULL);
	self = (GeeAbstractMultiSet*) gee_abstract_collection_construct (object_type, g_type, (GBoxedCopyFunc) g_dup_func, (GDestroyNotify) g_destroy_func);
	self->priv->g_type = g_type;
	self->priv->g_dup_func = g_dup_func;
	self->priv->g_destroy_func = g_destroy_func;
	_tmp0_ = _g_object_ref0 (storage_map);
	_g_object_unref0 (self->_storage_map);
	self->_storage_map = _tmp0_;
	return self;
}

static gint
gee_abstract_multi_set_real_count (GeeMultiSet* base,
                                   gconstpointer item)
{
	GeeAbstractMultiSet * self;
	gint _result_ = 0;
	GeeMap* _tmp0_;
	gint result;
	self = (GeeAbstractMultiSet*) base;
	_result_ = 0;
	_tmp0_ = self->_storage_map;
	if (gee_map_has_key (_tmp0_, item)) {
		GeeMap* _tmp1_;
		gpointer _tmp2_;
		_tmp1_ = self->_storage_map;
		_tmp2_ = gee_map_get (_tmp1_, item);
		_result_ = (gint) ((gintptr) _tmp2_);
	}
	result = _result_;
	return result;
}

static gboolean
gee_abstract_multi_set_real_contains (GeeAbstractCollection* base,
                                      gconstpointer item)
{
	GeeAbstractMultiSet * self;
	GeeMap* _tmp0_;
	gboolean result;
	self = (GeeAbstractMultiSet*) base;
	_tmp0_ = self->_storage_map;
	result = gee_map_has_key (_tmp0_, item);
	return result;
}

static GeeIterator*
gee_abstract_multi_set_real_iterator (GeeAbstractCollection* base)
{
	GeeAbstractMultiSet * self;
	GeeAbstractMultiSetIterator* _tmp0_;
	GeeIterator* result;
	self = (GeeAbstractMultiSet*) base;
	_tmp0_ = gee_abstract_multi_set_iterator_new (self->priv->g_type, (GBoxedCopyFunc) self->priv->g_dup_func, (GDestroyNotify) self->priv->g_destroy_func, self);
	result = (GeeIterator*) _tmp0_;
	return result;
}

static gboolean
gee_abstract_multi_set_real_add (GeeAbstractCollection* base,
                                 gconstpointer item)
{
	GeeAbstractMultiSet * self;
	GeeMap* _tmp0_;
	gint _tmp5_;
	gboolean result;
	self = (GeeAbstractMultiSet*) base;
	_tmp0_ = self->_storage_map;
	if (gee_map_has_key (_tmp0_, item)) {
		gint current_count = 0;
		GeeMap* _tmp1_;
		gpointer _tmp2_;
		GeeMap* _tmp3_;
		_tmp1_ = self->_storage_map;
		_tmp2_ = gee_map_get (_tmp1_, item);
		current_count = (gint) ((gintptr) _tmp2_);
		_tmp3_ = self->_storage_map;
		gee_map_set (_tmp3_, item, (gpointer) ((gintptr) (current_count + 1)));
	} else {
		GeeMap* _tmp4_;
		_tmp4_ = self->_storage_map;
		gee_map_set (_tmp4_, item, (gpointer) ((gintptr) 1));
	}
	_tmp5_ = self->priv->_nitems;
	self->priv->_nitems = _tmp5_ + 1;
	result = TRUE;
	return result;
}

static gboolean
gee_abstract_multi_set_real_remove (GeeAbstractCollection* base,
                                    gconstpointer item)
{
	GeeAbstractMultiSet * self;
	gboolean _tmp0_ = FALSE;
	gboolean result;
	self = (GeeAbstractMultiSet*) base;
	if (self->priv->_nitems > 0) {
		GeeMap* _tmp1_;
		_tmp1_ = self->_storage_map;
		_tmp0_ = gee_map_has_key (_tmp1_, item);
	} else {
		_tmp0_ = FALSE;
	}
	if (_tmp0_) {
		gint current_count = 0;
		GeeMap* _tmp2_;
		gpointer _tmp3_;
		gint _tmp6_;
		_tmp2_ = self->_storage_map;
		_tmp3_ = gee_map_get (_tmp2_, item);
		current_count = (gint) ((gintptr) _tmp3_);
		if (current_count <= 1) {
			GeeMap* _tmp4_;
			_tmp4_ = self->_storage_map;
			gee_map_unset (_tmp4_, item, NULL);
		} else {
			GeeMap* _tmp5_;
			_tmp5_ = self->_storage_map;
			gee_map_set (_tmp5_, item, (gpointer) ((gintptr) (current_count - 1)));
		}
		_tmp6_ = self->priv->_nitems;
		self->priv->_nitems = _tmp6_ - 1;
		result = TRUE;
		return result;
	}
	result = FALSE;
	return result;
}

static void
gee_abstract_multi_set_real_clear (GeeAbstractCollection* base)
{
	GeeAbstractMultiSet * self;
	GeeMap* _tmp0_;
	self = (GeeAbstractMultiSet*) base;
	_tmp0_ = self->_storage_map;
	gee_map_clear (_tmp0_);
	self->priv->_nitems = 0;
}

static void
gee_abstract_multi_set_real_reserved0 (GeeAbstractMultiSet* self)
{
}

void
gee_abstract_multi_set_reserved0 (GeeAbstractMultiSet* self)
{
	GeeAbstractMultiSetClass* _klass_;
	g_return_if_fail (self != NULL);
	_klass_ = GEE_ABSTRACT_MULTI_SET_GET_CLASS (self);
	if (_klass_->reserved0) {
		_klass_->reserved0 (self);
	}
}

static void
gee_abstract_multi_set_real_reserved1 (GeeAbstractMultiSet* self)
{
}

void
gee_abstract_multi_set_reserved1 (GeeAbstractMultiSet* self)
{
	GeeAbstractMultiSetClass* _klass_;
	g_return_if_fail (self != NULL);
	_klass_ = GEE_ABSTRACT_MULTI_SET_GET_CLASS (self);
	if (_klass_->reserved1) {
		_klass_->reserved1 (self);
	}
}

static void
gee_abstract_multi_set_real_reserved2 (GeeAbstractMultiSet* self)
{
}

void
gee_abstract_multi_set_reserved2 (GeeAbstractMultiSet* self)
{
	GeeAbstractMultiSetClass* _klass_;
	g_return_if_fail (self != NULL);
	_klass_ = GEE_ABSTRACT_MULTI_SET_GET_CLASS (self);
	if (_klass_->reserved2) {
		_klass_->reserved2 (self);
	}
}

static void
gee_abstract_multi_set_real_reserved3 (GeeAbstractMultiSet* self)
{
}

void
gee_abstract_multi_set_reserved3 (GeeAbstractMultiSet* self)
{
	GeeAbstractMultiSetClass* _klass_;
	g_return_if_fail (self != NULL);
	_klass_ = GEE_ABSTRACT_MULTI_SET_GET_CLASS (self);
	if (_klass_->reserved3) {
		_klass_->reserved3 (self);
	}
}

static void
gee_abstract_multi_set_real_reserved4 (GeeAbstractMultiSet* self)
{
}

void
gee_abstract_multi_set_reserved4 (GeeAbstractMultiSet* self)
{
	GeeAbstractMultiSetClass* _klass_;
	g_return_if_fail (self != NULL);
	_klass_ = GEE_ABSTRACT_MULTI_SET_GET_CLASS (self);
	if (_klass_->reserved4) {
		_klass_->reserved4 (self);
	}
}

static void
gee_abstract_multi_set_real_reserved5 (GeeAbstractMultiSet* self)
{
}

void
gee_abstract_multi_set_reserved5 (GeeAbstractMultiSet* self)
{
	GeeAbstractMultiSetClass* _klass_;
	g_return_if_fail (self != NULL);
	_klass_ = GEE_ABSTRACT_MULTI_SET_GET_CLASS (self);
	if (_klass_->reserved5) {
		_klass_->reserved5 (self);
	}
}

static void
gee_abstract_multi_set_real_reserved6 (GeeAbstractMultiSet* self)
{
}

void
gee_abstract_multi_set_reserved6 (GeeAbstractMultiSet* self)
{
	GeeAbstractMultiSetClass* _klass_;
	g_return_if_fail (self != NULL);
	_klass_ = GEE_ABSTRACT_MULTI_SET_GET_CLASS (self);
	if (_klass_->reserved6) {
		_klass_->reserved6 (self);
	}
}

static void
gee_abstract_multi_set_real_reserved7 (GeeAbstractMultiSet* self)
{
}

void
gee_abstract_multi_set_reserved7 (GeeAbstractMultiSet* self)
{
	GeeAbstractMultiSetClass* _klass_;
	g_return_if_fail (self != NULL);
	_klass_ = GEE_ABSTRACT_MULTI_SET_GET_CLASS (self);
	if (_klass_->reserved7) {
		_klass_->reserved7 (self);
	}
}

static void
gee_abstract_multi_set_real_reserved8 (GeeAbstractMultiSet* self)
{
}

void
gee_abstract_multi_set_reserved8 (GeeAbstractMultiSet* self)
{
	GeeAbstractMultiSetClass* _klass_;
	g_return_if_fail (self != NULL);
	_klass_ = GEE_ABSTRACT_MULTI_SET_GET_CLASS (self);
	if (_klass_->reserved8) {
		_klass_->reserved8 (self);
	}
}

static gint
gee_abstract_multi_set_real_get_size (GeeAbstractCollection* base)
{
	gint result;
	GeeAbstractMultiSet* self;
	self = (GeeAbstractMultiSet*) base;
	result = self->priv->_nitems;
	return result;
}

static gboolean
gee_abstract_multi_set_real_get_read_only (GeeAbstractCollection* base)
{
	gboolean result;
	GeeAbstractMultiSet* self;
	self = (GeeAbstractMultiSet*) base;
	result = FALSE;
	return result;
}

GeeMultiSet*
gee_abstract_multi_set_get_read_only_view (GeeAbstractMultiSet* self)
{
	GeeAbstractMultiSetClass* _klass_;
	g_return_val_if_fail (self != NULL, NULL);
	_klass_ = GEE_ABSTRACT_MULTI_SET_GET_CLASS (self);
	if (_klass_->get_read_only_view) {
		return _klass_->get_read_only_view (self);
	}
	return NULL;
}

static GeeMultiSet*
gee_abstract_multi_set_real_get_read_only_view (GeeAbstractMultiSet* base)
{
	GeeMultiSet* result;
	GeeAbstractMultiSet* self;
	GeeMultiSet* instance = NULL;
	GObject* _tmp0_;
	GeeMultiSet* _tmp1_;
	self = base;
	_tmp0_ = g_weak_ref_get (&self->priv->_read_only_view);
	instance = G_TYPE_CHECK_INSTANCE_CAST (_tmp0_, GEE_TYPE_MULTI_SET, GeeMultiSet);
	_tmp1_ = instance;
	if (_tmp1_ == NULL) {
		GeeReadOnlyMultiSet* _tmp2_;
		GeeMultiSet* _tmp3_;
		_tmp2_ = gee_read_only_multi_set_new (self->priv->g_type, (GBoxedCopyFunc) self->priv->g_dup_func, (GDestroyNotify) self->priv->g_destroy_func, (GeeMultiSet*) self);
		_g_object_unref0 (instance);
		instance = (GeeMultiSet*) _tmp2_;
		_tmp3_ = instance;
		g_weak_ref_set (&self->priv->_read_only_view, (GObject*) _tmp3_);
	}
	result = instance;
	return result;
}

static inline gpointer
gee_abstract_multi_set_iterator_get_instance_private (GeeAbstractMultiSetIterator* self)
{
	return G_STRUCT_MEMBER_P (self, GeeAbstractMultiSetIterator_private_offset);
}

static GeeAbstractMultiSetIterator*
gee_abstract_multi_set_iterator_construct (GType object_type,
                                           GType g_type,
                                           GBoxedCopyFunc g_dup_func,
                                           GDestroyNotify g_destroy_func,
                                           GeeAbstractMultiSet* set)
{
	GeeAbstractMultiSetIterator * self = NULL;
	GeeAbstractMultiSet* _tmp0_;
	GeeAbstractMultiSet* _tmp1_;
	GeeMap* _tmp2_;
	GeeMapIterator* _tmp3_;
	g_return_val_if_fail (set != NULL, NULL);
	self = (GeeAbstractMultiSetIterator*) g_object_new (object_type, "g-type", g_type, "g-dup-func", g_dup_func, "g-destroy-func", g_destroy_func, NULL);
	self->priv->g_type = g_type;
	self->priv->g_dup_func = g_dup_func;
	self->priv->g_destroy_func = g_destroy_func;
	_tmp0_ = _g_object_ref0 (set);
	_g_object_unref0 (self->priv->_set);
	self->priv->_set = _tmp0_;
	_tmp1_ = self->priv->_set;
	_tmp2_ = _tmp1_->_storage_map;
	_tmp3_ = gee_map_map_iterator (_tmp2_);
	_g_object_unref0 (self->priv->_iter);
	self->priv->_iter = _tmp3_;
	return self;
}

static GeeAbstractMultiSetIterator*
gee_abstract_multi_set_iterator_new (GType g_type,
                                     GBoxedCopyFunc g_dup_func,
                                     GDestroyNotify g_destroy_func,
                                     GeeAbstractMultiSet* set)
{
	return gee_abstract_multi_set_iterator_construct (GEE_ABSTRACT_MULTI_SET_TYPE_ITERATOR, g_type, g_dup_func, g_destroy_func, set);
}

static gboolean
gee_abstract_multi_set_iterator_real_next (GeeIterator* base)
{
	GeeAbstractMultiSetIterator * self;
	gboolean result;
	self = (GeeAbstractMultiSetIterator*) base;
	self->priv->_removed = FALSE;
	if (self->priv->_pending == 0) {
		GeeMapIterator* _tmp0_;
		_tmp0_ = self->priv->_iter;
		if (gee_map_iterator_next (_tmp0_)) {
			GeeMapIterator* _tmp1_;
			gpointer _tmp2_;
			_tmp1_ = self->priv->_iter;
			_tmp2_ = gee_map_iterator_get_value (_tmp1_);
			self->priv->_pending = ((gint) ((gintptr) _tmp2_)) - 1;
			result = TRUE;
			return result;
		}
	} else {
		gint _tmp3_;
		_tmp3_ = self->priv->_pending;
		self->priv->_pending = _tmp3_ - 1;
		result = TRUE;
		return result;
	}
	result = FALSE;
	return result;
}

static gboolean
gee_abstract_multi_set_iterator_real_has_next (GeeIterator* base)
{
	GeeAbstractMultiSetIterator * self;
	gboolean _tmp0_ = FALSE;
	gboolean result;
	self = (GeeAbstractMultiSetIterator*) base;
	if (self->priv->_pending > 0) {
		_tmp0_ = TRUE;
	} else {
		GeeMapIterator* _tmp1_;
		_tmp1_ = self->priv->_iter;
		_tmp0_ = gee_map_iterator_has_next (_tmp1_);
	}
	result = _tmp0_;
	return result;
}

static gpointer
gee_abstract_multi_set_iterator_real_get (GeeIterator* base)
{
	GeeAbstractMultiSetIterator * self;
	GeeMapIterator* _tmp0_;
	gpointer _tmp1_;
	gpointer result;
	self = (GeeAbstractMultiSetIterator*) base;
	_vala_assert (!self->priv->_removed, "! _removed");
	_tmp0_ = self->priv->_iter;
	_tmp1_ = gee_map_iterator_get_key (_tmp0_);
	result = _tmp1_;
	return result;
}

static void
gee_abstract_multi_set_iterator_real_remove (GeeIterator* base)
{
	GeeAbstractMultiSetIterator * self;
	GeeMapIterator* _tmp0_;
	GeeMapIterator* _tmp1_;
	gpointer _tmp2_;
	GeeAbstractMultiSet* _tmp4_;
	gint _tmp5_;
	self = (GeeAbstractMultiSetIterator*) base;
	_vala_assert (!self->priv->_removed, "! _removed");
	_tmp0_ = self->priv->_iter;
	_tmp1_ = self->priv->_iter;
	_tmp2_ = gee_map_iterator_get_value (_tmp1_);
	self->priv->_pending = ((gint) ((gintptr) _tmp2_)) - 1;
	gee_map_iterator_set_value (_tmp0_, (gpointer) ((gintptr) self->priv->_pending));
	if (self->priv->_pending == 0) {
		GeeMapIterator* _tmp3_;
		_tmp3_ = self->priv->_iter;
		gee_map_iterator_unset (_tmp3_);
	}
	_tmp4_ = self->priv->_set;
	_tmp5_ = _tmp4_->priv->_nitems;
	_tmp4_->priv->_nitems = _tmp5_ - 1;
	self->priv->_removed = TRUE;
}

static gboolean
gee_abstract_multi_set_iterator_real_foreach (GeeTraversable* base,
                                              GeeForallFunc f,
                                              gpointer f_target)
{
	GeeAbstractMultiSetIterator * self;
	GeeMapIterator* _tmp0_;
	gboolean _tmp1_;
	gboolean _tmp2_;
	gboolean result;
	self = (GeeAbstractMultiSetIterator*) base;
	_tmp0_ = self->priv->_iter;
	_tmp1_ = gee_map_iterator_get_valid (_tmp0_);
	_tmp2_ = _tmp1_;
	if (_tmp2_) {
		if (!self->priv->_removed) {
			GeeMapIterator* _tmp3_;
			gpointer _tmp4_;
			_tmp3_ = self->priv->_iter;
			_tmp4_ = gee_map_iterator_get_key (_tmp3_);
			if (!f (_tmp4_, f_target)) {
				result = FALSE;
				return result;
			}
		}
		{
			gint i = 0;
			i = self->priv->_pending - 1;
			{
				gboolean _tmp5_ = FALSE;
				_tmp5_ = TRUE;
				while (TRUE) {
					GeeMapIterator* _tmp7_;
					gpointer _tmp8_;
					if (!_tmp5_) {
						gint _tmp6_;
						i = i - 1;
						_tmp6_ = i;
					}
					_tmp5_ = FALSE;
					if (!(i >= 0)) {
						break;
					}
					_tmp7_ = self->priv->_iter;
					_tmp8_ = gee_map_iterator_get_key (_tmp7_);
					if (!f (_tmp8_, f_target)) {
						self->priv->_pending = i;
						result = FALSE;
						return result;
					}
				}
			}
		}
	}
	while (TRUE) {
		GeeMapIterator* _tmp9_;
		_tmp9_ = self->priv->_iter;
		if (!gee_map_iterator_next (_tmp9_)) {
			break;
		}
		{
			gint i = 0;
			GeeMapIterator* _tmp10_;
			gpointer _tmp11_;
			_tmp10_ = self->priv->_iter;
			_tmp11_ = gee_map_iterator_get_value (_tmp10_);
			i = ((gint) ((gintptr) _tmp11_)) - 1;
			{
				gboolean _tmp12_ = FALSE;
				_tmp12_ = TRUE;
				while (TRUE) {
					GeeMapIterator* _tmp14_;
					gpointer _tmp15_;
					if (!_tmp12_) {
						gint _tmp13_;
						i = i - 1;
						_tmp13_ = i;
					}
					_tmp12_ = FALSE;
					if (!(i >= 0)) {
						break;
					}
					_tmp14_ = self->priv->_iter;
					_tmp15_ = gee_map_iterator_get_key (_tmp14_);
					if (!f (_tmp15_, f_target)) {
						self->priv->_removed = FALSE;
						self->priv->_pending = i;
						result = FALSE;
						return result;
					}
				}
			}
		}
	}
	self->priv->_removed = FALSE;
	self->priv->_pending = 0;
	result = TRUE;
	return result;
}

static gboolean
gee_abstract_multi_set_iterator_real_get_read_only (GeeIterator* base)
{
	gboolean result;
	GeeAbstractMultiSetIterator* self;
	self = (GeeAbstractMultiSetIterator*) base;
	result = FALSE;
	return result;
}

static gboolean
gee_abstract_multi_set_iterator_real_get_valid (GeeIterator* base)
{
	gboolean result;
	GeeAbstractMultiSetIterator* self;
	gboolean _tmp0_ = FALSE;
	self = (GeeAbstractMultiSetIterator*) base;
	if (!self->priv->_removed) {
		GeeMapIterator* _tmp1_;
		gboolean _tmp2_;
		gboolean _tmp3_;
		_tmp1_ = self->priv->_iter;
		_tmp2_ = gee_map_iterator_get_valid (_tmp1_);
		_tmp3_ = _tmp2_;
		_tmp0_ = _tmp3_;
	} else {
		_tmp0_ = FALSE;
	}
	result = _tmp0_;
	return result;
}

static void
gee_abstract_multi_set_iterator_class_init (GeeAbstractMultiSetIteratorClass * klass,
                                            gpointer klass_data)
{
	gee_abstract_multi_set_iterator_parent_class = g_type_class_peek_parent (klass);
	g_type_class_adjust_private_offset (klass, &GeeAbstractMultiSetIterator_private_offset);
	G_OBJECT_CLASS (klass)->get_property = _vala_gee_abstract_multi_set_iterator_get_property;
	G_OBJECT_CLASS (klass)->set_property = _vala_gee_abstract_multi_set_iterator_set_property;
	G_OBJECT_CLASS (klass)->finalize = gee_abstract_multi_set_iterator_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_ABSTRACT_MULTI_SET_ITERATOR_G_TYPE, g_param_spec_gtype ("g-type", "type", "type", G_TYPE_NONE, G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_ABSTRACT_MULTI_SET_ITERATOR_G_DUP_FUNC, g_param_spec_pointer ("g-dup-func", "dup func", "dup func", G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_ABSTRACT_MULTI_SET_ITERATOR_G_DESTROY_FUNC, g_param_spec_pointer ("g-destroy-func", "destroy func", "destroy func", G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_ABSTRACT_MULTI_SET_ITERATOR_READ_ONLY_PROPERTY, gee_abstract_multi_set_iterator_properties[GEE_ABSTRACT_MULTI_SET_ITERATOR_READ_ONLY_PROPERTY] = g_param_spec_boolean ("read-only", "read-only", "read-only", FALSE, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_ABSTRACT_MULTI_SET_ITERATOR_VALID_PROPERTY, gee_abstract_multi_set_iterator_properties[GEE_ABSTRACT_MULTI_SET_ITERATOR_VALID_PROPERTY] = g_param_spec_boolean ("valid", "valid", "valid", FALSE, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE));
}

static GType
gee_abstract_multi_set_iterator_gee_traversable_get_g_type (GeeAbstractMultiSetIterator* self)
{
	return self->priv->g_type;
}

static GBoxedCopyFunc
gee_abstract_multi_set_iterator_gee_traversable_get_g_dup_func (GeeAbstractMultiSetIterator* self)
{
	return self->priv->g_dup_func;
}

static GDestroyNotify
gee_abstract_multi_set_iterator_gee_traversable_get_g_destroy_func (GeeAbstractMultiSetIterator* self)
{
	return self->priv->g_destroy_func;
}

static void
gee_abstract_multi_set_iterator_gee_traversable_interface_init (GeeTraversableIface * iface,
                                                                gpointer iface_data)
{
	gee_abstract_multi_set_iterator_gee_traversable_parent_iface = g_type_interface_peek_parent (iface);
	iface->foreach = (gboolean (*) (GeeTraversable*, GeeForallFunc, gpointer)) gee_abstract_multi_set_iterator_real_foreach;
	iface->get_g_type = (GType (*) (GeeTraversable *)) gee_abstract_multi_set_iterator_gee_traversable_get_g_type;
	iface->get_g_dup_func = (GBoxedCopyFunc (*) (GeeTraversable *)) gee_abstract_multi_set_iterator_gee_traversable_get_g_dup_func;
	iface->get_g_destroy_func = (GDestroyNotify (*) (GeeTraversable *)) gee_abstract_multi_set_iterator_gee_traversable_get_g_destroy_func;
}

static void
gee_abstract_multi_set_iterator_gee_iterator_interface_init (GeeIteratorIface * iface,
                                                             gpointer iface_data)
{
	gee_abstract_multi_set_iterator_gee_iterator_parent_iface = g_type_interface_peek_parent (iface);
	iface->next = (gboolean (*) (GeeIterator*)) gee_abstract_multi_set_iterator_real_next;
	iface->has_next = (gboolean (*) (GeeIterator*)) gee_abstract_multi_set_iterator_real_has_next;
	iface->get = (gpointer (*) (GeeIterator*)) gee_abstract_multi_set_iterator_real_get;
	iface->remove = (void (*) (GeeIterator*)) gee_abstract_multi_set_iterator_real_remove;
	iface->get_read_only = gee_abstract_multi_set_iterator_real_get_read_only;
	iface->get_valid = gee_abstract_multi_set_iterator_real_get_valid;
}

static void
gee_abstract_multi_set_iterator_instance_init (GeeAbstractMultiSetIterator * self,
                                               gpointer klass)
{
	self->priv = gee_abstract_multi_set_iterator_get_instance_private (self);
	self->priv->_pending = 0;
	self->priv->_removed = FALSE;
}

static void
gee_abstract_multi_set_iterator_finalize (GObject * obj)
{
	GeeAbstractMultiSetIterator * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, GEE_ABSTRACT_MULTI_SET_TYPE_ITERATOR, GeeAbstractMultiSetIterator);
	_g_object_unref0 (self->priv->_set);
	_g_object_unref0 (self->priv->_iter);
	G_OBJECT_CLASS (gee_abstract_multi_set_iterator_parent_class)->finalize (obj);
}

static GType
gee_abstract_multi_set_iterator_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (GeeAbstractMultiSetIteratorClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) gee_abstract_multi_set_iterator_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GeeAbstractMultiSetIterator), 0, (GInstanceInitFunc) gee_abstract_multi_set_iterator_instance_init, NULL };
	static const GInterfaceInfo gee_traversable_info = { (GInterfaceInitFunc) gee_abstract_multi_set_iterator_gee_traversable_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
	static const GInterfaceInfo gee_iterator_info = { (GInterfaceInitFunc) gee_abstract_multi_set_iterator_gee_iterator_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
	GType gee_abstract_multi_set_iterator_type_id;
	gee_abstract_multi_set_iterator_type_id = g_type_register_static (G_TYPE_OBJECT, "GeeAbstractMultiSetIterator", &g_define_type_info, 0);
	g_type_add_interface_static (gee_abstract_multi_set_iterator_type_id, GEE_TYPE_TRAVERSABLE, &gee_traversable_info);
	g_type_add_interface_static (gee_abstract_multi_set_iterator_type_id, GEE_TYPE_ITERATOR, &gee_iterator_info);
	GeeAbstractMultiSetIterator_private_offset = g_type_add_instance_private (gee_abstract_multi_set_iterator_type_id, sizeof (GeeAbstractMultiSetIteratorPrivate));
	return gee_abstract_multi_set_iterator_type_id;
}

static GType
gee_abstract_multi_set_iterator_get_type (void)
{
	static volatile gsize gee_abstract_multi_set_iterator_type_id__once = 0;
	if (g_once_init_enter (&gee_abstract_multi_set_iterator_type_id__once)) {
		GType gee_abstract_multi_set_iterator_type_id;
		gee_abstract_multi_set_iterator_type_id = gee_abstract_multi_set_iterator_get_type_once ();
		g_once_init_leave (&gee_abstract_multi_set_iterator_type_id__once, gee_abstract_multi_set_iterator_type_id);
	}
	return gee_abstract_multi_set_iterator_type_id__once;
}

static void
_vala_gee_abstract_multi_set_iterator_get_property (GObject * object,
                                                    guint property_id,
                                                    GValue * value,
                                                    GParamSpec * pspec)
{
	GeeAbstractMultiSetIterator * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, GEE_ABSTRACT_MULTI_SET_TYPE_ITERATOR, GeeAbstractMultiSetIterator);
	switch (property_id) {
		case GEE_ABSTRACT_MULTI_SET_ITERATOR_READ_ONLY_PROPERTY:
		g_value_set_boolean (value, gee_iterator_get_read_only ((GeeIterator*) self));
		break;
		case GEE_ABSTRACT_MULTI_SET_ITERATOR_VALID_PROPERTY:
		g_value_set_boolean (value, gee_iterator_get_valid ((GeeIterator*) self));
		break;
		case GEE_ABSTRACT_MULTI_SET_ITERATOR_G_TYPE:
		g_value_set_gtype (value, self->priv->g_type);
		break;
		case GEE_ABSTRACT_MULTI_SET_ITERATOR_G_DUP_FUNC:
		g_value_set_pointer (value, self->priv->g_dup_func);
		break;
		case GEE_ABSTRACT_MULTI_SET_ITERATOR_G_DESTROY_FUNC:
		g_value_set_pointer (value, self->priv->g_destroy_func);
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

static void
_vala_gee_abstract_multi_set_iterator_set_property (GObject * object,
                                                    guint property_id,
                                                    const GValue * value,
                                                    GParamSpec * pspec)
{
	GeeAbstractMultiSetIterator * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, GEE_ABSTRACT_MULTI_SET_TYPE_ITERATOR, GeeAbstractMultiSetIterator);
	switch (property_id) {
		case GEE_ABSTRACT_MULTI_SET_ITERATOR_G_TYPE:
		self->priv->g_type = g_value_get_gtype (value);
		break;
		case GEE_ABSTRACT_MULTI_SET_ITERATOR_G_DUP_FUNC:
		self->priv->g_dup_func = g_value_get_pointer (value);
		break;
		case GEE_ABSTRACT_MULTI_SET_ITERATOR_G_DESTROY_FUNC:
		self->priv->g_destroy_func = g_value_get_pointer (value);
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

static GObject *
gee_abstract_multi_set_constructor (GType type,
                                    guint n_construct_properties,
                                    GObjectConstructParam * construct_properties)
{
	GObject * obj;
	GObjectClass * parent_class;
	GeeAbstractMultiSet * self;
	parent_class = G_OBJECT_CLASS (gee_abstract_multi_set_parent_class);
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, GEE_TYPE_ABSTRACT_MULTI_SET, GeeAbstractMultiSet);
	g_weak_ref_clear (&self->priv->_read_only_view);
	g_weak_ref_init (&self->priv->_read_only_view, NULL);
	return obj;
}

static void
gee_abstract_multi_set_class_init (GeeAbstractMultiSetClass * klass,
                                   gpointer klass_data)
{
	gee_abstract_multi_set_parent_class = g_type_class_peek_parent (klass);
	g_type_class_adjust_private_offset (klass, &GeeAbstractMultiSet_private_offset);
	((GeeAbstractCollectionClass *) klass)->contains = (gboolean (*) (GeeAbstractCollection*, gconstpointer)) gee_abstract_multi_set_real_contains;
	((GeeAbstractCollectionClass *) klass)->iterator = (GeeIterator* (*) (GeeAbstractCollection*)) gee_abstract_multi_set_real_iterator;
	((GeeAbstractCollectionClass *) klass)->add = (gboolean (*) (GeeAbstractCollection*, gconstpointer)) gee_abstract_multi_set_real_add;
	((GeeAbstractCollectionClass *) klass)->remove = (gboolean (*) (GeeAbstractCollection*, gconstpointer)) gee_abstract_multi_set_real_remove;
	((GeeAbstractCollectionClass *) klass)->clear = (void (*) (GeeAbstractCollection*)) gee_abstract_multi_set_real_clear;
	((GeeAbstractMultiSetClass *) klass)->reserved0 = (void (*) (GeeAbstractMultiSet*)) gee_abstract_multi_set_real_reserved0;
	((GeeAbstractMultiSetClass *) klass)->reserved1 = (void (*) (GeeAbstractMultiSet*)) gee_abstract_multi_set_real_reserved1;
	((GeeAbstractMultiSetClass *) klass)->reserved2 = (void (*) (GeeAbstractMultiSet*)) gee_abstract_multi_set_real_reserved2;
	((GeeAbstractMultiSetClass *) klass)->reserved3 = (void (*) (GeeAbstractMultiSet*)) gee_abstract_multi_set_real_reserved3;
	((GeeAbstractMultiSetClass *) klass)->reserved4 = (void (*) (GeeAbstractMultiSet*)) gee_abstract_multi_set_real_reserved4;
	((GeeAbstractMultiSetClass *) klass)->reserved5 = (void (*) (GeeAbstractMultiSet*)) gee_abstract_multi_set_real_reserved5;
	((GeeAbstractMultiSetClass *) klass)->reserved6 = (void (*) (GeeAbstractMultiSet*)) gee_abstract_multi_set_real_reserved6;
	((GeeAbstractMultiSetClass *) klass)->reserved7 = (void (*) (GeeAbstractMultiSet*)) gee_abstract_multi_set_real_reserved7;
	((GeeAbstractMultiSetClass *) klass)->reserved8 = (void (*) (GeeAbstractMultiSet*)) gee_abstract_multi_set_real_reserved8;
	GEE_ABSTRACT_COLLECTION_CLASS (klass)->get_size = gee_abstract_multi_set_real_get_size;
	GEE_ABSTRACT_COLLECTION_CLASS (klass)->get_read_only = gee_abstract_multi_set_real_get_read_only;
	GEE_ABSTRACT_MULTI_SET_CLASS (klass)->get_read_only_view = gee_abstract_multi_set_real_get_read_only_view;
	G_OBJECT_CLASS (klass)->get_property = _vala_gee_abstract_multi_set_get_property;
	G_OBJECT_CLASS (klass)->set_property = _vala_gee_abstract_multi_set_set_property;
	G_OBJECT_CLASS (klass)->constructor = gee_abstract_multi_set_constructor;
	G_OBJECT_CLASS (klass)->finalize = gee_abstract_multi_set_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_ABSTRACT_MULTI_SET_G_TYPE, g_param_spec_gtype ("g-type", "type", "type", G_TYPE_NONE, G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_ABSTRACT_MULTI_SET_G_DUP_FUNC, g_param_spec_pointer ("g-dup-func", "dup func", "dup func", G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_ABSTRACT_MULTI_SET_G_DESTROY_FUNC, g_param_spec_pointer ("g-destroy-func", "destroy func", "destroy func", G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_ABSTRACT_MULTI_SET_SIZE_PROPERTY, gee_abstract_multi_set_properties[GEE_ABSTRACT_MULTI_SET_SIZE_PROPERTY] = g_param_spec_int ("size", "size", "size", G_MININT, G_MAXINT, 0, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_ABSTRACT_MULTI_SET_READ_ONLY_PROPERTY, gee_abstract_multi_set_properties[GEE_ABSTRACT_MULTI_SET_READ_ONLY_PROPERTY] = g_param_spec_boolean ("read-only", "read-only", "read-only", FALSE, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE));
}

static GType
gee_abstract_multi_set_gee_multi_set_get_g_type (GeeAbstractMultiSet* self)
{
	return self->priv->g_type;
}

static GBoxedCopyFunc
gee_abstract_multi_set_gee_multi_set_get_g_dup_func (GeeAbstractMultiSet* self)
{
	return self->priv->g_dup_func;
}

static GDestroyNotify
gee_abstract_multi_set_gee_multi_set_get_g_destroy_func (GeeAbstractMultiSet* self)
{
	return self->priv->g_destroy_func;
}

static void
gee_abstract_multi_set_gee_multi_set_interface_init (GeeMultiSetIface * iface,
                                                     gpointer iface_data)
{
	gee_abstract_multi_set_gee_multi_set_parent_iface = g_type_interface_peek_parent (iface);
	iface->count = (gint (*) (GeeMultiSet*, gconstpointer)) gee_abstract_multi_set_real_count;
	iface->get_g_type = (GType (*) (GeeMultiSet *)) gee_abstract_multi_set_gee_multi_set_get_g_type;
	iface->get_g_dup_func = (GBoxedCopyFunc (*) (GeeMultiSet *)) gee_abstract_multi_set_gee_multi_set_get_g_dup_func;
	iface->get_g_destroy_func = (GDestroyNotify (*) (GeeMultiSet *)) gee_abstract_multi_set_gee_multi_set_get_g_destroy_func;
	iface->get_read_only_view = (GeeMultiSet* (*) (GeeMultiSet *)) gee_abstract_multi_set_get_read_only_view;
}

static void
gee_abstract_multi_set_instance_init (GeeAbstractMultiSet * self,
                                      gpointer klass)
{
	self->priv = gee_abstract_multi_set_get_instance_private (self);
	self->priv->_nitems = 0;
}

static void
gee_abstract_multi_set_finalize (GObject * obj)
{
	GeeAbstractMultiSet * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, GEE_TYPE_ABSTRACT_MULTI_SET, GeeAbstractMultiSet);
	_g_object_unref0 (self->_storage_map);
	g_weak_ref_clear (&self->priv->_read_only_view);
	G_OBJECT_CLASS (gee_abstract_multi_set_parent_class)->finalize (obj);
}

/**
 * Skeletal implementation of the {@link MultiSet} interface.
 *
 * @see HashMultiSet
 * @see TreeMultiSet
 */
static GType
gee_abstract_multi_set_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (GeeAbstractMultiSetClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) gee_abstract_multi_set_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GeeAbstractMultiSet), 0, (GInstanceInitFunc) gee_abstract_multi_set_instance_init, NULL };
	static const GInterfaceInfo gee_multi_set_info = { (GInterfaceInitFunc) gee_abstract_multi_set_gee_multi_set_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
	GType gee_abstract_multi_set_type_id;
	gee_abstract_multi_set_type_id = g_type_register_static (GEE_TYPE_ABSTRACT_COLLECTION, "GeeAbstractMultiSet", &g_define_type_info, G_TYPE_FLAG_ABSTRACT);
	g_type_add_interface_static (gee_abstract_multi_set_type_id, GEE_TYPE_MULTI_SET, &gee_multi_set_info);
	GeeAbstractMultiSet_private_offset = g_type_add_instance_private (gee_abstract_multi_set_type_id, sizeof (GeeAbstractMultiSetPrivate));
	return gee_abstract_multi_set_type_id;
}

GType
gee_abstract_multi_set_get_type (void)
{
	static volatile gsize gee_abstract_multi_set_type_id__once = 0;
	if (g_once_init_enter (&gee_abstract_multi_set_type_id__once)) {
		GType gee_abstract_multi_set_type_id;
		gee_abstract_multi_set_type_id = gee_abstract_multi_set_get_type_once ();
		g_once_init_leave (&gee_abstract_multi_set_type_id__once, gee_abstract_multi_set_type_id);
	}
	return gee_abstract_multi_set_type_id__once;
}

static void
_vala_gee_abstract_multi_set_get_property (GObject * object,
                                           guint property_id,
                                           GValue * value,
                                           GParamSpec * pspec)
{
	GeeAbstractMultiSet * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, GEE_TYPE_ABSTRACT_MULTI_SET, GeeAbstractMultiSet);
	switch (property_id) {
		case GEE_ABSTRACT_MULTI_SET_SIZE_PROPERTY:
		g_value_set_int (value, gee_abstract_collection_get_size ((GeeAbstractCollection*) self));
		break;
		case GEE_ABSTRACT_MULTI_SET_READ_ONLY_PROPERTY:
		g_value_set_boolean (value, gee_abstract_collection_get_read_only ((GeeAbstractCollection*) self));
		break;
		case GEE_ABSTRACT_MULTI_SET_G_TYPE:
		g_value_set_gtype (value, self->priv->g_type);
		break;
		case GEE_ABSTRACT_MULTI_SET_G_DUP_FUNC:
		g_value_set_pointer (value, self->priv->g_dup_func);
		break;
		case GEE_ABSTRACT_MULTI_SET_G_DESTROY_FUNC:
		g_value_set_pointer (value, self->priv->g_destroy_func);
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

static void
_vala_gee_abstract_multi_set_set_property (GObject * object,
                                           guint property_id,
                                           const GValue * value,
                                           GParamSpec * pspec)
{
	GeeAbstractMultiSet * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, GEE_TYPE_ABSTRACT_MULTI_SET, GeeAbstractMultiSet);
	switch (property_id) {
		case GEE_ABSTRACT_MULTI_SET_G_TYPE:
		self->priv->g_type = g_value_get_gtype (value);
		break;
		case GEE_ABSTRACT_MULTI_SET_G_DUP_FUNC:
		self->priv->g_dup_func = g_value_get_pointer (value);
		break;
		case GEE_ABSTRACT_MULTI_SET_G_DESTROY_FUNC:
		self->priv->g_destroy_func = g_value_get_pointer (value);
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

