/* readonlybidirsortedmap.c generated by valac 0.56.3, the Vala compiler
 * generated from readonlybidirsortedmap.vala, do not modify */

/* readonlybidirsortedmap.vala
 *
 * Copyright (C) 2012  Maciej Piechotka
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Maciej Piechotka <uzytkownik2@gmail.com>
 */

#include <glib-object.h>
#include "gee.h"
#include <glib.h>

#define GEE_TYPE_READ_ONLY_MAP (gee_read_only_map_get_type ())
#define GEE_READ_ONLY_MAP(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_READ_ONLY_MAP, GeeReadOnlyMap))
#define GEE_READ_ONLY_MAP_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_TYPE_READ_ONLY_MAP, GeeReadOnlyMapClass))
#define GEE_IS_READ_ONLY_MAP(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_READ_ONLY_MAP))
#define GEE_IS_READ_ONLY_MAP_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEE_TYPE_READ_ONLY_MAP))
#define GEE_READ_ONLY_MAP_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_TYPE_READ_ONLY_MAP, GeeReadOnlyMapClass))

typedef struct _GeeReadOnlyMap GeeReadOnlyMap;
typedef struct _GeeReadOnlyMapClass GeeReadOnlyMapClass;
typedef struct _GeeReadOnlyMapPrivate GeeReadOnlyMapPrivate;

#define GEE_TYPE_READ_ONLY_SORTED_MAP (gee_read_only_sorted_map_get_type ())
#define GEE_READ_ONLY_SORTED_MAP(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_READ_ONLY_SORTED_MAP, GeeReadOnlySortedMap))
#define GEE_READ_ONLY_SORTED_MAP_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_TYPE_READ_ONLY_SORTED_MAP, GeeReadOnlySortedMapClass))
#define GEE_IS_READ_ONLY_SORTED_MAP(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_READ_ONLY_SORTED_MAP))
#define GEE_IS_READ_ONLY_SORTED_MAP_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEE_TYPE_READ_ONLY_SORTED_MAP))
#define GEE_READ_ONLY_SORTED_MAP_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_TYPE_READ_ONLY_SORTED_MAP, GeeReadOnlySortedMapClass))

typedef struct _GeeReadOnlySortedMap GeeReadOnlySortedMap;
typedef struct _GeeReadOnlySortedMapClass GeeReadOnlySortedMapClass;
typedef struct _GeeReadOnlySortedMapPrivate GeeReadOnlySortedMapPrivate;

#define GEE_TYPE_READ_ONLY_BIDIR_SORTED_MAP (gee_read_only_bidir_sorted_map_get_type ())
#define GEE_READ_ONLY_BIDIR_SORTED_MAP(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_READ_ONLY_BIDIR_SORTED_MAP, GeeReadOnlyBidirSortedMap))
#define GEE_READ_ONLY_BIDIR_SORTED_MAP_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_TYPE_READ_ONLY_BIDIR_SORTED_MAP, GeeReadOnlyBidirSortedMapClass))
#define GEE_IS_READ_ONLY_BIDIR_SORTED_MAP(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_READ_ONLY_BIDIR_SORTED_MAP))
#define GEE_IS_READ_ONLY_BIDIR_SORTED_MAP_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEE_TYPE_READ_ONLY_BIDIR_SORTED_MAP))
#define GEE_READ_ONLY_BIDIR_SORTED_MAP_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_TYPE_READ_ONLY_BIDIR_SORTED_MAP, GeeReadOnlyBidirSortedMapClass))

typedef struct _GeeReadOnlyBidirSortedMap GeeReadOnlyBidirSortedMap;
typedef struct _GeeReadOnlyBidirSortedMapClass GeeReadOnlyBidirSortedMapClass;
typedef struct _GeeReadOnlyBidirSortedMapPrivate GeeReadOnlyBidirSortedMapPrivate;
enum  {
	GEE_READ_ONLY_BIDIR_SORTED_MAP_0_PROPERTY,
	GEE_READ_ONLY_BIDIR_SORTED_MAP_K_TYPE,
	GEE_READ_ONLY_BIDIR_SORTED_MAP_K_DUP_FUNC,
	GEE_READ_ONLY_BIDIR_SORTED_MAP_K_DESTROY_FUNC,
	GEE_READ_ONLY_BIDIR_SORTED_MAP_V_TYPE,
	GEE_READ_ONLY_BIDIR_SORTED_MAP_V_DUP_FUNC,
	GEE_READ_ONLY_BIDIR_SORTED_MAP_V_DESTROY_FUNC,
	GEE_READ_ONLY_BIDIR_SORTED_MAP_READ_ONLY_VIEW_PROPERTY,
	GEE_READ_ONLY_BIDIR_SORTED_MAP_NUM_PROPERTIES
};
static GParamSpec* gee_read_only_bidir_sorted_map_properties[GEE_READ_ONLY_BIDIR_SORTED_MAP_NUM_PROPERTIES];

#define GEE_READ_ONLY_MAP_TYPE_MAP_ITERATOR (gee_read_only_map_map_iterator_get_type ())
#define GEE_READ_ONLY_MAP_MAP_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_READ_ONLY_MAP_TYPE_MAP_ITERATOR, GeeReadOnlyMapMapIterator))
#define GEE_READ_ONLY_MAP_MAP_ITERATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_READ_ONLY_MAP_TYPE_MAP_ITERATOR, GeeReadOnlyMapMapIteratorClass))
#define GEE_READ_ONLY_MAP_IS_MAP_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_READ_ONLY_MAP_TYPE_MAP_ITERATOR))
#define GEE_READ_ONLY_MAP_IS_MAP_ITERATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEE_READ_ONLY_MAP_TYPE_MAP_ITERATOR))
#define GEE_READ_ONLY_MAP_MAP_ITERATOR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_READ_ONLY_MAP_TYPE_MAP_ITERATOR, GeeReadOnlyMapMapIteratorClass))

typedef struct _GeeReadOnlyMapMapIterator GeeReadOnlyMapMapIterator;
typedef struct _GeeReadOnlyMapMapIteratorClass GeeReadOnlyMapMapIteratorClass;

#define GEE_READ_ONLY_BIDIR_SORTED_MAP_TYPE_BIDIR_MAP_ITERATOR (gee_read_only_bidir_sorted_map_bidir_map_iterator_get_type ())
#define GEE_READ_ONLY_BIDIR_SORTED_MAP_BIDIR_MAP_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_READ_ONLY_BIDIR_SORTED_MAP_TYPE_BIDIR_MAP_ITERATOR, GeeReadOnlyBidirSortedMapBidirMapIterator))
#define GEE_READ_ONLY_BIDIR_SORTED_MAP_BIDIR_MAP_ITERATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_READ_ONLY_BIDIR_SORTED_MAP_TYPE_BIDIR_MAP_ITERATOR, GeeReadOnlyBidirSortedMapBidirMapIteratorClass))
#define GEE_READ_ONLY_BIDIR_SORTED_MAP_IS_BIDIR_MAP_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_READ_ONLY_BIDIR_SORTED_MAP_TYPE_BIDIR_MAP_ITERATOR))
#define GEE_READ_ONLY_BIDIR_SORTED_MAP_IS_BIDIR_MAP_ITERATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEE_READ_ONLY_BIDIR_SORTED_MAP_TYPE_BIDIR_MAP_ITERATOR))
#define GEE_READ_ONLY_BIDIR_SORTED_MAP_BIDIR_MAP_ITERATOR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_READ_ONLY_BIDIR_SORTED_MAP_TYPE_BIDIR_MAP_ITERATOR, GeeReadOnlyBidirSortedMapBidirMapIteratorClass))

typedef struct _GeeReadOnlyBidirSortedMapBidirMapIterator GeeReadOnlyBidirSortedMapBidirMapIterator;
typedef struct _GeeReadOnlyBidirSortedMapBidirMapIteratorClass GeeReadOnlyBidirSortedMapBidirMapIteratorClass;
#define _g_object_unref0(var) ((var == NULL) ? NULL : (var = (g_object_unref (var), NULL)))
typedef struct _GeeReadOnlyMapMapIteratorPrivate GeeReadOnlyMapMapIteratorPrivate;
typedef struct _GeeReadOnlyBidirSortedMapBidirMapIteratorPrivate GeeReadOnlyBidirSortedMapBidirMapIteratorPrivate;
enum  {
	GEE_READ_ONLY_BIDIR_SORTED_MAP_BIDIR_MAP_ITERATOR_0_PROPERTY,
	GEE_READ_ONLY_BIDIR_SORTED_MAP_BIDIR_MAP_ITERATOR_K_TYPE,
	GEE_READ_ONLY_BIDIR_SORTED_MAP_BIDIR_MAP_ITERATOR_K_DUP_FUNC,
	GEE_READ_ONLY_BIDIR_SORTED_MAP_BIDIR_MAP_ITERATOR_K_DESTROY_FUNC,
	GEE_READ_ONLY_BIDIR_SORTED_MAP_BIDIR_MAP_ITERATOR_V_TYPE,
	GEE_READ_ONLY_BIDIR_SORTED_MAP_BIDIR_MAP_ITERATOR_V_DUP_FUNC,
	GEE_READ_ONLY_BIDIR_SORTED_MAP_BIDIR_MAP_ITERATOR_V_DESTROY_FUNC,
	GEE_READ_ONLY_BIDIR_SORTED_MAP_BIDIR_MAP_ITERATOR_NUM_PROPERTIES
};
static GParamSpec* gee_read_only_bidir_sorted_map_bidir_map_iterator_properties[GEE_READ_ONLY_BIDIR_SORTED_MAP_BIDIR_MAP_ITERATOR_NUM_PROPERTIES];

struct _GeeReadOnlyMap {
	GObject parent_instance;
	GeeReadOnlyMapPrivate * priv;
	GeeMap* _map;
};

struct _GeeReadOnlyMapClass {
	GObjectClass parent_class;
	GeeMap* (*get_read_only_view) (GeeReadOnlyMap* self);
};

struct _GeeReadOnlySortedMap {
	GeeReadOnlyMap parent_instance;
	GeeReadOnlySortedMapPrivate * priv;
};

struct _GeeReadOnlySortedMapClass {
	GeeReadOnlyMapClass parent_class;
};

struct _GeeReadOnlyBidirSortedMap {
	GeeReadOnlySortedMap parent_instance;
	GeeReadOnlyBidirSortedMapPrivate * priv;
};

struct _GeeReadOnlyBidirSortedMapClass {
	GeeReadOnlySortedMapClass parent_class;
};

struct _GeeReadOnlyBidirSortedMapPrivate {
	GType k_type;
	GBoxedCopyFunc k_dup_func;
	GDestroyNotify k_destroy_func;
	GType v_type;
	GBoxedCopyFunc v_dup_func;
	GDestroyNotify v_destroy_func;
};

struct _GeeReadOnlyMapMapIterator {
	GObject parent_instance;
	GeeReadOnlyMapMapIteratorPrivate * priv;
	GeeMapIterator* _iter;
};

struct _GeeReadOnlyMapMapIteratorClass {
	GObjectClass parent_class;
};

struct _GeeReadOnlyBidirSortedMapBidirMapIterator {
	GeeReadOnlyMapMapIterator parent_instance;
	GeeReadOnlyBidirSortedMapBidirMapIteratorPrivate * priv;
};

struct _GeeReadOnlyBidirSortedMapBidirMapIteratorClass {
	GeeReadOnlyMapMapIteratorClass parent_class;
};

struct _GeeReadOnlyBidirSortedMapBidirMapIteratorPrivate {
	GType k_type;
	GBoxedCopyFunc k_dup_func;
	GDestroyNotify k_destroy_func;
	GType v_type;
	GBoxedCopyFunc v_dup_func;
	GDestroyNotify v_destroy_func;
};

static gint GeeReadOnlyBidirSortedMap_private_offset;
static gpointer gee_read_only_bidir_sorted_map_parent_class = NULL;
static gint GeeReadOnlyBidirSortedMapBidirMapIterator_private_offset;
static gpointer gee_read_only_bidir_sorted_map_bidir_map_iterator_parent_class = NULL;
static GeeBidirMapIteratorIface * gee_read_only_bidir_sorted_map_bidir_map_iterator_gee_bidir_map_iterator_parent_iface = NULL;
static GeeBidirSortedMapIface * gee_read_only_bidir_sorted_map_gee_bidir_sorted_map_parent_iface = NULL;

 G_GNUC_INTERNAL GType gee_read_only_map_get_type (void) G_GNUC_CONST  G_GNUC_UNUSED ;
 G_GNUC_INTERNAL GType gee_read_only_sorted_map_get_type (void) G_GNUC_CONST  G_GNUC_UNUSED ;
 G_GNUC_INTERNAL GType gee_read_only_bidir_sorted_map_get_type (void) G_GNUC_CONST  G_GNUC_UNUSED ;
 G_GNUC_INTERNAL GeeReadOnlyBidirSortedMap* gee_read_only_bidir_sorted_map_new (GType k_type,
                                                               GBoxedCopyFunc k_dup_func,
                                                               GDestroyNotify k_destroy_func,
                                                               GType v_type,
                                                               GBoxedCopyFunc v_dup_func,
                                                               GDestroyNotify v_destroy_func,
                                                               GeeBidirSortedMap* map);
 G_GNUC_INTERNAL GeeReadOnlyBidirSortedMap* gee_read_only_bidir_sorted_map_construct (GType object_type,
                                                                     GType k_type,
                                                                     GBoxedCopyFunc k_dup_func,
                                                                     GDestroyNotify k_destroy_func,
                                                                     GType v_type,
                                                                     GBoxedCopyFunc v_dup_func,
                                                                     GDestroyNotify v_destroy_func,
                                                                     GeeBidirSortedMap* map);
 G_GNUC_INTERNAL GeeReadOnlySortedMap* gee_read_only_sorted_map_new (GType k_type,
                                                    GBoxedCopyFunc k_dup_func,
                                                    GDestroyNotify k_destroy_func,
                                                    GType v_type,
                                                    GBoxedCopyFunc v_dup_func,
                                                    GDestroyNotify v_destroy_func,
                                                    GeeMap* map);
 G_GNUC_INTERNAL GeeReadOnlySortedMap* gee_read_only_sorted_map_construct (GType object_type,
                                                          GType k_type,
                                                          GBoxedCopyFunc k_dup_func,
                                                          GDestroyNotify k_destroy_func,
                                                          GType v_type,
                                                          GBoxedCopyFunc v_dup_func,
                                                          GDestroyNotify v_destroy_func,
                                                          GeeMap* map);
static GeeBidirMapIterator* gee_read_only_bidir_sorted_map_real_bidir_map_iterator (GeeBidirSortedMap* base);
 G_GNUC_INTERNAL GeeReadOnlyBidirSortedMapBidirMapIterator* gee_read_only_bidir_sorted_map_bidir_map_iterator_new (GType k_type,
                                                                                                  GBoxedCopyFunc k_dup_func,
                                                                                                  GDestroyNotify k_destroy_func,
                                                                                                  GType v_type,
                                                                                                  GBoxedCopyFunc v_dup_func,
                                                                                                  GDestroyNotify v_destroy_func,
                                                                                                  GeeBidirMapIterator* iterator);
 G_GNUC_INTERNAL GeeReadOnlyBidirSortedMapBidirMapIterator* gee_read_only_bidir_sorted_map_bidir_map_iterator_construct (GType object_type,
                                                                                                        GType k_type,
                                                                                                        GBoxedCopyFunc k_dup_func,
                                                                                                        GDestroyNotify k_destroy_func,
                                                                                                        GType v_type,
                                                                                                        GBoxedCopyFunc v_dup_func,
                                                                                                        GDestroyNotify v_destroy_func,
                                                                                                        GeeBidirMapIterator* iterator);
 G_GNUC_INTERNAL GType gee_read_only_map_map_iterator_get_type (void) G_GNUC_CONST  G_GNUC_UNUSED ;
VALA_EXTERN GType gee_read_only_bidir_sorted_map_bidir_map_iterator_get_type (void) G_GNUC_CONST ;
 G_GNUC_INTERNAL GeeReadOnlyMapMapIterator* gee_read_only_map_map_iterator_new (GType k_type,
                                                               GBoxedCopyFunc k_dup_func,
                                                               GDestroyNotify k_destroy_func,
                                                               GType v_type,
                                                               GBoxedCopyFunc v_dup_func,
                                                               GDestroyNotify v_destroy_func,
                                                               GeeMapIterator* iterator);
 G_GNUC_INTERNAL GeeReadOnlyMapMapIterator* gee_read_only_map_map_iterator_construct (GType object_type,
                                                                     GType k_type,
                                                                     GBoxedCopyFunc k_dup_func,
                                                                     GDestroyNotify k_destroy_func,
                                                                     GType v_type,
                                                                     GBoxedCopyFunc v_dup_func,
                                                                     GDestroyNotify v_destroy_func,
                                                                     GeeMapIterator* iterator);
static gboolean gee_read_only_bidir_sorted_map_bidir_map_iterator_real_first (GeeBidirMapIterator* base);
static gboolean gee_read_only_bidir_sorted_map_bidir_map_iterator_real_previous (GeeBidirMapIterator* base);
static gboolean gee_read_only_bidir_sorted_map_bidir_map_iterator_real_has_previous (GeeBidirMapIterator* base);
static gboolean gee_read_only_bidir_sorted_map_bidir_map_iterator_real_last (GeeBidirMapIterator* base);
static GType gee_read_only_bidir_sorted_map_bidir_map_iterator_get_type_once (void);
static void _vala_gee_read_only_bidir_sorted_map_bidir_map_iterator_get_property (GObject * object,
                                                                           guint property_id,
                                                                           GValue * value,
                                                                           GParamSpec * pspec);
static void _vala_gee_read_only_bidir_sorted_map_bidir_map_iterator_set_property (GObject * object,
                                                                           guint property_id,
                                                                           const GValue * value,
                                                                           GParamSpec * pspec);
static GType gee_read_only_bidir_sorted_map_get_type_once (void);
static void _vala_gee_read_only_bidir_sorted_map_get_property (GObject * object,
                                                        guint property_id,
                                                        GValue * value,
                                                        GParamSpec * pspec);
static void _vala_gee_read_only_bidir_sorted_map_set_property (GObject * object,
                                                        guint property_id,
                                                        const GValue * value,
                                                        GParamSpec * pspec);

static inline gpointer
gee_read_only_bidir_sorted_map_get_instance_private (GeeReadOnlyBidirSortedMap* self)
{
	return G_STRUCT_MEMBER_P (self, GeeReadOnlyBidirSortedMap_private_offset);
}

/**
 * Constructs a read-only map that mirrors the content of the specified map.
 *
 * @param set the set to decorate.
 */
 G_GNUC_INTERNAL GeeReadOnlyBidirSortedMap*
gee_read_only_bidir_sorted_map_construct (GType object_type,
                                          GType k_type,
                                          GBoxedCopyFunc k_dup_func,
                                          GDestroyNotify k_destroy_func,
                                          GType v_type,
                                          GBoxedCopyFunc v_dup_func,
                                          GDestroyNotify v_destroy_func,
                                          GeeBidirSortedMap* map)
{
	GeeReadOnlyBidirSortedMap * self = NULL;
	g_return_val_if_fail (map != NULL, NULL);
	self = (GeeReadOnlyBidirSortedMap*) gee_read_only_sorted_map_construct (object_type, k_type, (GBoxedCopyFunc) k_dup_func, (GDestroyNotify) k_destroy_func, v_type, (GBoxedCopyFunc) v_dup_func, (GDestroyNotify) v_destroy_func, (GeeMap*) map);
	self->priv->k_type = k_type;
	self->priv->k_dup_func = k_dup_func;
	self->priv->k_destroy_func = k_destroy_func;
	self->priv->v_type = v_type;
	self->priv->v_dup_func = v_dup_func;
	self->priv->v_destroy_func = v_destroy_func;
	return self;
}

 G_GNUC_INTERNAL GeeReadOnlyBidirSortedMap*
gee_read_only_bidir_sorted_map_new (GType k_type,
                                    GBoxedCopyFunc k_dup_func,
                                    GDestroyNotify k_destroy_func,
                                    GType v_type,
                                    GBoxedCopyFunc v_dup_func,
                                    GDestroyNotify v_destroy_func,
                                    GeeBidirSortedMap* map)
{
	return gee_read_only_bidir_sorted_map_construct (GEE_TYPE_READ_ONLY_BIDIR_SORTED_MAP, k_type, k_dup_func, k_destroy_func, v_type, v_dup_func, v_destroy_func, map);
}

/**
 * {@inheritDoc}
 */
static GeeBidirMapIterator*
gee_read_only_bidir_sorted_map_real_bidir_map_iterator (GeeBidirSortedMap* base)
{
	GeeReadOnlyBidirSortedMap * self;
	GeeMap* _tmp0_;
	GeeBidirMapIterator* _tmp1_;
	GeeBidirMapIterator* _tmp2_;
	GeeReadOnlyBidirSortedMapBidirMapIterator* _tmp3_;
	GeeBidirMapIterator* _tmp4_;
	GeeBidirMapIterator* result;
	self = (GeeReadOnlyBidirSortedMap*) base;
	_tmp0_ = ((GeeReadOnlyMap*) self)->_map;
	_tmp1_ = gee_bidir_sorted_map_bidir_map_iterator (G_TYPE_CHECK_INSTANCE_CAST (_tmp0_, GEE_TYPE_BIDIR_SORTED_MAP, GeeBidirSortedMap));
	_tmp2_ = _tmp1_;
	_tmp3_ = gee_read_only_bidir_sorted_map_bidir_map_iterator_new (self->priv->k_type, (GBoxedCopyFunc) self->priv->k_dup_func, (GDestroyNotify) self->priv->k_destroy_func, self->priv->v_type, (GBoxedCopyFunc) self->priv->v_dup_func, (GDestroyNotify) self->priv->v_destroy_func, _tmp2_);
	_tmp4_ = (GeeBidirMapIterator*) _tmp3_;
	_g_object_unref0 (_tmp2_);
	result = _tmp4_;
	return result;
}

static gpointer
_g_object_ref0 (gpointer self)
{
	return self ? g_object_ref (self) : NULL;
}

static GeeBidirSortedMap*
gee_read_only_bidir_sorted_map_real_get_read_only_view (GeeBidirSortedMap* base)
{
	GeeBidirSortedMap* result;
	GeeReadOnlyBidirSortedMap* self;
	GeeBidirSortedMap* _tmp0_;
	self = (GeeReadOnlyBidirSortedMap*) base;
	_tmp0_ = _g_object_ref0 ((GeeBidirSortedMap*) self);
	result = _tmp0_;
	return result;
}

static inline gpointer
gee_read_only_bidir_sorted_map_bidir_map_iterator_get_instance_private (GeeReadOnlyBidirSortedMapBidirMapIterator* self)
{
	return G_STRUCT_MEMBER_P (self, GeeReadOnlyBidirSortedMapBidirMapIterator_private_offset);
}

 G_GNUC_INTERNAL GeeReadOnlyBidirSortedMapBidirMapIterator*
gee_read_only_bidir_sorted_map_bidir_map_iterator_construct (GType object_type,
                                                             GType k_type,
                                                             GBoxedCopyFunc k_dup_func,
                                                             GDestroyNotify k_destroy_func,
                                                             GType v_type,
                                                             GBoxedCopyFunc v_dup_func,
                                                             GDestroyNotify v_destroy_func,
                                                             GeeBidirMapIterator* iterator)
{
	GeeReadOnlyBidirSortedMapBidirMapIterator * self = NULL;
	g_return_val_if_fail (iterator != NULL, NULL);
	self = (GeeReadOnlyBidirSortedMapBidirMapIterator*) gee_read_only_map_map_iterator_construct (object_type, k_type, (GBoxedCopyFunc) k_dup_func, (GDestroyNotify) k_destroy_func, v_type, (GBoxedCopyFunc) v_dup_func, (GDestroyNotify) v_destroy_func, (GeeMapIterator*) iterator);
	self->priv->k_type = k_type;
	self->priv->k_dup_func = k_dup_func;
	self->priv->k_destroy_func = k_destroy_func;
	self->priv->v_type = v_type;
	self->priv->v_dup_func = v_dup_func;
	self->priv->v_destroy_func = v_destroy_func;
	return self;
}

 G_GNUC_INTERNAL GeeReadOnlyBidirSortedMapBidirMapIterator*
gee_read_only_bidir_sorted_map_bidir_map_iterator_new (GType k_type,
                                                       GBoxedCopyFunc k_dup_func,
                                                       GDestroyNotify k_destroy_func,
                                                       GType v_type,
                                                       GBoxedCopyFunc v_dup_func,
                                                       GDestroyNotify v_destroy_func,
                                                       GeeBidirMapIterator* iterator)
{
	return gee_read_only_bidir_sorted_map_bidir_map_iterator_construct (GEE_READ_ONLY_BIDIR_SORTED_MAP_TYPE_BIDIR_MAP_ITERATOR, k_type, k_dup_func, k_destroy_func, v_type, v_dup_func, v_destroy_func, iterator);
}

static gboolean
gee_read_only_bidir_sorted_map_bidir_map_iterator_real_first (GeeBidirMapIterator* base)
{
	GeeReadOnlyBidirSortedMapBidirMapIterator * self;
	GeeMapIterator* _tmp0_;
	gboolean result;
	self = (GeeReadOnlyBidirSortedMapBidirMapIterator*) base;
	_tmp0_ = ((GeeReadOnlyMapMapIterator*) self)->_iter;
	result = gee_bidir_map_iterator_first (G_TYPE_CHECK_INSTANCE_CAST (_tmp0_, GEE_TYPE_BIDIR_MAP_ITERATOR, GeeBidirMapIterator));
	return result;
}

static gboolean
gee_read_only_bidir_sorted_map_bidir_map_iterator_real_previous (GeeBidirMapIterator* base)
{
	GeeReadOnlyBidirSortedMapBidirMapIterator * self;
	GeeMapIterator* _tmp0_;
	gboolean result;
	self = (GeeReadOnlyBidirSortedMapBidirMapIterator*) base;
	_tmp0_ = ((GeeReadOnlyMapMapIterator*) self)->_iter;
	result = gee_bidir_map_iterator_previous (G_TYPE_CHECK_INSTANCE_CAST (_tmp0_, GEE_TYPE_BIDIR_MAP_ITERATOR, GeeBidirMapIterator));
	return result;
}

static gboolean
gee_read_only_bidir_sorted_map_bidir_map_iterator_real_has_previous (GeeBidirMapIterator* base)
{
	GeeReadOnlyBidirSortedMapBidirMapIterator * self;
	GeeMapIterator* _tmp0_;
	gboolean result;
	self = (GeeReadOnlyBidirSortedMapBidirMapIterator*) base;
	_tmp0_ = ((GeeReadOnlyMapMapIterator*) self)->_iter;
	result = gee_bidir_map_iterator_has_previous (G_TYPE_CHECK_INSTANCE_CAST (_tmp0_, GEE_TYPE_BIDIR_MAP_ITERATOR, GeeBidirMapIterator));
	return result;
}

static gboolean
gee_read_only_bidir_sorted_map_bidir_map_iterator_real_last (GeeBidirMapIterator* base)
{
	GeeReadOnlyBidirSortedMapBidirMapIterator * self;
	GeeMapIterator* _tmp0_;
	gboolean result;
	self = (GeeReadOnlyBidirSortedMapBidirMapIterator*) base;
	_tmp0_ = ((GeeReadOnlyMapMapIterator*) self)->_iter;
	result = gee_bidir_map_iterator_last (G_TYPE_CHECK_INSTANCE_CAST (_tmp0_, GEE_TYPE_BIDIR_MAP_ITERATOR, GeeBidirMapIterator));
	return result;
}

static void
gee_read_only_bidir_sorted_map_bidir_map_iterator_class_init (GeeReadOnlyBidirSortedMapBidirMapIteratorClass * klass,
                                                              gpointer klass_data)
{
	gee_read_only_bidir_sorted_map_bidir_map_iterator_parent_class = g_type_class_peek_parent (klass);
	g_type_class_adjust_private_offset (klass, &GeeReadOnlyBidirSortedMapBidirMapIterator_private_offset);
	G_OBJECT_CLASS (klass)->get_property = _vala_gee_read_only_bidir_sorted_map_bidir_map_iterator_get_property;
	G_OBJECT_CLASS (klass)->set_property = _vala_gee_read_only_bidir_sorted_map_bidir_map_iterator_set_property;
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_READ_ONLY_BIDIR_SORTED_MAP_BIDIR_MAP_ITERATOR_K_TYPE, g_param_spec_gtype ("k-type", "type", "type", G_TYPE_NONE, G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_READ_ONLY_BIDIR_SORTED_MAP_BIDIR_MAP_ITERATOR_K_DUP_FUNC, g_param_spec_pointer ("k-dup-func", "dup func", "dup func", G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_READ_ONLY_BIDIR_SORTED_MAP_BIDIR_MAP_ITERATOR_K_DESTROY_FUNC, g_param_spec_pointer ("k-destroy-func", "destroy func", "destroy func", G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_READ_ONLY_BIDIR_SORTED_MAP_BIDIR_MAP_ITERATOR_V_TYPE, g_param_spec_gtype ("v-type", "type", "type", G_TYPE_NONE, G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_READ_ONLY_BIDIR_SORTED_MAP_BIDIR_MAP_ITERATOR_V_DUP_FUNC, g_param_spec_pointer ("v-dup-func", "dup func", "dup func", G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_READ_ONLY_BIDIR_SORTED_MAP_BIDIR_MAP_ITERATOR_V_DESTROY_FUNC, g_param_spec_pointer ("v-destroy-func", "destroy func", "destroy func", G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
}

static GType
gee_read_only_bidir_sorted_map_bidir_map_iterator_gee_bidir_map_iterator_get_k_type (GeeReadOnlyBidirSortedMapBidirMapIterator* self)
{
	return self->priv->k_type;
}

static GBoxedCopyFunc
gee_read_only_bidir_sorted_map_bidir_map_iterator_gee_bidir_map_iterator_get_k_dup_func (GeeReadOnlyBidirSortedMapBidirMapIterator* self)
{
	return self->priv->k_dup_func;
}

static GDestroyNotify
gee_read_only_bidir_sorted_map_bidir_map_iterator_gee_bidir_map_iterator_get_k_destroy_func (GeeReadOnlyBidirSortedMapBidirMapIterator* self)
{
	return self->priv->k_destroy_func;
}

static GType
gee_read_only_bidir_sorted_map_bidir_map_iterator_gee_bidir_map_iterator_get_v_type (GeeReadOnlyBidirSortedMapBidirMapIterator* self)
{
	return self->priv->v_type;
}

static GBoxedCopyFunc
gee_read_only_bidir_sorted_map_bidir_map_iterator_gee_bidir_map_iterator_get_v_dup_func (GeeReadOnlyBidirSortedMapBidirMapIterator* self)
{
	return self->priv->v_dup_func;
}

static GDestroyNotify
gee_read_only_bidir_sorted_map_bidir_map_iterator_gee_bidir_map_iterator_get_v_destroy_func (GeeReadOnlyBidirSortedMapBidirMapIterator* self)
{
	return self->priv->v_destroy_func;
}

static void
gee_read_only_bidir_sorted_map_bidir_map_iterator_gee_bidir_map_iterator_interface_init (GeeBidirMapIteratorIface * iface,
                                                                                         gpointer iface_data)
{
	gee_read_only_bidir_sorted_map_bidir_map_iterator_gee_bidir_map_iterator_parent_iface = g_type_interface_peek_parent (iface);
	iface->first = (gboolean (*) (GeeBidirMapIterator*)) gee_read_only_bidir_sorted_map_bidir_map_iterator_real_first;
	iface->previous = (gboolean (*) (GeeBidirMapIterator*)) gee_read_only_bidir_sorted_map_bidir_map_iterator_real_previous;
	iface->has_previous = (gboolean (*) (GeeBidirMapIterator*)) gee_read_only_bidir_sorted_map_bidir_map_iterator_real_has_previous;
	iface->last = (gboolean (*) (GeeBidirMapIterator*)) gee_read_only_bidir_sorted_map_bidir_map_iterator_real_last;
	iface->get_k_type = (GType (*) (GeeBidirMapIterator *)) gee_read_only_bidir_sorted_map_bidir_map_iterator_gee_bidir_map_iterator_get_k_type;
	iface->get_k_dup_func = (GBoxedCopyFunc (*) (GeeBidirMapIterator *)) gee_read_only_bidir_sorted_map_bidir_map_iterator_gee_bidir_map_iterator_get_k_dup_func;
	iface->get_k_destroy_func = (GDestroyNotify (*) (GeeBidirMapIterator *)) gee_read_only_bidir_sorted_map_bidir_map_iterator_gee_bidir_map_iterator_get_k_destroy_func;
	iface->get_v_type = (GType (*) (GeeBidirMapIterator *)) gee_read_only_bidir_sorted_map_bidir_map_iterator_gee_bidir_map_iterator_get_v_type;
	iface->get_v_dup_func = (GBoxedCopyFunc (*) (GeeBidirMapIterator *)) gee_read_only_bidir_sorted_map_bidir_map_iterator_gee_bidir_map_iterator_get_v_dup_func;
	iface->get_v_destroy_func = (GDestroyNotify (*) (GeeBidirMapIterator *)) gee_read_only_bidir_sorted_map_bidir_map_iterator_gee_bidir_map_iterator_get_v_destroy_func;
}

static void
gee_read_only_bidir_sorted_map_bidir_map_iterator_instance_init (GeeReadOnlyBidirSortedMapBidirMapIterator * self,
                                                                 gpointer klass)
{
	self->priv = gee_read_only_bidir_sorted_map_bidir_map_iterator_get_instance_private (self);
}

static GType
gee_read_only_bidir_sorted_map_bidir_map_iterator_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (GeeReadOnlyBidirSortedMapBidirMapIteratorClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) gee_read_only_bidir_sorted_map_bidir_map_iterator_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GeeReadOnlyBidirSortedMapBidirMapIterator), 0, (GInstanceInitFunc) gee_read_only_bidir_sorted_map_bidir_map_iterator_instance_init, NULL };
	static const GInterfaceInfo gee_bidir_map_iterator_info = { (GInterfaceInitFunc) gee_read_only_bidir_sorted_map_bidir_map_iterator_gee_bidir_map_iterator_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
	GType gee_read_only_bidir_sorted_map_bidir_map_iterator_type_id;
	gee_read_only_bidir_sorted_map_bidir_map_iterator_type_id = g_type_register_static (GEE_READ_ONLY_MAP_TYPE_MAP_ITERATOR, "GeeReadOnlyBidirSortedMapBidirMapIterator", &g_define_type_info, 0);
	g_type_add_interface_static (gee_read_only_bidir_sorted_map_bidir_map_iterator_type_id, GEE_TYPE_BIDIR_MAP_ITERATOR, &gee_bidir_map_iterator_info);
	GeeReadOnlyBidirSortedMapBidirMapIterator_private_offset = g_type_add_instance_private (gee_read_only_bidir_sorted_map_bidir_map_iterator_type_id, sizeof (GeeReadOnlyBidirSortedMapBidirMapIteratorPrivate));
	return gee_read_only_bidir_sorted_map_bidir_map_iterator_type_id;
}

GType
gee_read_only_bidir_sorted_map_bidir_map_iterator_get_type (void)
{
	static volatile gsize gee_read_only_bidir_sorted_map_bidir_map_iterator_type_id__once = 0;
	if (g_once_init_enter (&gee_read_only_bidir_sorted_map_bidir_map_iterator_type_id__once)) {
		GType gee_read_only_bidir_sorted_map_bidir_map_iterator_type_id;
		gee_read_only_bidir_sorted_map_bidir_map_iterator_type_id = gee_read_only_bidir_sorted_map_bidir_map_iterator_get_type_once ();
		g_once_init_leave (&gee_read_only_bidir_sorted_map_bidir_map_iterator_type_id__once, gee_read_only_bidir_sorted_map_bidir_map_iterator_type_id);
	}
	return gee_read_only_bidir_sorted_map_bidir_map_iterator_type_id__once;
}

static void
_vala_gee_read_only_bidir_sorted_map_bidir_map_iterator_get_property (GObject * object,
                                                                      guint property_id,
                                                                      GValue * value,
                                                                      GParamSpec * pspec)
{
	GeeReadOnlyBidirSortedMapBidirMapIterator * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, GEE_READ_ONLY_BIDIR_SORTED_MAP_TYPE_BIDIR_MAP_ITERATOR, GeeReadOnlyBidirSortedMapBidirMapIterator);
	switch (property_id) {
		case GEE_READ_ONLY_BIDIR_SORTED_MAP_BIDIR_MAP_ITERATOR_K_TYPE:
		g_value_set_gtype (value, self->priv->k_type);
		break;
		case GEE_READ_ONLY_BIDIR_SORTED_MAP_BIDIR_MAP_ITERATOR_K_DUP_FUNC:
		g_value_set_pointer (value, self->priv->k_dup_func);
		break;
		case GEE_READ_ONLY_BIDIR_SORTED_MAP_BIDIR_MAP_ITERATOR_K_DESTROY_FUNC:
		g_value_set_pointer (value, self->priv->k_destroy_func);
		break;
		case GEE_READ_ONLY_BIDIR_SORTED_MAP_BIDIR_MAP_ITERATOR_V_TYPE:
		g_value_set_gtype (value, self->priv->v_type);
		break;
		case GEE_READ_ONLY_BIDIR_SORTED_MAP_BIDIR_MAP_ITERATOR_V_DUP_FUNC:
		g_value_set_pointer (value, self->priv->v_dup_func);
		break;
		case GEE_READ_ONLY_BIDIR_SORTED_MAP_BIDIR_MAP_ITERATOR_V_DESTROY_FUNC:
		g_value_set_pointer (value, self->priv->v_destroy_func);
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

static void
_vala_gee_read_only_bidir_sorted_map_bidir_map_iterator_set_property (GObject * object,
                                                                      guint property_id,
                                                                      const GValue * value,
                                                                      GParamSpec * pspec)
{
	GeeReadOnlyBidirSortedMapBidirMapIterator * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, GEE_READ_ONLY_BIDIR_SORTED_MAP_TYPE_BIDIR_MAP_ITERATOR, GeeReadOnlyBidirSortedMapBidirMapIterator);
	switch (property_id) {
		case GEE_READ_ONLY_BIDIR_SORTED_MAP_BIDIR_MAP_ITERATOR_K_TYPE:
		self->priv->k_type = g_value_get_gtype (value);
		break;
		case GEE_READ_ONLY_BIDIR_SORTED_MAP_BIDIR_MAP_ITERATOR_K_DUP_FUNC:
		self->priv->k_dup_func = g_value_get_pointer (value);
		break;
		case GEE_READ_ONLY_BIDIR_SORTED_MAP_BIDIR_MAP_ITERATOR_K_DESTROY_FUNC:
		self->priv->k_destroy_func = g_value_get_pointer (value);
		break;
		case GEE_READ_ONLY_BIDIR_SORTED_MAP_BIDIR_MAP_ITERATOR_V_TYPE:
		self->priv->v_type = g_value_get_gtype (value);
		break;
		case GEE_READ_ONLY_BIDIR_SORTED_MAP_BIDIR_MAP_ITERATOR_V_DUP_FUNC:
		self->priv->v_dup_func = g_value_get_pointer (value);
		break;
		case GEE_READ_ONLY_BIDIR_SORTED_MAP_BIDIR_MAP_ITERATOR_V_DESTROY_FUNC:
		self->priv->v_destroy_func = g_value_get_pointer (value);
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

static void
gee_read_only_bidir_sorted_map_class_init (GeeReadOnlyBidirSortedMapClass * klass,
                                           gpointer klass_data)
{
	gee_read_only_bidir_sorted_map_parent_class = g_type_class_peek_parent (klass);
	g_type_class_adjust_private_offset (klass, &GeeReadOnlyBidirSortedMap_private_offset);
	G_OBJECT_CLASS (klass)->get_property = _vala_gee_read_only_bidir_sorted_map_get_property;
	G_OBJECT_CLASS (klass)->set_property = _vala_gee_read_only_bidir_sorted_map_set_property;
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_READ_ONLY_BIDIR_SORTED_MAP_K_TYPE, g_param_spec_gtype ("k-type", "type", "type", G_TYPE_NONE, G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_READ_ONLY_BIDIR_SORTED_MAP_K_DUP_FUNC, g_param_spec_pointer ("k-dup-func", "dup func", "dup func", G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_READ_ONLY_BIDIR_SORTED_MAP_K_DESTROY_FUNC, g_param_spec_pointer ("k-destroy-func", "destroy func", "destroy func", G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_READ_ONLY_BIDIR_SORTED_MAP_V_TYPE, g_param_spec_gtype ("v-type", "type", "type", G_TYPE_NONE, G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_READ_ONLY_BIDIR_SORTED_MAP_V_DUP_FUNC, g_param_spec_pointer ("v-dup-func", "dup func", "dup func", G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_READ_ONLY_BIDIR_SORTED_MAP_V_DESTROY_FUNC, g_param_spec_pointer ("v-destroy-func", "destroy func", "destroy func", G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	/**
	 * {@inheritDoc}
	 */
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_READ_ONLY_BIDIR_SORTED_MAP_READ_ONLY_VIEW_PROPERTY, gee_read_only_bidir_sorted_map_properties[GEE_READ_ONLY_BIDIR_SORTED_MAP_READ_ONLY_VIEW_PROPERTY] = g_param_spec_object ("read-only-view", "read-only-view", "read-only-view", GEE_TYPE_BIDIR_SORTED_MAP, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE));
}

static GType
gee_read_only_bidir_sorted_map_gee_bidir_sorted_map_get_k_type (GeeReadOnlyBidirSortedMap* self)
{
	return self->priv->k_type;
}

static GBoxedCopyFunc
gee_read_only_bidir_sorted_map_gee_bidir_sorted_map_get_k_dup_func (GeeReadOnlyBidirSortedMap* self)
{
	return self->priv->k_dup_func;
}

static GDestroyNotify
gee_read_only_bidir_sorted_map_gee_bidir_sorted_map_get_k_destroy_func (GeeReadOnlyBidirSortedMap* self)
{
	return self->priv->k_destroy_func;
}

static GType
gee_read_only_bidir_sorted_map_gee_bidir_sorted_map_get_v_type (GeeReadOnlyBidirSortedMap* self)
{
	return self->priv->v_type;
}

static GBoxedCopyFunc
gee_read_only_bidir_sorted_map_gee_bidir_sorted_map_get_v_dup_func (GeeReadOnlyBidirSortedMap* self)
{
	return self->priv->v_dup_func;
}

static GDestroyNotify
gee_read_only_bidir_sorted_map_gee_bidir_sorted_map_get_v_destroy_func (GeeReadOnlyBidirSortedMap* self)
{
	return self->priv->v_destroy_func;
}

static void
gee_read_only_bidir_sorted_map_gee_bidir_sorted_map_interface_init (GeeBidirSortedMapIface * iface,
                                                                    gpointer iface_data)
{
	gee_read_only_bidir_sorted_map_gee_bidir_sorted_map_parent_iface = g_type_interface_peek_parent (iface);
	iface->bidir_map_iterator = (GeeBidirMapIterator* (*) (GeeBidirSortedMap*)) gee_read_only_bidir_sorted_map_real_bidir_map_iterator;
	iface->get_k_type = (GType (*) (GeeBidirSortedMap *)) gee_read_only_bidir_sorted_map_gee_bidir_sorted_map_get_k_type;
	iface->get_k_dup_func = (GBoxedCopyFunc (*) (GeeBidirSortedMap *)) gee_read_only_bidir_sorted_map_gee_bidir_sorted_map_get_k_dup_func;
	iface->get_k_destroy_func = (GDestroyNotify (*) (GeeBidirSortedMap *)) gee_read_only_bidir_sorted_map_gee_bidir_sorted_map_get_k_destroy_func;
	iface->get_v_type = (GType (*) (GeeBidirSortedMap *)) gee_read_only_bidir_sorted_map_gee_bidir_sorted_map_get_v_type;
	iface->get_v_dup_func = (GBoxedCopyFunc (*) (GeeBidirSortedMap *)) gee_read_only_bidir_sorted_map_gee_bidir_sorted_map_get_v_dup_func;
	iface->get_v_destroy_func = (GDestroyNotify (*) (GeeBidirSortedMap *)) gee_read_only_bidir_sorted_map_gee_bidir_sorted_map_get_v_destroy_func;
	iface->get_read_only_view = gee_read_only_bidir_sorted_map_real_get_read_only_view;
}

static void
gee_read_only_bidir_sorted_map_instance_init (GeeReadOnlyBidirSortedMap * self,
                                              gpointer klass)
{
	self->priv = gee_read_only_bidir_sorted_map_get_instance_private (self);
}

/**
 * Read-only view for {@link BidirSortedMap} collections.
 *
 * This class decorates any class which implements the {@link BidirSortedMap}
 * interface by making it read only. Any method which normally modify data will
 * throw an error.
 *
 * @see BidirSortedMap
 */
static GType
gee_read_only_bidir_sorted_map_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (GeeReadOnlyBidirSortedMapClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) gee_read_only_bidir_sorted_map_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GeeReadOnlyBidirSortedMap), 0, (GInstanceInitFunc) gee_read_only_bidir_sorted_map_instance_init, NULL };
	static const GInterfaceInfo gee_bidir_sorted_map_info = { (GInterfaceInitFunc) gee_read_only_bidir_sorted_map_gee_bidir_sorted_map_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
	GType gee_read_only_bidir_sorted_map_type_id;
	gee_read_only_bidir_sorted_map_type_id = g_type_register_static (GEE_TYPE_READ_ONLY_SORTED_MAP, "GeeReadOnlyBidirSortedMap", &g_define_type_info, 0);
	g_type_add_interface_static (gee_read_only_bidir_sorted_map_type_id, GEE_TYPE_BIDIR_SORTED_MAP, &gee_bidir_sorted_map_info);
	GeeReadOnlyBidirSortedMap_private_offset = g_type_add_instance_private (gee_read_only_bidir_sorted_map_type_id, sizeof (GeeReadOnlyBidirSortedMapPrivate));
	return gee_read_only_bidir_sorted_map_type_id;
}

 G_GNUC_INTERNAL GType
gee_read_only_bidir_sorted_map_get_type (void)
{
	static volatile gsize gee_read_only_bidir_sorted_map_type_id__once = 0;
	if (g_once_init_enter (&gee_read_only_bidir_sorted_map_type_id__once)) {
		GType gee_read_only_bidir_sorted_map_type_id;
		gee_read_only_bidir_sorted_map_type_id = gee_read_only_bidir_sorted_map_get_type_once ();
		g_once_init_leave (&gee_read_only_bidir_sorted_map_type_id__once, gee_read_only_bidir_sorted_map_type_id);
	}
	return gee_read_only_bidir_sorted_map_type_id__once;
}

static void
_vala_gee_read_only_bidir_sorted_map_get_property (GObject * object,
                                                   guint property_id,
                                                   GValue * value,
                                                   GParamSpec * pspec)
{
	GeeReadOnlyBidirSortedMap * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, GEE_TYPE_READ_ONLY_BIDIR_SORTED_MAP, GeeReadOnlyBidirSortedMap);
	switch (property_id) {
		case GEE_READ_ONLY_BIDIR_SORTED_MAP_READ_ONLY_VIEW_PROPERTY:
		g_value_take_object (value, gee_bidir_sorted_map_get_read_only_view ((GeeBidirSortedMap*) self));
		break;
		case GEE_READ_ONLY_BIDIR_SORTED_MAP_K_TYPE:
		g_value_set_gtype (value, self->priv->k_type);
		break;
		case GEE_READ_ONLY_BIDIR_SORTED_MAP_K_DUP_FUNC:
		g_value_set_pointer (value, self->priv->k_dup_func);
		break;
		case GEE_READ_ONLY_BIDIR_SORTED_MAP_K_DESTROY_FUNC:
		g_value_set_pointer (value, self->priv->k_destroy_func);
		break;
		case GEE_READ_ONLY_BIDIR_SORTED_MAP_V_TYPE:
		g_value_set_gtype (value, self->priv->v_type);
		break;
		case GEE_READ_ONLY_BIDIR_SORTED_MAP_V_DUP_FUNC:
		g_value_set_pointer (value, self->priv->v_dup_func);
		break;
		case GEE_READ_ONLY_BIDIR_SORTED_MAP_V_DESTROY_FUNC:
		g_value_set_pointer (value, self->priv->v_destroy_func);
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

static void
_vala_gee_read_only_bidir_sorted_map_set_property (GObject * object,
                                                   guint property_id,
                                                   const GValue * value,
                                                   GParamSpec * pspec)
{
	GeeReadOnlyBidirSortedMap * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, GEE_TYPE_READ_ONLY_BIDIR_SORTED_MAP, GeeReadOnlyBidirSortedMap);
	switch (property_id) {
		case GEE_READ_ONLY_BIDIR_SORTED_MAP_K_TYPE:
		self->priv->k_type = g_value_get_gtype (value);
		break;
		case GEE_READ_ONLY_BIDIR_SORTED_MAP_K_DUP_FUNC:
		self->priv->k_dup_func = g_value_get_pointer (value);
		break;
		case GEE_READ_ONLY_BIDIR_SORTED_MAP_K_DESTROY_FUNC:
		self->priv->k_destroy_func = g_value_get_pointer (value);
		break;
		case GEE_READ_ONLY_BIDIR_SORTED_MAP_V_TYPE:
		self->priv->v_type = g_value_get_gtype (value);
		break;
		case GEE_READ_ONLY_BIDIR_SORTED_MAP_V_DUP_FUNC:
		self->priv->v_dup_func = g_value_get_pointer (value);
		break;
		case GEE_READ_ONLY_BIDIR_SORTED_MAP_V_DESTROY_FUNC:
		self->priv->v_destroy_func = g_value_get_pointer (value);
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

