/* readonlysortedmap.c generated by valac 0.56.3, the Vala compiler
 * generated from readonlysortedmap.vala, do not modify */

/* readonlysortedmap.vala
 *
 * Copyright (C) 2009-2011  Maciej Piechotka
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Maciej Piechotka <uzytkownik2@gmail.com>
 */

#include <glib-object.h>
#include "gee.h"
#include <glib.h>

#define GEE_TYPE_READ_ONLY_MAP (gee_read_only_map_get_type ())
#define GEE_READ_ONLY_MAP(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_READ_ONLY_MAP, GeeReadOnlyMap))
#define GEE_READ_ONLY_MAP_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_TYPE_READ_ONLY_MAP, GeeReadOnlyMapClass))
#define GEE_IS_READ_ONLY_MAP(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_READ_ONLY_MAP))
#define GEE_IS_READ_ONLY_MAP_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEE_TYPE_READ_ONLY_MAP))
#define GEE_READ_ONLY_MAP_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_TYPE_READ_ONLY_MAP, GeeReadOnlyMapClass))

typedef struct _GeeReadOnlyMap GeeReadOnlyMap;
typedef struct _GeeReadOnlyMapClass GeeReadOnlyMapClass;
typedef struct _GeeReadOnlyMapPrivate GeeReadOnlyMapPrivate;

#define GEE_TYPE_READ_ONLY_SORTED_MAP (gee_read_only_sorted_map_get_type ())
#define GEE_READ_ONLY_SORTED_MAP(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_READ_ONLY_SORTED_MAP, GeeReadOnlySortedMap))
#define GEE_READ_ONLY_SORTED_MAP_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_TYPE_READ_ONLY_SORTED_MAP, GeeReadOnlySortedMapClass))
#define GEE_IS_READ_ONLY_SORTED_MAP(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_READ_ONLY_SORTED_MAP))
#define GEE_IS_READ_ONLY_SORTED_MAP_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEE_TYPE_READ_ONLY_SORTED_MAP))
#define GEE_READ_ONLY_SORTED_MAP_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_TYPE_READ_ONLY_SORTED_MAP, GeeReadOnlySortedMapClass))

typedef struct _GeeReadOnlySortedMap GeeReadOnlySortedMap;
typedef struct _GeeReadOnlySortedMapClass GeeReadOnlySortedMapClass;
typedef struct _GeeReadOnlySortedMapPrivate GeeReadOnlySortedMapPrivate;
enum  {
	GEE_READ_ONLY_SORTED_MAP_0_PROPERTY,
	GEE_READ_ONLY_SORTED_MAP_K_TYPE,
	GEE_READ_ONLY_SORTED_MAP_K_DUP_FUNC,
	GEE_READ_ONLY_SORTED_MAP_K_DESTROY_FUNC,
	GEE_READ_ONLY_SORTED_MAP_V_TYPE,
	GEE_READ_ONLY_SORTED_MAP_V_DUP_FUNC,
	GEE_READ_ONLY_SORTED_MAP_V_DESTROY_FUNC,
	GEE_READ_ONLY_SORTED_MAP_ASCENDING_KEYS_PROPERTY,
	GEE_READ_ONLY_SORTED_MAP_ASCENDING_ENTRIES_PROPERTY,
	GEE_READ_ONLY_SORTED_MAP_READ_ONLY_VIEW_PROPERTY,
	GEE_READ_ONLY_SORTED_MAP_NUM_PROPERTIES
};
static GParamSpec* gee_read_only_sorted_map_properties[GEE_READ_ONLY_SORTED_MAP_NUM_PROPERTIES];
#define _g_object_unref0(var) ((var == NULL) ? NULL : (var = (g_object_unref (var), NULL)))

struct _GeeReadOnlyMap {
	GObject parent_instance;
	GeeReadOnlyMapPrivate * priv;
	GeeMap* _map;
};

struct _GeeReadOnlyMapClass {
	GObjectClass parent_class;
	GeeMap* (*get_read_only_view) (GeeReadOnlyMap* self);
};

struct _GeeReadOnlySortedMap {
	GeeReadOnlyMap parent_instance;
	GeeReadOnlySortedMapPrivate * priv;
};

struct _GeeReadOnlySortedMapClass {
	GeeReadOnlyMapClass parent_class;
};

struct _GeeReadOnlySortedMapPrivate {
	GType k_type;
	GBoxedCopyFunc k_dup_func;
	GDestroyNotify k_destroy_func;
	GType v_type;
	GBoxedCopyFunc v_dup_func;
	GDestroyNotify v_destroy_func;
};

static gint GeeReadOnlySortedMap_private_offset;
static gpointer gee_read_only_sorted_map_parent_class = NULL;
static GeeSortedMapIface * gee_read_only_sorted_map_gee_sorted_map_parent_iface = NULL;

 G_GNUC_INTERNAL GType gee_read_only_map_get_type (void) G_GNUC_CONST  G_GNUC_UNUSED ;
 G_GNUC_INTERNAL GType gee_read_only_sorted_map_get_type (void) G_GNUC_CONST  G_GNUC_UNUSED ;
 G_GNUC_INTERNAL GeeReadOnlySortedMap* gee_read_only_sorted_map_new (GType k_type,
                                                    GBoxedCopyFunc k_dup_func,
                                                    GDestroyNotify k_destroy_func,
                                                    GType v_type,
                                                    GBoxedCopyFunc v_dup_func,
                                                    GDestroyNotify v_destroy_func,
                                                    GeeMap* map);
 G_GNUC_INTERNAL GeeReadOnlySortedMap* gee_read_only_sorted_map_construct (GType object_type,
                                                          GType k_type,
                                                          GBoxedCopyFunc k_dup_func,
                                                          GDestroyNotify k_destroy_func,
                                                          GType v_type,
                                                          GBoxedCopyFunc v_dup_func,
                                                          GDestroyNotify v_destroy_func,
                                                          GeeMap* map);
 G_GNUC_INTERNAL GeeReadOnlyMap* gee_read_only_map_new (GType k_type,
                                       GBoxedCopyFunc k_dup_func,
                                       GDestroyNotify k_destroy_func,
                                       GType v_type,
                                       GBoxedCopyFunc v_dup_func,
                                       GDestroyNotify v_destroy_func,
                                       GeeMap* map);
 G_GNUC_INTERNAL GeeReadOnlyMap* gee_read_only_map_construct (GType object_type,
                                             GType k_type,
                                             GBoxedCopyFunc k_dup_func,
                                             GDestroyNotify k_destroy_func,
                                             GType v_type,
                                             GBoxedCopyFunc v_dup_func,
                                             GDestroyNotify v_destroy_func,
                                             GeeMap* map);
static GeeSortedMap* gee_read_only_sorted_map_real_head_map (GeeSortedMap* base,
                                                      gconstpointer before);
static GeeSortedMap* gee_read_only_sorted_map_real_tail_map (GeeSortedMap* base,
                                                      gconstpointer after);
static GeeSortedMap* gee_read_only_sorted_map_real_sub_map (GeeSortedMap* base,
                                                     gconstpointer from,
                                                     gconstpointer to);
static GType gee_read_only_sorted_map_get_type_once (void);
static void _vala_gee_read_only_sorted_map_get_property (GObject * object,
                                                  guint property_id,
                                                  GValue * value,
                                                  GParamSpec * pspec);
static void _vala_gee_read_only_sorted_map_set_property (GObject * object,
                                                  guint property_id,
                                                  const GValue * value,
                                                  GParamSpec * pspec);

static inline gpointer
gee_read_only_sorted_map_get_instance_private (GeeReadOnlySortedMap* self)
{
	return G_STRUCT_MEMBER_P (self, GeeReadOnlySortedMap_private_offset);
}

/**
 * Constructs a read-only map that mirrors the content of the specified map.
 *
 * @param map the map to decorate.
 */
 G_GNUC_INTERNAL GeeReadOnlySortedMap*
gee_read_only_sorted_map_construct (GType object_type,
                                    GType k_type,
                                    GBoxedCopyFunc k_dup_func,
                                    GDestroyNotify k_destroy_func,
                                    GType v_type,
                                    GBoxedCopyFunc v_dup_func,
                                    GDestroyNotify v_destroy_func,
                                    GeeMap* map)
{
	GeeReadOnlySortedMap * self = NULL;
	g_return_val_if_fail (map != NULL, NULL);
	self = (GeeReadOnlySortedMap*) gee_read_only_map_construct (object_type, k_type, (GBoxedCopyFunc) k_dup_func, (GDestroyNotify) k_destroy_func, v_type, (GBoxedCopyFunc) v_dup_func, (GDestroyNotify) v_destroy_func, map);
	self->priv->k_type = k_type;
	self->priv->k_dup_func = k_dup_func;
	self->priv->k_destroy_func = k_destroy_func;
	self->priv->v_type = v_type;
	self->priv->v_dup_func = v_dup_func;
	self->priv->v_destroy_func = v_destroy_func;
	return self;
}

 G_GNUC_INTERNAL GeeReadOnlySortedMap*
gee_read_only_sorted_map_new (GType k_type,
                              GBoxedCopyFunc k_dup_func,
                              GDestroyNotify k_destroy_func,
                              GType v_type,
                              GBoxedCopyFunc v_dup_func,
                              GDestroyNotify v_destroy_func,
                              GeeMap* map)
{
	return gee_read_only_sorted_map_construct (GEE_TYPE_READ_ONLY_SORTED_MAP, k_type, k_dup_func, k_destroy_func, v_type, v_dup_func, v_destroy_func, map);
}

/**
 * {@inheritDoc}
 */
static GeeSortedMap*
gee_read_only_sorted_map_real_head_map (GeeSortedMap* base,
                                        gconstpointer before)
{
	GeeReadOnlySortedMap * self;
	GeeMap* _tmp0_;
	GeeSortedMap* _tmp1_;
	GeeSortedMap* _tmp2_;
	GeeSortedMap* _tmp3_;
	GeeSortedMap* _tmp4_;
	GeeSortedMap* _tmp5_;
	GeeSortedMap* result;
	self = (GeeReadOnlySortedMap*) base;
	_tmp0_ = ((GeeReadOnlyMap*) self)->_map;
	_tmp1_ = gee_sorted_map_head_map (G_TYPE_CHECK_INSTANCE_CAST (_tmp0_, GEE_TYPE_SORTED_MAP, GeeSortedMap), before);
	_tmp2_ = _tmp1_;
	_tmp3_ = gee_sorted_map_get_read_only_view (_tmp2_);
	_tmp4_ = _tmp3_;
	_tmp5_ = _tmp4_;
	_g_object_unref0 (_tmp2_);
	result = _tmp5_;
	return result;
}

/**
 * {@inheritDoc}
 */
static GeeSortedMap*
gee_read_only_sorted_map_real_tail_map (GeeSortedMap* base,
                                        gconstpointer after)
{
	GeeReadOnlySortedMap * self;
	GeeMap* _tmp0_;
	GeeSortedMap* _tmp1_;
	GeeSortedMap* _tmp2_;
	GeeSortedMap* _tmp3_;
	GeeSortedMap* _tmp4_;
	GeeSortedMap* _tmp5_;
	GeeSortedMap* result;
	self = (GeeReadOnlySortedMap*) base;
	_tmp0_ = ((GeeReadOnlyMap*) self)->_map;
	_tmp1_ = gee_sorted_map_tail_map (G_TYPE_CHECK_INSTANCE_CAST (_tmp0_, GEE_TYPE_SORTED_MAP, GeeSortedMap), after);
	_tmp2_ = _tmp1_;
	_tmp3_ = gee_sorted_map_get_read_only_view (_tmp2_);
	_tmp4_ = _tmp3_;
	_tmp5_ = _tmp4_;
	_g_object_unref0 (_tmp2_);
	result = _tmp5_;
	return result;
}

/**
 * {@inheritDoc}
 */
static GeeSortedMap*
gee_read_only_sorted_map_real_sub_map (GeeSortedMap* base,
                                       gconstpointer from,
                                       gconstpointer to)
{
	GeeReadOnlySortedMap * self;
	GeeMap* _tmp0_;
	GeeSortedMap* _tmp1_;
	GeeSortedMap* _tmp2_;
	GeeSortedMap* _tmp3_;
	GeeSortedMap* _tmp4_;
	GeeSortedMap* _tmp5_;
	GeeSortedMap* result;
	self = (GeeReadOnlySortedMap*) base;
	_tmp0_ = ((GeeReadOnlyMap*) self)->_map;
	_tmp1_ = gee_sorted_map_sub_map (G_TYPE_CHECK_INSTANCE_CAST (_tmp0_, GEE_TYPE_SORTED_MAP, GeeSortedMap), from, to);
	_tmp2_ = _tmp1_;
	_tmp3_ = gee_sorted_map_get_read_only_view (_tmp2_);
	_tmp4_ = _tmp3_;
	_tmp5_ = _tmp4_;
	_g_object_unref0 (_tmp2_);
	result = _tmp5_;
	return result;
}

static GeeSortedSet*
gee_read_only_sorted_map_real_get_ascending_keys (GeeSortedMap* base)
{
	GeeSortedSet* result;
	GeeReadOnlySortedMap* self;
	GeeMap* _tmp0_;
	GeeSortedSet* _tmp1_;
	GeeSortedSet* _tmp2_;
	GeeSortedSet* _tmp3_;
	GeeSortedSet* _tmp4_;
	GeeSortedSet* _tmp5_;
	GeeSortedSet* _tmp6_;
	self = (GeeReadOnlySortedMap*) base;
	_tmp0_ = ((GeeReadOnlyMap*) self)->_map;
	_tmp1_ = gee_sorted_map_get_ascending_keys (G_TYPE_CHECK_INSTANCE_CAST (_tmp0_, GEE_TYPE_SORTED_MAP, GeeSortedMap));
	_tmp2_ = _tmp1_;
	_tmp3_ = _tmp2_;
	_tmp4_ = gee_sorted_set_get_read_only_view (_tmp3_);
	_tmp5_ = _tmp4_;
	_tmp6_ = _tmp5_;
	_g_object_unref0 (_tmp3_);
	result = _tmp6_;
	return result;
}

static GeeSortedSet*
gee_read_only_sorted_map_real_get_ascending_entries (GeeSortedMap* base)
{
	GeeSortedSet* result;
	GeeReadOnlySortedMap* self;
	GeeMap* _tmp0_;
	GeeSortedSet* _tmp1_;
	GeeSortedSet* _tmp2_;
	GeeSortedSet* _tmp3_;
	GeeSortedSet* _tmp4_;
	GeeSortedSet* _tmp5_;
	GeeSortedSet* _tmp6_;
	self = (GeeReadOnlySortedMap*) base;
	_tmp0_ = ((GeeReadOnlyMap*) self)->_map;
	_tmp1_ = gee_sorted_map_get_ascending_entries (G_TYPE_CHECK_INSTANCE_CAST (_tmp0_, GEE_TYPE_SORTED_MAP, GeeSortedMap));
	_tmp2_ = _tmp1_;
	_tmp3_ = _tmp2_;
	_tmp4_ = gee_sorted_set_get_read_only_view (_tmp3_);
	_tmp5_ = _tmp4_;
	_tmp6_ = _tmp5_;
	_g_object_unref0 (_tmp3_);
	result = _tmp6_;
	return result;
}

static gpointer
_g_object_ref0 (gpointer self)
{
	return self ? g_object_ref (self) : NULL;
}

static GeeSortedMap*
gee_read_only_sorted_map_real_get_read_only_view (GeeSortedMap* base)
{
	GeeSortedMap* result;
	GeeReadOnlySortedMap* self;
	GeeSortedMap* _tmp0_;
	self = (GeeReadOnlySortedMap*) base;
	_tmp0_ = _g_object_ref0 ((GeeSortedMap*) self);
	result = _tmp0_;
	return result;
}

static void
gee_read_only_sorted_map_class_init (GeeReadOnlySortedMapClass * klass,
                                     gpointer klass_data)
{
	gee_read_only_sorted_map_parent_class = g_type_class_peek_parent (klass);
	g_type_class_adjust_private_offset (klass, &GeeReadOnlySortedMap_private_offset);
	G_OBJECT_CLASS (klass)->get_property = _vala_gee_read_only_sorted_map_get_property;
	G_OBJECT_CLASS (klass)->set_property = _vala_gee_read_only_sorted_map_set_property;
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_READ_ONLY_SORTED_MAP_K_TYPE, g_param_spec_gtype ("k-type", "type", "type", G_TYPE_NONE, G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_READ_ONLY_SORTED_MAP_K_DUP_FUNC, g_param_spec_pointer ("k-dup-func", "dup func", "dup func", G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_READ_ONLY_SORTED_MAP_K_DESTROY_FUNC, g_param_spec_pointer ("k-destroy-func", "destroy func", "destroy func", G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_READ_ONLY_SORTED_MAP_V_TYPE, g_param_spec_gtype ("v-type", "type", "type", G_TYPE_NONE, G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_READ_ONLY_SORTED_MAP_V_DUP_FUNC, g_param_spec_pointer ("v-dup-func", "dup func", "dup func", G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_READ_ONLY_SORTED_MAP_V_DESTROY_FUNC, g_param_spec_pointer ("v-destroy-func", "destroy func", "destroy func", G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	/**
	 * {@inheritDoc}
	 */
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_READ_ONLY_SORTED_MAP_ASCENDING_KEYS_PROPERTY, gee_read_only_sorted_map_properties[GEE_READ_ONLY_SORTED_MAP_ASCENDING_KEYS_PROPERTY] = g_param_spec_object ("ascending-keys", "ascending-keys", "ascending-keys", GEE_TYPE_SORTED_SET, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE));
	/**
	 * {@inheritDoc}
	 */
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_READ_ONLY_SORTED_MAP_ASCENDING_ENTRIES_PROPERTY, gee_read_only_sorted_map_properties[GEE_READ_ONLY_SORTED_MAP_ASCENDING_ENTRIES_PROPERTY] = g_param_spec_object ("ascending-entries", "ascending-entries", "ascending-entries", GEE_TYPE_SORTED_SET, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE));
	/**
	 * {@inheritDoc}
	 */
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_READ_ONLY_SORTED_MAP_READ_ONLY_VIEW_PROPERTY, gee_read_only_sorted_map_properties[GEE_READ_ONLY_SORTED_MAP_READ_ONLY_VIEW_PROPERTY] = g_param_spec_object ("read-only-view", "read-only-view", "read-only-view", GEE_TYPE_SORTED_MAP, G_PARAM_STATIC_STRINGS | G_PARAM_READABLE));
}

static void
gee_read_only_sorted_map_gee_sorted_map_interface_init (GeeSortedMapIface * iface,
                                                        gpointer iface_data)
{
	gee_read_only_sorted_map_gee_sorted_map_parent_iface = g_type_interface_peek_parent (iface);
	iface->head_map = (GeeSortedMap* (*) (GeeSortedMap*, gconstpointer)) gee_read_only_sorted_map_real_head_map;
	iface->tail_map = (GeeSortedMap* (*) (GeeSortedMap*, gconstpointer)) gee_read_only_sorted_map_real_tail_map;
	iface->sub_map = (GeeSortedMap* (*) (GeeSortedMap*, gconstpointer, gconstpointer)) gee_read_only_sorted_map_real_sub_map;
	iface->get_ascending_keys = gee_read_only_sorted_map_real_get_ascending_keys;
	iface->get_ascending_entries = gee_read_only_sorted_map_real_get_ascending_entries;
	iface->get_read_only_view = gee_read_only_sorted_map_real_get_read_only_view;
}

static void
gee_read_only_sorted_map_instance_init (GeeReadOnlySortedMap * self,
                                        gpointer klass)
{
	self->priv = gee_read_only_sorted_map_get_instance_private (self);
}

/**
 * Read-only view for {@link SortedMap} collections.
 *
 * This class decorates any class which implements the {@link SortedMap} interface
 * by making it read only. Any method which normally modify data will throw an
 * error.
 *
 * @see SortedMap
 */
static GType
gee_read_only_sorted_map_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (GeeReadOnlySortedMapClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) gee_read_only_sorted_map_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GeeReadOnlySortedMap), 0, (GInstanceInitFunc) gee_read_only_sorted_map_instance_init, NULL };
	static const GInterfaceInfo gee_sorted_map_info = { (GInterfaceInitFunc) gee_read_only_sorted_map_gee_sorted_map_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
	GType gee_read_only_sorted_map_type_id;
	gee_read_only_sorted_map_type_id = g_type_register_static (GEE_TYPE_READ_ONLY_MAP, "GeeReadOnlySortedMap", &g_define_type_info, 0);
	g_type_add_interface_static (gee_read_only_sorted_map_type_id, GEE_TYPE_SORTED_MAP, &gee_sorted_map_info);
	GeeReadOnlySortedMap_private_offset = g_type_add_instance_private (gee_read_only_sorted_map_type_id, sizeof (GeeReadOnlySortedMapPrivate));
	return gee_read_only_sorted_map_type_id;
}

 G_GNUC_INTERNAL GType
gee_read_only_sorted_map_get_type (void)
{
	static volatile gsize gee_read_only_sorted_map_type_id__once = 0;
	if (g_once_init_enter (&gee_read_only_sorted_map_type_id__once)) {
		GType gee_read_only_sorted_map_type_id;
		gee_read_only_sorted_map_type_id = gee_read_only_sorted_map_get_type_once ();
		g_once_init_leave (&gee_read_only_sorted_map_type_id__once, gee_read_only_sorted_map_type_id);
	}
	return gee_read_only_sorted_map_type_id__once;
}

static void
_vala_gee_read_only_sorted_map_get_property (GObject * object,
                                             guint property_id,
                                             GValue * value,
                                             GParamSpec * pspec)
{
	GeeReadOnlySortedMap * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, GEE_TYPE_READ_ONLY_SORTED_MAP, GeeReadOnlySortedMap);
	switch (property_id) {
		case GEE_READ_ONLY_SORTED_MAP_ASCENDING_KEYS_PROPERTY:
		g_value_take_object (value, gee_sorted_map_get_ascending_keys ((GeeSortedMap*) self));
		break;
		case GEE_READ_ONLY_SORTED_MAP_ASCENDING_ENTRIES_PROPERTY:
		g_value_take_object (value, gee_sorted_map_get_ascending_entries ((GeeSortedMap*) self));
		break;
		case GEE_READ_ONLY_SORTED_MAP_READ_ONLY_VIEW_PROPERTY:
		g_value_take_object (value, gee_sorted_map_get_read_only_view ((GeeSortedMap*) self));
		break;
		case GEE_READ_ONLY_SORTED_MAP_K_TYPE:
		g_value_set_gtype (value, self->priv->k_type);
		break;
		case GEE_READ_ONLY_SORTED_MAP_K_DUP_FUNC:
		g_value_set_pointer (value, self->priv->k_dup_func);
		break;
		case GEE_READ_ONLY_SORTED_MAP_K_DESTROY_FUNC:
		g_value_set_pointer (value, self->priv->k_destroy_func);
		break;
		case GEE_READ_ONLY_SORTED_MAP_V_TYPE:
		g_value_set_gtype (value, self->priv->v_type);
		break;
		case GEE_READ_ONLY_SORTED_MAP_V_DUP_FUNC:
		g_value_set_pointer (value, self->priv->v_dup_func);
		break;
		case GEE_READ_ONLY_SORTED_MAP_V_DESTROY_FUNC:
		g_value_set_pointer (value, self->priv->v_destroy_func);
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

static void
_vala_gee_read_only_sorted_map_set_property (GObject * object,
                                             guint property_id,
                                             const GValue * value,
                                             GParamSpec * pspec)
{
	GeeReadOnlySortedMap * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, GEE_TYPE_READ_ONLY_SORTED_MAP, GeeReadOnlySortedMap);
	switch (property_id) {
		case GEE_READ_ONLY_SORTED_MAP_K_TYPE:
		self->priv->k_type = g_value_get_gtype (value);
		break;
		case GEE_READ_ONLY_SORTED_MAP_K_DUP_FUNC:
		self->priv->k_dup_func = g_value_get_pointer (value);
		break;
		case GEE_READ_ONLY_SORTED_MAP_K_DESTROY_FUNC:
		self->priv->k_destroy_func = g_value_get_pointer (value);
		break;
		case GEE_READ_ONLY_SORTED_MAP_V_TYPE:
		self->priv->v_type = g_value_get_gtype (value);
		break;
		case GEE_READ_ONLY_SORTED_MAP_V_DUP_FUNC:
		self->priv->v_dup_func = g_value_get_pointer (value);
		break;
		case GEE_READ_ONLY_SORTED_MAP_V_DESTROY_FUNC:
		self->priv->v_destroy_func = g_value_get_pointer (value);
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

