/* readonlybidirsortedset.c generated by valac 0.56.3, the Vala compiler
 * generated from readonlybidirsortedset.vala, do not modify */

/* readonlybidirsortedset.vala
 *
 * Copyright (C) 2012  Maciej Piechotka
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Maciej Piechotka <uzytkownik2@gmail.com>
 */

#include <glib-object.h>
#include "gee.h"
#include <glib.h>

#define GEE_TYPE_READ_ONLY_COLLECTION (gee_read_only_collection_get_type ())
#define GEE_READ_ONLY_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_READ_ONLY_COLLECTION, GeeReadOnlyCollection))
#define GEE_READ_ONLY_COLLECTION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_TYPE_READ_ONLY_COLLECTION, GeeReadOnlyCollectionClass))
#define GEE_IS_READ_ONLY_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_READ_ONLY_COLLECTION))
#define GEE_IS_READ_ONLY_COLLECTION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEE_TYPE_READ_ONLY_COLLECTION))
#define GEE_READ_ONLY_COLLECTION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_TYPE_READ_ONLY_COLLECTION, GeeReadOnlyCollectionClass))

typedef struct _GeeReadOnlyCollection GeeReadOnlyCollection;
typedef struct _GeeReadOnlyCollectionClass GeeReadOnlyCollectionClass;
typedef struct _GeeReadOnlyCollectionPrivate GeeReadOnlyCollectionPrivate;

#define GEE_TYPE_READ_ONLY_SET (gee_read_only_set_get_type ())
#define GEE_READ_ONLY_SET(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_READ_ONLY_SET, GeeReadOnlySet))
#define GEE_READ_ONLY_SET_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_TYPE_READ_ONLY_SET, GeeReadOnlySetClass))
#define GEE_IS_READ_ONLY_SET(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_READ_ONLY_SET))
#define GEE_IS_READ_ONLY_SET_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEE_TYPE_READ_ONLY_SET))
#define GEE_READ_ONLY_SET_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_TYPE_READ_ONLY_SET, GeeReadOnlySetClass))

typedef struct _GeeReadOnlySet GeeReadOnlySet;
typedef struct _GeeReadOnlySetClass GeeReadOnlySetClass;
typedef struct _GeeReadOnlySetPrivate GeeReadOnlySetPrivate;

#define GEE_TYPE_READ_ONLY_SORTED_SET (gee_read_only_sorted_set_get_type ())
#define GEE_READ_ONLY_SORTED_SET(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_READ_ONLY_SORTED_SET, GeeReadOnlySortedSet))
#define GEE_READ_ONLY_SORTED_SET_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_TYPE_READ_ONLY_SORTED_SET, GeeReadOnlySortedSetClass))
#define GEE_IS_READ_ONLY_SORTED_SET(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_READ_ONLY_SORTED_SET))
#define GEE_IS_READ_ONLY_SORTED_SET_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEE_TYPE_READ_ONLY_SORTED_SET))
#define GEE_READ_ONLY_SORTED_SET_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_TYPE_READ_ONLY_SORTED_SET, GeeReadOnlySortedSetClass))

typedef struct _GeeReadOnlySortedSet GeeReadOnlySortedSet;
typedef struct _GeeReadOnlySortedSetClass GeeReadOnlySortedSetClass;
typedef struct _GeeReadOnlySortedSetPrivate GeeReadOnlySortedSetPrivate;

#define GEE_TYPE_READ_ONLY_BIDIR_SORTED_SET (gee_read_only_bidir_sorted_set_get_type ())
#define GEE_READ_ONLY_BIDIR_SORTED_SET(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_READ_ONLY_BIDIR_SORTED_SET, GeeReadOnlyBidirSortedSet))
#define GEE_READ_ONLY_BIDIR_SORTED_SET_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_TYPE_READ_ONLY_BIDIR_SORTED_SET, GeeReadOnlyBidirSortedSetClass))
#define GEE_IS_READ_ONLY_BIDIR_SORTED_SET(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_READ_ONLY_BIDIR_SORTED_SET))
#define GEE_IS_READ_ONLY_BIDIR_SORTED_SET_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEE_TYPE_READ_ONLY_BIDIR_SORTED_SET))
#define GEE_READ_ONLY_BIDIR_SORTED_SET_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_TYPE_READ_ONLY_BIDIR_SORTED_SET, GeeReadOnlyBidirSortedSetClass))

typedef struct _GeeReadOnlyBidirSortedSet GeeReadOnlyBidirSortedSet;
typedef struct _GeeReadOnlyBidirSortedSetClass GeeReadOnlyBidirSortedSetClass;
typedef struct _GeeReadOnlyBidirSortedSetPrivate GeeReadOnlyBidirSortedSetPrivate;
enum  {
	GEE_READ_ONLY_BIDIR_SORTED_SET_0_PROPERTY,
	GEE_READ_ONLY_BIDIR_SORTED_SET_G_TYPE,
	GEE_READ_ONLY_BIDIR_SORTED_SET_G_DUP_FUNC,
	GEE_READ_ONLY_BIDIR_SORTED_SET_G_DESTROY_FUNC,
	GEE_READ_ONLY_BIDIR_SORTED_SET_NUM_PROPERTIES
};
static GParamSpec* gee_read_only_bidir_sorted_set_properties[GEE_READ_ONLY_BIDIR_SORTED_SET_NUM_PROPERTIES];

#define GEE_READ_ONLY_COLLECTION_TYPE_ITERATOR (gee_read_only_collection_iterator_get_type ())
#define GEE_READ_ONLY_COLLECTION_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_READ_ONLY_COLLECTION_TYPE_ITERATOR, GeeReadOnlyCollectionIterator))
#define GEE_READ_ONLY_COLLECTION_ITERATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_READ_ONLY_COLLECTION_TYPE_ITERATOR, GeeReadOnlyCollectionIteratorClass))
#define GEE_READ_ONLY_COLLECTION_IS_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_READ_ONLY_COLLECTION_TYPE_ITERATOR))
#define GEE_READ_ONLY_COLLECTION_IS_ITERATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEE_READ_ONLY_COLLECTION_TYPE_ITERATOR))
#define GEE_READ_ONLY_COLLECTION_ITERATOR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_READ_ONLY_COLLECTION_TYPE_ITERATOR, GeeReadOnlyCollectionIteratorClass))

typedef struct _GeeReadOnlyCollectionIterator GeeReadOnlyCollectionIterator;
typedef struct _GeeReadOnlyCollectionIteratorClass GeeReadOnlyCollectionIteratorClass;

#define GEE_READ_ONLY_BIDIR_SORTED_SET_TYPE_BIDIR_ITERATOR (gee_read_only_bidir_sorted_set_bidir_iterator_get_type ())
#define GEE_READ_ONLY_BIDIR_SORTED_SET_BIDIR_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_READ_ONLY_BIDIR_SORTED_SET_TYPE_BIDIR_ITERATOR, GeeReadOnlyBidirSortedSetBidirIterator))
#define GEE_READ_ONLY_BIDIR_SORTED_SET_BIDIR_ITERATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_READ_ONLY_BIDIR_SORTED_SET_TYPE_BIDIR_ITERATOR, GeeReadOnlyBidirSortedSetBidirIteratorClass))
#define GEE_READ_ONLY_BIDIR_SORTED_SET_IS_BIDIR_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_READ_ONLY_BIDIR_SORTED_SET_TYPE_BIDIR_ITERATOR))
#define GEE_READ_ONLY_BIDIR_SORTED_SET_IS_BIDIR_ITERATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEE_READ_ONLY_BIDIR_SORTED_SET_TYPE_BIDIR_ITERATOR))
#define GEE_READ_ONLY_BIDIR_SORTED_SET_BIDIR_ITERATOR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_READ_ONLY_BIDIR_SORTED_SET_TYPE_BIDIR_ITERATOR, GeeReadOnlyBidirSortedSetBidirIteratorClass))

typedef struct _GeeReadOnlyBidirSortedSetBidirIterator GeeReadOnlyBidirSortedSetBidirIterator;
typedef struct _GeeReadOnlyBidirSortedSetBidirIteratorClass GeeReadOnlyBidirSortedSetBidirIteratorClass;
#define _g_object_unref0(var) ((var == NULL) ? NULL : (var = (g_object_unref (var), NULL)))
typedef struct _GeeReadOnlyCollectionIteratorPrivate GeeReadOnlyCollectionIteratorPrivate;
typedef struct _GeeReadOnlyBidirSortedSetBidirIteratorPrivate GeeReadOnlyBidirSortedSetBidirIteratorPrivate;
enum  {
	GEE_READ_ONLY_BIDIR_SORTED_SET_BIDIR_ITERATOR_0_PROPERTY,
	GEE_READ_ONLY_BIDIR_SORTED_SET_BIDIR_ITERATOR_G_TYPE,
	GEE_READ_ONLY_BIDIR_SORTED_SET_BIDIR_ITERATOR_G_DUP_FUNC,
	GEE_READ_ONLY_BIDIR_SORTED_SET_BIDIR_ITERATOR_G_DESTROY_FUNC,
	GEE_READ_ONLY_BIDIR_SORTED_SET_BIDIR_ITERATOR_NUM_PROPERTIES
};
static GParamSpec* gee_read_only_bidir_sorted_set_bidir_iterator_properties[GEE_READ_ONLY_BIDIR_SORTED_SET_BIDIR_ITERATOR_NUM_PROPERTIES];

struct _GeeReadOnlyCollection {
	GObject parent_instance;
	GeeReadOnlyCollectionPrivate * priv;
	GeeCollection* _collection;
};

struct _GeeReadOnlyCollectionClass {
	GObjectClass parent_class;
	GeeCollection* (*get_read_only_view) (GeeReadOnlyCollection* self);
};

struct _GeeReadOnlySet {
	GeeReadOnlyCollection parent_instance;
	GeeReadOnlySetPrivate * priv;
};

struct _GeeReadOnlySetClass {
	GeeReadOnlyCollectionClass parent_class;
	GeeSet* (*get_read_only_view) (GeeReadOnlySet* self);
};

struct _GeeReadOnlySortedSet {
	GeeReadOnlySet parent_instance;
	GeeReadOnlySortedSetPrivate * priv;
};

struct _GeeReadOnlySortedSetClass {
	GeeReadOnlySetClass parent_class;
};

struct _GeeReadOnlyBidirSortedSet {
	GeeReadOnlySortedSet parent_instance;
	GeeReadOnlyBidirSortedSetPrivate * priv;
};

struct _GeeReadOnlyBidirSortedSetClass {
	GeeReadOnlySortedSetClass parent_class;
};

struct _GeeReadOnlyBidirSortedSetPrivate {
	GType g_type;
	GBoxedCopyFunc g_dup_func;
	GDestroyNotify g_destroy_func;
};

struct _GeeReadOnlyCollectionIterator {
	GObject parent_instance;
	GeeReadOnlyCollectionIteratorPrivate * priv;
	GeeIterator* _iter;
};

struct _GeeReadOnlyCollectionIteratorClass {
	GObjectClass parent_class;
};

struct _GeeReadOnlyBidirSortedSetBidirIterator {
	GeeReadOnlyCollectionIterator parent_instance;
	GeeReadOnlyBidirSortedSetBidirIteratorPrivate * priv;
};

struct _GeeReadOnlyBidirSortedSetBidirIteratorClass {
	GeeReadOnlyCollectionIteratorClass parent_class;
};

struct _GeeReadOnlyBidirSortedSetBidirIteratorPrivate {
	GType g_type;
	GBoxedCopyFunc g_dup_func;
	GDestroyNotify g_destroy_func;
};

static gint GeeReadOnlyBidirSortedSet_private_offset;
static gpointer gee_read_only_bidir_sorted_set_parent_class = NULL;
static gint GeeReadOnlyBidirSortedSetBidirIterator_private_offset;
static gpointer gee_read_only_bidir_sorted_set_bidir_iterator_parent_class = NULL;
static GeeBidirIteratorIface * gee_read_only_bidir_sorted_set_bidir_iterator_gee_bidir_iterator_parent_iface = NULL;
static GeeBidirSortedSetIface * gee_read_only_bidir_sorted_set_gee_bidir_sorted_set_parent_iface = NULL;

 G_GNUC_INTERNAL GType gee_read_only_collection_get_type (void) G_GNUC_CONST  G_GNUC_UNUSED ;
 G_GNUC_INTERNAL GType gee_read_only_set_get_type (void) G_GNUC_CONST  G_GNUC_UNUSED ;
 G_GNUC_INTERNAL GType gee_read_only_sorted_set_get_type (void) G_GNUC_CONST  G_GNUC_UNUSED ;
 G_GNUC_INTERNAL GType gee_read_only_bidir_sorted_set_get_type (void) G_GNUC_CONST  G_GNUC_UNUSED ;
 G_GNUC_INTERNAL GeeReadOnlyBidirSortedSet* gee_read_only_bidir_sorted_set_new (GType g_type,
                                                               GBoxedCopyFunc g_dup_func,
                                                               GDestroyNotify g_destroy_func,
                                                               GeeBidirSortedSet* set);
 G_GNUC_INTERNAL GeeReadOnlyBidirSortedSet* gee_read_only_bidir_sorted_set_construct (GType object_type,
                                                                     GType g_type,
                                                                     GBoxedCopyFunc g_dup_func,
                                                                     GDestroyNotify g_destroy_func,
                                                                     GeeBidirSortedSet* set);
 G_GNUC_INTERNAL GeeReadOnlySortedSet* gee_read_only_sorted_set_new (GType g_type,
                                                    GBoxedCopyFunc g_dup_func,
                                                    GDestroyNotify g_destroy_func,
                                                    GeeSortedSet* set);
 G_GNUC_INTERNAL GeeReadOnlySortedSet* gee_read_only_sorted_set_construct (GType object_type,
                                                          GType g_type,
                                                          GBoxedCopyFunc g_dup_func,
                                                          GDestroyNotify g_destroy_func,
                                                          GeeSortedSet* set);
static GeeBidirIterator* gee_read_only_bidir_sorted_set_real_bidir_iterator (GeeBidirSortedSet* base);
 G_GNUC_INTERNAL GeeReadOnlyBidirSortedSetBidirIterator* gee_read_only_bidir_sorted_set_bidir_iterator_new (GType g_type,
                                                                                           GBoxedCopyFunc g_dup_func,
                                                                                           GDestroyNotify g_destroy_func,
                                                                                           GeeBidirIterator* iterator);
 G_GNUC_INTERNAL GeeReadOnlyBidirSortedSetBidirIterator* gee_read_only_bidir_sorted_set_bidir_iterator_construct (GType object_type,
                                                                                                 GType g_type,
                                                                                                 GBoxedCopyFunc g_dup_func,
                                                                                                 GDestroyNotify g_destroy_func,
                                                                                                 GeeBidirIterator* iterator);
VALA_EXTERN GType gee_read_only_collection_iterator_get_type (void) G_GNUC_CONST ;
VALA_EXTERN GType gee_read_only_bidir_sorted_set_bidir_iterator_get_type (void) G_GNUC_CONST ;
 G_GNUC_INTERNAL GeeReadOnlyCollectionIterator* gee_read_only_collection_iterator_new (GType g_type,
                                                                      GBoxedCopyFunc g_dup_func,
                                                                      GDestroyNotify g_destroy_func,
                                                                      GeeIterator* iterator);
 G_GNUC_INTERNAL GeeReadOnlyCollectionIterator* gee_read_only_collection_iterator_construct (GType object_type,
                                                                            GType g_type,
                                                                            GBoxedCopyFunc g_dup_func,
                                                                            GDestroyNotify g_destroy_func,
                                                                            GeeIterator* iterator);
static gboolean gee_read_only_bidir_sorted_set_bidir_iterator_real_first (GeeBidirIterator* base);
static gboolean gee_read_only_bidir_sorted_set_bidir_iterator_real_previous (GeeBidirIterator* base);
static gboolean gee_read_only_bidir_sorted_set_bidir_iterator_real_has_previous (GeeBidirIterator* base);
static gboolean gee_read_only_bidir_sorted_set_bidir_iterator_real_last (GeeBidirIterator* base);
static GType gee_read_only_bidir_sorted_set_bidir_iterator_get_type_once (void);
static void _vala_gee_read_only_bidir_sorted_set_bidir_iterator_get_property (GObject * object,
                                                                       guint property_id,
                                                                       GValue * value,
                                                                       GParamSpec * pspec);
static void _vala_gee_read_only_bidir_sorted_set_bidir_iterator_set_property (GObject * object,
                                                                       guint property_id,
                                                                       const GValue * value,
                                                                       GParamSpec * pspec);
static GType gee_read_only_bidir_sorted_set_get_type_once (void);
static void _vala_gee_read_only_bidir_sorted_set_get_property (GObject * object,
                                                        guint property_id,
                                                        GValue * value,
                                                        GParamSpec * pspec);
static void _vala_gee_read_only_bidir_sorted_set_set_property (GObject * object,
                                                        guint property_id,
                                                        const GValue * value,
                                                        GParamSpec * pspec);

static inline gpointer
gee_read_only_bidir_sorted_set_get_instance_private (GeeReadOnlyBidirSortedSet* self)
{
	return G_STRUCT_MEMBER_P (self, GeeReadOnlyBidirSortedSet_private_offset);
}

/**
 * Constructs a read-only set that mirrors the content of the specified set.
 *
 * @param set the set to decorate.
 */
 G_GNUC_INTERNAL GeeReadOnlyBidirSortedSet*
gee_read_only_bidir_sorted_set_construct (GType object_type,
                                          GType g_type,
                                          GBoxedCopyFunc g_dup_func,
                                          GDestroyNotify g_destroy_func,
                                          GeeBidirSortedSet* set)
{
	GeeReadOnlyBidirSortedSet * self = NULL;
	g_return_val_if_fail (set != NULL, NULL);
	self = (GeeReadOnlyBidirSortedSet*) gee_read_only_sorted_set_construct (object_type, g_type, (GBoxedCopyFunc) g_dup_func, (GDestroyNotify) g_destroy_func, (GeeSortedSet*) set);
	self->priv->g_type = g_type;
	self->priv->g_dup_func = g_dup_func;
	self->priv->g_destroy_func = g_destroy_func;
	return self;
}

 G_GNUC_INTERNAL GeeReadOnlyBidirSortedSet*
gee_read_only_bidir_sorted_set_new (GType g_type,
                                    GBoxedCopyFunc g_dup_func,
                                    GDestroyNotify g_destroy_func,
                                    GeeBidirSortedSet* set)
{
	return gee_read_only_bidir_sorted_set_construct (GEE_TYPE_READ_ONLY_BIDIR_SORTED_SET, g_type, g_dup_func, g_destroy_func, set);
}

/**
 * {@inheritDoc}
 */
static GeeBidirIterator*
gee_read_only_bidir_sorted_set_real_bidir_iterator (GeeBidirSortedSet* base)
{
	GeeReadOnlyBidirSortedSet * self;
	GeeCollection* _tmp0_;
	GeeBidirIterator* _tmp1_;
	GeeBidirIterator* _tmp2_;
	GeeReadOnlyBidirSortedSetBidirIterator* _tmp3_;
	GeeBidirIterator* _tmp4_;
	GeeBidirIterator* result;
	self = (GeeReadOnlyBidirSortedSet*) base;
	_tmp0_ = ((GeeReadOnlyCollection*) self)->_collection;
	_tmp1_ = gee_bidir_sorted_set_bidir_iterator (G_TYPE_CHECK_INSTANCE_CAST (_tmp0_, GEE_TYPE_BIDIR_SORTED_SET, GeeBidirSortedSet));
	_tmp2_ = _tmp1_;
	_tmp3_ = gee_read_only_bidir_sorted_set_bidir_iterator_new (self->priv->g_type, (GBoxedCopyFunc) self->priv->g_dup_func, (GDestroyNotify) self->priv->g_destroy_func, _tmp2_);
	_tmp4_ = (GeeBidirIterator*) _tmp3_;
	_g_object_unref0 (_tmp2_);
	result = _tmp4_;
	return result;
}

static inline gpointer
gee_read_only_bidir_sorted_set_bidir_iterator_get_instance_private (GeeReadOnlyBidirSortedSetBidirIterator* self)
{
	return G_STRUCT_MEMBER_P (self, GeeReadOnlyBidirSortedSetBidirIterator_private_offset);
}

 G_GNUC_INTERNAL GeeReadOnlyBidirSortedSetBidirIterator*
gee_read_only_bidir_sorted_set_bidir_iterator_construct (GType object_type,
                                                         GType g_type,
                                                         GBoxedCopyFunc g_dup_func,
                                                         GDestroyNotify g_destroy_func,
                                                         GeeBidirIterator* iterator)
{
	GeeReadOnlyBidirSortedSetBidirIterator * self = NULL;
	g_return_val_if_fail (iterator != NULL, NULL);
	self = (GeeReadOnlyBidirSortedSetBidirIterator*) gee_read_only_collection_iterator_construct (object_type, g_type, (GBoxedCopyFunc) g_dup_func, (GDestroyNotify) g_destroy_func, (GeeIterator*) iterator);
	self->priv->g_type = g_type;
	self->priv->g_dup_func = g_dup_func;
	self->priv->g_destroy_func = g_destroy_func;
	return self;
}

 G_GNUC_INTERNAL GeeReadOnlyBidirSortedSetBidirIterator*
gee_read_only_bidir_sorted_set_bidir_iterator_new (GType g_type,
                                                   GBoxedCopyFunc g_dup_func,
                                                   GDestroyNotify g_destroy_func,
                                                   GeeBidirIterator* iterator)
{
	return gee_read_only_bidir_sorted_set_bidir_iterator_construct (GEE_READ_ONLY_BIDIR_SORTED_SET_TYPE_BIDIR_ITERATOR, g_type, g_dup_func, g_destroy_func, iterator);
}

static gboolean
gee_read_only_bidir_sorted_set_bidir_iterator_real_first (GeeBidirIterator* base)
{
	GeeReadOnlyBidirSortedSetBidirIterator * self;
	GeeIterator* _tmp0_;
	gboolean result;
	self = (GeeReadOnlyBidirSortedSetBidirIterator*) base;
	_tmp0_ = ((GeeReadOnlyCollectionIterator*) self)->_iter;
	result = gee_bidir_iterator_first (G_TYPE_CHECK_INSTANCE_CAST (_tmp0_, GEE_TYPE_BIDIR_ITERATOR, GeeBidirIterator));
	return result;
}

static gboolean
gee_read_only_bidir_sorted_set_bidir_iterator_real_previous (GeeBidirIterator* base)
{
	GeeReadOnlyBidirSortedSetBidirIterator * self;
	GeeIterator* _tmp0_;
	gboolean result;
	self = (GeeReadOnlyBidirSortedSetBidirIterator*) base;
	_tmp0_ = ((GeeReadOnlyCollectionIterator*) self)->_iter;
	result = gee_bidir_iterator_previous (G_TYPE_CHECK_INSTANCE_CAST (_tmp0_, GEE_TYPE_BIDIR_ITERATOR, GeeBidirIterator));
	return result;
}

static gboolean
gee_read_only_bidir_sorted_set_bidir_iterator_real_has_previous (GeeBidirIterator* base)
{
	GeeReadOnlyBidirSortedSetBidirIterator * self;
	GeeIterator* _tmp0_;
	gboolean result;
	self = (GeeReadOnlyBidirSortedSetBidirIterator*) base;
	_tmp0_ = ((GeeReadOnlyCollectionIterator*) self)->_iter;
	result = gee_bidir_iterator_has_previous (G_TYPE_CHECK_INSTANCE_CAST (_tmp0_, GEE_TYPE_BIDIR_ITERATOR, GeeBidirIterator));
	return result;
}

static gboolean
gee_read_only_bidir_sorted_set_bidir_iterator_real_last (GeeBidirIterator* base)
{
	GeeReadOnlyBidirSortedSetBidirIterator * self;
	GeeIterator* _tmp0_;
	gboolean result;
	self = (GeeReadOnlyBidirSortedSetBidirIterator*) base;
	_tmp0_ = ((GeeReadOnlyCollectionIterator*) self)->_iter;
	result = gee_bidir_iterator_last (G_TYPE_CHECK_INSTANCE_CAST (_tmp0_, GEE_TYPE_BIDIR_ITERATOR, GeeBidirIterator));
	return result;
}

static void
gee_read_only_bidir_sorted_set_bidir_iterator_class_init (GeeReadOnlyBidirSortedSetBidirIteratorClass * klass,
                                                          gpointer klass_data)
{
	gee_read_only_bidir_sorted_set_bidir_iterator_parent_class = g_type_class_peek_parent (klass);
	g_type_class_adjust_private_offset (klass, &GeeReadOnlyBidirSortedSetBidirIterator_private_offset);
	G_OBJECT_CLASS (klass)->get_property = _vala_gee_read_only_bidir_sorted_set_bidir_iterator_get_property;
	G_OBJECT_CLASS (klass)->set_property = _vala_gee_read_only_bidir_sorted_set_bidir_iterator_set_property;
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_READ_ONLY_BIDIR_SORTED_SET_BIDIR_ITERATOR_G_TYPE, g_param_spec_gtype ("g-type", "type", "type", G_TYPE_NONE, G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_READ_ONLY_BIDIR_SORTED_SET_BIDIR_ITERATOR_G_DUP_FUNC, g_param_spec_pointer ("g-dup-func", "dup func", "dup func", G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_READ_ONLY_BIDIR_SORTED_SET_BIDIR_ITERATOR_G_DESTROY_FUNC, g_param_spec_pointer ("g-destroy-func", "destroy func", "destroy func", G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
}

static GType
gee_read_only_bidir_sorted_set_bidir_iterator_gee_bidir_iterator_get_g_type (GeeReadOnlyBidirSortedSetBidirIterator* self)
{
	return self->priv->g_type;
}

static GBoxedCopyFunc
gee_read_only_bidir_sorted_set_bidir_iterator_gee_bidir_iterator_get_g_dup_func (GeeReadOnlyBidirSortedSetBidirIterator* self)
{
	return self->priv->g_dup_func;
}

static GDestroyNotify
gee_read_only_bidir_sorted_set_bidir_iterator_gee_bidir_iterator_get_g_destroy_func (GeeReadOnlyBidirSortedSetBidirIterator* self)
{
	return self->priv->g_destroy_func;
}

static void
gee_read_only_bidir_sorted_set_bidir_iterator_gee_bidir_iterator_interface_init (GeeBidirIteratorIface * iface,
                                                                                 gpointer iface_data)
{
	gee_read_only_bidir_sorted_set_bidir_iterator_gee_bidir_iterator_parent_iface = g_type_interface_peek_parent (iface);
	iface->first = (gboolean (*) (GeeBidirIterator*)) gee_read_only_bidir_sorted_set_bidir_iterator_real_first;
	iface->previous = (gboolean (*) (GeeBidirIterator*)) gee_read_only_bidir_sorted_set_bidir_iterator_real_previous;
	iface->has_previous = (gboolean (*) (GeeBidirIterator*)) gee_read_only_bidir_sorted_set_bidir_iterator_real_has_previous;
	iface->last = (gboolean (*) (GeeBidirIterator*)) gee_read_only_bidir_sorted_set_bidir_iterator_real_last;
	iface->get_g_type = (GType (*) (GeeBidirIterator *)) gee_read_only_bidir_sorted_set_bidir_iterator_gee_bidir_iterator_get_g_type;
	iface->get_g_dup_func = (GBoxedCopyFunc (*) (GeeBidirIterator *)) gee_read_only_bidir_sorted_set_bidir_iterator_gee_bidir_iterator_get_g_dup_func;
	iface->get_g_destroy_func = (GDestroyNotify (*) (GeeBidirIterator *)) gee_read_only_bidir_sorted_set_bidir_iterator_gee_bidir_iterator_get_g_destroy_func;
}

static void
gee_read_only_bidir_sorted_set_bidir_iterator_instance_init (GeeReadOnlyBidirSortedSetBidirIterator * self,
                                                             gpointer klass)
{
	self->priv = gee_read_only_bidir_sorted_set_bidir_iterator_get_instance_private (self);
}

static GType
gee_read_only_bidir_sorted_set_bidir_iterator_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (GeeReadOnlyBidirSortedSetBidirIteratorClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) gee_read_only_bidir_sorted_set_bidir_iterator_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GeeReadOnlyBidirSortedSetBidirIterator), 0, (GInstanceInitFunc) gee_read_only_bidir_sorted_set_bidir_iterator_instance_init, NULL };
	static const GInterfaceInfo gee_bidir_iterator_info = { (GInterfaceInitFunc) gee_read_only_bidir_sorted_set_bidir_iterator_gee_bidir_iterator_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
	GType gee_read_only_bidir_sorted_set_bidir_iterator_type_id;
	gee_read_only_bidir_sorted_set_bidir_iterator_type_id = g_type_register_static (GEE_READ_ONLY_COLLECTION_TYPE_ITERATOR, "GeeReadOnlyBidirSortedSetBidirIterator", &g_define_type_info, 0);
	g_type_add_interface_static (gee_read_only_bidir_sorted_set_bidir_iterator_type_id, GEE_TYPE_BIDIR_ITERATOR, &gee_bidir_iterator_info);
	GeeReadOnlyBidirSortedSetBidirIterator_private_offset = g_type_add_instance_private (gee_read_only_bidir_sorted_set_bidir_iterator_type_id, sizeof (GeeReadOnlyBidirSortedSetBidirIteratorPrivate));
	return gee_read_only_bidir_sorted_set_bidir_iterator_type_id;
}

GType
gee_read_only_bidir_sorted_set_bidir_iterator_get_type (void)
{
	static volatile gsize gee_read_only_bidir_sorted_set_bidir_iterator_type_id__once = 0;
	if (g_once_init_enter (&gee_read_only_bidir_sorted_set_bidir_iterator_type_id__once)) {
		GType gee_read_only_bidir_sorted_set_bidir_iterator_type_id;
		gee_read_only_bidir_sorted_set_bidir_iterator_type_id = gee_read_only_bidir_sorted_set_bidir_iterator_get_type_once ();
		g_once_init_leave (&gee_read_only_bidir_sorted_set_bidir_iterator_type_id__once, gee_read_only_bidir_sorted_set_bidir_iterator_type_id);
	}
	return gee_read_only_bidir_sorted_set_bidir_iterator_type_id__once;
}

static void
_vala_gee_read_only_bidir_sorted_set_bidir_iterator_get_property (GObject * object,
                                                                  guint property_id,
                                                                  GValue * value,
                                                                  GParamSpec * pspec)
{
	GeeReadOnlyBidirSortedSetBidirIterator * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, GEE_READ_ONLY_BIDIR_SORTED_SET_TYPE_BIDIR_ITERATOR, GeeReadOnlyBidirSortedSetBidirIterator);
	switch (property_id) {
		case GEE_READ_ONLY_BIDIR_SORTED_SET_BIDIR_ITERATOR_G_TYPE:
		g_value_set_gtype (value, self->priv->g_type);
		break;
		case GEE_READ_ONLY_BIDIR_SORTED_SET_BIDIR_ITERATOR_G_DUP_FUNC:
		g_value_set_pointer (value, self->priv->g_dup_func);
		break;
		case GEE_READ_ONLY_BIDIR_SORTED_SET_BIDIR_ITERATOR_G_DESTROY_FUNC:
		g_value_set_pointer (value, self->priv->g_destroy_func);
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

static void
_vala_gee_read_only_bidir_sorted_set_bidir_iterator_set_property (GObject * object,
                                                                  guint property_id,
                                                                  const GValue * value,
                                                                  GParamSpec * pspec)
{
	GeeReadOnlyBidirSortedSetBidirIterator * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, GEE_READ_ONLY_BIDIR_SORTED_SET_TYPE_BIDIR_ITERATOR, GeeReadOnlyBidirSortedSetBidirIterator);
	switch (property_id) {
		case GEE_READ_ONLY_BIDIR_SORTED_SET_BIDIR_ITERATOR_G_TYPE:
		self->priv->g_type = g_value_get_gtype (value);
		break;
		case GEE_READ_ONLY_BIDIR_SORTED_SET_BIDIR_ITERATOR_G_DUP_FUNC:
		self->priv->g_dup_func = g_value_get_pointer (value);
		break;
		case GEE_READ_ONLY_BIDIR_SORTED_SET_BIDIR_ITERATOR_G_DESTROY_FUNC:
		self->priv->g_destroy_func = g_value_get_pointer (value);
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

static void
gee_read_only_bidir_sorted_set_class_init (GeeReadOnlyBidirSortedSetClass * klass,
                                           gpointer klass_data)
{
	gee_read_only_bidir_sorted_set_parent_class = g_type_class_peek_parent (klass);
	g_type_class_adjust_private_offset (klass, &GeeReadOnlyBidirSortedSet_private_offset);
	G_OBJECT_CLASS (klass)->get_property = _vala_gee_read_only_bidir_sorted_set_get_property;
	G_OBJECT_CLASS (klass)->set_property = _vala_gee_read_only_bidir_sorted_set_set_property;
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_READ_ONLY_BIDIR_SORTED_SET_G_TYPE, g_param_spec_gtype ("g-type", "type", "type", G_TYPE_NONE, G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_READ_ONLY_BIDIR_SORTED_SET_G_DUP_FUNC, g_param_spec_pointer ("g-dup-func", "dup func", "dup func", G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_READ_ONLY_BIDIR_SORTED_SET_G_DESTROY_FUNC, g_param_spec_pointer ("g-destroy-func", "destroy func", "destroy func", G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
}

static GType
gee_read_only_bidir_sorted_set_gee_bidir_sorted_set_get_g_type (GeeReadOnlyBidirSortedSet* self)
{
	return self->priv->g_type;
}

static GBoxedCopyFunc
gee_read_only_bidir_sorted_set_gee_bidir_sorted_set_get_g_dup_func (GeeReadOnlyBidirSortedSet* self)
{
	return self->priv->g_dup_func;
}

static GDestroyNotify
gee_read_only_bidir_sorted_set_gee_bidir_sorted_set_get_g_destroy_func (GeeReadOnlyBidirSortedSet* self)
{
	return self->priv->g_destroy_func;
}

static void
gee_read_only_bidir_sorted_set_gee_bidir_sorted_set_interface_init (GeeBidirSortedSetIface * iface,
                                                                    gpointer iface_data)
{
	gee_read_only_bidir_sorted_set_gee_bidir_sorted_set_parent_iface = g_type_interface_peek_parent (iface);
	iface->bidir_iterator = (GeeBidirIterator* (*) (GeeBidirSortedSet*)) gee_read_only_bidir_sorted_set_real_bidir_iterator;
	iface->get_g_type = (GType (*) (GeeBidirSortedSet *)) gee_read_only_bidir_sorted_set_gee_bidir_sorted_set_get_g_type;
	iface->get_g_dup_func = (GBoxedCopyFunc (*) (GeeBidirSortedSet *)) gee_read_only_bidir_sorted_set_gee_bidir_sorted_set_get_g_dup_func;
	iface->get_g_destroy_func = (GDestroyNotify (*) (GeeBidirSortedSet *)) gee_read_only_bidir_sorted_set_gee_bidir_sorted_set_get_g_destroy_func;
	iface->get_read_only_view = (GeeBidirSortedSet* (*) (GeeBidirSortedSet *)) gee_sorted_set_get_read_only_view;
}

static void
gee_read_only_bidir_sorted_set_instance_init (GeeReadOnlyBidirSortedSet * self,
                                              gpointer klass)
{
	self->priv = gee_read_only_bidir_sorted_set_get_instance_private (self);
}

/**
 * Read-only view for {@link BidirSortedSet} collections.
 *
 * This class decorates any class which implements the {@link BidirSortedSet}
 * interface by making it read only. Any method which normally modify data will
 * throw an error.
 *
 * @see BidirSortedSet
 */
static GType
gee_read_only_bidir_sorted_set_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (GeeReadOnlyBidirSortedSetClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) gee_read_only_bidir_sorted_set_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GeeReadOnlyBidirSortedSet), 0, (GInstanceInitFunc) gee_read_only_bidir_sorted_set_instance_init, NULL };
	static const GInterfaceInfo gee_bidir_sorted_set_info = { (GInterfaceInitFunc) gee_read_only_bidir_sorted_set_gee_bidir_sorted_set_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
	GType gee_read_only_bidir_sorted_set_type_id;
	gee_read_only_bidir_sorted_set_type_id = g_type_register_static (GEE_TYPE_READ_ONLY_SORTED_SET, "GeeReadOnlyBidirSortedSet", &g_define_type_info, 0);
	g_type_add_interface_static (gee_read_only_bidir_sorted_set_type_id, GEE_TYPE_BIDIR_SORTED_SET, &gee_bidir_sorted_set_info);
	GeeReadOnlyBidirSortedSet_private_offset = g_type_add_instance_private (gee_read_only_bidir_sorted_set_type_id, sizeof (GeeReadOnlyBidirSortedSetPrivate));
	return gee_read_only_bidir_sorted_set_type_id;
}

 G_GNUC_INTERNAL GType
gee_read_only_bidir_sorted_set_get_type (void)
{
	static volatile gsize gee_read_only_bidir_sorted_set_type_id__once = 0;
	if (g_once_init_enter (&gee_read_only_bidir_sorted_set_type_id__once)) {
		GType gee_read_only_bidir_sorted_set_type_id;
		gee_read_only_bidir_sorted_set_type_id = gee_read_only_bidir_sorted_set_get_type_once ();
		g_once_init_leave (&gee_read_only_bidir_sorted_set_type_id__once, gee_read_only_bidir_sorted_set_type_id);
	}
	return gee_read_only_bidir_sorted_set_type_id__once;
}

static void
_vala_gee_read_only_bidir_sorted_set_get_property (GObject * object,
                                                   guint property_id,
                                                   GValue * value,
                                                   GParamSpec * pspec)
{
	GeeReadOnlyBidirSortedSet * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, GEE_TYPE_READ_ONLY_BIDIR_SORTED_SET, GeeReadOnlyBidirSortedSet);
	switch (property_id) {
		case GEE_READ_ONLY_BIDIR_SORTED_SET_G_TYPE:
		g_value_set_gtype (value, self->priv->g_type);
		break;
		case GEE_READ_ONLY_BIDIR_SORTED_SET_G_DUP_FUNC:
		g_value_set_pointer (value, self->priv->g_dup_func);
		break;
		case GEE_READ_ONLY_BIDIR_SORTED_SET_G_DESTROY_FUNC:
		g_value_set_pointer (value, self->priv->g_destroy_func);
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

static void
_vala_gee_read_only_bidir_sorted_set_set_property (GObject * object,
                                                   guint property_id,
                                                   const GValue * value,
                                                   GParamSpec * pspec)
{
	GeeReadOnlyBidirSortedSet * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, GEE_TYPE_READ_ONLY_BIDIR_SORTED_SET, GeeReadOnlyBidirSortedSet);
	switch (property_id) {
		case GEE_READ_ONLY_BIDIR_SORTED_SET_G_TYPE:
		self->priv->g_type = g_value_get_gtype (value);
		break;
		case GEE_READ_ONLY_BIDIR_SORTED_SET_G_DUP_FUNC:
		self->priv->g_dup_func = g_value_get_pointer (value);
		break;
		case GEE_READ_ONLY_BIDIR_SORTED_SET_G_DESTROY_FUNC:
		self->priv->g_destroy_func = g_value_get_pointer (value);
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

